@@ -3,8 +3,7 @@
 eval 'exec /usr/bin/perl -w -S $0 ${1+"$@"}'
     if 0; # not running under some shell
 use strict;
-# $Id: translate_seq.PLS,v 1.4 2005/01/24 02:31:57 jason Exp $
-
+# $Id: translate_seq.PLS 15088 2008-12-04 02:49:09Z bosborne $
 
 =head1 NAME
 
@@ -26,8 +25,8 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
 =head2 Reporting Bugs
 
@@ -35,7 +34,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 email or the web:
 
- http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
@@ -63,3 +62,5 @@ while( (my $seq = $seqin->next_seq()) ) {
 	my $pseq = $seq->translate();
 	$seqout->write_seq($pseq);
 }
+
+__END__
