@@ -1,7 +1,9 @@
-# $Id: AlignI.pm,v 1.9 2004/05/13 15:33:24 avilella Exp $
+# $Id: AlignI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Align::AlignI
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
@@ -19,9 +21,9 @@ Bio::Align::AlignI - An interface for describing sequence alignments.
   # get a Bio::Align::AlignI somehow - typically using Bio::AlignIO system
   # some descriptors
   print $aln->length, "\n";
-  print $aln->no_residues, "\n";
+  print $aln->num_residues, "\n";
   print $aln->is_flush, "\n";
-  print $aln->no_sequences, "\n";
+  print $aln->num_sequences, "\n";
   print $aln->percentage_identity, "\n";
   print $aln->consensus_string(50), "\n";
 
@@ -49,17 +51,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via
-email or the web:
+of the bugs and their resolution. Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -68,7 +80,7 @@ Email jason@bioperl.org
 =head1 CONTRIBUTORS
 
 Ewan Birney, birney@ebi.ac.uk
-Heikki Lehvaslaiho, heikki@ebi.ac.uk
+Heikki Lehvaslaiho, heikki-at-bioperl-dot-org
 
 =head1 APPENDIX
 
@@ -82,12 +94,10 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Align::AlignI;
-use vars qw(@ISA);
 use strict;
 
-use Bio::Root::RootI;
 
-@ISA = qw(Bio::Root::RootI);
+use base qw(Bio::Root::RootI);
 
 =head1 Modifier methods
 
@@ -100,7 +110,7 @@ sequences.
  Usage     : $myalign->add_seq($newseq);
  Function  : Adds another sequence to the alignment. *Does not* align
              it - just adds it to the hashes.
- Returns   : nothing
+ Returns   : None
  Argument  : a Bio::LocatableSeq object
              order (optional)
 
@@ -157,10 +167,10 @@ sub purge {
  Usage     : $ali->sort_alphabetically
  Function  : 
 
-             Changes the order of the alignemnt to alphabetical on name 
+             Changes the order of the alignment to alphabetical on name 
              followed by numerical by number.
 
- Returns   : 
+ Returns   : an array
  Argument  : 
 
 =cut
@@ -237,10 +247,10 @@ sub each_seq_with_id {
 
              Gets a sequence based on its position in the alignment.
              Numbering starts from 1.  Sequence positions larger than
-             no_sequences() will thow an error.
+             num_sequences() will throw an error.
 
  Returns   : a Bio::LocatableSeq object
- Argument  : positive integer for the sequence osition
+ Argument  : positive integer for the sequence position
 
 =cut
 
@@ -262,7 +272,7 @@ current MSE.
 
              Creates a new alignment from a continuous subset of
              sequences.  Numbering starts from 1.  Sequence positions
-             larger than no_sequences() will thow an error.
+             larger than num_sequences() will throw an error.
 
  Returns   : a Bio::SimpleAlign object
  Argument  : positive integer for the first sequence
@@ -284,7 +294,7 @@ sub select {
 
              Creates a new alignment from a subset of
              sequences.  Numbering starts from 1.  Sequence positions
-             larger than no_sequences() will thow an error.
+             larger than num_sequences() will throw an error.
 
  Returns   : a Bio::SimpleAlign object
  Args      : array of integers for the sequences
@@ -321,7 +331,7 @@ sub slice {
 
 =head1 Change sequences within the MSE
 
-These methods affect characters in all sequences without changeing the
+These methods affect characters in all sequences without changing the
 alignment.
 
 
@@ -334,11 +344,11 @@ alignment.
              Does a s/$arg1/$arg2/ on the sequences. Useful for gap
              characters
 
-             Notice that the from (arg1) is interpretted as a regex,
+             Notice that the from (arg1) is interpreted as a regex,
              so be careful about quoting meta characters (eg
              $ali->map_chars('.','-') wont do what you want)
 
- Returns   : 
+ Returns   : None
  Argument  : 'from' rexexp
              'to' string
 
@@ -520,8 +530,8 @@ These read only methods describe the MSE in various ways.
  Title     : consensus_string
  Usage     : $str = $ali->consensus_string($threshold_percent)
  Function  : Makes a strict consensus 
- Returns   : 
- Argument  : Optional treshold ranging from 0 to 100.
+ Returns   : consensus string
+ Argument  : Optional threshold ranging from 0 to 100.
              The consensus residue has to appear at least threshold %
              of the sequences at a given location, otherwise a '?'
              character will be placed at that location.
@@ -587,7 +597,7 @@ sub is_flush {
  Usage     : $len = $ali->length() 
  Function  : Returns the maximum length of the alignment.
              To be sure the alignment is a block, use is_flush
- Returns   : 
+ Returns   : integer
  Argument  : 
 
 =cut
@@ -597,10 +607,10 @@ sub length {
     $self->throw_not_implemented();
 }
 
-=head2 maxdisplayname_length
+=head2 maxname_length
 
- Title     : maxdisplayname_length
- Usage     : $ali->maxdisplayname_length()
+ Title     : maxname_length
+ Usage     : $ali->maxname_length()
  Function  : 
 
              Gets the maximum length of the displayname in the
@@ -616,32 +626,34 @@ sub maxname_length {
     $self->throw_not_implemented();
 }
 
-=head2 no_residues
+=head2 num_residues
 
- Title     : no_residues
- Usage     : $no = $ali->no_residues
+ Title     : num_residues
+ Usage     : $no = $ali->num_residues
  Function  : number of residues in total in the alignment
  Returns   : integer
- Argument  : 
+ Argument  :
+ Note      : replaces no_residues
 
 =cut
 
-sub no_residues {
+sub num_residues {
     my ($self) = @_;
     $self->throw_not_implemented();
 }
 
-=head2 no_sequences
+=head2 num_sequences
 
- Title     : no_sequences
- Usage     : $depth = $ali->no_sequences
+ Title     : num_sequences
+ Usage     : $depth = $ali->num_sequences
  Function  : number of sequence in the sequence alignment
  Returns   : integer
  Argument  : None
+ Note      : replaces no_sequences
 
 =cut
 
-sub no_sequences {
+sub num_sequences {
     my ($self) = @_;
     $self->throw_not_implemented();
 }
@@ -652,20 +664,20 @@ sub no_sequences {
  Usage   : $id = $align->percentage_identity
  Function: The function calculates the percentage identity of the alignment
  Returns : The percentage identity of the alignment (as defined by the 
-						     implementation)
+	   implementation)
  Argument: None
 
 =cut
 
 sub percentage_identity{
     my ($self) = @_;
-    $self->throw_not_implemeneted();
+    $self->throw_not_implemented();
 }
 
 =head2 overall_percentage_identity
 
- Title   : percentage_identity
- Usage   : $id = $align->percentage_identity
+ Title   : overall_percentage_identity
+ Usage   : $id = $align->overall_percentage_identity
  Function: The function calculates the percentage identity of 
            the conserved columns
  Returns : The percentage identity of the conserved columns
@@ -723,12 +735,12 @@ L<Bio::LocatableSeq::location_from_column>:
   	     Seq2/24-30 ACGG.RTY..
   	     Seq3/43-51 AC.DDEFGHI
 
-           column_from_residue_number( "Seq1", 94 ) returns 5.
+           column_from_residue_number( "Seq1", 94 ) returns 6.
            column_from_residue_number( "Seq2", 25 ) returns 2.
            column_from_residue_number( "Seq3", 50 ) returns 9.
 
            An exception is thrown if the residue number would lie
-           outside the length of the aligment
+           outside the length of the alignment
            (e.g. column_from_residue_number( "Seq2", 22 )
 
 	  Note: If the parent sequence is represented by more than one
@@ -821,4 +833,38 @@ sub set_displayname_normal {
     $self->throw_not_implemented();
 }
 
+=head1 Deprecated methods
+
+=head2 no_residues
+
+ Title     : no_residues
+ Usage     : $no = $ali->no_residues
+ Function  : number of residues in total in the alignment
+ Returns   : integer
+ Argument  : 
+ Note      : deprecated in favor of num_residues()
+
+=cut
+
+sub no_residues {
+    # immediate deprecation
+    shift->deprecated();
+}
+
+=head2 no_sequences
+
+ Title     : no_sequences
+ Usage     : $depth = $ali->no_sequences
+ Function  : number of sequence in the sequence alignment
+ Returns   : integer
+ Argument  : None
+ Note      : deprecated in favor of num_sequences() 
+
+=cut
+
+sub no_sequences {
+    # immediate deprecation
+    shift->deprecated();
+}
+
 1;
