@@ -1,8 +1,10 @@
-# $Id: DNAStatistics.pm,v 1.22 2004/07/04 18:51:22 jason Exp $
+# $Id: DNAStatistics.pm 16147 2009-09-22 01:26:32Z cjfields $
 #
 # BioPerl module for Bio::Align::DNAStatistics
 #
-# Cared for by Jason Stajich <jason@bioperl.org>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Jason Stajich <jason-AT-bioperl.org>
 #
 # Copyright Jason Stajich
 #
@@ -19,8 +21,8 @@ Bio::Align::DNAStatistics - Calculate some statistics for a DNA alignment
   use Bio::AlignIO;
   use Bio::Align::DNAStatistics;
 
-  my $stats = new Bio::Align::DNAStatistics;
-  my $alignin = new Bio::AlignIO(-format => 'emboss',
+  my $stats = Bio::Align::DNAStatistics->new();
+  my $alignin = Bio::AlignIO->new(-format => 'emboss',
                                  -file   => 't/data/insulin.water');
   my $aln = $alignin->next_aln;
   my $jcmatrix = $stats->distance(-align => $aln, 
@@ -29,7 +31,7 @@ Bio::Align::DNAStatistics - Calculate some statistics for a DNA alignment
   print $jcmatrix->print_matrix;
   ## and for measurements of synonymous /nonsynonymous substitutions ##
 
-  my $in = new Bio::AlignIO(-format => 'fasta',
+  my $in = Bio::AlignIO->new(-format => 'fasta',
                             -file   => 't/data/nei_gojobori_test.aln');
   my $alnobj = $in->next_aln;
   my ($seq1id,$seq2id) = map { $_->display_id } $alnobj->each_seq;
@@ -71,21 +73,37 @@ in brackets are the pattern which will match
 
 =over 3
 
-=item  JukesCantor [jc|jukes|jukescantor|jukes-cantor]
+=item *
 
-=item Uncorrected [jcuncor|uncorrected]
+JukesCantor [jc|jukes|jukescantor|jukes-cantor]
 
-=item F81 [f81|felsenstein]
+=item *
 
-=item Kimura [k2|k2p|k80|kimura]
+Uncorrected [jcuncor|uncorrected]
 
-=item Tamura [t92|tamura|tamura92]
+=item *
 
-=item F84 [f84|felsenstein84]
+F81 [f81|felsenstein]
 
-=item TajimaNei [tajimanei|tajima\-nei]
+=item *
 
-=item JinNei [jinnei|jin\-nei] (not implemented)
+Kimura [k2|k2p|k80|kimura]
+
+=item *
+
+Tamura [t92|tamura|tamura92]
+
+=item *
+
+F84 [f84|felsenstein84]
+
+=item *
+
+TajimaNei [tajimanei|tajima\-nei]
+
+=item *
+
+JinNei [jinnei|jin\-nei] (not implemented)
 
 =back
 
@@ -102,7 +120,7 @@ several pre-requisites for the alignment.
 =item 1
 
 DNA alignment must be based on protein alignment. Use the subroutine
-L<aa_to_dna_aln> in Bio::Align::Utilities to achieve this.
+L<Bio::Align::Utilities/aa_to_dna_aln> to achieve this.
 
 =item 2
 
@@ -116,7 +134,7 @@ achieve meaningful results
 
 =item 4
 
-ALignment must therefore be a multiple of 3 nucleotides long.
+Alignment must therefore be a multiple of 3 nucleotides long.
 
 =item 5
 
@@ -138,78 +156,80 @@ comparisons in an MSA.  The statistics returned are:
 
 =over 3
 
-=item S_d
+=item *
 
-Number of synonymous mutations between the 2 sequences.
+S_d - Number of synonymous mutations between the 2 sequences.
 
-=item N_d
+=item *
 
-Number of non-synonymous mutations between the 2 sequences.
+N_d - Number of non-synonymous mutations between the 2 sequences.
 
-=item S
+=item *
 
-Mean number of  synonymous sites in both sequences.
+S -  Mean number of  synonymous sites in both sequences.
 
-=item N
+=item *
 
-mean number of  synonymous sites in both sequences.
+N -  mean number of  synonymous sites in both sequences.
 
-=item P_s
+=item *
 
-proportion of synonymous differences in both sequences given by P_s = S_d/S.
+P_s - proportion of synonymous differences in both sequences given by
+P_s = S_d/S.
 
-=item P_n
+=item *
 
-proportion of non-synonymous differences in both sequences given by P_n = S_n/S.
+P_n - proportion of non-synonymous differences in both sequences given
+by P_n = S_n/S.
 
-=item D_s
+=item *
 
-estimation of synonymous mutations per synonymous site (by Jukes-CAntor).
+D_s - estimation of synonymous mutations per synonymous site (by
+Jukes-Cantor).
 
-=item D_n
+=item *
 
-estimation of non-synonymous mutations per non-synonymous site (by Jukes-CAntor).
+D_n - estimation of non-synonymous mutations per non-synonymous site (by
+Jukes-Cantor).
 
-=item D_n_var
+=item *
 
-estimation of variance of D_n .
+D_n_var - estimation of variance of D_n .
 
-=item D_s_var
+=item *
 
-estimation of variance of S_n.
+D_s_var - estimation of variance of S_n.
 
-=item z_value
+=item *
 
-calculation of z value.Positive value indicates D_n E<gt> D_s,
+z_value - calculation of z value.Positive value indicates D_n E<gt> D_s,
 negative value indicates D_s E<gt> D_n.
 
 =back
 
-=head1 calc_average_KaKs
-
 The statistics returned by calc_average_KaKs are:
 
 =over 3
 
-=item D_s
+=item *
 
-Average number of synonymous mutations/synonymous site.
+D_s - Average number of synonymous mutations/synonymous site.
 
-=item D_n
+=item *
 
-Average number of non-synonymous mutations/non-synonymous site.
+D_n - Average number of non-synonymous mutations/non-synonymous site.
 
-=item D_s_var
+=item *
 
-Estimated variance of Ds from bootstrapped alignments.
+D_s_var - Estimated variance of Ds from bootstrapped alignments.
 
-=item D_n_var
+=item *
 
-Estimated variance of Dn from bootstrapped alignments.
+D_n_var - Estimated variance of Dn from bootstrapped alignments.
 
-=item z_score
+=item *
 
-calculation of z value. POsitive value indicates D_n E<gt>D_s,
+z_score - calculation of z value. Positive value indicates D_n E<gt>D_s,
 negative values vice versa.
 
 =back
@@ -222,7 +242,6 @@ the book, and reproduce those results. If people like having this sort
 of analysis in BioPerl other methods for estimating Ds and Dn can be
 provided later.
 
-
 Much of the DNA distance code is based on implementations in EMBOSS
 (Rice et al, www.emboss.org) [distmat.c] and PHYLIP (J. Felsenstein et
 al) [dnadist.c].  Insight also gained from Eddy, Durbin, Krogh, &
@@ -232,26 +251,36 @@ Mitchison.
 
 =over 3
 
-=item D_JukesCantor 
+=item *
+
+D_JukesCantor 
 
 "Phylogenetic Inference", Swoffrod, Olsen, Waddell and Hillis, in
 Mol. Systematics, 2nd ed, 1996, Ch 11.  Derived from "Evolution of
 Protein Molecules", Jukes & Cantor, in Mammalian Prot. Metab., III,
 1969, pp. 21-132.
 
-=item D_Tamura
+=item *
+
+D_Tamura
 
 K Tamura, Mol. Biol. Evol. 1992, 9, 678.
 
-=item D_Kimura 
+=item *
+
+D_Kimura 
 
 M Kimura, J. Mol. Evol., 1980, 16, 111.
 
-=item JinNei 
+=item *
+
+JinNei 
 
 Jin and Nei, Mol. Biol. Evol. 82, 7, 1990.
 
-=item D_TajimaNei
+=item *
+
+D_TajimaNei
 
 Tajima and Nei, Mol. Biol. Evol. 1984, 1, 269.
 
@@ -265,27 +294,34 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via
-email or the web:
+of the bugs and their resolution. Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich
 
-Email jason@bioperl.org
-
-Describe contact details here
+Email jason-AT-bioperl.org
 
 =head1 CONTRIBUTORS
 
-Additional contributors names and emails here
 Richard Adams, richard.adams@ed.ac.uk
 
 =head1 APPENDIX
@@ -300,12 +336,11 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Align::DNAStatistics;
-use vars qw(@ISA %DNAChanges @Nucleotides %NucleotideIndexes
+use vars qw(%DNAChanges @Nucleotides %NucleotideIndexes
 	    $GapChars $SeqCount $DefaultGapPenalty %DistanceMethods
             $CODONS %synchanges $synsites $Precision $GCChhars);
 use strict;
 use Bio::Align::PairwiseStatistics;
-use Bio::Root::Root;
 use Bio::Matrix::PhylipDist;
 use Bio::Tools::IUPAC;
 
@@ -365,14 +400,14 @@ BEGIN {
 			 'jinnei|jin\-nei'       => 'JinNei');
 
 }
-@ISA = qw( Bio::Root::Root Bio::Align::StatisticsI );
+use base qw(Bio::Root::Root Bio::Align::StatisticsI);
 
 ## generate look up hashes for Nei_Gojobori methods##
 $CODONS = get_codons();
 my @t = split '', "FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG";
 #create look up hash of number of possible synonymous mutations per codon
 $synsites = get_syn_sites();
-#create refernce look up hash of single basechanges in codons
+#create reference look up hash of single basechanges in codons
 %synchanges = get_syn_changes();
 
 
@@ -380,7 +415,7 @@ $synsites = get_syn_sites();
 =head2 new
 
  Title   : new
- Usage   : my $obj = new Bio::Align::DNAStatistics();
+ Usage   : my $obj = Bio::Align::DNAStatistics->new();
  Function: Builds a new Bio::Align::DNAStatistics object 
  Returns : Bio::Align::DNAStatistics
  Args    : none
@@ -392,7 +427,7 @@ sub new {
     my ($class,@args) = @_;
     my $self = $class->SUPER::new(@args);
     
-    $self->pairwise_stats( new Bio::Align::PairwiseStatistics());
+    $self->pairwise_stats( Bio::Align::PairwiseStatistics->new());
 
     return $self;
 }
@@ -405,10 +440,11 @@ sub new {
 		 			       -method => $method);
  Function: Calculates a distance matrix for all pairwise distances of
            sequences in an alignment.
- Returns : Array ref
+ Returns : L<Bio::Matrix::PhylipDist> object
  Args    : -align  => Bio::Align::AlignI object
            -method => String specifying specific distance method 
                       (implementing class may assume a default)
+See also: L<Bio::Matrix::PhylipDist>
 
 =cut
 
@@ -427,7 +463,7 @@ sub distance{
    }
    $self->warn("Unrecognized distance method $method must be one of [".
 	       join(',',$self->available_distance_methods())."]");
-   return undef;
+   return;
 }
 
 =head2 available_distance_methods
@@ -658,11 +694,19 @@ sub D_Kimura {
 	   }
 	   my $P = $self->transitions($pairwise) / $L;
 	   my $Q = $self->transversions($pairwise) / $L;
-	   
+	   my $K = 0;
+	   my $denom = ( 1 - (2 * $P) - $Q);
+	   if( $denom == 0 ) {
+	       $self->throw("cannot find distance for ",$i+1,
+			    ",",$j+1," $P, $Q\n");
+	   }
 	   my $a = 1 / ( 1 - (2 * $P) - $Q);
 	   my $b = 1 / ( 1 - 2 * $Q );
-	   my $K = (1/2) * log ( $a ) + (1/4) * log($b);
-
+	   if( $a < 0 || $b < 0 ) { 
+	       $K = -1;
+	   } else{ 
+	       $K = (1/2) * log ( $a ) + (1/4) * log($b);
+	   }
 	   # fwd and rev lookup
 	   $dist{$names[$i]}->{$names[$j]} = [$i,$j];
 	   $dist{$names[$j]}->{$names[$i]} = [$i,$j];	   
@@ -793,6 +837,7 @@ sub D_Tamura {
    for my $t1 ( @seqs ) {
        $j = 0;
        for my $t2 ( @seqs ) {
+	   $gap[$i][$j] = 0;
 	   for( my $k = 0; $k < $length; $k++ ) {
 	       my ($c1,$c2) = ( substr($seqs[$i],$k,1),
 				substr($seqs[$j],$k,1) );
@@ -858,7 +903,7 @@ sub D_Tamura {
 sub D_F84 {
    my ($self,$aln,$gappenalty) = @_;
    return 0 unless $self->_check_arg($aln);
-   $self->throw("not implemented\n");
+   $self->throw_not_implemented();
    # ambiguities ignored at this point
    my (@seqs,@names,@values,%dist);
    my $seqct = 0;
@@ -970,12 +1015,19 @@ sub D_TajimaNei{
 	   my $m = ( $matrix->[0]->[0] + $matrix->[1]->[1] + 
 		     $matrix->[2]->[2] + $matrix->[3]->[3] );
 	   my $D = 1 - ( $m / $slen);
-	   
-	   my $b = (1 - $fij2 + (($D**2)/$h)) / 2;
-	   $self->debug("h is $h fij2 is $fij2 b is $b\n");
+	   my $d;
+	   if( $h == 0 ) {
+	       $d = -1;
+	   } else {
+	       my $b = (1 - $fij2 + (($D**2)/$h)) / 2;
+	       my $c = 1- $D/ $b;
 
-	   my $d = (-1 * $b) * log ( 1 - $D/ $b);
-	   
+	       if( $c < 0 ) {
+		   $d = -1;
+	       } else { 
+		   $d = (-1 * $b) * log ( $c);
+	       }
+	   }
 	   # fwd and rev lookup
 	   $dist{$names[$i]}->{$names[$j]} = [$i,$j];
 	   $dist{$names[$j]}->{$names[$i]} = [$i,$j];	   
@@ -1010,7 +1062,7 @@ sub D_TajimaNei{
 sub D_JinNei{
    my ($self,@args) = @_;
    $self->warn("JinNei implementation not completed");
-   return undef;
+   return;
 }
 
 =head2 transversions
@@ -1052,8 +1104,8 @@ sub _trans_count_helper {
     return 0 unless( $self->_check_arg($aln) );
     if( ! $aln->is_flush ) { $self->throw("must be flush") }
     my (@tcount);
-    my ($first,$second) = ( $aln->get_seq_by_pos(1)->seq(),
-			    $aln->get_seq_by_pos(2)->seq() );
+    my ($first,$second) = ( uc $aln->get_seq_by_pos(1)->seq(),
+			    uc $aln->get_seq_by_pos(2)->seq() );
     my $alen = $aln->length; 
     for (my $i = 0;$i<$alen; $i++ ) { 
 	my ($c1,$c2) = ( substr($first,$i,1),
@@ -1097,7 +1149,7 @@ sub _build_nt_matrix {
 	my $tj_index = $NucleotideIndexes{$tj};	    
 
 	if( ! defined $ti_index ) {
-	    print "ti_index not defined for $ti\n";
+	    $self->warn("ti_index not defined for $ti\n");
 	    next;
 	}
 	
@@ -1243,7 +1295,7 @@ sub calc_all_KaKs_pairs {
 
 sub calc_average_KaKs {
 #calculates global value for sequences in alignment using bootstrapping
-#this is quite slow (~10 sexonds per  3 X 200nt seqs); 
+#this is quite slow (~10 seconds per  3 X 200nt seqs); 
     my ($self, $aln, $bootstrap_rpt) = @_;
     $bootstrap_rpt ||= 1000;
     $self->throw ("This calculation needs a Bio::Align::AlignI compatible object, not a [ " . ref($aln) . " ]object") unless $aln->isa('Bio::Align::AlignI');
@@ -1441,7 +1493,7 @@ sub analyse_mutations {
 
 	#ignore codon if beeing compared with gaps! 
 	if ($input{'cod1'} =~ /\-/ || $input{'cod2'} =~ /\-/){
-	    $gap_cnt += 3; #just increments once if there is a apair of gaps
+	    $gap_cnt += 3; #just increments once if there is a pair of gaps
 	    next;
 	}
 
@@ -1543,7 +1595,7 @@ sub count_diffs {
 =head2 get_syn_changes
 
  Title   : get_syn_changes
- Usage   : Bio::Align::DNAStatitics->get_syn_chnages
+ Usage   : Bio::Align::DNAStatitics->get_syn_changes
  Function: Generate a hashref of all pairwise combinations of codns
            differing by 1
  Returns : Symetic matrix using hashes
@@ -1552,7 +1604,7 @@ sub count_diffs {
            the values of which describe type of change.
            my $type = $hash{$codon1}->{$codon2};
            values are :
-             1   synonomous
+             1   synonymous
              0   non-syn
             -1   either codon is a stop codon
  Args    : none
@@ -1592,12 +1644,28 @@ sub get_syn_changes {
 	}
     }
     return %results;
-}			
+}
+
+=head2 dnds_pattern_number
+
+ Title   : dnds_pattern_number
+ Usage   : my $patterns = $stats->dnds_pattern_number($alnobj);
+ Function: Counts the number of codons with no gaps in the MSA
+ Returns : Number of codons with no gaps ('patterns' in PAML notation)
+ Args    : A Bio::Align::AlignI compliant object such as a
+            Bio::SimpleAlign object.
+
+=cut
+
+sub dnds_pattern_number{
+    my ($self, $aln) = @_;
+    return ($aln->remove_gaps->length)/3;
+}
 
 sub count_syn_sites {
     #counts the number of possible synonymous changes for sequence
     my ($seq, $synsite) = @_;
-    die "not integral number of codons" if length($seq) % 3 != 0;
+    __PACKAGE__->throw("not integral number of codons") if length($seq) % 3 != 0;
     my $S = 0;
     for (my $i = 0; $i< length($seq); $i+=3) {
 	my $cod = substr($seq, $i, 3);
