@@ -1,7 +1,9 @@
-# $Id: medlinexml.pm,v 1.7 2004/11/16 17:45:38 jason Exp $
+# $Id: medlinexml.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module Bio::Biblio::IO::medlinexml.pm
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Martin Senger <senger@ebi.ac.uk>
 # For copyright and disclaimer see below.
 
@@ -17,7 +19,7 @@ Do not use this object directly, it is recommended to access it and use
 it through the I<Bio::Biblio::IO> module:
 
   use Bio::Biblio::IO;
-  my $io = new Bio::Biblio::IO (-format => 'medlinexml');
+  my $io = Bio::Biblio::IO->new(-format => 'medlinexml');
 
 =head1 DESCRIPTION
 
@@ -33,8 +35,19 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
@@ -42,7 +55,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
@@ -74,21 +87,15 @@ with an underscore _.
 
 
 package Bio::Biblio::IO::medlinexml;
-use vars qw(@ISA $Revision);
 use vars qw(@Citations $Callback $Convert @ObjectStack @PCDataStack);
 use vars qw(%PCDATA_NAMES %SIMPLE_TREATMENT %POP_DATA_AND_PEEK_OBJ %POP_OBJ_AND_PEEK_OBJ);
 use vars qw(%POP_AND_ADD_ELEMENT %POP_AND_ADD_DATA_ELEMENT);
 
 use strict;
 
-use Bio::Biblio::IO;
 use XML::Parser;
 
-@ISA = qw(Bio::Biblio::IO);
-
-BEGIN {
-    $Revision = q[$Id: medlinexml.pm,v 1.7 2004/11/16 17:45:38 jason Exp $];
-}
+use base qw(Bio::Biblio::IO);
 
 # -----------------------------------------------------------------------------
 
@@ -466,7 +473,7 @@ sub handle_start {
     if ($e eq 'QualifierName' or
 	$e eq 'SubHeading') {
 	my %p = ();
-	$p{'majorTopic'} = "Y" if $attrs{'MajorTopicYN'};
+	$p{'majorTopic'} = $attrs{'MajorTopicYN'} if $attrs{'MajorTopicYN'};
 	push (@ObjectStack, \%p);
     }
 
