@@ -1,7 +1,9 @@
-# $Id: UniGene.pm,v 1.31 2004/08/11 07:25:00 lapp Exp $
+# $Id: UniGene.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Cluster::UniGene.pm
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Andrew Macgregor <andrew at cbbc.murdoch.edu.au>
 #
 # Copyright Andrew Macgregor, Jo-Ann Stanton, David Green
@@ -140,17 +142,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists. Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bio.perl.org/MailList.html  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Andrew Macgregor
 
@@ -172,25 +184,15 @@ methods. Internal methods are usually preceded with a "_".
 
 
 package Bio::Cluster::UniGene;
-use vars qw(@ISA);
 use strict;
 
-
-use Bio::Root::Root;
-use Bio::IdentifiableI;
-use Bio::DescribableI;
-use Bio::AnnotatableI;
 use Bio::Annotation::Collection;
 use Bio::Annotation::DBLink;
 use Bio::Annotation::SimpleValue;
 use Bio::Species;
-use Bio::Factory::SequenceStreamI;
 use Bio::Seq::SeqFactory;
-use Bio::Cluster::UniGeneI;
 
-@ISA = qw(Bio::Root::Root Bio::Cluster::UniGeneI
-	  Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI
-	  Bio::Factory::SequenceStreamI);
+use base qw(Bio::Root::Root Bio::Cluster::UniGeneI Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI Bio::Factory::SequenceStreamI);
 
 my %species_map = (
 		   'Aga' => "Anopheles gambiae",
@@ -202,6 +204,8 @@ my %species_map = (
 		   'Cfa' => "Canine familiaris",
 		   'Cin' => "Ciona intestinalis",
 		   'Cre' => "Chlamydomonas reinhardtii",
+		   'Csa' => "Ciona savignyi",
+		   'Csi' => "Citrus sinensis",
 		   'Ddi' => "Dictyostelium discoideum",
 		   'Dr'  => "Danio rerio",
 		   'Dm'  => "Drosophila melanogaster",
@@ -215,8 +219,10 @@ my %species_map = (
 		   'Les' => "Lycopersicon esculentum",
 		   'Lsa' => "Lactuca sativa",
 		   'Mdo' => "Malus x domestica",
+                   'Mgr' => "Magnaporthe grisea",
 		   'Mm'  => "Mus musculus",
 		   'Mtr' => "Medicago truncatula",
+                   'Ncr' => "Neurospora crassa",
 		   'Oar' => "Ovis aries",
 		   'Omy' => "Oncorhynchus mykiss",
 		   'Os'  => "Oryza sativa",
@@ -235,11 +241,12 @@ my %species_map = (
 		   'Stu' => "Solanum tuberosum",
 		   'Ta'  => "Triticum aestivum",
 		   'Tgo' => "Toxoplasma gondii",
+                   'Tru' => "Takifugu rubripes",
 		   'Vvi' => "Vitis vinifera",
 		   'Xl'  => "Xenopus laevis",
 		   'Zm'  => "Zea mays",
 		   );
-		 
+
 
 =head2 new
 
@@ -280,7 +287,7 @@ sub new {
     $self->authority($auth || 'NCBI');
     $self->version($v) if defined($v);
     if( ! defined $seqfact ) {
-	$seqfact = new Bio::Seq::SeqFactory
+	$seqfact = Bio::Seq::SeqFactory->new
 	    (-verbose => $self->verbose(), 
 	     -type => 'Bio::Seq::RichSeq');
     }
@@ -452,6 +459,23 @@ sub homol {
 }
 
 
+=head2 restr_expr
+
+ Title   : restr_expr
+ Usage   : restr_expr();
+ Function: Returns the restr_expr entry associated with the object.
+ Example : $restr_expr = $unigene->restr_expr or $unigene->restr_expr($restr_expr)
+ Returns : A string
+ Args    : None or a restr_expr entry
+
+=cut
+
+sub restr_expr {
+    my $self = shift;
+    return $self->_annotation_value('restr_expr', @_);
+}
+
+
 =head2 gnm_terminus
 
  Title   : gnm_terminus
@@ -1039,7 +1063,7 @@ sub _next_element{
     if(! @$queue) {
 	# yes, remove queue and signal to the caller
 	delete $self->{$queuename};
-	return undef;
+	return;
     }
     return shift(@$queue);
 }
@@ -1220,7 +1244,7 @@ sub next_seq {
     if(! @$queue) {
 	# yes, remove queue and signal to the caller
 	delete $obj->{'_seq_queue'};
-	return undef;
+	return;
     }
     # no, still data in the queue: get the next one from the queue
     my $seq_h = shift(@$queue);
@@ -1307,12 +1331,12 @@ sub _annotation_value{
 	}
     }
     ($ann) = $self->annotation->get_Annotations($key);
-    if($ann && (! $val)) {
+    if(defined $ann && (! $val)) {
 	# get mode and exists
 	$val = $ann->value();
     } elsif($val) {
 	# set mode
-	if(! $ann) {
+	if(!defined $ann) {
 	    $ann = Bio::Annotation::SimpleValue->new(-tagname => $key);
 	    $self->annotation->add_Annotation($ann);
 	}
