@@ -1,8 +1,10 @@
 #
-# $Id: Flat.pm,v 1.19 2004/11/10 16:03:02 amackey Exp $
+# $Id: Flat.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::DB::Flat
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Lincoln Stein <lstein@cshl.org>
 #
 # You may distribute this module under the same terms as perl itself
@@ -44,17 +46,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via
-email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Lincoln Stein
 
@@ -71,12 +83,9 @@ methods are usually preceded with an "_" (underscore).
 # Let the code begin...
 package Bio::DB::Flat;
 
-use Bio::DB::RandomAccessI;
-use Bio::Root::Root;
-use Bio::Root::IO;
-use vars '@ISA';
+use File::Spec;
 
-@ISA = qw(Bio::Root::Root Bio::DB::RandomAccessI);
+use base qw(Bio::Root::Root Bio::DB::RandomAccessI);
 
 use constant CONFIG_FILE_NAME => 'config.dat';
 
@@ -151,7 +160,7 @@ sub new {
     unless -e $flat_directory;
   $self->throw("$flat_directory isn't a directory")
     unless -d _;
-  my $dbpath = Bio::Root::IO->catfile($flat_directory,$dbname);
+  my $dbpath = File::Spec->catfile($flat_directory,$dbname);
   unless (-d $dbpath) {
     $self->debug("creating db directory $dbpath\n");
     mkdir $dbpath,0777 or $self->throw("Can't create $dbpath: $!");
@@ -379,34 +388,34 @@ sub write_config {
   $self->write_flag or $self->throw("cannot write configuration file because write_flag is not set");
   my $path = $self->_config_path;
 
-  open (F,">$path") or $self->throw("open error on $path: $!");
+  open (my $F,">$path") or $self->throw("open error on $path: $!");
 
   my $index_type = $self->indexing_scheme;
-  print F "index\t$index_type\n";
+  print $F "index\t$index_type\n";
 
   my $format     = $self->file_format;
   my $alphabet   = $self->alphabet;
   my $alpha      = $alphabet ? "/$alphabet" : '';
-  print F "format\tURN:LSID:open-bio.org:${format}${alpha}\n";
+  print $F "format\tURN:LSID:open-bio.org:${format}${alpha}\n";
 
   my @filenos = $self->_filenos or $self->throw("cannot write config file because no flat files defined");
   for my $nf (@filenos) {
     my $path = $self->{flat_flat_file_path}{$nf};
     my $size = -s $path;
-    print F join("\t","fileid_$nf",$path,$size),"\n";
+    print $F join("\t","fileid_$nf",$path,$size),"\n";
   }
 
   # write primary namespace
   my $primary_ns = $self->primary_namespace
     or $self->throw('cannot write config file because no primary namespace defined');
 
-  print F join("\t",'primary_namespace',$primary_ns),"\n";
+  print $F join("\t",'primary_namespace',$primary_ns),"\n";
 
   # write secondary namespaces
   my @secondary = $self->secondary_namespaces;
-  print F join("\t",'secondary_namespaces',@secondary),"\n";
+  print $F join("\t",'secondary_namespaces',@secondary),"\n";
 
-  close F or $self->throw("close error on $path: $!");
+  close $F or $self->throw("close error on $path: $!");
 }
 
 sub files {
@@ -458,14 +467,14 @@ sub _read_config {
   my $path = $self->_config_path;
   return unless -e $path;
 
-  open (F,$path) or $self->throw("open error on $path: $!");
+  open (my $F,$path) or $self->throw("open error on $path: $!");
   my %config;
-  while (<F>) {
+  while (<$F>) {
     chomp;
     my ($tag,@values) = split "\t";
     $config{$tag} = \@values;
   }
-  CORE::close F or $self->throw("close error on $path: $!");
+  CORE::close $F or $self->throw("close error on $path: $!");
 
   $config{index}[0] =~ m~(flat/1|BerkeleyDB/1)~
     or $self->throw("invalid configuration file $path: no index line");
@@ -508,7 +517,7 @@ sub _config_path {
 sub _catfile {
   my $self = shift;
   my $component = shift;
-  Bio::Root::IO->catfile($self->directory,$self->dbname,$component);
+  File::Spec->catfile($self->directory,$self->dbname,$component);
 }
 
 sub _config_name { CONFIG_FILE_NAME }
