@@ -1,7 +1,9 @@
-# $Id: BinarySearch.pm,v 1.15 2004/10/05 18:49:48 bosborne Exp $
+# $Id: BinarySearch.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::DB::Flat::BinarySearch
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Michele Clamp <michele@sanger.ac.uk>>
 #
 # You may distribute this module under the same terms as perl itself
@@ -10,12 +12,11 @@
 
 =head1 NAME
 
-Bio::DB::Flat::BinarySearch - BinarySearch search indexing system for
-sequence files
+Bio::DB::Flat::BinarySearch - BinarySearch search indexing system for sequence files
 
 =head1 SYNOPSIS
 
-  # See below
+  TODO: SYNOPSIS NEEDED!
 
 =head1 DESCRIPTION
 
@@ -46,13 +47,13 @@ A string also has to be entered to defined what the primary key
 
 The index can now be created using 
 
-    my $index = new Bio::DB::Flat::BinarySearch(
+    my $index = Bio::DB::Flat::BinarySearch->new(
              -directory         => "/home/max/",
              -dbname            => "mydb",
-	     -start_pattern     => $start_pattern,
-	     -primary_pattern   => $primary_pattern,
+	          -start_pattern     => $start_pattern,
+	          -primary_pattern   => $primary_pattern,
              -primary_namespace => "ID",
-	     -format            => "fasta" );
+	          -format            => "fasta" );
 
     my @files = ("file1","file2","file3");
 
@@ -100,10 +101,10 @@ id (1433_CAEEL) as the secondary id.  The index is created as follows
 
     $secondary_patterns{"ID"} = '^ID   (\S+)';
 
-    my $index = new Bio::DB::Flat::BinarySearch(
+    my $index = Bio::DB::Flat::BinarySearch->new(
                 -directory          => $index_directory,
-		-dbname             => "ppp",
-		-write_flag         => 1,
+		          -dbname             => "ppp",
+		          -write_flag         => 1,
                 -verbose            => 1,
                 -start_pattern      => $start_pattern,
                 -primary_pattern    => $primary_pattern,
@@ -120,7 +121,7 @@ memory.
 To fetch sequences using an existing index first of all create your sequence 
 object 
 
-    my $index = new Bio::DB::Flat::BinarySearch(
+    my $index = Bio::DB::Flat::BinarySearch->new(
                   -directory => $index_directory);
 
 Now you can happily fetch sequences either by the primary key or
@@ -138,8 +139,8 @@ Other ways of getting sequences are
 This can then be passed to a seqio object for output or converting
 into objects.
 
-    my $seq = new Bio::SeqIO(-fh     => $fh,
-			     -format => 'fasta');
+    my $seq = Bio::SeqIO->new(-fh     => $fh,
+			                    -format => 'fasta');
 
 The last way is to retrieve a sequence directly.  This is the
 slowest way of extracting as the sequence objects need to be made.
@@ -167,17 +168,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via
-email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Michele Clamp
 
@@ -197,30 +208,30 @@ methods are usually preceded with an "_" (underscore).
 package Bio::DB::Flat::BinarySearch;
 
 use strict;
-use vars qw(@ISA);
 
 use Fcntl qw(SEEK_END SEEK_CUR);
 # rather than using tell which might be buffered
 sub systell{ sysseek($_[0], 0, SEEK_CUR) }
 sub syseof{ sysseek($_[0], 0, SEEK_END) }
 
-use Bio::DB::RandomAccessI;
+use File::Spec;
 use Bio::Root::RootI;
 use Bio::SeqIO;
 use Bio::Seq;
 
-@ISA = qw(Bio::DB::RandomAccessI);
+use base qw(Bio::DB::RandomAccessI);
 
 use constant CONFIG_FILE_NAME => 'config.dat';
 use constant HEADER_SIZE      => 4;
 use constant DEFAULT_FORMAT   => 'fasta';
+
 my @formats = ['FASTA','SWISSPROT','EMBL'];
 
 =head2 new
 
  Title   : new
  Usage   : For reading 
-             my $index = new Bio::DB::Flat::BinarySearch(
+             my $index = Bio::DB::Flat::BinarySearch->new(
                      -directory => '/Users/michele/indices/dbest',
 		     -dbname    => 'mydb',
                      -format    => 'fasta');
@@ -228,7 +239,7 @@ my @formats = ['FASTA','SWISSPROT','EMBL'];
            For writing 
 
              my %secondary_patterns = {"ACC" => "^>\\S+ +(\\S+)"}
-             my $index = new Bio::DB::Flat::BinarySearch(
+             my $index = Bio::DB::Flat::BinarySearch->new(
 		     -directory          => '/Users/michele/indices',
                      -dbname             => 'mydb',
 		     -primary_pattern    => "^>(\\S+)",
@@ -307,7 +318,7 @@ sub new_from_registry {
     my $dbname   = $config{'dbname'};
     my $location = $config{'location'};
     
-    my $index =  new Bio::DB::Flat::BinarySearch(-dbname    => $dbname,
+    my $index =  Bio::DB::Flat::BinarySearch->new(-dbname    => $dbname,
 						 -index_dir => $location,
 						 );
 }
@@ -339,7 +350,7 @@ sub get_Seq_by_id {
 
     unless ( defined($self->{_seqio}) ) {
 
-	$self->{_seqio} = new Bio::SeqIO(-fh => $fh,
+	$self->{_seqio} = Bio::SeqIO->new(-fh => $fh,
 					 -format => $self->format);
     } else {
 	$self->{_seqio}->fh($fh);
@@ -414,17 +425,15 @@ sub get_stream_by_id {
       return;
     }
 
-    my $fh = $self->get_filehandle_by_fileid($fileid);
     my $file = $self->{_file}{$fileid};
 
-    open (IN,"<$file");
-    $fh = \*IN;
+    open (my $IN,"<$file");
 
     my $entry;
     
-    sysseek($fh,$pos,0);
+    sysseek($IN,$pos,0);
 
-    return ($fh,$length);
+    return ($IN,$length);
 }
 
 =head2 get_Seq_by_acc
@@ -728,7 +737,7 @@ sub build_index {
 	$self->throw("Index directory [$rootdir] is not a directory. Cant' build indices");
     }
 
-    my $dbpath = Bio::Root::IO->catfile($rootdir,$self->dbname);
+    my $dbpath = File::Spec->catfile($rootdir,$self->dbname);
     if (! -d $dbpath) {
       warn "Creating directory $dbpath\n";
       mkdir $dbpath,0777 or $self->throw("Couldn't create $dbpath: $!");
@@ -783,112 +792,113 @@ sub build_index {
 =cut
 
 sub _index_file {
-    my ($self,$file) = @_;
-    my $v = $self->verbose;
-    open(FILE,"<$file") || $self->throw("Can't open file [$file]");
+	my ($self,$file) = @_;
+	my $v = $self->verbose;
+	open(my $FILE,"<", $file) || $self->throw("Can't open file [$file]");
 
-    my $recstart = 0;
-    my $fileid = $self->get_fileid_by_filename($file);
-    my $found = 0;
-    my $id;
-    my $count = 0;
+	my $recstart = 0;
+	my $fileid = $self->get_fileid_by_filename($file);
+	my $found = 0;
+	my $id;
+	my $count = 0;
 
-    my $primary       = $self->primary_pattern;
-    my $start_pattern = $self->start_pattern;
+	my $primary       = $self->primary_pattern;
+	my $start_pattern = $self->start_pattern;
 
-    my $pos = 0;
+	my $pos = 0;
 
-    my $new_primary_entry;
+	my $new_primary_entry;
 
-    my $length;
-    #my $pos = 0;
-    my $fh = \*FILE;
+	my $length;
 
-    my $done = -1;
+	my $fh = $FILE;
 
-    my @secondary_names = $self->secondary_namespaces;
-    my %secondary_id;
-    my $last_one;
+	my $done = -1;
 
-    while (<$fh>) {
+	my @secondary_names = $self->secondary_namespaces;
+	my %secondary_id;
+	my $last_one;
+
+	while (<$fh>) {
       $last_one = $_;
       $self->{alphabet} ||= $self->guess_alphabet($_);		
       if ($_ =~ /$start_pattern/) {
-	 if ($done == 0) {
-	    $id = $new_primary_entry;
-	    $self->{alphabet} ||= $self->guess_alphabet($_);
+			if ($done == 0) {
+				$id = $new_primary_entry;
+				$self->{alphabet} ||= $self->guess_alphabet($_);
 	  
-	  my $tmplen = (tell $fh) - length($_);
+				my $tmplen = (tell $fh) - length($_);
 
-	  $length = $tmplen  - $pos;
+				$length = $tmplen  - $pos;
 		
-	  unless( defined($id)) {
-	    $self->throw("No id defined for sequence");
-	  }
-	  unless( defined($fileid)) {
-	    $self->throw("No fileid defined for file $file");
-	  }
-	  unless( defined($pos)) {
-	    $self->throw("No position defined for " . $id . "\n");
-	  }
-	  unless( defined($length)) {
-	    $self->throw("No length defined for " . $id . "\n");
-	  }
-	  $self->_add_id_position($id,$pos,$fileid,$length,\%secondary_id);
+				unless( defined($id)) {
+					$self->throw("No id defined for sequence");
+				}
+				unless( defined($fileid)) {
+					$self->throw("No fileid defined for file $file");
+				}
+				unless( defined($pos)) {
+					$self->throw("No position defined for " . $id . "\n");
+				}
+				unless( defined($length)) {
+					$self->throw("No length defined for " . $id . "\n");
+				}
+				$self->_add_id_position($id,$pos,$fileid,$length,\%secondary_id);
 
-	  $pos   = $tmplen;
+				$pos   = $tmplen;
 		
-	  if ($count > 0 && $count%1000 == 0) {
-	    $self->debug( "Indexed $count ids\n") if $v > 0;
-	  }
+				if ($count > 0 && $count%1000 == 0) {
+					$self->debug( "Indexed $count ids\n") if $v > 0;
+				}
 	    
-	  $count++;
-	} else {
-	  $done = 0;
-	}
+				$count++;
+			} else {
+				$done = 0;
+			}
       }
 
       if ($_ =~ /$primary/) {
-	$new_primary_entry = $1;    
+			$new_primary_entry = $1;    
       }
 
       my $secondary_patterns = $self->secondary_patterns;
 
       foreach my $sec (@secondary_names) {
-	my $pattern = $secondary_patterns->{$sec};
+			my $pattern = $secondary_patterns->{$sec};
 
-	if ($_ =~ /$pattern/) {
-	  $secondary_id{$sec} = $1;
-	}
+			if ($_ =~ /$pattern/) {
+				$secondary_id{$sec} = $1;
+			}
       }
-	
-    }
+		
+	}
 
-    # Remember to add in the last one
+	# Remember to add in the last one
 
-    $id = $new_primary_entry;
-    my $tmplen = (tell $fh) - length($last_one);
+	$id = $new_primary_entry;
+	# my $tmplen = (tell $fh) - length($last_one);
+	my $tmplen = (tell $fh);
 
-    $length = $tmplen  - $pos;
+	$length = $tmplen - $pos;
     
-    if (!defined($id)) {
-	$self->throw("No id defined for sequence");
-    }
-    if (!defined($fileid)) {
-	$self->throw("No fileid defined for file $file");
-    }
-    if (!defined($pos)) {
-	$self->throw("No position defined for " . $id . "\n");
-    }
-    if (!defined($length)) {
-	$self->throw("No length defined for " . $id . "\n");
-    }
+	if (!defined($id)) {
+		$self->throw("No id defined for sequence");
+	}
+	if (!defined($fileid)) {
+		$self->throw("No fileid defined for file $file");
+	}
+	if (!defined($pos)) {
+		$self->throw("No position defined for " . $id . "\n");
+	}
+	if (!defined($length)) {
+		$self->throw("No length defined for " . $id . "\n");
+	}
     
-    $self->_add_id_position($id,$pos,$fileid,$length,\%secondary_id);
-    $count++;
+	$self->_add_id_position($id,$pos,$fileid,$length,\%secondary_id);
+	$count++;
     
-    close(FILE);
-    $count;
+	close(FILE);
+	$count;
 }
 
 =head2 write_primary_index
@@ -904,45 +914,43 @@ sub _index_file {
 =cut
 
 sub write_primary_index {
-    my ($self) = @_;
+	my ($self) = @_;
 
-    my @ids = keys %{$self->{_id}};
+	my @ids = keys %{$self->{_id}};
 
-    @ids = sort {$a cmp $b} @ids;
+	@ids = sort {$a cmp $b} @ids;
 
-    open (INDEX,">" . $self->primary_index_file) || 
-	$self->throw("Can't open primary index file [" . 
-		     $self->primary_index_file . "]");
+	open (my $INDEX,">" . $self->primary_index_file) || 
+	  $self->throw("Can't open primary index file [" . 
+						$self->primary_index_file . "]");
 
-    my $recordlength = $self->{_maxidlength} +
-	               $self->{_maxfileidlength} + 
-	               $self->{_maxposlength} +
-   		       $self->{_maxlengthlength} + 3;
-	
+	my $recordlength = $self->{_maxidlength} +
+	                   $self->{_maxfileidlength} + 
+		                $self->{_maxposlength} +
+			             $self->{_maxlengthlength} + 3;
     
-    print INDEX sprintf("%4d",$recordlength);
+	print $INDEX sprintf("%04d",$recordlength);
 
-    foreach my $id (@ids) {
+	foreach my $id (@ids) {
 
-	if (!defined($self->{_id}{$id}{_fileid})) {
-	    $self->throw("No fileid for $id\n");
-	}
-	if (!defined($self->{_id}{$id}{_pos})) {
-	    $self->throw("No position for $id\n");
-	}
-	if (!defined($self->{_id}{$id}{_length})) {
-	    $self->throw("No length for $id");
-	}
+		if (!defined($self->{_id}{$id}{_fileid})) {
+			$self->throw("No fileid for $id\n");
+		}
+		if (!defined($self->{_id}{$id}{_pos})) {
+			$self->throw("No position for $id\n");
+		}
+		if (!defined($self->{_id}{$id}{_length})) {
+			$self->throw("No length for $id");
+		}
 
-	my $record =  $id              . "\t" . 
-	    $self->{_id}{$id}{_fileid} . "\t" .
-	    $self->{_id}{$id}{_pos}    . "\t" .
-	    $self->{_id}{$id}{_length};
+		my $record =  $id              . "\t" . 
+		  $self->{_id}{$id}{_fileid} . "\t" .
+			 $self->{_id}{$id}{_pos}    . "\t" .
+				$self->{_id}{$id}{_length};
 
-	print INDEX sprintf("%-${recordlength}s",$record);
+		print $INDEX sprintf("%-${recordlength}s",$record);
 
     }
-    close(INDEX);
 }
 
 =head2 write_secondary_indices
@@ -988,7 +996,7 @@ sub write_secondary_indices {
 	
 	my $fh = $self->new_secondary_filehandle($name);	
 
-	print $fh sprintf("%4d",$length);
+	print $fh sprintf("%04d",$length);
 	@seconds = sort @seconds;
 	
 	foreach my $second (@seconds) {
@@ -1028,10 +1036,9 @@ sub new_secondary_filehandle {
 
     my $indexdir = $self->_config_path;
 
-    my $secindex = Bio::Root::IO->catfile($indexdir,"id_$name.index");
+    my $secindex = File::Spec->catfile($indexdir,"id_$name.index");
 
-    my $fh;
-    open($fh,">$secindex") || $self->throw($!);
+    open(my $fh,">", $secindex) || $self->throw($!);
     return $fh;
 }
 
@@ -1059,8 +1066,7 @@ sub open_secondary_index {
 	    $self->throw("Index is not present for namespace [$name]\n");
 	}
 
-        my $newfh;
-	open($newfh,"<$secindex") || $self->throw($!);
+	open(my $newfh,"<", $secindex) || $self->throw($!);
 	my $reclen = $self->read_header($newfh);
 
 	$self->{_secondary_filehandle} {$name} = $newfh;
@@ -1084,45 +1090,47 @@ sub open_secondary_index {
 =cut
 
 sub _add_id_position {
-  my ($self,$id,$pos,$fileid,$length,$secondary_id) = @_;
+	my ($self,$id,$pos,$fileid,$length,$secondary_id) = @_;
 
-  if (!defined($id)) {
-    $self->throw("No id defined. Can't add id position");
-  }
-  if (!defined($pos)) {
-    $self->throw("No position defined. Can't add id position");
-  }
-  if ( ! defined($fileid)) {
-    $self->throw("No fileid defined. Can't add id position");
-  }
-  if (! defined($length) || $length <= 0) {
-    $self->throw("No length defined or <= 0 [$length]. Can't add id position");
-  }
+	if (!defined($id)) {
+		$self->throw("No id defined. Can't add id position");
+	}
+	if (!defined($pos)) {
+		$self->throw("No position defined. Can't add id position");
+	}
+	if ( ! defined($fileid)) {
+		$self->throw("No fileid defined. Can't add id position");
+	}
+	if (! defined($length) || $length <= 0) {
+		$self->throw("No length defined or <= 0 [$length]. Can't add id position");
+	}
 
-  $self->{_id}{$id}{_pos}    = $pos;
-  $self->{_id}{$id}{_length} = $length;
-  $self->{_id}{$id}{_fileid} = $fileid;
+	$self->{_id}{$id}{_pos}    = $pos;
+	$self->{_id}{$id}{_length} = $length;
+	$self->{_id}{$id}{_fileid} = $fileid;
 
-  # Now the secondary ids
+	# Now the secondary ids
 
-  foreach my $sec (keys (%$secondary_id)) {
-    my $value = $secondary_id->{$sec};
+	foreach my $sec (keys (%$secondary_id)) {
+		my $value = $secondary_id->{$sec};
+		$self->{_secondary_id}{$sec}{$value}{$id} = 1;
+	}
 
-    $self->{_secondary_id}{$sec}{$value}{$id} = 1;
-  }
+	$self->{_maxidlength} = length($id)
+	  if !exists $self->{_maxidlength} or 
+		 length($id) >= $self->{_maxidlength};
 
-  $self->{_maxidlength} = length($id)
-    if !exists $self->{_maxidlength} or length($id) >= $self->{_maxidlength};
+	$self->{_maxfileidlength} = length($fileid)
+	  if !exists $self->{_maxfileidlength} or 
+		 length($fileid) >= $self->{_maxfileidlength};
 
-  $self->{_maxfileidlength} = length($fileid)
-    if !exists $self->{_maxfileidlength} or length($fileid) >= $self->{_maxidlength};
-
-  $self->{_maxposlength} = length($pos)
-    if !exists $self->{_maxposlength} or length($pos) >= $self->{_maxposlength};
-
-  $self->{_maxlengthlength} = length($length)
-      if !exists $self->{_maxlengthlength} or length($length) >= $self->{_maxlengthlength};
+	$self->{_maxposlength} = length($pos)
+	  if !exists $self->{_maxposlength} or 
+		 length($pos) >= $self->{_maxposlength};
 
+	$self->{_maxlengthlength} = length($length)
+	  if !exists $self->{_maxlengthlength} or 
+		 length($length) >= $self->{_maxlengthlength};
 }
 
 =head2 make_config_file
@@ -1143,10 +1151,10 @@ sub make_config_file {
 
     my $configfile = $self->_config_file;
 
-    open(CON,">$configfile") || $self->throw("Can't create config file [$configfile]");
+    open(my $CON,">", $configfile) || $self->throw("Can't create config file [$configfile]");
 
     # First line must be the type of index - in this case flat
-    print CON "index\tflat/1\n";
+    print $CON "index\tflat/1\n";
 
     # Now the fileids
 
@@ -1156,21 +1164,19 @@ sub make_config_file {
 
 	my $size = -s $file;
 
-	print CON "fileid_$count\t$file\t$size\n";
+	print $CON "fileid_$count\t$file\t$size\n";
 
 	my $fh;
-	open($fh,"<$file") || $self->throw($!);
-	$self->{_fileid}{$count}   = $fh;
+	open($fh,"<", $file) || $self->throw($!);
 	$self->{_file}  {$count}   = $file;
 	$self->{_dbfile}{$file}    = $count;
 	$self->{_size}{$count}     = $size; 
-	
 	$count++;
     }
 
     # Now the namespaces
 
-    print CON "primary_namespace\t" .$self->primary_namespace. "\n";
+    print $CON "primary_namespace\t" .$self->primary_namespace. "\n";
     
     # Needs fixing for the secondary stuff
 
@@ -1179,12 +1185,12 @@ sub make_config_file {
     my @second = keys %$second_patterns;
 
     if ((@second))  {
-	print CON "secondary_namespaces";
+	print $CON "secondary_namespaces";
 
 	foreach my $second (@second) {
-	    print CON "\t$second";
+	    print $CON "\t$second";
 	}
-        print CON "\n";
+        print $CON "\n";
     }
 
     # Now the config format
@@ -1195,9 +1201,9 @@ sub make_config_file {
 	my $format = $self->format;
 	my $alphabet = $self->alphabet;
 	my $alpha    = $alphabet ? "/$alphabet" : '';
-	print CON "format\t" . "$format\n";
+	print $CON "format\t" . "$format\n";
      }
-    close(CON);
+    close($CON);
 }
 
 =head2 read_config_file
@@ -1209,101 +1215,95 @@ sub make_config_file {
  Returns : value of read_config_file
  Args    : newvalue (optional)
 
-
 =cut
 
 sub read_config_file {
-    my ($self) = @_;
+	my ($self) = @_;
+	my $configfile = $self->_config_file;
+	return unless -e $configfile;
 
-    my $configfile = $self->_config_file;
-    return unless -e $configfile;
+	open(my $CON,"<", $configfile) || $self->throw("Can't open configfile [$configfile]");
 
-    open(CON,"<$configfile") || $self->throw("Can't open configfile [$configfile]");
+	# First line must be type
+	my $line = <$CON>; 
+	chomp($line);
+	my $version;
 
-    # First line must be type
+	# This is hard coded as we only index flatfiles here
+	if ($line =~ m{index\tflat/(\d+)}) {
+		$version = $1;
+	} else {
+		$self->throw("First line not compatible with flat file index.  Should be something like\n\nindex\tflat/1");
+	}
 
-    my $line = <CON>; chomp($line);
-    my $version;
+	$self->index_type("flat");
+	$self->index_version($version);
 
-    # This is hard coded as we only index flatfiles here
-    if ($line =~ /index\tflat\/(\d+)/) {
-	$version = $1;
-    } else {
-	$self->throw("First line not compatible with flat file index.  Should be something like\n\nindex\tflat/1");
-    }
+	while (<$CON>) {
+		chomp;
 
-    $self->index_type("flat");
-    $self->index_version($version);
+		# Look for fileid lines
+		if ($_ =~ /^fileid_(\d+)\t(.+)\t(\d+)/) {
+			my $fileid   = $1;
+			my $filename = $2;
+			my $filesize = $3;
 
-    while (<CON>) {
-	chomp;
-
-	# Look for fileid lines
-	if ($_ =~ /^fileid_(\d+)\t(\S+)\t(\d+)/) {
-	    my $fileid   = $1;
-	    my $filename = $2;
-	    my $filesize = $3;
-	    
-	    if (! -e $filename) {
-		$self->throw("File [$filename] does not exist!");
-	    }
-	    if (-s $filename != $filesize) {
-		$self->throw("Flatfile size for $filename differs from what the index thinks it is. Real size [" . (-s $filename) . "] Index thinks it is [" . $filesize  . "]");
-	    }
+			if (! -e $filename) {
+				$self->throw("File [$filename] does not exist!");
+			}
+			if (-s $filename != $filesize) {
+				$self->throw("Flatfile size for $filename differs from what the index thinks it is. Real size [" . (-s $filename) . "] Index thinks it is [" . $filesize  . "]");
+			}
 		
-	    my $fh;
-	    open($fh,"<$filename") || $self->throw($!);
-
-	    $self->{_fileid}{$fileid}   = $fh;
-	    $self->{_file}  {$fileid}   = $filename;
-	    $self->{_dbfile}{$filename} = $fileid;
-            $self->{_size}  {$fileid}   = $filesize; 
-
-	}
-
-	# Look for namespace lines
-	if (/(.*)_namespaces?\t(.+)/) {
-	    if ($1 eq "primary") {
-		$self->primary_namespace($2);
-	    } elsif ($1 eq "secondary") {
-		$self->secondary_namespaces(split "\t",$2);
-	    } else {
-		$self->throw("Unknown namespace name in config file [$1");
-	    }
-	}
+			my $fh;
+			open($fh,"<", $filename) || $self->throw($!);
+			$self->{_file}  {$fileid}   = $filename;
+			$self->{_dbfile}{$filename} = $fileid;
+			$self->{_size}  {$fileid}   = $filesize; 
+		}
+		
+		# Look for namespace lines
+		if ( /(.*)_namespaces?\t(.+)/ ) {
+	      if ($1 eq "primary") {
+		       $self->primary_namespace($2);
+	      } elsif ($1 eq "secondary") {
+		       $self->secondary_namespaces(split "\t",$2);
+	      } else {
+		       $self->throw("Unknown namespace name in config file [$1");
+	      }
+	   }
 	
-	# Look for format lines
+	   # Look for format lines
+	   if ($_ =~ /format\t(\S+)/) {
+	      # Check the format here?
+	      my $format = $1;
 
-	if ($_ =~ /format\t(\S+)/) {
-
-	  # Check the format here?
-	  my $format = $1;
-
-	  # handle LSID format
-	  if ($format =~ /^URN:LSID:open-bio\.org:(\w+)(?:\/(\w+))?/) {
-	      $self->format($1);
-	      $self->alphabet($2);
-	  } else {  # compatibility with older versions
-	    $self->format($1);
-	  }
-	}
+	      # handle LSID format
+	      if ($format =~ /^URN:LSID:open-bio\.org:(\w+)(?:\/(\w+))?/) {
+	         $self->format($1);
+	         $self->alphabet($2);
+	      } else {  # compatibility with older versions
+	         $self->format($1);
+	      }
+	    }
     }
-    close(CON);
+    
+    close($CON);
 
     # Now check we have all that we need
 
-    my @fileid_keys = keys (%{$self->{_fileid}});
+    my @fileid_keys = keys (%{$self->{_file}});
 
     if (!(@fileid_keys)) {
-	$self->throw("No flatfile fileid files in config - check the index has been made correctly");
+	     $self->throw("No flatfile fileid files in config - check the index has been made correctly");
     }
 
     if (!defined($self->primary_namespace)) {
-	$self->throw("No primary namespace exists");
+	    $self->throw("No primary namespace exists");
     }
 
     if (! -e $self->primary_index_file) {
-	$self->throw("Primary index file [" . $self->primary_index_file . "] doesn't exist");
+	    $self->throw("Primary index file [" . $self->primary_index_file . "] doesn't exist");
     }
 
     1;
@@ -1318,7 +1318,6 @@ sub read_config_file {
  Returns : value of get_fileid_by_filename
  Args    : newvalue (optional)
 
-
 =cut
 
 sub get_fileid_by_filename {
@@ -1341,17 +1340,18 @@ sub get_fileid_by_filename {
  Returns : value of get_filehandle_by_fileid
  Args    : newvalue (optional)
 
-
 =cut
 
 sub get_filehandle_by_fileid {
     my ($self,$fileid) = @_;
 
-    if (!defined($self->{_fileid}{$fileid})) {
+    if (!defined($self->{_file}{$fileid})) {
 	$self->throw("ERROR: undefined fileid in index [$fileid]");
     }
    
-    return $self->{_fileid}{$fileid};
+   	my $fh;
+	open($fh,"<", $self->{_file}{$fileid}) || $self->throw($!);
+    return $fh;
 }
 
 =head2 primary_index_file
@@ -1369,7 +1369,7 @@ sub get_filehandle_by_fileid {
 sub primary_index_file {
     my ($self) = @_;
 
-    return Bio::Root::IO->catfile($self->_config_path,"key_" . $self->primary_namespace . ".key");
+    return File::Spec->catfile($self->_config_path,"key_" . $self->primary_namespace . ".key");
 }
 
 =head2 primary_index_filehandle
@@ -1477,7 +1477,7 @@ sub index_directory {
     my ($self,$arg) = @_;
 
     if (defined($arg)) {
-	if ($arg !~ /\/$/) {
+	if ($arg !~ m{/$}) {
 	    $arg .= "/";
 	}
 	$self->{_index_directory} = $arg;
@@ -1490,13 +1490,13 @@ sub _config_path {
   my $self = shift;
   my $root = $self->index_directory;
   my $dbname = $self->dbname;
-  Bio::Root::IO->catfile($root,$dbname);
+  File::Spec->catfile($root,$dbname);
 }
 
 sub _config_file {
   my $self = shift;
   my $path = $self->_config_path;
-  Bio::Root::IO->catfile($path,CONFIG_FILE_NAME);
+  File::Spec->catfile($path,CONFIG_FILE_NAME);
 }
 
 =head2 record_size
@@ -1669,7 +1669,7 @@ sub new_SWISSPROT_index {
 
     $secondary_patterns{"ID"} = $start_pattern;
 
-    my $index =  new Bio::DB::Flat::BinarySearch
+    my $index =  Bio::DB::Flat::BinarySearch->new
 	(-index_dir          => $index_dir,
 	 -format             => 'swissprot',
 	 -primary_pattern    => $primary_pattern,
@@ -1691,7 +1691,7 @@ sub new_EMBL_index {
 
    $secondary_patterns{"ID"} = $start_pattern;
 
-   my $index = new Bio::DB::Flat::BinarySearch
+   my $index = Bio::DB::Flat::BinarySearch->new
        (-index_dir          => $index_dir,
 	-format             => 'embl',
 	-primary_pattern    => $primary_pattern,
@@ -1715,7 +1715,7 @@ sub new_FASTA_index {
 
    $secondary_patterns{"ID"} = "^>\\S+ +(\\S+)";
 
-   my $index =  new Bio::DB::Flat::BinarySearch
+   my $index =  Bio::DB::Flat::BinarySearch->new
        (-index_dir          => $index_dir,
 	-format             => 'fasta',
 	-primary_pattern    => $primary_pattern,
