@@ -1,4 +1,4 @@
-# $Id: dbi.pm,v 1.53 2004/07/21 12:57:40 lstein Exp $
+# $Id: dbi.pm 14716 2008-06-11 06:48:28Z heikki $
 
 =head1 NAME
 
@@ -22,21 +22,19 @@ package Bio::DB::GFF::Adaptor::dbi;
 use strict;
 
 use DBI;
-use Bio::DB::GFF;
 use Bio::DB::GFF::Util::Rearrange; # for rearrange()
 use Bio::DB::GFF::Util::Binning;
 use Bio::DB::GFF::Adaptor::dbi::iterator;
 use Bio::DB::GFF::Adaptor::dbi::caching_handle;
-use vars qw(@ISA);
 
-@ISA =  qw(Bio::DB::GFF);
+use base qw(Bio::DB::GFF);
 
 # constants for choosing
 
-use constant MAX_SEGMENT => 100_000_000;  # the largest a segment can get
+use constant MAX_SEGMENT => 1_000_000_000;  # the largest a segment can get
 
 # this is the largest that any reference sequence can be (100 megabases)
-use constant MAX_BIN    => 100_000_000;
+use constant MAX_BIN    => 1_000_000_000;
 
 # this is the smallest bin (1 K)
 use constant MIN_BIN    => 1000;
@@ -47,6 +45,10 @@ use constant STRAIGHT_JOIN_LIMIT => 200_000;
 # this is the size to which DNA should be shredded
 use constant DNA_CHUNK_SIZE  => 2000;
 
+# for debugging fbin optimization
+use constant EPSILON  => 1e-7;  # set to zero if you trust mysql's floating point comparisons
+use constant OPTIMIZE => 1;     # set to zero to turn off optimization completely
+
 ##############################################################################
 
 
@@ -80,7 +82,7 @@ sub new {
   my ($features_db,$username,$auth,$other) = rearrange([
 							[qw(FEATUREDB DB DSN)],
 							[qw(USERNAME USER)],
-							[qw(PASSWORD PASS)],
+							[qw(PASSWORD PASSWD PASS)],
 						       ],@_);
 
   $features_db  || $class->throw("new(): Provide a data source or DBI database");
@@ -362,6 +364,7 @@ by make_feature().  Internally, it invokes the following abstract procedures:
  make_features_select_part
  make_features_from_part
  make_features_by_name_where_part
+ make_features_by_alias_where_part  (for aliases)
  make_features_join_part
 
 =cut
@@ -380,16 +383,28 @@ sub _feature_by_name {
 								 class =>'',
 								 start=>$location->[1],
 								 stop =>$location->[2]}) if $location;
-  my $query  = "SELECT $select FROM $from WHERE $where AND $join";
-  $query    .= " AND $range" if $range;
-  my $sth    = $self->dbh->do_query($query,@args);
+  # group query
+  my $query1  = "SELECT $select FROM $from WHERE $where AND $join";
+  $query1    .= " AND $range" if $range;
+
+  # alias query
+  $from  = $self->make_features_from_part(undef,{attributes=>1});
+  ($where,@args) = $self->make_features_by_alias_where_part($class,$name);  # potential bug - @args1==@args2?
+
+  my $query2  = "SELECT $select FROM $from WHERE $where AND $join";
+  $query2    .= " AND $range" if $range;
 
   my $count = 0;
-  while (my @row = $sth->fetchrow_array) {
-    $callback->(@row);
-    $count++;
+
+  for my $query ($query1,$query2) {
+    my $sth    = $self->dbh->do_query($query,@args);
+    while (my @row = $sth->fetchrow_array) {
+      $callback->(@row);
+      $count++;
+    }
+    $sth->finish;
   }
-  $sth->finish;
+
   return $count;
 }
 
@@ -780,6 +795,7 @@ Each row of the returned array is a arrayref containing the following fields:
   column 1     A Bio::DB::GFF::Featname object, suitable for passing to segment()
   column 2     The text of the note
   column 3     A relevance score.
+  column 4     A Bio::DB::GFF::Typename object
 
 =cut
 
@@ -787,28 +803,32 @@ sub search_notes {
   my $self = shift;
   my ($search_string,$limit) = @_;
 
+  $search_string =~ tr/*?//d; 
+
   my @words  = $search_string =~ /(\w+)/g;
   my $regex  = join '|',@words;
   my @searches = map {"fattribute_value LIKE '%${_}%'"} @words;
   my $search   = join(' OR ',@searches);
 
   my $query = <<END;
-SELECT distinct gclass,gname,fattribute_value 
-  FROM fgroup,fattribute_to_feature,fdata
+SELECT distinct gclass,gname,fattribute_value,fmethod,fsource
+  FROM fgroup,fattribute_to_feature,fdata,ftype
   WHERE fgroup.gid=fdata.gid
      AND fdata.fid=fattribute_to_feature.fid
+     AND fdata.ftypeid=ftype.ftypeid
      AND ($search)
 END
 ;
 
   my $sth = $self->dbh->do_query($query);
   my @results;
-  while (my ($class,$name,$note) = $sth->fetchrow_array) {
+  while (my ($class,$name,$note,$method,$source) = $sth->fetchrow_array) {
      next unless $class && $name;    # sorry, ignore NULL objects
      my @matches = $note =~ /($regex)/g;
      my $relevance = 10*@matches;
      my $featname = Bio::DB::GFF::Featname->new($class=>$name);
-     push @results,[$featname,$note,$relevance];
+     my $type     = Bio::DB::GFF::Typename->new($method,$source);
+     push @results,[$featname,$note,$relevance,$type];
      last if $limit && @results >= $limit;
   }
   @results;
@@ -1127,6 +1147,25 @@ sub drop_all {
   }
 }
 
+=head2 clone
+
+The clone() method should be used when you want to pass the
+Bio::DB::GFF object to a child process across a fork(). The child must
+call clone() before making any queries.
+
+This method does two things: (1) it sets the underlying database
+handle's InactiveDestroy parameter to 1, thereby preventing the
+database connection from being destroyed in the parent when the dbh's
+destructor is called; (2) it replaces the dbh with the result of
+dbh-E<gt>clone(), so that we now have an independent handle.
+
+=cut
+
+sub clone {
+    my $self = shift;
+    $self->features_db->clone;
+}
+
 
 =head1 QUERIES TO IMPLEMENT
 
@@ -1251,6 +1290,8 @@ sub make_features_by_name_where_part {
   my $self = shift;
   my ($class,$name) = @_;
   if ($name =~ /\*/) {
+    $name =~ s/%/\\%/g;
+    $name =~ s/_/\\_/g;
     $name =~ tr/*/%/;
     return ("fgroup.gclass=? AND fgroup.gname LIKE ?",$class,$name);
   } else {
@@ -1258,6 +1299,19 @@ sub make_features_by_name_where_part {
   }
 }
 
+sub make_features_by_alias_where_part {
+  my $self = shift;
+  my ($class,$name) = @_;
+  if ($name =~ /\*/) {
+    $name =~ tr/*/%/;
+    $name =~ s/_/\\_/g;
+    return ("fgroup.gclass=? AND fattribute_to_feature.fattribute_value LIKE ? AND fgroup.gid=fdata.gid AND fattribute.fattribute_name in ('Alias','Name') AND fattribute_to_feature.fattribute_id=fattribute.fattribute_id AND fattribute_to_feature.fid=fdata.fid AND ftype.ftypeid=fdata.ftypeid",$class,$name)
+  } else {
+    return ("fgroup.gclass=? AND fattribute_to_feature.fattribute_value=? AND fgroup.gid=fdata.gid AND fattribute.fattribute_name in ('Alias','Name') AND fattribute_to_feature.fattribute_id=fattribute.fattribute_id AND fattribute_to_feature.fid=fdata.fid AND ftype.ftypeid=fdata.ftypeid",$class,$name);
+  }
+
+}
+
 sub make_features_by_attribute_where_part {
   my $self = shift;
   my $attributes = shift;
@@ -1622,12 +1676,16 @@ sub types_query {
     my ($method,$source) = @$type;
     my ($mlike, $slike) = (0, 0);
     if ($method && $method =~ m/\.\*/) {
-	$method =~ s/\.\*\??/%/g;
-	$mlike++;
+      $method =~ s/%/\\%/g;
+      $method =~ s/_/\\_/g;
+      $method =~ s/\.\*\??/%/g;
+      $mlike++;
     }
     if ($source && $source =~ m/\.\*/) {
-	$source =~ s/\.\*\??/%/g;
-	$slike++;
+      $source =~ s/%/\\%/g;
+      $source =~ s/_/\\_/g;
+      $source =~ s/\.\*\??/%/g;
+      $slike++;
     }
     my @pair;
     if (defined $method && length $method) {
@@ -1875,6 +1933,8 @@ sub make_abscoord_query {
   my $query = $self->getseqcoords_query();
   my $getforcedseqcoords = $self->getforcedseqcoords_query() ;
   if ($name =~ /\*/) {
+    $name =~ s/%/\\%/g;
+    $name =~ s/_/\\_/g;
     $name =~ tr/*/%/;
     $query =~ s/gname=\?/gname LIKE ?/;
   }
@@ -1888,6 +1948,8 @@ sub make_aliasabscoord_query {
   #my $query = GETALIASCOORDS;
   my $query = $self->getaliascoords_query();
   if ($name =~ /\*/) {
+    $name =~ s/%/\\%/g;
+    $name =~ s/_/\\_/g;
     $name =~ tr/*/%/;
     $query =~ s/gname=\?/gname LIKE ?/;
   }
@@ -1903,6 +1965,20 @@ sub getaliascoords_query {
 }
 
 sub bin_query {
+  my $self = shift;
+  my ($start,$stop,$minbin,$maxbin) = @_;
+  if ($start < 0 && $stop > 0) {  # split the queries
+    my ($lower_query,@lower_args) = $self->_bin_query($start,0,$minbin,$maxbin);
+    my ($upper_query,@upper_args) = $self->_bin_query(0,$stop,$minbin,$maxbin);
+    my $query = "$lower_query\n\t OR $upper_query";
+    my @args  = (@lower_args,@upper_args);
+    return wantarray ? ($query,@args) : $self->dbh->dbi_quote($query,@args);
+  } else {
+    return $self->_bin_query($start,$stop,$minbin,$maxbin);
+  }
+}
+
+sub _bin_query {
   my $self = shift;
   my ($start,$stop,$minbin,$maxbin) = @_;
   my ($query,@args);
@@ -1915,7 +1991,8 @@ sub bin_query {
   $maxbin = defined $maxbin ? $maxbin : $self->max_bin;
   my $tier = $maxbin;
   while ($tier >= $minbin) {
-    my ($tier_start,$tier_stop) = (bin_bot($tier,$start),bin_top($tier,$stop));
+    my ($tier_start,$tier_stop) = (bin_bot($tier,$start)-EPSILON(),bin_top($tier,$stop)+EPSILON());
+    ($tier_start,$tier_stop)    = ($tier_stop,$tier_start) if $tier_start > $tier_stop;  # can happen when working with negative coordinates
     if ($tier_start == $tier_stop) {
       push @bins,'fbin=?';
       push @args,$tier_start;
@@ -1935,10 +2012,17 @@ sub overlap_query {
   my $self = shift;
   my ($start,$stop) = @_;
 
-  my ($bq,@bargs)   = $self->bin_query($start,$stop);
-  my ($iq,@iargs) = $self->overlap_query_nobin($start,$stop);
-  my $query = "($bq)\n\tAND $iq";
-  my @args  = (@bargs,@iargs);
+  my ($query,@args);
+  my ($iq,@iargs)   = $self->overlap_query_nobin($start,$stop);
+  if (OPTIMIZE) {
+    my ($bq,@bargs)   = $self->bin_query($start,$stop);
+    $query = "($bq)\n\tAND $iq";
+    @args  = (@bargs,@iargs);
+  }
+  else {
+    $query = $iq;
+    @args  = @iargs;
+  }
 
   return wantarray ? ($query,@args) : $self->dbh->dbi_quote($query,@args);
 }
@@ -1965,12 +2049,30 @@ sub contained_in_query {
   return wantarray ? ($query,@args) : $self->dbh->dbi_quote($query,@args);
 }
 
+# implement the _delete_fattribute_to_feature() method
+sub _delete_fattribute_to_feature {
+  my $self         = shift;
+  my @feature_ids  = @_;
+  my $dbh          = $self->features_db;
+  my $fields       = join ',',map{$dbh->quote($_)} @feature_ids;
+
+  my $query = "delete from fattribute_to_feature where fid in ($fields)";
+  warn "$query\n" if $self->debug;
+  my $result = $dbh->do($query);
+  defined $result or $self->throw($dbh->errstr);
+  $result;
+}
+
 # implement the _delete_features() method
 sub _delete_features {
   my $self = shift;
   my @feature_ids = @_;
   my $dbh          = $self->features_db;
   my $fields       = join ',',map{$dbh->quote($_)} @feature_ids;
+
+  # delete from fattribute_to_feature
+  $self->_delete_fattribute_to_feature(@feature_ids);
+
   my $query = "delete from fdata where fid in ($fields)";
   warn "$query\n" if $self->debug;
   my $result = $dbh->do($query);
@@ -1985,15 +2087,15 @@ sub _delete_groups {
   my $dbh          = $self->features_db;
   my $fields       = join ',',map{$dbh->quote($_)} @group_ids;
 
-  my $query = "delete from fdata  where gid in ($fields)";
+  foreach my $gid (@group_ids){
+      my @features = $self->get_feature_by_gid($gid);
+      $self->delete_features(@features);
+  }
+
+  my $query  = "delete from fgroup where gid in ($fields)";
   warn "$query\n" if $self->debug;
   my $result = $dbh->do($query);
   defined $result or $self->throw($dbh->errstr);
-
-  $query  = "delete from fgroup where gid in ($fields)";
-  warn "$query\n" if $self->debug;
-  $result = $dbh->do($query);
-  defined $result or $self->throw($dbh->errstr);
   $result;
 }
 
