@@ -1,7 +1,9 @@
-# $Id: WebDBSeqI.pm,v 1.44 2004/11/29 20:45:03 bosborne Exp $
+# $Id: WebDBSeqI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::DB::WebDBSeqI
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
@@ -9,12 +11,12 @@
 # You may distribute this module under the same terms as perl itself
 #
 # POD documentation - main docs before the code
-#
+#  
 
 =head1 NAME
 
 Bio::DB::WebDBSeqI - Object Interface to generalize Web Databases
-  for retrieving sequences
+for retrieving sequences
 
 =head1 SYNOPSIS
 
@@ -25,10 +27,10 @@ Bio::DB::WebDBSeqI - Object Interface to generalize Web Databases
 =head1 DESCRIPTION
 
 Provides core set of functionality for connecting to a web based
-database for retrieving sequences.
+database for retriving sequences.
 
 Users wishing to add another Web Based Sequence Dabatase will need to
-extend this class (see Bio::DB::SwissProt or Bio::DB::NCBIHelper for
+extend this class (see L<Bio::DB::SwissProt> or L<Bio::DB::NCBIHelper> for
 examples) and implement the get_request method which returns a
 HTTP::Request for the specified uids (accessions, ids, etc depending
 on what query types the database accepts).
@@ -43,18 +45,27 @@ your comments and suggestions preferably to one
 of the Bioperl mailing lists. Your participation
 is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to
 help us keep track the bugs and their resolution.
-Bug reports can be submitted via email or the
-web:
+Bug reports can be submitted via the web.
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -72,26 +83,27 @@ preceded with a _
 
 package Bio::DB::WebDBSeqI;
 use strict;
-use vars qw(@ISA $MODVERSION %RETRIEVAL_TYPES $DEFAULT_RETRIEVAL_TYPE
-	    $DEFAULTFORMAT $LAST_INVOCATION_TIME);
+use vars qw($MODVERSION %RETRIEVAL_TYPES $DEFAULT_RETRIEVAL_TYPE
+	    $DEFAULTFORMAT $LAST_INVOCATION_TIME @ATTRIBUTES);
 
-use Bio::DB::RandomAccessI;
 use Bio::SeqIO;
 use Bio::Root::IO;
 use LWP::UserAgent;
+use POSIX 'setsid';
 use HTTP::Request::Common;
 use HTTP::Response;
 use File::Spec;
+use IO::Pipe;
 use IO::String;
 use Bio::Root::Root;
 
-@ISA = qw(Bio::DB::RandomAccessI);
+use base qw(Bio::DB::RandomAccessI);
 
 BEGIN {
-	$MODVERSION = $Bio::Root::Version::VERSION;
-	%RETRIEVAL_TYPES = ( 'io_string' => 1,
-								'tempfile'  => 1,
-								'pipeline'  => 1,
+	$MODVERSION = '0.8';
+	%RETRIEVAL_TYPES = ('io_string' => 1,
+								 'tempfile'  => 1,
+								 'pipeline'  => 1,
 							 );
 	$DEFAULT_RETRIEVAL_TYPE = 'pipeline';
 	$DEFAULTFORMAT = 'fasta';
@@ -99,35 +111,32 @@ BEGIN {
 }
 
 sub new {
-	my ($class, @args) = @_;
-	my $self = $class->SUPER::new(@args);
-	my ($baseaddress, $params, $ret_type, $format,$delay,$db) =
-	  $self->_rearrange([qw(BASEADDRESS PARAMS RETRIEVALTYPE FORMAT DELAY DB)],
-			  @args);
+    my ($class, @args) = @_;
+    my $self = $class->SUPER::new(@args);
+    my ($baseaddress, $params, $ret_type, $format,$delay,$db) =
+	 $self->_rearrange([qw(BASEADDRESS PARAMS RETRIEVALTYPE FORMAT DELAY DB)],
+							 @args);
 
-	$ret_type = $DEFAULT_RETRIEVAL_TYPE unless ( $ret_type);
-	$baseaddress   && $self->url_base_address($baseaddress);
-	$params        && $self->url_params($params);
-	$db            && $self->db($db);
-	$ret_type      && $self->retrieval_type($ret_type);
-	$self->retrieval_type('io_string') if $self->retrieval_type =~ /pipeline/
-	  && $^O =~ /^MSWin/;	# MSWin can't do pipes
-	$delay          = $self->delay_policy unless defined $delay;
-	$self->delay($delay);
+    $ret_type = $DEFAULT_RETRIEVAL_TYPE unless ( $ret_type);
+    $baseaddress   && $self->url_base_address($baseaddress);
+    $params        && $self->url_params($params);
+    $db            && $self->db($db);
+    $ret_type      && $self->retrieval_type($ret_type);
+    $delay          = $self->delay_policy unless defined $delay;
+    $self->delay($delay);
+    
 
-	# insure we always have a default format set for retrieval
-	# even though this will be immedietly overwritten by most sub classes
-	$format = $self->default_format unless ( defined $format &&
+    # insure we always have a default format set for retrieval
+    # even though this will be immedietly overwritten by most sub classes
+    $format = $self->default_format unless ( defined $format && 
 					     $format ne '' );
 
-	$self->request_format($format);
-	my $ua = new LWP::UserAgent(env_proxy => 1);
-	my $nm = ref($self);
-	$nm =~ s/::/_/g;
-	$ua->agent("bioperl-$nm/$MODVERSION");
-	$self->ua($ua);
-	$self->{'_authentication'} = [];
-	return $self;
+    $self->request_format($format);
+    my $ua = new LWP::UserAgent(env_proxy => 1);
+    $ua->agent(ref($self) ."/$MODVERSION");
+    $self->ua($ua);  
+    $self->{'_authentication'} = [];
+    return $self;
 }
 
 # from Bio::DB::RandomAccessI
@@ -145,20 +154,19 @@ sub new {
 =cut
 
 sub get_Seq_by_id {
-	my ($self,$seqid) = @_;
-	$self->_sleep;
-	my $seqio = $self->get_Stream_by_id([$seqid]);
-	unless( defined $seqio ) {
-		$self->warn("id ($seqid) does not exist");
-		return undef;
-	}
-	my @seqs;
-	while( my $seq = $seqio->next_seq() ) { push @seqs, $seq; }
-	unless( @seqs ) {
-		$self->warn("id ($seqid) does not exist");
-		return undef;
-	}
-	if( wantarray ) { return @seqs } else { return shift @seqs }
+    my ($self,$seqid) = @_;
+    $self->_sleep;
+    my $seqio = $self->get_Stream_by_id([$seqid]);
+    $self->throw("id does not exist") if( !defined $seqio ) ;
+    if ($self->can('complexity') &&  defined $self->complexity && $self->complexity==0) {
+        $self->warn("When complexity is set to 0, use get_Stream_by_id\n".
+                    "Returning Bio::SeqIO object");
+        return $seqio;
+    }
+    my @seqs;
+    while( my $seq = $seqio->next_seq() ) { push @seqs, $seq; }
+    $self->throw("id does not exist") unless @seqs;
+    if( wantarray ) { return @seqs } else { return shift @seqs }
 }
 
 =head2 get_Seq_by_acc
@@ -176,16 +184,15 @@ sub get_Seq_by_acc {
    my ($self,$seqid) = @_;
    $self->_sleep;
    my $seqio = $self->get_Stream_by_acc($seqid);
-   if( ! defined $seqio ) { 
-		$self->warn("acc ($seqid) does not exist");
-		return undef;
-   }
+   $self->throw("acc $seqid does not exist") if( ! defined $seqio );
+    if ($self->can('complexity') &&  defined $self->complexity && $self->complexity==0) {
+        $self->warn("When complexity is set to 0, use get_Stream_by_acc\n".
+                    "Returning Bio::SeqIO object");
+        return $seqio;
+    }
    my @seqs;
    while( my $seq = $seqio->next_seq() ) { push @seqs, $seq; }
-   unless( @seqs ) {
-       $self->warn("acc ($seqid) does not exist");
-       return undef;
-   }
+   $self->throw("acc $seqid does not exist") unless @seqs;
    if( wantarray ) { return @seqs } else { return shift @seqs }
 }
 
@@ -203,18 +210,17 @@ sub get_Seq_by_acc {
 
 sub get_Seq_by_gi {
    my ($self,$seqid) = @_;
-	$self->_sleep;
+    $self->_sleep;
    my $seqio = $self->get_Stream_by_gi($seqid);
-   unless( defined $seqio ) {
-		$self->warn("gi ($seqid) does not exist");
-		return undef;
-   }
+   $self->throw("gi does not exist") if( !defined $seqio );
+    if ($self->can('complexity') &&  defined $self->complexity && $self->complexity==0) {
+        $self->warn("When complexity is set to 0, use get_Stream_by_gi\n".
+                    "Returning Bio::SeqIO object");
+        return $seqio;
+    }
    my @seqs;
    while( my $seq = $seqio->next_seq() ) { push @seqs, $seq; }
-   unless( @seqs ) {
-		$self->warn("gi ($seqid) does not exist");
-		return undef;
-   }
+   $self->throw("gi does not exist") unless @seqs;
    if( wantarray ) { return @seqs } else { return shift @seqs }
 }
 
@@ -231,18 +237,17 @@ sub get_Seq_by_gi {
 
 sub get_Seq_by_version {
    my ($self,$seqid) = @_;
-	$self->_sleep;
+    $self->_sleep;
    my $seqio = $self->get_Stream_by_version($seqid);
-   unless( defined $seqio ) {
-		$self->warn("accession.version ($seqid) does not exist");
-		return undef;
-   }
-   my @seqs;
+   $self->throw("accession.version does not exist") if( !defined $seqio );
+    if ($self->can('complexity') &&  defined $self->complexity && $self->complexity==0) {
+        $self->warn("When complexity is set to 0, use get_Stream_by_version\n".
+                    "Returning Bio::SeqIO object");
+        return $seqio;
+    }
+    my @seqs;
    while( my $seq = $seqio->next_seq() ) { push @seqs, $seq; }
-   unless( @seqs ) {
-		$self->warn("accession.version ($seqid) does not exist");
-		return undef;
-   }
+   $self->throw("accession.version does not exist") unless @seqs;
    if( wantarray ) { return @seqs } else { return shift @seqs }
 }
 
@@ -259,9 +264,9 @@ sub get_Seq_by_version {
 =cut
 
 sub get_request {
-	my ($self) = @_;
-	my $msg = "Implementing class must define method get_request in class WebDBSeqI";
-	$self->throw($msg);
+    my ($self) = @_;
+    my $msg = "Implementing class must define method get_request in class WebDBSeqI";
+    $self->throw($msg);
 }
 
 # class methods
@@ -279,16 +284,16 @@ sub get_request {
 =cut
 
 sub get_Stream_by_id {
-	my ($self, $ids) = @_;
-	my ($webfmt,$localfmt) = $self->request_format;
-	return $self->get_seq_stream('-uids' => $ids, '-mode' => 'single',
-										  '-format' => $webfmt);
+    my ($self, $ids) = @_;
+    my ($webfmt,$localfmt) = $self->request_format;
+    return $self->get_seq_stream('-uids' => $ids, '-mode' => 'single',
+				 '-format' => $webfmt);
 }
 
 *get_Stream_by_batch = sub {
-	my $self = shift;
-	$self->deprecated('get_Stream_by_batch() is deprecated; use get_Stream_by_id() instead');
-	$self->get_Stream_by_id(@_) 
+  my $self = shift;
+  $self->deprecated('get_Stream_by_batch() is deprecated; use get_Stream_by_id() instead');
+  $self->get_Stream_by_id(@_) 
 };
 
 
@@ -305,8 +310,8 @@ sub get_Stream_by_id {
 =cut
 
 sub get_Stream_by_acc {
-	my ($self, $ids ) = @_;
-	return $self->get_seq_stream('-uids' => $ids, '-mode' => 'single');
+    my ($self, $ids ) = @_;
+    return $self->get_seq_stream('-uids' => $ids, '-mode' => 'single');
 }
 
 
@@ -323,8 +328,8 @@ sub get_Stream_by_acc {
 =cut
 
 sub get_Stream_by_gi {
-	my ($self, $ids ) = @_;
-	return $self->get_seq_stream('-uids' => $ids, '-mode' => 'gi');
+    my ($self, $ids ) = @_;
+    return $self->get_seq_stream('-uids' => $ids, '-mode' => 'gi');
 }
 
 =head2 get_Stream_by_version
@@ -340,10 +345,9 @@ sub get_Stream_by_gi {
 =cut
 
 sub get_Stream_by_version {
-	my ($self, $ids ) = @_;
+    my ($self, $ids ) = @_;
 #    $self->throw("Implementing class should define this method!"); 
-	return $self->get_seq_stream('-uids' => $ids, '-mode' => 'version'); 
-	# how it should work
+    return $self->get_seq_stream('-uids' => $ids, '-mode' => 'version'); # how it should work
 }
 
 =head2 get_Stream_by_query
@@ -380,10 +384,10 @@ sub default_format {
 
 # sorry, but this is hacked in because of BioFetch problems...
 sub db {
-	my $self = shift;
-	my $d    = $self->{_db};
-	$self->{_db} = shift if @_;
-	$d;
+  my $self = shift;
+  my $d    = $self->{_db};
+  $self->{_db} = shift if @_;
+  $d;
 }
 
 =head2 request_format
@@ -401,12 +405,12 @@ sub db {
 =cut
 
 sub request_format {
-	my ($self, $value) = @_;
+    my ($self, $value) = @_;
 
-	if( defined $value ) {
-		$self->{'_format'} = [ $value, $value];
-	}
-	return @{$self->{'_format'}};
+    if( defined $value ) {
+	$self->{'_format'} = [ $value, $value];
+    }
+    return @{$self->{'_format'}};
 }
 
 =head2 get_seq_stream
@@ -425,22 +429,27 @@ sub get_seq_stream {
 	my ($rformat, $ioformat) = $self->request_format();
 	my $seen = 0;
 	foreach my $key ( keys %qualifiers ) {
-      if( $key =~ /format/i ) {
+		if( $key =~ /format/i ) {
 			$rformat = $qualifiers{$key};
 			$seen = 1;
 		}
 	}
-	$qualifiers{'-format'} = $rformat if( ! $seen);
-	$qualifiers{'-offset'} = 0;
+	$qualifiers{'-format'} = $rformat if( !$seen);
 	($rformat, $ioformat) = $self->request_format($rformat);
+	# These parameters are implemented for Bio::DB::GenBank objects only
+	if($self->isa('Bio::DB::GenBank')) {
+		$self->seq_start() &&  ($qualifiers{'-seq_start'} = $self->seq_start());
+		$self->seq_stop() && ($qualifiers{'-seq_stop'} = $self->seq_stop());
+		$self->strand() && ($qualifiers{'-strand'} = $self->strand());
+		defined $self->complexity() && ($qualifiers{'-complexity'} = $self->complexity());
+	}
+	my $request = $self->get_request(%qualifiers);
+	$request->proxy_authorization_basic($self->authentication)
+	  if ( $self->authentication);
+	$self->debug("request is ". $request->as_string(). "\n");
 
 	# workaround for MSWin systems
-	# can this be removed now this is implemented in the "new" sub?
-	$self->retrieval_type('io_string') 
-	  if $self->retrieval_type =~ /pipeline/ && $^O =~ /^MSWin/;
-
-	my $expected = $qualifiers{-query} && ref $qualifiers{-query} ? $qualifiers{-query}->count : 0;
-	my $got      = 0;
+	$self->retrieval_type('io_string') if $self->retrieval_type =~ /pipeline/ && $^O =~ /^MSWin/;
 
 	if ($self->retrieval_type =~ /pipeline/) {
 		# Try to create a stream using POSIX fork-and-pipe facility.
@@ -451,30 +460,18 @@ sub get_seq_stream {
 		# If this fails (Windows, MacOS 9), we fall back to non-pipelined access.
 
 		# fork and pipe: _stream_request()=><STREAM>
-		my $result =  eval { open(STREAM,"-|") };
+		my ($result,$stream) = $self->_open_pipe();
 
 		if (defined $result) {
-			$DB::fork_TTY = '/dev/null'; # prevents complaints from debugger
-
-			if (!$result) {	# in child process
-				do {
-					$qualifiers{-offset} = $got;
-					my $request = $self->get_request(%qualifiers);
-					$request->proxy_authorization_basic($self->authentication)
-					  if ( $self->authentication);
-					$self->debug("request is ". $request->as_string(). "\n");
-					$got += $self->_stream_request($request);
-					$self->debug("expected $expected, got $got");
-				} until $got >= $expected;
-				kill 9=>$$; 
-				# to prevent END{} blocks from executing in forked children
-				exit 0;
+			$DB::fork_TTY = File::Spec->devnull; # prevents complaints from debugge
+			if (!$result) { # in child process
+			        $self->_stream_request($request,$stream);
+			        POSIX::_exit(0); #prevent END blocks from executing in this forked child
 			}
-
 			else {
 				return Bio::SeqIO->new('-verbose' => $self->verbose,
 											  '-format'  => $ioformat,
-											  '-fh'      => \*STREAM);
+											  '-fh'      => $stream);
 			}
 		}
 		else {
@@ -486,36 +483,25 @@ sub get_seq_stream {
 		my $dir = $self->io->tempdir( CLEANUP => 1);
 		my ( $fh, $tmpfile) = $self->io()->tempfile( DIR => $dir );
 		close $fh;
-
-		my $request = $self->get_request(%qualifiers);
-		$request->proxy_authorization_basic($self->authentication)
-		  if ( $self->authentication);
-		$self->debug("request is ". $request->as_string(). "\n");
-
-		my $resp = $self->_request($request, $tmpfile);
+		my $resp = $self->_request($request, $tmpfile);		
 		if( ! -e $tmpfile || -z $tmpfile || ! $resp->is_success() ) {
 			$self->throw("WebDBSeqI Error - check query sequences!\n");
 		}
 		$self->postprocess_data('type' => 'file',
-										'location' => $tmpfile);
+										'location' => $tmpfile);	
 		# this may get reset when requesting batch mode
 		($rformat,$ioformat) = $self->request_format();
 		if( $self->verbose > 0 ) {
-			open(ERR, "<$tmpfile");
-			while(<ERR>) { $self->debug($_);}
+			open(my $ERR, "<", $tmpfile);
+			while(<$ERR>) { $self->debug($_);}
 		}
+
 		return Bio::SeqIO->new('-verbose' => $self->verbose,
 									  '-format' => $ioformat,
 									  '-file'   => $tmpfile);
 	}
 
 	if ($self->retrieval_type =~ /io_string/i ) {
-
-		my $request = $self->get_request(%qualifiers);
-		$request->proxy_authorization_basic($self->authentication)
-		  if ( $self->authentication);
-		$self->debug("request is ". $request->as_string(). "\n");
-
 		my $resp = $self->_request($request);
 		my $content = $resp->content_ref;
 		$self->debug( "content is $$content\n");
@@ -532,7 +518,7 @@ sub get_seq_stream {
 	}
 
 	# if we got here, we don't know how to handle the retrieval type
-	$self->throw("retrieval type " . $self->retrieval_type .
+	$self->throw("retrieval type " . $self->retrieval_type . 
 					 " unsupported\n");
 }
 
@@ -548,10 +534,10 @@ sub get_seq_stream {
 =cut
 
 sub url_base_address {
-	my $self = shift;
-	my $d = $self->{'_baseaddress'};
-	$self->{'_baseaddress'} = shift if @_;
-	$d;
+    my $self = shift;
+    my $d = $self->{'_baseaddress'};
+    $self->{'_baseaddress'} = shift if @_;
+    $d;
 }
 
 
@@ -570,12 +556,12 @@ sub url_base_address {
 =cut
 
 sub proxy {
-	my ($self,$protocol,$proxy,$username,$password) = @_;
-	return undef if ( !defined $self->ua || !defined $protocol 
-							|| !defined $proxy );
-	$self->authentication($username, $password)
-	  if ($username && $password);
-	return $self->ua->proxy($protocol,$proxy);
+    my ($self,$protocol,$proxy,$username,$password) = @_;
+    return if ( !defined $self->ua || !defined $protocol 
+		      || !defined $proxy );
+    $self->authentication($username, $password) 	
+	if ($username && $password);
+    return $self->ua->proxy($protocol,$proxy);
 }
 
 =head2 authentication
@@ -593,7 +579,7 @@ sub authentication{
    my ($self,$u,$p) = @_;
 
    if( defined $u && defined $p ) {
-		$self->{'_authentication'} = [ $u,$p];
+       $self->{'_authentication'} = [ $u,$p];
    }
    return @{$self->{'_authentication'}};
 }
@@ -632,17 +618,17 @@ pipelining is not available.
 =cut
 
 sub retrieval_type {
-	my ($self, $value) = @_;
-	if( defined $value ) {
-		$value = lc $value;
-		if( ! $RETRIEVAL_TYPES{$value} ) {
-			$self->warn("invalid retrieval type $value must be one of (" . 
-							join(",", keys %RETRIEVAL_TYPES), ")"); 
-			$value = $DEFAULT_RETRIEVAL_TYPE;
-		}
-		$self->{'_retrieval_type'} = $value;
+    my ($self, $value) = @_;
+    if( defined $value ) {
+	$value = lc $value;
+	if( ! $RETRIEVAL_TYPES{$value} ) {
+	    $self->warn("invalid retrieval type $value must be one of (" . 
+			join(",", keys %RETRIEVAL_TYPES), ")"); 
+	    $value = $DEFAULT_RETRIEVAL_TYPE;
 	}
-	return $self->{'_retrieval_type'};
+	$self->{'_retrieval_type'} = $value;
+    }
+    return $self->{'_retrieval_type'};
 }
 
 =head2 url_params
@@ -660,7 +646,7 @@ sub url_params {
 	my ($self, $value) = @_;
 	if( defined $value ) {
 		$self->{'_urlparams'} = $value;
-	}
+	}    
 }
 
 =head2 ua
@@ -696,8 +682,8 @@ sub ua {
 =cut
 
 sub postprocess_data {
-    my ( $self, %args) = @_;
-    return;
+	my ( $self, %args) = @_;
+	return;
 }
 
 # private methods
@@ -706,69 +692,102 @@ sub _request {
 	my ($resp);
 	if( defined $tmpfile && $tmpfile ne '' ) { 
 		$resp =  $self->ua->request($url, $tmpfile);
-	} else {
+	} else { 
 		$resp =  $self->ua->request($url); 
-	}
-
+	} 
+	
 	if( $resp->is_error  ) {
 		$self->throw("WebDBSeqI Request Error:\n".$resp->as_string);
 	}
 	return $resp;
 }
 
-# send web request to stdout for streaming purposes
+#mod_perl-safe replacement for the open(BLEH,'-|') call.  if running
+#under mod_perl, detects it and closes the child's STDIN and STDOUT
+#handles
+sub _open_pipe {
+  my ($self) = @_;
+  # is mod_perl running?  Which API?
+  my $mp = $self->mod_perl_api;
+  if($mp and ! our $loaded_apache_sp) {
+    my $load_api = ($mp == 1) ? 'use Apache::SubProcess': 'use Apache2::SubProcess';
+    eval $load_api;
+    $@ and $self->throw("$@\n$load_api module required for running under mod_perl");
+    $loaded_apache_sp = 1;
+  }
+
+  my $pipe = IO::Pipe->new();
+
+  local $SIG{CHLD} = 'IGNORE';
+  defined(my $pid = fork)
+    or $self->throw("Couldn't fork: $!");
+
+  unless($pid) {
+    #CHILD
+    $pipe->writer();
+
+    #if we're running under mod_perl, clean up some things after this fork
+    if ($ENV{MOD_PERL} and my $r = eval{Apache->request} ) {
+      $r->cleanup_for_exec;
+      #don't read or write the mod_perl parent's tied filehandles
+      close STDIN; close STDOUT;
+      setsid() or $self->throw('Could not detach from parent');
+    }
+  } else {
+    #PARENT
+    $pipe->reader();
+  }
+  return ( $pid, $pipe );
+}
+
+# send web request to specified filehandle, or stdout, for streaming purposes
 sub _stream_request {
-	my $self    = shift;
-	my $request = shift;
+  my $self    = shift;
+  my $request = shift;
+  my $dest_fh = shift || \*STDOUT;
 
-	# fork so as to pipe output of fetch process through to
-	# postprocess_data method call.
-	my $child = open (FETCH,"-|");
-	$self->throw("Couldn't fork: $!") unless defined $child;
+  # fork so as to pipe output of fetch process through to
+  # postprocess_data method call.
+  my ($child,$fetch) = $self->_open_pipe();
 
-	my ($rformat, $ioformat) = $self->request_format;
-	my $is_fasta = $ioformat eq 'fasta';
-
-	if ($child) { # in parent
-		local ($/) =  $is_fasta ? ">" : "//\n";  # assume genbank/swiss format
-		$| = 1;
-		my $records = 0;
-		while (my $record = <FETCH>) {
-			chomp($record);
-			next unless $record =~ /\S/;
-			$records++;
-			$self->postprocess_data('type'     => 'string',
-											'location' => \$record);
-			print STDOUT $is_fasta ? ">$record" : "$record//\n";
-		}
-		$/ = "\n"; # reset to be safe;
-		close FETCH;
-		return $records;
-	}
-	else {
-		$| = 1;
-		my $resp =  $self->ua->request($request,
-												 sub { print shift }
-												);
-		if( $resp->is_error  ) {
-			$self->throw("WebDBSeqI Request Error:\n".$resp->as_string);
-		}
-
-		close STDOUT;
-		kill 9=>$$;  # to prevent END{} blocks from executing in forked children
-		sleep;
-	}
-	exit 0;
+  if ($child) {
+    #PARENT
+    local ($/) = "//\n";  # assume genbank/swiss format
+    $| = 1;
+    my $records = 0;
+    while (my $record = <$fetch>) {
+      $records++;
+      $self->postprocess_data('type'     => 'string',
+			      'location' => \$record);
+      print $dest_fh $record;
+    }
+    $/ = "\n"; # reset to be safe;
+    close $dest_fh; #must explicitly close here, because the hard
+                    #exits don't cloes them for us
+  }
+  else {
+    #CHILD
+    $| = 1;
+    my $resp =  $self->ua->request($request,
+				   sub { print $fetch $_[0] }
+				   );
+    if( $resp->is_error  ) {
+      $self->throw("WebDBSeqI Request Error:\n".$resp->as_string);
+    }
+    close $fetch; #must explicitly close here, because the hard exists
+                  #don't close them for us
+    POSIX::_exit(0);
+  }
 }
 
 sub io {
-	my ($self,$io) = @_;
+    my ($self,$io) = @_;
 
-	if(defined($io) || (! exists($self->{'_io'}))) {
-		$io = Bio::Root::IO->new() unless $io;
-		$self->{'_io'} = $io;
-	}
-	return $self->{'_io'};
+    if(defined($io) || (! exists($self->{'_io'}))) {
+	$io = Bio::Root::IO->new() unless $io;
+	$self->{'_io'} = $io;
+    }
+    return $self->{'_io'};
 }
 
 
@@ -831,10 +850,30 @@ sub _sleep {
    my $last_invocation = $LAST_INVOCATION_TIME;
    if (time - $LAST_INVOCATION_TIME < $self->delay) {
       my $delay = $self->delay - (time - $LAST_INVOCATION_TIME);
-      warn "sleeping for $delay seconds\n" if $self->verbose;
+      warn "sleeping for $delay seconds\n" if $self->verbose > 0;
       sleep $delay;
    }
    $LAST_INVOCATION_TIME = time;
 }
 
+=head2 mod_perl_api
+
+ Title   : mod_perl_api
+ Usage   : $version = self->mod_perl_api
+ Function: Returns API version of mod_perl being used based on set env. variables
+ Returns : mod_perl API version; if mod_perl isn't loaded, returns 0
+ Args    : none
+
+=cut
+
+sub mod_perl_api {
+    my $self = shift;
+    my $v = $ENV{MOD_PERL} ?
+            ( exists $ENV{MOD_PERL_API_VERSION} && $ENV{MOD_PERL_API_VERSION} >= 2 ) ?
+            2 :
+            1 
+        : 0;
+    return $v;
+}
+
 1;
