@@ -2,22 +2,22 @@
 
 =head1 NAME
 
-Bio::FeatureIO::gff - DESCRIPTION of Object
+Bio::FeatureIO::gff - read/write GFF feature files
 
 =head1 SYNOPSIS
 
   my $feature; #get a Bio::SeqFeature::Annotated somehow
-  my $featureOut = Bio::FeatureIO->new(-format => 'gff',
-                                       -version => 3,
-                                       -fh => \*STDOUT,
-                                       -validate_terms => 1, #boolean. validate ontology
-                                                             #terms online?  default 0 (false).
-                                      );
+  my $featureOut = Bio::FeatureIO->new(
+    -format => 'gff',
+    -version => 3,
+    -fh => \*STDOUT,
+    -validate_terms => 1, #boolean. validate ontology terms online?  default 0 (false).
+  );
   $featureOut->write_feature($feature);
 
 =head1 DESCRIPTION
 
- currently implemented:
+ Currently implemented:
 
  version         read?   write?
  ------------------------------
@@ -34,8 +34,19 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                 - General discussion
+  http://bioperl.org/wiki/Mailing_list  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
@@ -43,14 +54,12 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 the web:
 
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
  Allen Day, <allenday@ucla.edu>
 
-Describe contact details here
-
 =head1 CONTRIBUTORS
 
  Steffen Grossmann, <grossman@molgen.mpg.de>
@@ -85,6 +94,7 @@ use URI::Escape;
 use base qw(Bio::FeatureIO);
 
 use constant DEFAULT_VERSION => 3;
+my $RESERVED_TAGS   = "ID|Name|Alias|Parent|Target|Gap|Derives_from|Note|Dbxref|dbxref|Ontology_term|Index|CRUD";
 
 sub _initialize {
   my($self,%arg) = @_;
@@ -94,21 +104,23 @@ sub _initialize {
   $self->version( $arg{-version}        || DEFAULT_VERSION);
   $self->validate($arg{-validate_terms} || 0);
 
-  #read headers
-  my $directive;
-  while(($directive = $self->_readline()) && ( $directive =~ /^##/ || $directive =~ /^>/)){
-    $self->_handle_directive($directive);
-  }
-  $self->_pushback($directive);
-
   if ($arg{-file} =~ /^>.*/ ) {
     $self->_print("##gff-version " . $self->version() . "\n");
   }
-
+  else {
+    my $directive;
+    while(($directive = $self->_readline()) && ($directive =~ /^##/) ){
+      $self->_handle_directive($directive);
+    }
+    $self->_pushback($directive);
+  }
+  
   #need to validate against SOFA, no SO
-  $self->so(
-            Bio::Ontology::OntologyStore->get_ontology('Sequence Ontology Feature Annotation')
-           );
+  if ($self->validate) {
+    $self->so(
+              Bio::Ontology::OntologyStore->get_ontology('Sequence Ontology Feature Annotation')
+              );
+  }
 }
 
 =head2 next_feature()
@@ -129,7 +141,7 @@ sub next_feature {
     return $f;
   }
 
-  return undef if $self->fasta_mode();
+  return if $self->fasta_mode();
 
   # be graceful about empty lines or comments, and make sure we return undef
   # if the input is consumed
@@ -139,13 +151,13 @@ sub next_feature {
     last;
   }
 
-  return undef unless $gff_string;
+  return unless $gff_string;
 
   # looks like we went into FASTA mode without a directive.
   if($gff_string =~ /^>/){
     $self->_pushback($gff_string);
     $self->fasta_mode(1);
-    return undef;
+    return;
   }
 
   # got a directive
@@ -196,7 +208,8 @@ sub next_feature_group {
   while ($self->{group_not_done} && ($feat = $self->next_feature()) && defined($feat)) {
 	# we start by collecting all features in the group and
 	# memorizing those which have an ID attribute
-	if(my $anno_ID = $feat->get_Annotations('ID')) {
+    my $anno_ID = $feat->get_Annotations('ID');
+	if(ref($anno_ID)) {
       my $attr_ID = $anno_ID->value;
       $self->throw("Oops! ID $attr_ID exists more than once in your file!")
 		if (exists($seen_ids{$attr_ID}));
@@ -231,7 +244,7 @@ will return undef if not all features in the stream have been handled
 
 sub next_seq() {
   my $self = shift;
-  return undef unless $self->fasta_mode();
+  return unless $self->fasta_mode();
 
   #first time next_seq has been called.  initialize Bio::SeqIO instance
   if(!$self->seqio){
@@ -327,10 +340,10 @@ sub sequence_region {
     return $v;
   }
   elsif(defined($k)){
-    return $self->{'sequence-region'}{$k};
+    return $self->{'sequence_region'}{$k};
   }
   else {
-    return undef;
+    return;
   }
 }
 
@@ -382,7 +395,7 @@ sub version {
   my $self = shift;
   my $val = shift;
   my %valid = map {$_=>1} (1, 2, 2.5, 3);
-  if($valid{$val}){
+  if(defined $val && $valid{$val}){
     return $self->{'version'} = $val;
   }
   elsif(defined($val)){
@@ -417,7 +430,7 @@ sub _buffer_feature {
     return shift @{ $self->{'buffer'} };
   }
   else {
-    return undef;
+    return;
   }
 }
 
@@ -446,6 +459,8 @@ sub _handle_directive {
     # for these we want to store the seqid, start, and end. Then when we validate
     # we want to make sure that the features are within the seqid/start/end
 
+    $self->throw('Both start and end for sequence region should be defined')
+      unless $arg[1] && $arg[2];
     my $fta = Bio::Annotation::OntologyTerm->new();
     $fta->name( 'region');
 
@@ -479,13 +494,17 @@ sub _handle_directive {
   elsif($directive eq 'FASTA' or $directive =~ /^>/){
     #next_seq() will take care of this.
     $self->fasta_mode(1);
-    return undef;
+    return;
   }
 
   elsif($directive eq '#'){
     #all forward references resolved
     $self->{group_not_done} = 0;
-    $self->warn("'##$directive' directive handling not yet implemented");
+  }
+
+  elsif($directive eq 'organism') {
+    my $organism = $arg[0];
+    $self->organism($organism);
   }
 
   else {
@@ -508,7 +527,7 @@ sub _handle_feature {
   my($seq,$source,$type,$start,$end,$score,$strand,$phase,$attribute_string) = split /\t/, $feature_string;
 
   $feat->seq_id($seq);
-  $feat->source($source);
+  $feat->source_tag($source);
   $feat->start($start) unless $start eq '.';
   $feat->end($end) unless $end eq '.';
   $feat->strand($strand eq '+' ? 1 : $strand eq '-' ? -1 : 0);
@@ -582,21 +601,32 @@ sub _handle_feature {
 
   my %attr = ();
   chomp $attribute_string;
-  my @attributes = split ';', $attribute_string;
-  foreach my $attribute (@attributes){
-    my($key,$values) = split '=', $attribute;
 
-    # remove leading and trailing quotes from values
-    $values =~ s/^["']//;
-    $values =~ s/["']$//;
+  unless ( $attribute_string eq '.' ) {
+    my @attributes = split ';', $attribute_string;
+    foreach my $attribute (@attributes){
+      my($key,$values) = split '=', $attribute;
 
-    my @values = map{uri_unescape($_)} split ',', $values;
-    $attr{$key} = [@values];
+      # remove leading and trailing quotes from values
+      $values =~ s/^["']//;
+      $values =~ s/["']$//; #' terminate the quote for emacs
+
+      my @values = map{uri_unescape($_)} split ',', $values;
+
+     #minor hack to allow for multiple instances of the same tag
+      if ($attr{$key}) {
+        my @tmparray = @{$attr{$key}};
+        push @tmparray, @values;
+        $attr{$key} = [@tmparray];
+      } else {
+        $attr{$key} = [@values];
+      }
+    }
   }
 
   #Handle Dbxref attributes
-  if($attr{Dbxref}){
-    foreach my $value (@{ $attr{Dbxref} }){
+  if($attr{Dbxref} or $attr{dbxref}){
+    foreach my $value (@{ $attr{Dbxref} }, @{ $attr{dbxref} }){
       my $a = Bio::Annotation::DBLink->new();
       my($db,$accession) = $value =~ /^(.+?):(.+)$/;
 
@@ -631,7 +661,11 @@ sub _handle_feature {
 
   #Handle Gap attributes
   if($attr{Gap}){
-    $self->warn("Warning for line:\n$feature_string\nGap attribute handling not yet implemented, skipping it");
+     for my $value (@{ $attr{Gap} }) {
+       my $a = Bio::Annotation::SimpleValue->new();
+       $a->value($value);
+       $feat->add_Annotation('Gap',$a);
+     }
   }
 
   #Handle Target attributes
@@ -639,7 +673,14 @@ sub _handle_feature {
     my $target_collection = Bio::Annotation::Collection->new();
 
     foreach my $target_string (@{ $attr{Target} } ) {
-      $target_string =~ s/\+/ /g; 
+
+      #only replace + for space if + has been used in place of it
+      #that is, + could also mean plus strand, and we don't want
+      #to accidentally remove it
+ 
+      #presumably you can't use + for space and + for strand in the same string.      
+      $target_string =~ s/\+/ /g unless $target_string =~ / /; 
+
       my ($t_id,$tstart,$tend,$strand,$extra) = split /\s+/, $target_string; 
       if (!$tend || $extra) { # too much or too little stuff in the string
         $self->throw("The value in the Target string, $target_string, does not conform to the GFF3 specification");
@@ -675,7 +716,7 @@ sub _handle_feature {
     if ($self->{'allIDs'}->{${$attr{ID}}[0]} && $self->validate()) {
       $self->throw("Validation Error: The ID ${$attr{ID}}[0] occurs more than once in the file, but should be unique");
     }
-    $self->{'allIDs'}->{${$attr{ID}}[0]} = $feat;
+    $self->{'allIDs'}->{${$attr{ID}}[0]} = 1;
 
 
     my $a = Bio::Annotation::SimpleValue->new();
@@ -694,7 +735,7 @@ sub _handle_feature {
     $feat->add_Annotation('Name',$a);
   }
 
-  foreach my $other_canonical (qw(Alias Parent Note)){
+  foreach my $other_canonical (qw(Alias Parent Note Derives_from Index CRUD)){
     if($attr{$other_canonical}){
       foreach my $value (@{ $attr{$other_canonical} }){
         my $a = Bio::Annotation::SimpleValue->new();
@@ -706,11 +747,21 @@ sub _handle_feature {
 
   my @non_reserved_tags = grep {/^[a-z]/} keys %attr;
   foreach my $non_reserved_tag (@non_reserved_tags) {
+    next if ($non_reserved_tag eq 'dbxref');
     foreach my $value (@{ $attr{$non_reserved_tag} }){
       $feat = $self->_handle_non_reserved_tag($feat,$non_reserved_tag,$value);
     }
   }
 
+  my @illegal_tags = grep 
+ {!/($RESERVED_TAGS)/} 
+ grep {/^[A-Z]/} keys %attr;
+
+  if (@illegal_tags > 0) {
+      my $tags = join(", ", @illegal_tags);
+      $self->throw("The following tag(s) are illegal and are causing this parser to die: $tags");
+  }
+
   return $feat;
 }
 
@@ -735,13 +786,33 @@ sub _handle_non_reserved_tag {
   #  do something different
   # else { do what is below
 
-  my $a = Bio::Annotation::SimpleValue->new();
-  $a->value($value);
+  my $a;
+  if ($tag eq 'comment') {
+    $a = Bio::Annotation::Comment->new();
+  }
+  else {
+    $a = Bio::Annotation::SimpleValue->new();
+  }
+  $a->value($value); 
   $feat->add_Annotation($tag,$a);
-
+  
   return $feat;
 }
 
+=head1 organims
+
+Gets/sets the organims from the organism directive
+
+=cut
+
+sub organism {
+    my $self = shift;
+    my $organism = shift if defined(@_);
+    return $self->{'organism'} = $organism if defined($organism);
+    return $self->{'organism'};
+}
+
+
 =head1 _write_feature_1()
 
 write a feature in GFF v1 format.  currently not implemented.
@@ -774,25 +845,29 @@ sub _write_feature_25 {
   my($self,$feature,$group) = @_;
 
   #the top-level feature is an aggregate of all subfeatures
+  my ($transcript_id, $gene_id) = (($feature->get_Annotations('transcript_id'))[0], ($feature->get_Annotations('gene_id'))[0]);
   if(!defined($group)){
-    $group = ($feature->get_Annotations('ID'))[0]->value;
+    $group = ($feature->get_Annotations('ID'))[0];
+    $transcript_id ||= $group;
+    $gene_id ||= $group;
   }
+  
 
-  my $seq    = $feature->seq_id->value;
+  my $seq    = ref($feature->seq_id) ? $feature->seq_id->value : $feature->seq_id;
   my $source = $feature->source->value;
   my $type   = $feature->type->name;
   $type = 'EXON' if $type eq 'exon'; #a GTF peculiarity, incosistent with the sequence ontology.
   my $min    = $feature->start   || '.';
   my $max    = $feature->end     || '.';
   my $strand = $feature->strand == 1 ? '+' : $feature->strand == -1 ? '-' : '.';
-  my $score  = $feature->score->value;
-  my $phase  = $feature->phase->value;
+  my $score  = defined($feature->score) ? (ref($feature->score) ? $feature->score->value : $feature->score) : '.'; # score is optional
+  my $frame  = defined($feature->frame) ? (ref($feature->frame) ? $feature->frame->value : $feature->frame) : (ref($feature->phase) ? $feature->phase->value : $feature->phase);
 
   #these are the only valid types in a GTF document
   if($type eq 'EXON' or $type eq 'CDS' or $type eq 'start_codon' or $type eq 'stop_codon'){
-    my $attr = sprintf('gene_id "%s"; transcript_id "%s";',$group,$group);
+    my $attr = sprintf('gene_id "%s"; transcript_id "%s";',$gene_id ? $gene_id->value : '',$transcript_id ? $transcript_id->value : '');
     my $outstring = sprintf("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
-                            $seq,$source,$type,$min,$max,$score,$strand,$phase,$attr);
+                            $seq,$source,$type,$min,$max,$score,$strand,$frame eq '.' ? 0 : $frame,$attr);
 
     $self->_print($outstring);
   }
@@ -810,7 +885,7 @@ write a feature in GFF v3 format.
 
 sub _write_feature_3 {
   my($self,$feature) = @_;
-  my $seq    = $feature->seq_id->value;
+  my $seq    = ref($feature->seq_id) ? $feature->seq_id->value : $feature->seq_id;
   my $source;
   if ($feature->source()) {
     $source = $feature->source->value;
@@ -824,7 +899,7 @@ sub _write_feature_3 {
   my $min    = $feature->start   || '.';
   my $max    = $feature->end     || '.';
   my $strand = $feature->strand == 1 ? '+' : $feature->strand == -1 ? '-' : '.';
-  my $score  = $feature->score->value;
+  my $score  = defined($feature->score) ? (ref($feature->score) ? $feature->score->value : $feature->score) : undef;
   my $phase  = $feature->phase->value;
 
   my @attr;
@@ -854,7 +929,10 @@ sub _write_feature_3 {
     push @attr, "Note=$vstring";
   }
   if(my @v = ($feature->get_Annotations('Target'))){
-    my $vstring = join ',', map {uri_escape($_->target_id).' '.$_->start.' '.$_->end.($_->strand =~ /^[\+\-]$/ ? ' '.$_->strand : '')} @v;
+    my %strand_map = ( 1=>'+', 0=>'', -1=>'-', '+' => '+', '-' => '-' );
+    my $vstring = join ',', map {
+      uri_escape($_->target_id).' '.$_->start.' '.$_->end.(defined $_->strand ? ' '.$strand_map{$_->strand} : '')
+    } @v;
     push @attr, "Target=$vstring";
   }
 
