@@ -1,7 +1,9 @@
-# $Id: Loader.pm,v 1.17 2004/12/17 12:31:04 bosborne Exp $
+# $Id: Loader.pm 15635 2009-04-14 19:11:13Z cjfields $
 #
 # bioperl module for Bio::LiveSeq::IO::Loader
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Joseph Insana <insana@ebi.ac.uk> <jinsana@gmx.net>
 #
 # Copyright Joseph Insana
@@ -20,7 +22,7 @@ Bio::LiveSeq::IO::Loader - Parent Loader for LiveSeq
 
 =head1 DESCRIPTION
 
-This package holds common methods used by BioPerl, SRS and file loaders.
+This package holds common methods used by BioPerl and file loaders.
 It contains methods to create LiveSeq objects out of entire entries or from a
 localized sequence region surrounding a particular gene.
 
@@ -28,12 +30,6 @@ localized sequence region surrounding a particular gene.
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
 
-Address: 
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom 
-
 =head1 APPENDIX
 
 The rest of the documentation details each of the object
@@ -47,7 +43,6 @@ package Bio::LiveSeq::IO::Loader;
 
 use strict;
 use Carp qw(cluck croak carp);
-use vars qw(@ISA);
 use Bio::LiveSeq::DNA;
 use Bio::LiveSeq::Exon;
 use Bio::LiveSeq::Transcript ;
@@ -60,8 +55,6 @@ use Bio::LiveSeq::Repeat_Unit;
 use Bio::LiveSeq::AARange;
 use Bio::Tools::CodonTable;
 
-#@ISA=qw(Bio::LiveSeq::); # not useful now
-
 =head2 entry2liveseq
 
   Title   : entry2liveseq
@@ -161,7 +154,7 @@ sub gene2liveseq {
     return (0);
   }
   if (($gene_name)&&($cds_position)) {
-    carp "Gene_Name and Position cannot be given together, use one";
+    carp "Gene_Name and Position cannot be given together";
     return (0);
   } elsif ($gene_name) {
     $input=$gene_name;
@@ -505,11 +498,8 @@ sub transexonscreation {
             loader. Mainly used for testing purposes.
   Args    : a hashref containing the SWISSPROT entry datas
   Note    : the hashref can be obtained with a call to the method
-               $loader->get_swisshash()      (only with SRS loader or
-                                              BioPerl via Bio::DB::EMBL.pm)
-	    that takes as argument a string like "SWISS-PROT:P10275" or
-	    from $loader->swissprot2hash() that takes an SRS query string
-	    as its argument (e.g. "swissprot-acc:P10275")
+               $loader->get_swisshash()      (BioPerl via Bio::DB::EMBL.pm)
+	    that takes as argument a string like "SWISS-PROT:P10275"
 
 =cut
 
@@ -888,12 +878,12 @@ sub _get_alignment {
   my $fastafile2="/tmp/tmpfastafile2";
   my $grepcut='egrep -v "[[:digit:]]|^ *$|sequences" | cut -c8-'; # grep/cut
   my $alignprogram="/usr/local/etc/bioinfo/fasta2/align -s /usr/local/etc/bioinfo/fasta2/idnaa.mat $fastafile1 $fastafile2 2>/dev/null | $grepcut"; # ALIGN
-  open TMPFASTAFILE1,">$fastafile1" || croak "Cannot write into $fastafile1 for aa alignment";
-  open TMPFASTAFILE2,">$fastafile2" || croak "Cannot write into $fastafile1 for aa alignment";
-  print TMPFASTAFILE1 ">firstseq\n$seq1\n";
-  print TMPFASTAFILE2 ">secondseq\n$seq2\n";
-  close TMPFASTAFILE1;
-  close TMPFASTAFILE2;
+  open my $TMPFASTAFILE1,">$fastafile1" || croak "Cannot write into $fastafile1 for aa alignment";
+  open my $TMPFASTAFILE2,">$fastafile2" || croak "Cannot write into $fastafile1 for aa alignment";
+  print $TMPFASTAFILE1 ">firstseq\n$seq1\n";
+  print $TMPFASTAFILE2 ">secondseq\n$seq2\n";
+  close $TMPFASTAFILE1;
+  close $TMPFASTAFILE2;
   my $alignment=`$alignprogram`;
   my @alignlines=split(/\n/,$alignment);
   my ($linecount,$seq1_aligned,$seq2_aligned,$codes);
