@@ -1,7 +1,9 @@
-# $Id: Repeat_Region.pm,v 1.8 2003/06/04 08:36:40 heikki Exp $
+# $Id: Repeat_Region.pm 15635 2009-04-14 19:11:13Z cjfields $
 #
 # bioperl module for Bio::LiveSeq::Repeat_Region
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Joseph Insana <insana@ebi.ac.uk> <jinsana@gmx.net>
 #
 # Copyright Joseph Insana
@@ -28,12 +30,6 @@ The strand could be 1 (forward strand, default), -1 (reverse strand).
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
 
-Address: 
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, Regioned Kingdom 
-
 =head1 APPENDIX
 
 The rest of the documentation details each of the object
@@ -47,9 +43,7 @@ package Bio::LiveSeq::Repeat_Region;
 
 
 use strict;
-use vars qw(@ISA);
-use Bio::LiveSeq::Range; # uses Range, inherits from it
-@ISA=qw(Bio::LiveSeq::Range);
+use base qw(Bio::LiveSeq::Range);
 
 =head2 new
 
