@@ -1,7 +1,9 @@
-# $Id: SeqI.pm,v 1.26 2003/06/04 08:36:40 heikki Exp $
+# $Id: SeqI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # bioperl module for Bio::LiveSeq::SeqI
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Joseph Insana <insana@ebi.ac.uk> <jinsana@gmx.net>
 #
 # Copyright Joseph Insana
@@ -40,28 +42,32 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Joseph A.L. Insana
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
 
-Address: 
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom 
-
 =head1 APPENDIX
 
 The rest of the documentation details each of the object
@@ -84,12 +90,9 @@ Some note on the terminology/notation of method names:
 
 package Bio::LiveSeq::SeqI;
 use strict;
-use vars qw(@ISA);
-use Bio::LiveSeq::ChainI; # to inherit from it
 use Bio::Tools::CodonTable; # for the translate() function
-use Bio::PrimarySeqI;
 
-@ISA=qw(Bio::Root::Root Bio::LiveSeq::ChainI Bio::PrimarySeqI ); # inherit from ChainI
+use base qw(Bio::Root::Root Bio::LiveSeq::ChainI Bio::PrimarySeqI);
 
 =head2 seq
 
@@ -762,7 +765,6 @@ sub strand {
 
  Returns : a string either 'dna','rna','protein'.
  Args    : none
- Note    : "circular dna" is set as dna
 
 =cut
 
@@ -771,7 +773,8 @@ sub alphabet {
   my %valid_type = map {$_, 1} qw( dna rna protein );
   my ($self,$value) = @_;
   if (defined $value) {
-    $value =~ s/circular dna/dna/;
+    $value = 'dna' if $value =~ /dna/i;
+    $value = 'rna' if $value =~ /rna/i;
     unless ( $valid_type{$value} ) {
       $self->warn("Molecular type '$value' is not a valid type");
     }
