@@ -1,7 +1,9 @@
-# $Id: Transcript.pm,v 1.19 2003/06/04 08:36:40 heikki Exp $
+# $Id: Transcript.pm 15635 2009-04-14 19:11:13Z cjfields $
 #
 # bioperl module for Bio::LiveSeq::Transcript
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Joseph Insana <insana@ebi.ac.uk> <jinsana@gmx.net>
 #
 # Copyright Joseph Insana
@@ -28,12 +30,6 @@ Exon objects, inferring from them the nucleotide structure and sequence.
 
 Email:  Insana@ebi.ac.uk, jinsana@gmx.net
 
-Address: 
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom 
-
 =head1 APPENDIX
 
 The rest of the documentation details each of the object
@@ -47,10 +43,8 @@ package Bio::LiveSeq::Transcript;
 
 use strict;
 # use Carp qw(carp cluck);
-use vars qw(@ISA);
-use Bio::LiveSeq::SeqI; # uses SeqI, inherits from it
 use Bio::LiveSeq::Exon; # uses Exon to create new exon in case of deletion
-@ISA=qw(Bio::LiveSeq::SeqI);
+use base qw(Bio::LiveSeq::SeqI);
 
 =head2 new
 
@@ -783,7 +777,7 @@ sub translation_table {
     $self->{'translation_table'} = $value;
   }
   unless (exists $self->{'translation_table'}) {
-    return (undef);
+    return;
   } else {
     return $self->{'translation_table'};
   }
