@@ -1,10 +1,12 @@
-# $Id: Term.pm,v 1.23 2004/10/20 08:25:15 allenday Exp $
+# $Id: Term.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Ontology::Term
 #
-# Cared for by Christian M. Zmasek <czmasek@gnf.org> or <cmzmasek@yahoo.com>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
 #
-# (c) Christian M. Zmasek, czmasek@gnf.org, 2002.
+# Cared for by Christian M. Zmasek <czmasek-at-burnham.org> or <cmzmasek@yahoo.com>
+#
+# (c) Christian M. Zmasek, czmasek-at-burnham.org, 2002.
 # (c) GNF, Genomics Institute of the Novartis Research Foundation, 2002.
 #
 # You may distribute this module under the same terms as perl itself.
@@ -20,10 +22,9 @@
 
 # POD documentation - main docs before the code
 
-
 =head1 NAME
 
-Term - interface for ontology terms
+Bio::Ontology::Term - implementation of the interface for ontology terms
 
 =head1 SYNOPSIS
 
@@ -41,9 +42,9 @@ Term - interface for ontology terms
 
 =head1 DESCRIPTION
 
-This is "dumb" interface for ontology terms providing basic methods
-(it provides no functionality related to graphs). It implements the
-L<Bio::Ontology::TermI> interface.
+This is a simple implementation for ontology terms providing basic
+methods (it provides no functionality related to graphs). It
+implements the L<Bio::Ontology::TermI> interface.
 
 This class also implements L<Bio::IdentifiableI> and
 L<Bio::DescribableI>.
@@ -56,25 +57,34 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
 Christian M. Zmasek
 
-Email: czmasek@gnf.org  or  cmzmasek@yahoo.com
+Email: czmasek-at-burnham.org  or  cmzmasek@yahoo.com
 
-WWW:   http://www.genetics.wustl.edu/eddy/people/zmasek/
+WWW:   http://monochrome-effect.net/
 
 Address:
 
@@ -93,30 +103,21 @@ methods.
 # Let the code begin...
 
 package Bio::Ontology::Term;
-use vars qw( @ISA );
 use strict;
-use Bio::Root::Object;
-use Bio::Ontology::TermI;
 use Bio::Ontology::Ontology;
 use Bio::Ontology::OntologyStore;
-use Bio::IdentifiableI;
-use Bio::DescribableI;
+use Bio::Annotation::DBLink;
+use Data::Dumper;
 
 use constant TRUE    => 1;
 use constant FALSE   => 0;
 
-@ISA = qw( Bio::Root::Root
-           Bio::Ontology::TermI
-           Bio::IdentifiableI
-           Bio::DescribableI
-         );
-
-
+use base qw(Bio::Root::Root Bio::Ontology::TermI Bio::IdentifiableI Bio::DescribableI);
 
 =head2 new
 
  Title   : new
- Usage   : $term = Bio::Ontology::Term->new( 
+ Usage   : $term = Bio::Ontology::Term->new(
                 -identifier  => "16847",
                 -name        => "1-aminocyclopropane-1-carboxylate synthase",
                 -definition  => "Catalysis of ...",
@@ -128,14 +129,17 @@ use constant FALSE   => 0;
            -name                  => the name of this term [scalar]
            -definition            => the definition of this term [scalar]
            -ontology              => the ontology this term lives in
-                                     (a L<Bio::Ontology::OntologyI> object)
+                                     (a Bio::Ontology::OntologyI object)
            -version               => version information [scalar]
            -is_obsolete           => the obsoleteness of this term [0 or 1]
            -comment               => a comment [scalar]
-           -dblinks               => L<Bio::Annotation::DBLink> objects
-                                     [reference to array] 
-           -references            => L<Bio::Annotation::Reference> objects
+           -dblinks               => Bio::Annotation::DBLink objects
                                      [reference to array]
+           -references            => Bio::Annotation::Reference objects
+                                     [reference to array]
+
+See L<Bio::Ontology::OntologyI>, L<Bio::Annotation::Reference>,
+L<Bio::Annotation::DBLink>.
 
 =cut
 
@@ -144,33 +148,33 @@ sub new {
     my( $class,@args ) = @_;
 
     my $self = $class->SUPER::new( @args );
-#    return $self;
-#}
-
-#sub _initialize {
-#    my ($self, @args) =@_;
     my ( $identifier,
-         $name,
-         $definition,
-         $category,
-	 $ont,
-         $version,
-         $is_obsolete,
-         $comment,
-	 $dblinks, $references)
-	= $self->_rearrange( [ qw( IDENTIFIER
-				   NAME
-				   DEFINITION
-				   CATEGORY
-                   ONTOLOGY
-				   VERSION
-				   IS_OBSOLETE
-				   COMMENT
-                   DBLINKS REFERENCES
+        $name,
+        $definition,
+        $category,
+        $ont,
+        $version,
+        $is_obsolete,
+        $comment,
+        $dblinks,
+        $dbxrefs,
+        $references)
+        = $self->_rearrange( [
+        qw(IDENTIFIER
+        NAME
+        DEFINITION
+        CATEGORY
+        ONTOLOGY
+        VERSION
+        IS_OBSOLETE
+        COMMENT
+        DBLINKS
+        DBXREFS
+        REFERENCES
        ) ], @args );
 
     $self->init();
-    
+
     defined($identifier)   && $self->identifier( $identifier );
     defined($name)         && $self->name( $name );
     defined($definition)   && $self->definition( $definition );
@@ -179,6 +183,8 @@ sub new {
     defined($version)      && $self->version( $version );
     defined($is_obsolete)  && $self->is_obsolete( $is_obsolete );
     defined($comment)      && $self->comment( $comment  );
+    defined($dbxrefs)      && $self->add_dbxref(-dbxrefs => $dbxrefs);
+    # deprecated methods, allow to pass on to get the dep. notification
     ref($dblinks)          && $self->add_dblink(@$dblinks);
     ref($references)       && $self->add_reference(@$references);
 
@@ -198,7 +204,7 @@ sub init {
     $self->ontology(undef);
     $self->is_obsolete(0);
     $self->remove_synonyms();
-    $self->remove_dblinks();
+    $self->remove_dbxrefs();
     $self->remove_references;
     $self->remove_secondary_ids();
 
@@ -209,7 +215,7 @@ sub init {
 =head2 identifier
 
  Title   : identifier
- Usage   : $term->identifier( "0003947" );
+ Usage   : $term->identifier( "GO:0003947" );
            or
            print $term->identifier();
  Function: Set/get for the identifier of this Term.
@@ -226,8 +232,6 @@ sub identifier {
 } # identifier
 
 
-
-
 =head2 name
 
  Title   : name
@@ -248,9 +252,6 @@ sub name {
 } # name
 
 
-
-
-
 =head2 definition
 
  Title   : definition
@@ -280,13 +281,15 @@ sub definition {
  Function: Get the ontology this term is in.
 
            Note that with the ontology in hand you can query for all
-           related terms etc. See L<Bio::Ontology::OntologyI>.
+           related terms etc.
 
- Returns : The ontology of this Term as a L<Bio::Ontology::OntologyI>
+ Returns : The ontology of this Term as a Bio::Ontology::OntologyI
            implementing object.
- Args    : On set, the  ontology of this Term as a L<Bio::Ontology::OntologyI>
+ Args    : On set, the  ontology of this Term as a Bio::Ontology::OntologyI
            implementing object or a string representing its name.
 
+See L<Bio::Ontology::OntologyI>.
+
 =cut
 
 sub ontology {
@@ -294,20 +297,19 @@ sub ontology {
     my $ont;
 
     if(@_) {
-	$ont = shift;
-	if($ont) {
-	    $ont = Bio::Ontology::Ontology->new(-name => $ont) if ! ref($ont);
-	    if(! $ont->isa("Bio::Ontology::OntologyI")) {
-		$self->throw(ref($ont)." does not implement ".
-			     "Bio::Ontology::OntologyI. Bummer.");
-	    }
-	}
-	return $self->{"_ontology"} = $ont;
+        $ont = shift;
+        if($ont) {
+            $ont = Bio::Ontology::Ontology->new(-name => $ont) if ! ref($ont);
+            if(! $ont->isa("Bio::Ontology::OntologyI")) {
+                $self->throw(ref($ont)." does not implement ".
+                             "Bio::Ontology::OntologyI. Bummer.");
+            }
+        }
+        return $self->{"_ontology"} = $ont;
     }
     return $self->{"_ontology"};
 } # ontology
 
-
 =head2 version
 
  Title   : version
@@ -327,8 +329,6 @@ sub version {
     return $self->{'version'};
 } # version
 
-
-
 =head2 is_obsolete
 
  Title   : is_obsolete
@@ -349,9 +349,6 @@ sub is_obsolete{
 } # is_obsolete
 
 
-
-
-
 =head2 comment
 
  Title   : comment
@@ -371,9 +368,6 @@ sub comment{
     return $self->{'comment'};
 } # comment
 
-
-
-
 =head2 get_synonyms
 
  Title   : get_synonyms
@@ -411,8 +405,8 @@ sub add_synonym {
 
     # avoid duplicates
     foreach my $syn (@values) {
-	next if grep { $_ eq $syn; } @{$self->{ "_synonyms" }};
-	push( @{ $self->{ "_synonyms" } }, $syn );
+        next if grep { $_ eq $syn; } @{$self->{ "_synonyms" }};
+        push( @{ $self->{ "_synonyms" } }, $syn );
     }
 
 } # add_synonym
@@ -443,19 +437,52 @@ sub remove_synonyms {
  Usage   : @ds = $term->get_dblinks();
  Function: Returns a list of each dblinks of this GO term.
  Returns : A list of dblinks [array of [scalars]].
- Args    : context. If omitted, the default/context-less one will be used.
+ Args    : A scalar indicating the context (optional).
+           If omitted, all dblinks will be returned.
+ Note    : deprecated method due to past use of mixed data types; use
+           get_dbxrefs() instead, which handles both strings and DBLink
+           instances
 
 =cut
 
 sub get_dblinks {
-    my $self = shift;
-    my $context=shift ||'_default';
-    
-    return @{$self->{_dblinks}->{$context}} 
-        if exists($self->{_dblinks}->{$context});
-    return ();
+    my ($self, $context) = @_;
+    $self->deprecated("Use of get_dblinks is deprecated.  Note that prior use\n".
+                      "of this method could return either simple scalar values\n".
+                      "or Bio::Annotation::DBLink instances; only \n".
+                      "Bio::Annotation::DBLink is now supported.\n ".
+                      "Use get_dbxrefs() instead");
+    $self->get_dbxrefs($context);
 } # get_dblinks
 
+=head2 get_dbxrefs
+
+ Title   : get_dbxrefs()
+ Usage   : @ds = $term->get_dbxrefs();
+ Function: Returns a list of each link for this term.
+
+           If an implementor of this interface permits modification of
+           this array property, the class should define at least
+           methods add_dbxref() and remove_dbxrefs(), with obvious
+           functionality.
+
+ Returns : A list of L<Bio::Annotation::DBLink> instances
+ Args    : [optional] string which specifies context (default : returns all dbxrefs)
+
+=cut
+
+sub get_dbxrefs {
+    my ($self, $context) = shift;
+    my @dbxrefs;
+    if (defined($context)) {
+        if (exists($self->{_dblinks}->{$context})) {
+            @dbxrefs =  @{$self->{_dblinks}->{$context}};
+        }
+    } else {
+        @dbxrefs = map { @$_ } values %{$self->{_dblinks}} ;
+    }
+    return @dbxrefs;
+} # get_dbxrefs
 
 =head2 get_dblink_context
 
@@ -464,10 +491,28 @@ sub get_dblinks {
   Function: Return all context existing in Term
   Returns : a list of scalar
   Args    : [none]
+  Note    : deprecated method due to past use of mixed data types; use
+            get_dbxref_context() instead
 
-=cut 
+=cut
 
 sub get_dblink_context {
+    my $self=shift;
+    $self->deprecated("Use of get_dblink_context() is deprecated; use get_dbxref_context() instead");
+    return $self->get_dbxref_context(@_);
+}
+
+=head2 get_dbxref_context
+
+  Title   : get_dbxref_context
+  Usage   : @context = $term->get_dbxref_context;
+  Function: Return all context strings existing in Term
+  Returns : a list of scalars
+  Args    : [none]
+
+=cut
+
+sub get_dbxref_context {
     my $self=shift;
     return keys %{$self->{_dblinks}};
 }
@@ -480,49 +525,130 @@ sub get_dblink_context {
            $term->add_dblink( $dbl );
  Function: Pushes one or more dblinks onto the list of dblinks.
  Returns :
- Args    : One  dblink [scalar] or a list of
-            dblinks [array of [scalars]].
- Note    : For back compatibility, we keep it since it takes an array of 
-            arguments.
+ Args    : One or more L<Bio::Annotation::DBLink> instances
+ Note    : deprecated method due to past use of mixed data types; use
+           add_dbxref() instead, which handles both strings and
+           DBLink instances
+
 =cut
 
 sub add_dblink {
-    my ( $self, @values ) = @_;
-    my $context = '_default';
-    return unless( @values );
-
-    # avoid duplicates
-    foreach my $dbl (@values) {
-	    next if grep { $_ eq $dbl; } @{$self->{_dblinks}->{$context}};
-	    push( @{ $self->{_dblinks}->{$context} }, $dbl );
+    my $self = shift;
+    $self->deprecated("Use of simple strings and add_dblink() is deprecated; use\n".
+                      "Bio::Annotation::DBLink instances and add_dbxref() instead");
+    # here we're assuming the data is in a simple DB:ID format
+    my @dbxrefs;
+    for my $string (@_) {
+        my ($db, $id) = split(':',$string);
+        push @dbxrefs, Bio::Annotation::DBLink->new(-database => $db, -primary_id => $id);
     }
-
+    return $self->add_dbxref(-dbxrefs => \@dbxrefs, -context => '_default');
 } # add_dblink
 
-=head2 add_dblink_context 
+=head2 add_dbxref
 
-  Title   : add_db_link_context
+ Title   : add_dbxref
+ Usage   : $term->add_dbxref( @dbls );
+           or
+           $term->add_dbxref( $dbl );
+ Function: Pushes one or more dblinks onto the list of dblinks.
+ Returns :
+ Args    : -dbxrefs : array ref of Bio::Annotation::DBLink instances
+           -context : string designating the context for the DBLink
+                       (default : '_default' - contextless)
+
+=cut
+
+sub add_dbxref {
+    my $self = shift;
+    my ($links, $context) = $self->_rearrange([qw(DBXREFS CONTEXT)],@_);
+    return unless defined $links;
+    $context ||= '_default';
+    $self->throw("DBLinks must be passed as an array reference") if ref $links ne 'ARRAY';
+    foreach my $dbxref (@{$links}) {
+        $self->throw("$dbxref is not a DBLink") unless ref $dbxref &&
+            $dbxref->isa('Bio::Annotation::DBLink');
+        $self->throw("'all' is a reserved word for context.") if $context eq 'all';
+        if (! exists($self->{_dblinks}->{$context})) {
+            $self->{_dblinks}->{$context} = [];
+        }
+        my $linktext = ref $dbxref ? $dbxref->display_text : $dbxref;
+        if (grep {$_->display_text eq $linktext}
+            @{$self->{_dblinks}->{$context}})
+        {
+            $self->warn("DBLink exists in the dblink of $context");
+        }
+        push @{$self->{_dblinks}->{$context}}, $dbxref;    
+    }
+} # add_dbxref
+
+=head2 has_dblink
+
+  Title   : has_dblink
+  Usage   : $term->has_dblink($dblink);
+  Function: Checks if a DBXref is already existing in the OBOterm object
+  Return  : TRUE/FALSE
+  Args    : [arg1] A DBxref identifier
+  Note    : deprecated method due to past use of mixed data types; use
+            has_dbxref() instead, which handles both strings and
+            DBLink instances
+
+=cut
+
+sub has_dblink {
+    my ( $self, $value ) = @_;
+    $self->deprecated("use of has_dblink() is deprecated; use has_dbxref() instead");
+    return $self->has_dbxref($value);
+}
+
+=head2 has_dbxref
+
+  Title   : has_dbxref
+  Usage   : $term->has_dbxref($dbxref);
+  Function: Checks if a dbxref string is already existing in the OBOterm object
+  Return  : TRUE/FALSE
+  Args    : [arg1] A DBxref identifier (string).
+            Bio::Annotation::DBLink::display_text() is used for comparison
+            against the string.
+
+=cut
+
+sub has_dbxref {
+    my ( $self, $value ) = @_;
+    return unless defined $value;
+    my $context = "_default";
+    $self->throw("'all' is a reserved word for context.") if $context eq 'all';
+    $context ||= '_default';
+    if ( ( $self->{_dblinks}->{$context} ) &&
+        grep { $_->display_text eq $value } 
+        @{ $self->{_dblinks}->{$context} } )
+    {
+        return TRUE;
+    }
+    else {
+        return FALSE;
+    }
+}
+
+=head2 add_dblink_context
+
+  Title   : add_dblink_context
   Usage   : $term->add_dblink_context($db, $context);
   Function: add a dblink with its context
   Return  : [none]
-  Args    : [arg1] an object of Bio::Annotation::DBLink
-            [arg2] a string for context
-            context. If omitted, the default/context-less one will be used.
+  Args    : [arg1] a Bio::Annotation::DBLink instance
+            [arg2] a string for context; if omitted, the
+                   default/context-less one will be used.
+  Note    : deprecated method due to past use of mixed data types; use
+            add_dbxref() instead
 
 =cut
 
 sub add_dblink_context {
-    my ($self, $value, $context)=@_;
-    return unless defined $value;
-    $self->throw("'all' is a reserved word for context.") if $context eq 'all';
-    $context ||= '_default';
-    $self->{_dblink_context}->{$context}={}
-        unless exist $self->{_dblink_context}->{$context};
-        
-    if(grep {$_ eq $value} @{$self->{_dblink_context}->{$context}}){
-        $self->warn("$value exists in the dblink of $context");
-    }
-    push @{$self->{_dblink_context}->{$context}}, $value;
+    my ($self, $value, $context) = @_;
+    $self->deprecated("Use of simple strings and add_dblink_context() is deprecated; use\n
+                      Bio::Annotation::DBLink instances and add_dbxref() instead");
+    return $self->add_dbxref([$value],$context);
 }
 
 =head2 remove_dblinks
@@ -531,26 +657,42 @@ sub add_dblink_context {
  Usage   : $term->remove_dblinks();
  Function: Deletes (and returns) the definition references of this GO term.
  Returns : A list of definition references [array of [scalars]].
- Args    : context. If omitted, the default/context-less one will be used.
+ Args    : Context. If omitted or equal to 'all', all dblinks
+           will be removed.
+ Note    : deprecated method due to past use of mixed data types; use
+           remove_dblinks() instead, which handles both strings and
+           DBLink instances
 
 =cut
 
 sub remove_dblinks {
     my ($self, $context) = @_;
-    $context ||= '_default';
-    my @old;
-    if($context eq 'all'){
-        foreach my $sub_context($self->get_dblink_context){
-            push @old, $self->remove_db_links($sub_context);
-        }
-    }else{
-        push @old, $self->get_dblinks($context);
-        $self->{_dblinks}->{$context}=[];
-    }
-    return @old;
-
+    $self->deprecated("use of remove_dblinks() is deprecated; use remove_dbxrefs() instead");
+    return $self->remove_dbxrefs(@_);
 } # remove_dblinks
 
+=head2 remove_dbxrefs
+
+ Title   : remove_dbxrefs()
+ Usage   : $term->remove_dbxrefs();
+ Function: Deletes (and returns) the definition references of this GO term.
+ Returns : A list of definition references [array of [scalars]].
+ Args    : Context. If omitted or equal to 'all', all dblinks
+           will be removed.
+
+=cut
+
+sub remove_dbxrefs {
+    my ($self, $context) = @_;
+    $context = undef if $context && ($context eq "all");
+    my @old = $self->get_dbxrefs($context);
+    if (defined($context)) {
+        $self->{_dblinks}->{$context}=[];
+    } else {
+        $self->{_dblinks} = {};
+    }
+    return @old;
+} # remove_dbxrefs
 
 =head2 get_references
 
@@ -583,7 +725,9 @@ sub add_reference {
     return unless @values;
     # avoid duplicates and undefs
     foreach my $reference (@values){
-        next unless $reference;
+        $self->throw("Passed data not an Bio::Annotation::Reference") unless ref $reference &&
+            $reference->isa('Bio::AnnotationI');
+        next unless defined $reference;
         next if grep{$_ eq $reference} @{$self->{_references}};
         push @{$self->{_references}}, $reference;
     }
@@ -647,8 +791,8 @@ sub add_secondary_id {
 
     # avoid duplicates
     foreach my $id (@_) {
-	next if grep { !$_ or $_ eq $id; } @{$self->{ "_secondary_ids" }};
-	push( @{ $self->{ "_secondary_ids" } }, $id );
+        next if grep { !$_ or $_ eq $id; } @{$self->{ "_secondary_ids" }};
+        push( @{ $self->{ "_secondary_ids" } }, $id );
     }
 
 } # add_secondary_id
@@ -730,8 +874,8 @@ sub authority {
 
     return $ont->authority(@_) if $ont;
     $self->throw("cannot manipulate authority prior to ".
-		 "setting the namespace or ontology") if @_;
-    return undef;
+                 "setting the namespace or ontology") if @_;
+    return;
 }
 
 
@@ -767,8 +911,8 @@ sub namespace {
  Usage   : $string    = $obj->display_name()
  Function: A string which is what should be displayed to the user.
 
-           The definition in L<Bio::DescribableI> states that the
-           string should not contain spaces. As this isn't very
+           The definition in Bio::DescribableI states that the
+           string should not contain spaces. As this is not very
            sensible for ontology terms, we relax this here. The
            implementation just forwards to name().
 
@@ -794,7 +938,7 @@ sub display_name {
            This forwards to definition(). The caveat is that the text
            will often be longer for ontology term definitions than the
            255 characters stated in the definition in
-           L<Bio::DescribableI>.
+           Bio::DescribableI.
 
  Returns : A scalar
  Args    : on set, the new value (a scalar)
@@ -815,47 +959,9 @@ Used for looking up the methods that supercedes them.
 
 =cut
 
-=head2 category
-
- Title   : category
- Usage   :
- Function: This method is deprecated. Use ontology() instead.
- Example :
- Returns :
- Args    :
-
-
-=cut
-
-sub category {
-    my $self = shift;
-
-    $self->warn("TermI::category is deprecated and being phased out. ".
-		"Use TermI::ontology instead.");
-
-    # called in set mode?
-    if(@_) {
-	# yes; what is incompatible with ontology() is if we were given
-	# a TermI object
-	my $arg = shift;
-	$arg = $arg->name() if ref($arg) && $arg->isa("Bio::Ontology::TermI");
-	return $self->ontology($arg,@_);
-    } else {
-	# No, called in get mode. This is always incompatible with ontology()
-	# since category is supposed to return a TermI.
-	my $ont = $self->ontology();
-	my $term;
-	if(defined($ont)) {
-	    $term = Bio::Ontology::Term->new(-name => $ont->name(),
-					     -identifier =>$ont->identifier());
-	}
-	return $term;
-    }
-} # category
-
+sub each_dblink {shift->throw("use of each_dblink() is deprecated; use get_dbxrefs() instead")} 
+sub add_dblinks {shift->throw("use of add_dblinks() is deprecated; use add_dbxref() instead")}
 *each_synonym = \&get_synonyms;
 *add_synonyms = \&add_synonym;
-*each_dblink = \&get_dblinks;
-*add_dblinks = \&add_dblink;
 
 1;
