@@ -1,10 +1,12 @@
-# $Id: OMIMentry.pm,v 1.12 2004/08/26 07:01:02 juguang Exp $
+# $Id: OMIMentry.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Phenotype::OMIM::OMIMentry
 #
-# Cared for by Christian M. Zmasek <czmasek@gnf.org> or <cmzmasek@yahoo.com>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
 #
-# (c) Christian M. Zmasek, czmasek@gnf.org, 2002.
+# Cared for by Christian M. Zmasek <czmasek-at-burnham.org> or <cmzmasek@yahoo.com>
+#
+# (c) Christian M. Zmasek, czmasek-at-burnham.org, 2002.
 # (c) GNF, Genomics Institute of the Novartis Research Foundation, 2002.
 #
 # You may distribute this module under the same terms as perl itself.
@@ -22,7 +24,8 @@
 
 =head1 NAME
 
-OMIMentry - represents OMIM (Online Mendelian Inheritance in Man) database entries
+Bio::Phenotype::OMIM::OMIMentry - represents OMIM (Online Mendelian
+Inheritance in Man) database entries
 
 =head1 SYNOPSIS
 
@@ -44,25 +47,35 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the 
 Bioperl mailing lists  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                         - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.  Bug reports can be submitted via
- email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
 Christian M. Zmasek
 
-Email: czmasek@gnf.org  or  cmzmasek@yahoo.com
+Email: czmasek-at-burnham.org  or  cmzmasek@yahoo.com
 
-WWW:   http://www.genetics.wustl.edu/eddy/people/zmasek/
+WWW:   http://monochrome-effect.net/
 
 Address: 
 
@@ -82,10 +95,8 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::Phenotype::OMIM::OMIMentry;
-use vars qw( @ISA );
 use strict;
 
-use Bio::Phenotype::Phenotype;
 use Bio::Phenotype::OMIM::MiniMIMentry;
 use Bio::Phenotype::OMIM::OMIMentryAllelicVariant;
 
@@ -93,7 +104,7 @@ use constant TRUE              => 1;
 use constant FALSE             => 0;
 use constant DEFAULT_MIM_NUMER => 0;
 
-@ISA = qw( Bio::Phenotype::Phenotype );
+use base qw(Bio::Phenotype::Phenotype);
 
 
 
