@@ -1,10 +1,12 @@
-# $Id: OMIMparser.pm,v 1.14 2004/08/26 08:53:40 juguang Exp $
+# $Id: OMIMparser.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Phenotype::OMIM::OMIMparser
 #
-# Cared for by Christian M. Zmasek <czmasek@gnf.org> or <cmzmasek@yahoo.com>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
 #
-# (c) Christian M. Zmasek, czmasek@gnf.org, 2002.
+# Cared for by Christian M. Zmasek <czmasek-at-burnham.org> or <cmzmasek@yahoo.com>
+#
+# (c) Christian M. Zmasek, czmasek-at-burnham.org, 2002.
 # (c) GNF, Genomics Institute of the Novartis Research Foundation, 2002.
 #
 # You may distribute this module under the same terms as perl itself.
@@ -22,7 +24,7 @@
 
 =head1 NAME
 
-OMIMparser - parser for the OMIM database
+Bio::Phenotype::OMIM::OMIMparser - parser for the OMIM database
 
 =head1 SYNOPSIS
 
@@ -128,25 +130,35 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the 
 Bioperl mailing lists  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                         - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.  Bug reports can be submitted via
- email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
 Christian M. Zmasek
 
-Email: czmasek@gnf.org  or  cmzmasek@yahoo.com
+Email: czmasek-at-burnham.org  or  cmzmasek@yahoo.com
 
-WWW:   http://www.genetics.wustl.edu/eddy/people/zmasek/
+WWW:   http://monochrome-effect.net/
 
 Address: 
 
@@ -167,11 +179,9 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Phenotype::OMIM::OMIMparser;
 
-use vars qw( @ISA );
 use strict;
 
 use Bio::Root::IO;
-use Bio::Root::Root;
 use Bio::Species;
 use Bio::Annotation::Reference;
 use Bio::Map::CytoPosition;
@@ -179,7 +189,7 @@ use Bio::Phenotype::OMIM::OMIMentry;
 use Bio::Phenotype::OMIM::OMIMentryAllelicVariant;
 use Bio::Phenotype::Correlate;
 
-@ISA = qw( Bio::Root::Root );
+use base qw(Bio::Root::Root);
 
 
 use constant DEFAULT_STATE               => 0;
@@ -255,7 +265,7 @@ sub next_phenotype {
     }
     
     if ( $self->_done() == TRUE ) {
-        return undef;
+        return;
     }
 
     my $fieldtag          = "";
@@ -408,9 +418,7 @@ sub genemap_file_name {
 
     if ( defined $value ) {
         $self->{ "_genemap_file_name" } = $value;
-        if ( $value =~ /\W/ ) {
-            _genemap_hash( $self->_read_genemap( $value ) );
-        }
+        $self->_genemap_hash( $self->_read_genemap( $value ) );
     }
     
     return $self->{ "_genemap_file_name" };
@@ -435,7 +443,7 @@ sub omimtxt_file_name {
     if ( defined $value ) {
         $self->{ "_omimtxt_file_name" } = $value;
         if ( $value =~ /\W/ ) {
-            $self->_OMIM_text_file( new Bio::Root::IO->new( -file => $value ) );
+            $self->_OMIM_text_file( Bio::Root::IO->new->new( -file => $value ) );
         } 
     }
     
@@ -546,7 +554,7 @@ sub _finer_parse_symptoms {
     if( $text ) { 
 	my $part;
 	for my $line (split /\n/, $text){
-	    if($line =~ /^([\w\s]+)\:\s*$/){
+		if ($line =~ /^([\w\s,]+)\:\s*$/) {
 		$part = $1;
 	    } elsif( $line =~ /^\s+$/ ) {
 	    } elsif($line =~ /^(\s+)([^;]+)\;?\s*$/){
@@ -702,8 +710,8 @@ sub _create_allelic_variant {
          $symbol = $symbol_mut_line;
     }
     
-    if ( ! defined( $description ) ) { die( "undef desc" ); }
-    if ( ! defined( $mutation ) )   { die( "undef mutation" ); }
+    if ( ! defined( $description ) ) { $self->throw("undef desc"); }
+    if ( ! defined( $mutation ) )   { $self->throw("undef mutation"); }
   
     
     my $allelic_variant = Bio::Phenotype::OMIM::OMIMentryAllelicVariant->new();
@@ -877,7 +885,7 @@ sub _read_genemap {
     
     my $line         = "";
     my %genemap_hash = ();
-    my $genemap_file = new Bio::Root::IO->new( -file => $genemap_file_name );
+    my $genemap_file = Bio::Root::IO->new( -file => $genemap_file_name );
     my @a            = ();
     my %gm           = ();
     
@@ -885,7 +893,8 @@ sub _read_genemap {
         @a = split( /\|/, $line );
         unless( scalar( @a ) == 18 ) {
             $self->throw( "Gene map file \"".$self->genemap_file_name()
-            . "\" is not in the expected format" );
+            . "\" is not in the expected format."
+            . " Make sure there is a linebreak after the final line." );
         }
         $gm{ $a[ 9 ] } = $line;
     }
