@@ -1,8 +1,10 @@
-# $Id: PrimarySeq.pm,v 1.83 2004/10/15 02:31:57 bosborne Exp $
+# $Id: PrimarySeq.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # bioperl module for Bio::PrimarySeq
 #
-# Cared for by Ewan Birney <birney@sanger.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
 #
@@ -16,7 +18,7 @@ Bio::PrimarySeq - Bioperl lightweight Sequence Object
 
 =head1 SYNOPSIS
 
-  # The Bio::SeqIO for file reading, Bio::DB::GenBank for
+  # Bio::SeqIO for file reading, Bio::DB::GenBank for
   # database reading
 
   use Bio::Seq;
@@ -24,30 +26,30 @@ Bio::PrimarySeq - Bioperl lightweight Sequence Object
   use Bio::DB::GenBank;
 
   # make from memory
+
   $seqobj = Bio::PrimarySeq->new ( -seq => 'ATGGGGTGGGCGGTGGGTGGTTTG',
-				   -id  => 'GeneFragment-12',
-				   -accession_number => 'X78121',
-				   -alphabet => 'dna',
-				   -is_circular => 1
-				   );
-  print "Sequence ", $seqobj->id(), " with accession ", 
+                                   -id  => 'GeneFragment-12',
+                                   -accession_number => 'X78121',
+                                   -alphabet => 'dna',
+                                   -is_circular => 1 );
+  print "Sequence ", $seqobj->id(), " with accession ",
     $seqobj->accession_number, "\n";
 
   # read from file
-  $inputstream = Bio::SeqIO->new(-file => "myseq.fa",-format => 'Fasta');
+
+  $inputstream = Bio::SeqIO->new(-file => "myseq.fa",
+                                 -format => 'Fasta');
   $seqobj = $inputstream->next_seq();
   print "Sequence ", $seqobj->id(), " and desc ", $seqobj->desc, "\n";
 
-
   # to get out parts of the sequence.
 
-  print "Sequence ", $seqobj->id(), " with accession ", 
+  print "Sequence ", $seqobj->id(), " with accession ",
     $seqobj->accession_number, " and desc ", $seqobj->desc, "\n";
 
   $string  = $seqobj->seq();
   $string2 = $seqobj->subseq(1,40);
 
-
 =head1 DESCRIPTION
 
 PrimarySeq is a lightweight Sequence object, storing the sequence, its
@@ -83,23 +85,31 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney
 
-Email birney@sanger.ac.uk
-
-Describe contact details here
+Email birney@ebi.ac.uk
 
 =head1 APPENDIX
 
@@ -113,16 +123,14 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::PrimarySeq;
-use vars qw(@ISA);
+use vars qw($MATCHPATTERN $GAP_SYMBOLS);
 use strict;
 
-use Bio::Root::Root;
-use Bio::PrimarySeqI;
-use Bio::IdentifiableI;
-use Bio::DescribableI;
+$MATCHPATTERN = 'A-Za-z\-\.\*\?=~';
+$GAP_SYMBOLS = '-~';
 
-@ISA = qw(Bio::Root::Root Bio::PrimarySeqI
-	  Bio::IdentifiableI Bio::DescribableI);
+use base qw(Bio::Root::Root Bio::PrimarySeqI
+	    Bio::IdentifiableI Bio::DescribableI);
 
 #
 # setup the allowed values for alphabet()
@@ -148,9 +156,10 @@ my %valid_type = map {$_, 1} qw( dna rna protein );
            values.
  Returns : a new Bio::PrimarySeq object
  Args    : -seq         => sequence string
-           -display_id  => display id of the sequence (locus name) 
+           -display_id  => display id of the sequence (locus name)
            -accession_number => accession number
            -primary_id  => primary id (Genbank id)
+           -version     => version number
            -namespace   => the namespace for the accession
            -authority   => the authority for the namespace
            -description => description text
@@ -158,6 +167,9 @@ my %valid_type = map {$_, 1} qw( dna rna protein );
            -alphabet    => sequence type (alphabet) (dna|rna|protein)
            -id          => alias for display id
            -is_circular => boolean field for whether or not sequence is circular
+           -direct      => boolean field for directly setting sequence (requires alphabet also set)
+           -ref_to_seq  => boolean field indicating the sequence is a reference (?!?)
+           -nowarnonempty => boolean field for whether or not to warn when sequence is empty
 
 =cut
 
@@ -168,7 +180,7 @@ sub new {
 
     my($seq,$id,$acc,$pid,$ns,$auth,$v,$oid,
        $desc,$description,
-       $alphabet,$given_id,$is_circular,$direct,$ref_to_seq,$len) =
+       $alphabet,$given_id,$is_circular,$direct,$ref_to_seq,$len,$nowarnonempty) =
 	$self->_rearrange([qw(SEQ
 			      DISPLAY_ID
 			      ACCESSION_NUMBER
@@ -185,13 +197,18 @@ sub new {
 			      DIRECT
 			      REF_TO_SEQ
 			      LENGTH
+            NOWARNONEMPTY
 			      )],
 			  @args);
+  
+    # private var _nowarnonempty, need to be set before calling _guess_alphabet
+    $self->{'_nowarnonempty'} = $nowarnonempty; 
+
     if( defined $id && defined $given_id ) {
-		 if( $id ne $given_id ) {
-			 $self->throw("Provided both id and display_id constructor ".
-							  "functions. [$id] [$given_id]");
-		 }
+      if( $id ne $given_id ) {
+        $self->throw("Provided both id and display_id constructor ".
+            "functions. [$id] [$given_id]");
+      }
     }
     if( defined $given_id ) { $id = $given_id; }
 
@@ -203,21 +220,21 @@ sub new {
     # when the sequence is set
     $alphabet && $self->alphabet($alphabet);
 
-    # if there is an alphabet, and direct is passed in, assumme the alphabet
-    # and sequence is ok 
-
+    # if there is an alphabet, and direct is passed in, assume the alphabet
+    # and sequence is ok
+    
     if( $direct && $ref_to_seq) {
-		 $self->{'seq'} = $$ref_to_seq;
-		 if( ! $alphabet ) {
-			 $self->_guess_alphabet();
-		 } # else it has been set already above
+      $self->{'seq'} = $$ref_to_seq;
+        if( ! $alphabet ) {
+          $self->_guess_alphabet();
+        } # else it has been set already above
     } else {
 		 #	print STDERR "DEBUG: setting sequence to [$seq]\n";
 		 # note: the sequence string may be empty
 		 $self->seq($seq) if defined($seq);
 	 }
 
-    $id          && $self->display_id($id);
+    defined $id  && $self->display_id($id);
     $acc         && $self->accession_number($acc);
     defined $pid && $self->primary_id($pid);
     $desc        && $self->desc($desc);
@@ -228,13 +245,14 @@ sub new {
     defined($v)  && $self->version($v);
     defined($oid) && $self->object_id($oid);
 
+
     return $self;
 }
 
 sub direct_seq_set {
     my $obj = shift;
     return $obj->{'seq'} = shift if @_;
-    return undef;
+    return;
 }
 
 
@@ -249,8 +267,6 @@ sub direct_seq_set {
  Returns : A scalar
  Args    : Optionally on set the new value (a string). An optional second
            argument presets the alphabet (otherwise it will be guessed).
-           Both parameters may also be given in named paramater style
-           with -seq and -alphabet being the names.
 
 =cut
 
@@ -264,8 +280,8 @@ sub seq {
    my ($value,$alphabet) = @args;
 
    if(@args) {
-		if(defined($value) && (! $obj->validate_seq($value))) {
-			$obj->throw("Attempting to set the sequence to [$value] ".
+       if(defined($value) && (! $obj->validate_seq($value))) {
+	   $obj->throw("Attempting to set the sequence to [$value] ".
 							"which does not look healthy");
 		}
        # if a sequence was already set we make sure that we re-adjust the
@@ -296,7 +312,7 @@ sub seq {
 
  Title   : validate_seq
  Usage   : if(! $seq->validate_seq($seq_str) ) {
-                print "sequence $seq_str is not valid for an object of 
+                print "sequence $seq_str is not valid for an object of
                 alphabet ",$seq->alphabet, "\n";
 	   }
  Function: Validates a given sequence string. A validating sequence string
@@ -304,7 +320,8 @@ sub seq {
            lead to an exception if passed to seq().
 
            The implementation provided here does not take alphabet() into
-           account. Allowed are all letters (A-Z) and '-','.', '*' and '?'.
+           account. Allowed are all letters (A-Z) and '-','.','*','?','=',
+           and '~'.
 
  Example :
  Returns : 1 if the supplied sequence string is valid for the object, and
@@ -317,10 +334,11 @@ sub seq {
 sub validate_seq {
 	my ($self,$seqstr) = @_;
 	if( ! defined $seqstr ){ $seqstr = $self->seq(); }
-	return 0 unless( defined $seqstr); 
-	if((CORE::length($seqstr) > 0) && ($seqstr !~ /^([A-Za-z\-\.\*\?]+)$/)) {
-		$self->warn("seq doesn't validate, mismatch is " .
-						($seqstr =~ /([^A-Za-z\-\.\*\?]+)/g));
+	return 0 unless( defined $seqstr);
+	if((CORE::length($seqstr) > 0) &&
+	   ($seqstr !~ /^([$MATCHPATTERN]+)$/)) {
+	    $self->warn("seq doesn't validate, mismatch is " .
+			join(",",($seqstr =~ /([^$MATCHPATTERN]+)/g)));
 		return 0;
 	}
 	return 1;
@@ -330,27 +348,42 @@ sub validate_seq {
 
  Title   : subseq
  Usage   : $substring = $obj->subseq(10,40);
- Function: returns the subseq from start to end, where the first base
-           is 1 and the number is inclusive, ie 1-2 are the first two
-           bases of the sequence
+           $substring = $obj->subseq(10,40,NOGAP)
+           $substring = $obj->subseq(-START=>10,-END=>40,-REPLACE_WITH=>'tga')
+ Function: returns the subseq from start to end, where the first sequence
+           character has coordinate 1 number is inclusive, ie 1-2 are the 
+           first two characters of the sequence
  Returns : a string
  Args    : integer for start position
            integer for end position
                  OR
            Bio::LocationI location for subseq (strand honored)
+           Specify -NOGAP=>1 to return subseq with gap characters removed
+           Specify -REPLACE_WITH=>$new_subseq to replace the subseq returned
+           with $new_subseq in the sequence object
 
 =cut
 
 sub subseq {
-   my ($self,$start,$end,$replace) = @_;
+   my $self = shift;
+   my @args = @_;
+   my ($start,$end,$nogap,$replace) = $self->_rearrange([qw(START 
+                                                            END
+                                                            NOGAP
+                                                            REPLACE_WITH)],@args);
+   
+   # if $replace is specified, have the constructor validate it as seq
+   my $dummy = new Bio::PrimarySeq(-seq=>$replace, -alphabet=>$self->alphabet) if defined($replace);
 
    if( ref($start) && $start->isa('Bio::LocationI') ) {
        my $loc = $start;
-       $replace = $end; # do we really use this anywhere? scary. HL
        my $seq = "";
        foreach my $subloc ($loc->each_Location()) {
-	   my $piece = $self->subseq($subloc->start(),
-				     $subloc->end(), $replace);
+	   my $piece = $self->subseq(-START=>$subloc->start(),
+				     '-END'=>$subloc->end(), 
+				     -REPLACE_WITH=>$replace,
+	                             -NOGAP=>$nogap);
+	   $piece =~ s/[$GAP_SYMBOLS]//g if $nogap;
 	   if($subloc->strand() < 0) {
 	       $piece = Bio::PrimarySeq->new('-seq' => $piece)->revcom()->seq();
 	   }
@@ -371,14 +404,14 @@ sub subseq {
 
        # remove one from start, and then length is end-start
        $start--;
-       if( defined $replace ) {
-	   return substr( $self->seq(), $start, ($end-$start), $replace);
-       } else {
-	   return substr( $self->seq(), $start, ($end-$start));
-       }
+       my @ss_args = map { eval "defined $_"  ? $_ : () } qw( $self->{seq} $start $end-$start $replace);
+       my $seqstr = eval join( '', "substr(", join(',',@ss_args), ")");
+       $seqstr =~ s/[$GAP_SYMBOLS]//g if ($nogap);
+       return $seqstr;
+
    } else {
-       $self->warn("Incorrect parameters to subseq - must be two integers ".
-		   "or a Bio::LocationI object not ($start,$end)");
+       $self->warn("Incorrect parameters to subseq - must be two integers or a Bio::LocationI object. Got:", $self,$start,$end,$replace,$nogap);
+       return;
    }
 }
 
@@ -476,7 +509,7 @@ sub display_id {
 
            [Note this method name is likely to change in 1.3]
 
-           With the new Bio::IdentifiableI interface, this is aliased 
+           With the new Bio::IdentifiableI interface, this is aliased
            to object_id
 
  Returns : A string
@@ -632,6 +665,7 @@ sub is_circular{
     return $self->{'is_circular'};
 }
 
+
 =head1 Methods for Bio::IdentifiableI compliance
 
 =cut
@@ -681,7 +715,7 @@ sub version{
  Title   : authority
  Usage   : $authority    = $obj->authority()
  Function: A string which represents the organisation which
-           granted the namespace, written as the DNS name for  
+           granted the namespace, written as the DNS name for
            organisation (eg, wormbase.org).
 
  Returns : A scalar
@@ -730,7 +764,7 @@ This comprises of display_name and description.
  Function: A string which is what should be displayed to the user.
            The string should have no spaces (ideally, though a cautious
            user of this interface would not assumme this) and should be
-           less than thirty characters (though again, double checking 
+           less than thirty characters (though again, double checking
            this is a good idea).
 
            This is aliased to display_id().
@@ -746,7 +780,7 @@ sub display_name {
 
  Title   : description
  Usage   : $string    = $obj->description()
- Function: A text string suitable for displaying to the user a 
+ Function: A text string suitable for displaying to the user a
            description. This string is likely to have spaces, but
            should not have any newlines or formatting - just plain
            text. The string should not be greater than 255 characters
@@ -817,10 +851,10 @@ These are internal methods to PrimarySeq
 
  Title   : _guess_alphabet
  Usage   :
- Function:
+ Function: Determines (and sets) the type of sequence: dna, rna, protein
  Example :
- Returns :
- Args    :
+ Returns : one of strings 'dna', 'rna' or 'protein'.
+ Args    : none
 
 
 =cut
@@ -829,14 +863,19 @@ sub _guess_alphabet {
    my ($self) = @_;
    my $type;
 
+	#return if $self->alphabet;
+
    my $str = $self->seq();
 	# Remove char's that clearly denote ambiguity
-   $str =~ s/[-.?x]//gi;
+   $str =~ s/[-.?]//gi;
 
    my $total = CORE::length($str);
    if( $total == 0 ) {
-       $self->throw("Got a sequence with no letters in it ".
-		    "cannot guess alphabet [$str]");
+     if (!$self->{'_nowarnonempty'}) {
+       $self->warn("Got a sequence with no letters in it ".
+           "cannot guess alphabet");
+     }
+     return '';
    }
 
    my $u = ($str =~ tr/Uu//);
