@@ -1,8 +1,10 @@
-# $Id: PrimarySeqI.pm,v 1.55 2003/10/25 14:38:39 heikki Exp $
+# $Id: PrimarySeqI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::PrimarySeqI
 #
-# Cared for by Ewan Birney <birney@sanger.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
 #
@@ -12,45 +14,37 @@
 
 =head1 NAME
 
-Bio::PrimarySeqI [Developers] - Interface definition for a Bio::PrimarySeq
+Bio::PrimarySeqI - Interface definition for a Bio::PrimarySeq
 
 =head1 SYNOPSIS
 
-
     # Bio::PrimarySeqI is the interface class for sequences.
+    # If you are a newcomer to bioperl, you might want to start with
+    # Bio::Seq documentation.
 
-    # If you are a newcomer to bioperl, you should
-    # start with Bio::Seq documentation. This
-    # documentation is mainly for developers using
-    # Bioperl.
-
-    # to test this is a seq object
-
+    # Test if this is a seq object
     $obj->isa("Bio::PrimarySeqI") ||
       $obj->throw("$obj does not implement the Bio::PrimarySeqI interface");
 
-    # accessors
-
+    # Accessors
     $string    = $obj->seq();
     $substring = $obj->subseq(12,50);
-    $display   = $obj->display_id(); # for human display
-    $id        = $obj->primary_id(); # unique id for this object,
-                                     # implementation defined
-    $unique_key= $obj->accession_number();
-                       # unique biological id
+    $display   = $obj->display_id();       # for human display
+    $id        = $obj->primary_id();       # unique id for this object,
+                                           # implementation defined
+    $unique_key= $obj->accession_number(); # unique biological id
 
-    # object manipulation
 
+    # Object manipulation
     eval {
-	$rev    = $obj->revcom();
+	   $rev = $obj->revcom();
     };
     if( $@ ) {
-	$obj->throw("Could not reverse complement. ".
+	   $obj->throw("Could not reverse complement. ".
 		    "Probably not DNA. Actual exception\n$@\n");
     }
 
     $trunc = $obj->trunc(12,50);
-
     # $rev and $trunc are Bio::PrimarySeqI compliant objects
 
 
@@ -58,34 +52,31 @@ Bio::PrimarySeqI [Developers] - Interface definition for a Bio::PrimarySeq
 
 This object defines an abstract interface to basic sequence
 information - for most users of the package the documentation (and
-methods) in this class are not useful - this is a developers only
+methods) in this class are not useful - this is a developers-only
 class which defines what methods have to be implmented by other Perl
 objects to comply to the Bio::PrimarySeqI interface. Go "perldoc
 Bio::Seq" or "man Bio::Seq" for more information on the main class for
 sequences.
 
-
 PrimarySeq is an object just for the sequence and its name(s), nothing
 more. Seq is the larger object complete with features. There is a pure
-perl implementation of this in Bio::PrimarySeq. If you just want to
-use Bio::PrimarySeq objects, then please read that module first. This
+perl implementation of this in L<Bio::PrimarySeq>. If you just want to
+use L<Bio::PrimarySeq> objects, then please read that module first. This
 module defines the interface, and is of more interest to people who
 want to wrap their own Perl Objects/RDBs/FileSystems etc in way that
 they "are" bioperl sequence objects, even though it is not using Perl
 to store the sequence etc.
 
-
-This interface defines what bioperl consideres necessary to "be" a
-sequence, without providing an implementation of this. (An
-implementation is provided in Bio::PrimarySeq). If you want to provide
-a Bio::PrimarySeq 'compliant' object which in fact wraps another
+This interface defines what bioperl considers necessary to "be" a
+sequence, without providing an implementation of this, an
+implementation is provided in L<Bio::PrimarySeq>. If you want to provide
+a Bio::PrimarySeq-compliant object which in fact wraps another
 object/database/out-of-perl experience, then this is the correct thing
-to wrap, generally by providing a wrapper class which would inheriet
+to wrap, generally by providing a wrapper class which would inherit
 from your object and this Bio::PrimarySeqI interface. The wrapper class
 then would have methods lists in the "Implementation Specific
 Functions" which would provide these methods for your object.
 
-
 =head1 FEEDBACK
 
 =head2 Mailing Lists
@@ -94,21 +85,31 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                       - General discussion
-  http://bio.perl.org/MailList.html           - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney
 
-Email birney@sanger.ac.uk
+Email birney@ebi.ac.uk
 
 =head1 APPENDIX
 
@@ -122,12 +123,10 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::PrimarySeqI;
-use vars qw(@ISA );
 use strict;
-use Bio::Root::RootI;
 use Bio::Tools::CodonTable;
 
-@ISA = qw(Bio::Root::RootI);
+use base qw(Bio::Root::RootI);
 
 =head1 Implementation Specific Functions
 
@@ -137,13 +136,12 @@ define.
 =head2 seq
 
  Title   : seq
- Usage   : $string    = $obj->seq()
+ Usage   : $string = $obj->seq()
  Function: Returns the sequence as a string of letters. The
            case of the letters is left up to the implementer.
            Suggested cases are upper case for proteins and lower case for
-           DNA sequence (IUPAC standard),
-           but implementations are suggested to keep an open mind about
-           case (some users... want mixed case!)
+           DNA sequence (IUPAC standard), but implementations are suggested to
+           keep an open mind about case (some users... want mixed case!)
  Returns : A scalar
  Status  : Virtual
 
@@ -158,13 +156,13 @@ sub seq {
 
  Title   : subseq
  Usage   : $substring = $obj->subseq(10,40);
- Function: returns the subseq from start to end, where the first base
-           is 1 and the number is inclusive, ie 1-2 are the first two
-           bases of the sequence
+ Function: Returns the subseq from start to end, where the first base
+           is 1 and the number is inclusive, i.e. 1-2 are the first two
+           bases of the sequence.
 
-           Start cannot be larger than end but can be equal
+           Start cannot be larger than end but can be equal.
 
- Returns : a string
+ Returns : A string
  Args    :
  Status  : Virtual
 
@@ -179,7 +177,8 @@ sub subseq{
 
  Title   : display_id
  Usage   : $id_string = $obj->display_id();
- Function: returns the display id, aka the common name of the Sequence object.
+ Function: Returns the display id, also known as the common name of the Sequence
+           object.
 
            The semantics of this is that it is the most likely string
            to be used as an identifier of the sequence, and likely to
@@ -193,7 +192,7 @@ sub subseq{
            example, or extending the sequence object) to solve this.
 
            Notice that $seq->id() maps to this function, mainly for
-           legacy/convience issues
+           legacy/convenience reasons.
  Returns : A string
  Args    : None
  Status  : Virtual
@@ -246,8 +245,6 @@ sub accession_number {
            For sequences with no accession number, this method should
            return a stringified memory location.
 
-           [Note this method name is likely to change in 1.3]
-
  Returns : A string
  Args    : None
  Status  : Virtual
@@ -267,7 +264,7 @@ sub primary_id {
  Usage   : if( $obj->can_call_new ) {
              $newobj = $obj->new( %param );
 	 }
- Function: can_call_new returns 1 or 0 depending
+ Function: Can_call_new returns 1 or 0 depending
            on whether an implementation allows new
            constructor to be called. If a new constructor
            is allowed, then it should take the followed hashed
@@ -278,7 +275,6 @@ sub primary_id {
 			   -accession_number => $accession
 			   -alphabet => 'dna',
 			   );
- Example :
  Returns : 1 or 0
  Args    :
 
@@ -300,13 +296,13 @@ sub can_call_new{
  Function: Returns the type of sequence being one of
            'dna', 'rna' or 'protein'. This is case sensitive.
 
-           This is not called <type> because this would cause
+           This is not called "type" because this would cause
            upgrade problems from the 0.5 and earlier Seq objects.
 
- Returns : a string either 'dna','rna','protein'. NB - the object must
-           make a call of the type - if there is no type specified it
+ Returns : A string either 'dna','rna','protein'. NB - the object must
+           make a call of the alphabet, if there is no alphabet specified it
            has to guess.
- Args    : none
+ Args    : None
  Status  : Virtual
 
 
@@ -317,6 +313,13 @@ sub alphabet{
     $self->throw_not_implemented();
 }
 
+=head2 moltype
+
+ Title   : moltype
+ Usage   : Deprecated. Use alphabet() instead.
+
+=cut
+
 sub moltype{
    my ($self,@args) = @_;
 
@@ -332,7 +335,7 @@ implementing class does not need to provide these functions, as they
 will be provided by this class, but is free to override these
 functions.
 
-All of revcom(), trunc(), and translate() create new sequence
+The revcom(), trunc(), and translate() methods create new sequence
 objects. They will call new() on the class of the sequence object
 instance passed as argument, unless can_call_new() returns FALSE. In
 the latter case a Bio::PrimarySeq object will be created. Implementors
@@ -347,12 +350,12 @@ are encouraged to override these methods
  Function: Produces a new Bio::PrimarySeqI implementing object which
            is the reversed complement of the sequence. For protein
            sequences this throws an exception of "Sequence is a
-           protein. Cannot revcom"
+           protein. Cannot revcom".
 
            The id is the same id as the original sequence, and the
            accession number is also indentical. If someone wants to
            track that this sequence has be reversed, it needs to
-           define its own extensions
+           define its own extensionsj.
 
            To do an inplace edit of an object you can go:
 
@@ -363,7 +366,7 @@ are encouraged to override these methods
            efficient as an inplace edit.
 
  Returns : A new (fresh) Bio::PrimarySeqI object
- Args    : none
+ Args    : None
 
 
 =cut
@@ -416,6 +419,7 @@ sub revcom{
        $self->_attempt_to_load_Seq();
    }
    my $out = $seqclass->new( '-seq' => $revseq,
+			     '-is_circular'  => $self->is_circular,
 			     '-display_id'  => $self->display_id,
 			     '-accession_number' => $self->accession_number,
 			     '-alphabet' => $self->alphabet,
@@ -430,10 +434,8 @@ sub revcom{
 
  Title   : trunc
  Usage   : $subseq = $myseq->trunc(10,100);
- Function: Provides a truncation of a sequence,
-
- Example :
- Returns : a fresh Bio::PrimarySeqI implementing object
+ Function: Provides a truncation of a sequence.
+ Returns : A fresh Bio::PrimarySeqI implementing object.
  Args    : Two integers denoting first and last base of the sub-sequence.
 
 
@@ -480,126 +482,192 @@ sub trunc{
 
  Title   : translate
  Usage   : $protein_seq_obj = $dna_seq_obj->translate
-           #if full CDS expected:
-           $protein_seq_obj = $cds_seq_obj->translate(undef,undef,undef,undef,1);
- Function:
 
-           Provides the translation of the DNA sequence using full
+           Or if you expect a complete coding sequence (CDS) translation,
+           with inititator at the beginning and terminator at the end:
+
+           $protein_seq_obj = $cds_seq_obj->translate(-complete => 1);
+
+           Or if you want translate() to find the first initiation
+           codon and return the corresponding protein:
+
+           $protein_seq_obj = $cds_seq_obj->translate(-orf => 1);
+
+ Function: Provides the translation of the DNA sequence using full
            IUPAC ambiguities in DNA/RNA and amino acid codes.
 
-           The full CDS translation is identical to EMBL/TREMBL
+           The complete CDS translation is identical to EMBL/TREMBL
            database translation. Note that the trailing terminator
-           character is removed before returning the translation
+           character is removed before returning the translated protein
            object.
 
            Note: if you set $dna_seq_obj->verbose(1) you will get a
            warning if the first codon is not a valid initiator.
 
-           Added way of translating using a custom codon table.  This
-           has to be the final addition to this overloaded interface!
-
-
  Returns : A Bio::PrimarySeqI implementing object
- Args    : character for terminator (optional) defaults to '*'
-           character for unknown amino acid (optional) defaults to 'X'
-           frame (optional) valid values 0, 1, 2, defaults to 0
-           codon table id (optional) defaults to 1
-           complete coding sequence expected, defaults to 0 (false)
-           boolean, throw exception if not complete CDS (true) or defaults to warning (false)
-           codontable, a custom Bio::Tools::CodonTable object, optional
+ Args    : -terminator    - character for terminator        default is *
+           -unknown       - character for unknown           default is X
+           -frame         - frame                           default is 0
+           -codontable_id - codon table id                  default is 1
+           -complete      - complete CDS expected           default is 0
+           -throw         - throw exception if not complete default is 0
+           -orf           - find 1st ORF                    default is 0
+           -start         - alternative initiation codon
+           -codontable    - Bio::Tools::CodonTable object
+		   -offset        - offset for fuzzy locations      default is 0
+
+ Notes   : The -start argument only applies when -orf is set to 1. By default
+           all initiation codons found in the given codon table are used
+           but when "start" is set to some codon this codon will be used
+           exclusively as the initiation codon. Note that the default codon
+           table (NCBI "Standard") has 3 initiation codons!
+
+           By default translate() translates termination codons to
+           the some character (default is *), both internal and trailing
+           codons. Setting "-complete" to 1 tells translate() to remove
+           the trailing character.
+
+		   -offset is used for seqfeatures which contain the the \codon_start
+		   tag and can be set to 1, 2, or 3.  This is the offset by which the
+		   sequence translation starts relative to the first base of the
+		   feature
+
+For details on codon tables used by translate() see L<Bio::Tools::CodonTable>.
+
+           Deprecated argument set (v. 1.5.1 and prior versions)
+           where each argument is an element in an array:
+
+           1: character for terminator (optional), defaults to '*'.
+           2: character for unknown amino acid (optional), defaults to 'X'.
+           3: frame (optional), valid values are 0, 1, 2, defaults to 0.
+           4: codon table id (optional), defaults to 1.
+           5: complete coding sequence expected, defaults to 0 (false).
+           6: boolean, throw exception if not complete coding sequence
+              (true), defaults to warning (false)
+           7: codontable, a custom Bio::Tools::CodonTable object (optional).
 
 =cut
 
 sub translate {
-    my($self) = shift;
-    my($stop, $unknown, $frame, $tableid, $fullCDS, $throw, $codonTable) = @_;
-    my($i, $len, $output) = (0,0,'');
-    my($codon)   = "";
-    my $aa;
+	 my ($self,@args) = @_;
+	 my ($terminator, $unknown, $frame, $codonTableId, $complete, $throw,
+		  $codonTable, $orf, $start_codon, $offset);
 
-    ## User can pass in symbol for stop and unknown codons
-    unless(defined($stop) and $stop ne '')    { $stop = "*"; }
-    unless(defined($unknown) and $unknown ne '') { $unknown = "X"; }
-    unless(defined($frame) and $frame ne '') { $frame = 0; }
+	 ## new API with named parameters, post 1.5.1
+	 if ($args[0] && $args[0] =~ /^-[A-Z]+/i) {
+		 ($terminator, $unknown, $frame, $codonTableId, $complete, $throw,
+		  $codonTable, $orf, $start_codon, $offset) =
+			 $self->_rearrange([qw(TERMINATOR
+								UNKNOWN
+								FRAME
+								CODONTABLE_ID
+								COMPLETE
+								THROW
+								CODONTABLE
+								ORF
+								START
+								OFFSET)], @args);
+	 ## old API, 1.5.1 and preceding versions
+	 } else {
+		 ($terminator, $unknown, $frame, $codonTableId,
+		  $complete, $throw, $codonTable, $offset) = @args;
+	 }
 
-    ## the codon table ID
-    unless(defined($tableid) and $tableid ne '')    { $tableid = 1; }
+    ## Initialize termination codon, unknown codon, codon table id, frame
+    $terminator = '*'    unless (defined($terminator) and $terminator ne '');
+    $unknown = "X"       unless (defined($unknown) and $unknown ne '');
+    $frame = 0           unless (defined($frame) and $frame ne '');
+    $codonTableId = 1    unless (defined($codonTableId) and $codonTableId ne '');
 
-    ##Error if monomer is "Amino"
-    $self->throw("Can't translate an amino acid sequence.") if
-	($self->alphabet eq 'protein');
-
-    ##Error if frame is not 0, 1 or 2
-    $self->throw("Valid values for frame are 0, 1, 2, not [$frame].") unless
-	($frame == 0 or $frame == 1 or $frame == 2);
-
-    #warns if ID is invalid
+    ## Get a CodonTable, error if custom CodonTable is invalid
     if ($codonTable) {
-        $self->throw("Need a Bio::Tools::CodonTable object, not". $codonTable)
-            unless $codonTable->isa('Bio::Tools::CodonTable');
+		 $self->throw("Need a Bio::Tools::CodonTable object, not ". $codonTable)
+			unless $codonTable->isa('Bio::Tools::CodonTable');
     } else {
-        $codonTable = Bio::Tools::CodonTable->new( -id => $tableid);
+		 $codonTable = Bio::Tools::CodonTable->new( -id => $codonTableId);
+	 }
+
+    ## Error if alphabet is "protein"
+    $self->throw("Can't translate an amino acid sequence.") if
+		($self->alphabet =~ /protein/i);
+
+    ## Error if -start parameter isn't a valid codon
+	 if ($start_codon) {
+		 $self->throw("Invalid start codon: $start_codon.") if
+			( $start_codon !~ /^[A-Z]{3}$/i );
+	 }
+	 
+	 my $seq;
+	 
+	 if ($offset) {
+		$self->throw("Offset must be 1, 2, or 3.") if
+		    ( $offset !~ /^[123]$/ );
+		my ($start, $end) = ($offset, $self->length);
+		($seq) = $self->subseq($start, $end);
+	 } else {
+		($seq) = $self->seq();
+	 }
+
+    ## ignore frame if an ORF is supposed to be found
+	 if ($orf) {
+		 $seq = $self->_find_orf($seq,$codonTable,$start_codon);
+	 } else {
+	 ## use frame, error if frame is not 0, 1 or 2
+		 $self->throw("Valid values for frame are 0, 1, or 2, not $frame.")
+			unless ($frame == 0 or $frame == 1 or $frame == 2);
+		 $seq = substr($seq,$frame);
     }
 
-    my ($seq) = $self->seq();
-
-    # deal with frame offset.
-    if( $frame ) {
-	$seq = substr ($seq,$frame);
-    }
-
-    # Translate it
-    $output = $codonTable->translate($seq);
-    # Use user-input stop/unknown
-    $output =~ s/\*/$stop/g;
+    ## Translate it
+    my $output = $codonTable->translate($seq);
+    # Use user-input terminator/unknown
+    $output =~ s/\*/$terminator/g;
     $output =~ s/X/$unknown/g;
-	
-    # only if we are expecting to translate a complete coding region
-    if ($fullCDS) {
-	my $id = $self->display_id;
-	#remove the stop character
-	if( substr($output,-1,1) eq $stop ) {
-	    chop $output;
-	} else {
-	    $throw && $self->throw("Seq [$id]: Not using a valid terminator codon!");
-	    $self->warn("Seq [$id]: Not using a valid terminator codon!");
-	}
-	# test if there are terminator characters inside the protein sequence!
-	if ($output =~ /\*/) {
-	    $throw && $self->throw("Seq [$id]: Terminator codon inside CDS!");
-	    $self->warn("Seq [$id]: Terminator codon inside CDS!");
-	}
-	# if the initiator codon is not ATG, the amino acid needs to changed into M
-	if ( substr($output,0,1) ne 'M' ) {
-	    if ($codonTable->is_start_codon(substr($seq, 0, 3)) ) {
-		$output = 'M'. substr($output,1);
-	    }
-	    elsif ($throw) {
-		$self->throw("Seq [$id]: Not using a valid initiator codon!");
-	    } else {
-		$self->warn("Seq [$id]: Not using a valid initiator codon!");
-	    }
-	}
+
+    ## Only if we are expecting to translate a complete coding region
+    if ($complete) {
+		 my $id = $self->display_id;
+		 # remove the terminator character
+		 if( substr($output,-1,1) eq $terminator ) {
+			 chop $output;
+		 } else {
+			 $throw && $self->throw("Seq [$id]: Not using a valid terminator codon!");
+			 $self->warn("Seq [$id]: Not using a valid terminator codon!");
+		 }
+		 # test if there are terminator characters inside the protein sequence!
+		 if ($output =~ /\*/) {
+			 $throw && $self->throw("Seq [$id]: Terminator codon inside CDS!");
+			 $self->warn("Seq [$id]: Terminator codon inside CDS!");
+		 }
+		 # if the initiator codon is not ATG, the amino acid needs to be changed to M
+		 if ( substr($output,0,1) ne 'M' ) {
+			 if ($codonTable->is_start_codon(substr($seq, 0, 3)) ) {
+				 $output = 'M'. substr($output,1);
+			 }	elsif ($throw) {
+				 $self->throw("Seq [$id]: Not using a valid initiator codon!");
+			 } else {
+				 $self->warn("Seq [$id]: Not using a valid initiator codon!");
+			 }
+		 }
     }
 
     my $seqclass;
-    if($self->can_call_new()) {
-	$seqclass = ref($self);
+    if ($self->can_call_new()) {
+		 $seqclass = ref($self);
     } else {
-	$seqclass = 'Bio::PrimarySeq';
-	$self->_attempt_to_load_Seq();
+		 $seqclass = 'Bio::PrimarySeq';
+		 $self->_attempt_to_load_Seq();
     }
     my $out = $seqclass->new( '-seq' => $output,
-			      '-display_id'  => $self->display_id,
-			      '-accession_number' => $self->accession_number,
-			      # is there anything wrong with retaining the
-			      # description?
-			      '-desc' => $self->desc(),
-			      '-alphabet' => 'protein',
+										'-display_id'  => $self->display_id,
+										'-accession_number' => $self->accession_number,
+										# is there anything wrong with retaining the
+										# description?
+										'-desc' => $self->desc(),
+										'-alphabet' => 'protein',
                               '-verbose' => $self->verbose
 			      );
     return $out;
-
 }
 
 =head2 id
@@ -608,11 +676,9 @@ sub translate {
  Usage   : $id = $seq->id()
  Function: ID of the sequence. This should normally be (and actually is in
            the implementation provided here) just a synonym for display_id().
- Example :
  Returns : A string.
  Args    :
 
-
 =cut
 
 sub  id {
@@ -627,11 +693,9 @@ sub  id {
  Title   : length
  Usage   : $len = $seq->length()
  Function:
- Example :
- Returns : integer representing the length of the sequence.
+ Returns : Integer representing the length of the sequence.
  Args    :
 
-
 =cut
 
 sub  length {
@@ -645,11 +709,9 @@ sub  length {
  Usage   : $seq->desc($newval);
            $description = $seq->desc();
  Function: Get/set description text for a seq object
- Example :
- Returns : value of desc
+ Returns : Value of desc
  Args    : newvalue (optional)
 
-
 =cut
 
 sub desc {
@@ -676,6 +738,49 @@ sub is_circular{
 These are some private functions for the PrimarySeqI interface. You do not
 need to implement these functions
 
+=head2 _find_orf
+
+ Title   : _find_orf
+ Usage   :
+ Function: Finds ORF starting at 1st initiation codon in nucleotide sequence.
+           The ORF is not required to have a termination codon.
+ Example :
+ Returns : A nucleotide sequence or nothing, if no initiation codon is found.
+ Args    : Nucleotide sequence, CodonTable object, alternative initiation
+           codon (optional).
+
+=cut
+
+sub _find_orf {
+	my ($self,$sequence,$codonTable,$start_codon) = @_;
+
+	# find initiation codon and remove leading sequence
+	while ($sequence) {
+		my $codon = substr($sequence,0,3);
+		if ($start_codon) {
+			last if ( $codon =~ /$start_codon/i );
+		} else {
+			last if ($codonTable->is_start_codon($codon));
+		}
+		$sequence = substr($sequence,1);
+	}
+	return unless $sequence;
+
+	# find termination codon and remove trailing sequence
+	my $len = CORE::length($sequence);
+	my $offset = 3;
+	while ($offset < $len) {
+		my $codon = substr($sequence,$offset,3);
+		if ( $codonTable->is_ter_codon($codon) ){
+			$sequence = substr($sequence, 0, $offset + 3);
+			return $sequence;
+		}
+		$offset += 3;
+	}
+	$self->warn("No termination codon found, will translate - sequence:\n$sequence");
+	$sequence;
+}
+
 =head2 _attempt_to_load_Seq
 
  Title   : _attempt_to_load_Seq
