@@ -1,6 +1,9 @@
-# $Id: base.pm,v 1.5 2003/12/15 02:57:22 redwards Exp $
+# $Id: base.pm 16123 2009-09-17 12:57:27Z cjfields $
+#
 # BioPerl module for Bio::Restriction::IO::base
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Rob Edwards <redwards@utmem.edu>
 #
 # Copyright Rob Edwards
@@ -32,17 +35,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the
 Bioperl mailing lists Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                    - General discussion
-  http://bio.perl.org/MailList.html        - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR
 
@@ -50,7 +63,8 @@ Rob Edwards, redwards@utmem.edu
 
 =head1 CONTRIBUTORS
 
-Heikki Lehvaslaiho, heikki@ebi.ac.uk
+Heikki Lehvaslaiho, heikki-at-bioperl-dot-org
+Mark A. Jensen, maj-at-fortinbras-dot-us
 
 =head1 APPENDIX
 
@@ -63,30 +77,61 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Restriction::IO::base;
 
-use vars qw(@ISA);
 use strict;
 
-use Bio::Restriction::IO;
 use Bio::Restriction::Enzyme;
 use Bio::Restriction::EnzymeCollection;
-use Data::Dumper;
+use Bio::Restriction::Enzyme::MultiCut;
+use Bio::Restriction::Enzyme::MultiSite;
 
-@ISA = qw(Bio::Restriction::IO);
+use base qw(Bio::Restriction::IO);
 
 my $offset; # class variable
 
 sub new {
     my($class, @args) = @_;
+    $class = ref $class ? ref $class : $class;
     my $self = bless {}, $class;
     $self->_initialize(@args);
     return $self;
 }
 
+
+{
+    
+my %FILE_FORMAT = (
+            #'itype2'    => 'itype2', # itype2 format doesn't work with 'current'
+            #'8'         => 'itype2',
+            'withrefm'  => 'withrefm',
+            '31'        => 'withrefm',
+            #'bairoch'   => 'bairoch', # bairoch format doesn't work with 'current'
+            #'19'        => 'bairoch',
+            #'macvector' => 'bairoch',
+            #'vectorNTI' => 'bairoch',
+            'neo'       => 'neos',
+            'prototype' => 'proto'
+);
+
 sub _initialize {
     my($self,@args) = @_;
+    my ($current, $url, $file, $fh, $format, $verbose) =
+        $self->_rearrange([qw(CURRENT URL FILE FH FORMAT VERBOSE)],@args);
+    $verbose || 0;
+    $self->verbose($verbose);
+    if ($current && $format) {
+        $self->throw("Can't use -current with file, fh, or url set")  if ($url || $file || $fh);
+        $self->throw("Format $format not retrievable using 'current'") if (!exists $FILE_FORMAT{$format});
+        my $io = $self->new(-url => 'ftp://ftp.neb.com/pub/rebase/VERSION');
+        chomp (my $version = $io->_readline);
+        push @args, (-url => "ftp://ftp.neb.com/pub/rebase/$FILE_FORMAT{$format}.$version");
+    }
+
+    $self->_companies;
     return unless $self->SUPER::_initialize(@args);
 }
 
+}
+
 =head2 read
 
  Title   : read
@@ -97,20 +142,16 @@ sub _initialize {
 
 =cut
 
-
-
 sub read {
     my $self = shift;
 
-    my $renzs = new Bio::Restriction::EnzymeCollection(-empty => 1);
+    my $renzs = Bio::Restriction::EnzymeCollection->new(-empty => 1);
     seek DATA,($offset||=tell DATA), 0;
     while (<DATA>) {
         chomp;
         next if /^\s*$/;
         my ($name, $site, $cut) = split /\s+/;
-        #foreach my $key (keys %{$res}) {
-        #my ($site, $cut) = split /\s+/, $res->{$key};
-        my $re = new Bio::Restriction::Enzyme(-name => $name,
+        my $re = Bio::Restriction::Enzyme->new(-name => $name,
                                               -site => $site,
                                               -cut => $cut);
         $renzs->enzymes($re);
@@ -118,6 +159,21 @@ sub read {
     return $renzs;
 }
 
+=head2 _xln_sub
+
+ Title   : _xln_sub
+ Function: Translates withrefm coords to Bio::Restriction coords
+ Args    : Bio::Restriction::Enzyme object, scalar integer (cut posn)
+ Note    : Used internally; pass as a coderef to the B:R::Enzyme 
+           constructor
+ Note    : It is convenient for each format module to have its own 
+           version of this; not currently demanded by the interface.
+=cut
+
+sub _xln_sub { # for base.pm, a no-op
+    my ($z,$c) = @_; 
+    return $c;
+}
 
 
 =head2 write
@@ -142,6 +198,43 @@ sub write {
     }
 }
 
+=head2 verify_prototype
+
+ Title     : verify_prototype
+ Purpose   : checks enzyme against current prototype list (retrieved remotely)
+ Returns   : returns TRUE if enzyme is prototype
+ Argument  : Bio::Restriction::EnzymeI
+ Comments  : This is an auxiliary method to retrieve and check an enzyme
+             as a prototype.  It retrieves the current list, stores it
+             as a singleton instance, then uses it to check the prototype
+             and modify is_prototype() to true or false.  Use as follows:
+
+             my $col = $io->read;
+             for my $enz ($col->each_enzyme) {
+                 print $enz->name.":".$enz->site."\n";
+                 print "\t".$io->verify_prototype($enz)."\n";
+             }
+
+=cut
+
+my $protodb;
+
+sub verify_prototype {
+    my ($self, $enz) = @_;
+    $self->throw("Must pass a Bio::Restriction::EnzymeI") unless
+        $enz && ref $enz && $enz->isa("Bio::Restriction::EnzymeI");
+    if (!defined $protodb) {
+        my $io = Bio::Restriction::IO->new(-format => 'prototype',
+                      -current => 1);
+        $protodb = $io->read;
+    }
+    if ($protodb->get_enzyme($enz->name)) {
+        $enz->is_prototype(1);
+    } else {
+        $enz->is_prototype(0);
+    }
+    $enz->is_prototype;
+}
 
 =head2 Common REBASE parsing methods
 
@@ -159,6 +252,7 @@ class. (They are 'protected' in the sense the word is used in Java.)
            Does nothing to site if it does not have the cut string
  Returns : array of site_string, forward_cut_position, reverse_cut_position
  Args    : recognition site string
+ Note    : Not used in withrefm refactor/maj
 
 =cut
 
@@ -205,6 +299,7 @@ sub _meth {
  Returns : Cut position in correct coordinates
  Args    : 1. Original cut position
            2. Length of the recognition site
+ Note    : Not used in withrefm.pm refactor/maj
 
 =cut
 
@@ -217,46 +312,62 @@ sub _coordinate_shift_to_cut {
 =head2 _make_multisites
 
  Title   : _make_multisites
- Usage   : $self->_make_multisites($collection, $first_enzyme, \@sites, \@mets)
- Function: 
-
-           Bless a Bio::Restriction::Enzyme (which is already part of
-           the collection object) into
+ Usage   : $self->_make_multisites($first_enzyme, \@sites, \@mets)
+ Function: Bless a Bio::Restriction::Enzyme  into
            Bio::Restriction::Enzyme::MultiSite and clone it as many
-           times as there are alternative sites. The new objects are
-           added into the collection and into others list of sister
-           objects.
-
+           times as there are alternative sites.
  Returns : nothing, does in place editing
- Args    : 1. a Bio::Restriction::EnzymeCollection
-           2. a Bio::Restriction::Enzyme
-           3. reference to an array of recognition site strings
-           4. reference to an array of methylation code strings, optional
+ Args    : 1. a Bio::Restriction::Enzyme
+           2. reference to an array of recognition site strings
+           3. reference to an array of methylation code strings, optional
 
 =cut
 
+# removed the enzyme collection from arg list /maj
+
 sub _make_multisites {
-    my ($self, $renzs, $re, $sites, $meths) = @_;
+    my ($self, $re, $sites, $meths, $xln_sub) = @_;
 
     bless $re, 'Bio::Restriction::Enzyme::MultiSite';
-    #print Dumper $re, $sites, $meths;
 
     my $count = 0;
     while ($count < scalar @{$sites}) {
-        #print ">>>>>>>>>>>", scalar @{$sites}, ">>>>>>>>>>>>>>>>>>>>>> $count\n";
-        my $re2 = $re->clone;
-
+	# this should probably be refactored to use the constructor
+	# too, rather than the clone/accessor method /maj
+#        my $re2 = $re->clone;
+#	my $re2;
 
         my $site = @{$sites}[$count];
-        my ($cut, $comp_cut);
-        ($site, $cut, $comp_cut) = $self->_cuts_from_site($site);
-        $re2->site($site);
-
-        if ($cut) {
-            $re->cut($self->_coordinate_shift_to_cut(length($site), $cut));
-            $re->complementary_cut($self->_coordinate_shift_to_cut(length($site), $comp_cut));
-        }
+	my ($precut, $recog, $postcut) = ( $site =~ m/^(?:\((\w+\/\w+)\))?([\w^]+)(?:\((\w+\/\w+)\))?/ );
+	
+	# set the site attribute
+#	$re2->site($recog);
 
+	# set the recog attribute (which will make the regexp transformation
+	# if necessary:
+#	$re2->recog($recog);
+#	$recog = $re2->string;
+	
+# 	no warnings; # avoid 'uninitialized value' warning against $postcut
+#         my ($cut, $comp_cut) = ( $postcut =~  /(-?\d+)\/(-?\d+)/ );
+# 	use warnings;
+	
+	# note the following hard codes the coordinate transformation
+	# used for rebase/itype2 : this method will break on the 
+	# base.pm format. 
+#         if ($cut) {
+#             $re2->cut($cut + length $recog);
+#             $re2->complementary_cut($comp_cut + length $recog);
+# 	}
+	
+	my $re2 = Bio::Restriction::Enzyme::MultiSite->new(
+	    -name     => $re->name,
+	    -site     => $recog,
+	    -recog    => $recog,
+	    -precut   => $precut,
+	    -postcut  => $postcut,
+	    -xln_sub  => $xln_sub
+	    );
 
         if ($meths and @$meths) {
             $re2->purge_methylation_sites;
@@ -267,35 +378,27 @@ sub _make_multisites {
         $count++;
     }
 
-
     foreach my $enz ($re->others) {
         $enz->others($re, grep {$_ ne $enz} $re->others);
     }
 
-    #print Dumper $re;
-
     1;
 }
 
-
-
 =head2 _make_multicuts
 
  Title   : _make_multicuts
- Usage   : $self->_make_multicuts($collection, $first_enzyme, $precuts)
+ Usage   : $self->_make_multicuts($first_enzyme, $precuts)
  Function: 
 
-           Bless a Bio::Restriction::Enzyme (which is already part of
-           the collection object) into
+           Bless a Bio::Restriction::Enzyme into
            Bio::Restriction::Enzyme::MultiCut and clone it. The precut
            string is processed to replase the cut sites in the cloned
-           object which is added into the collection. Both object
-           refere to each other through others() method.
+           object. Both objects refer to each other through others() method.
 
  Returns : nothing, does in place editing
- Args    : 1. a Bio::Restriction::EnzymeCollection
-           2. a Bio::Restriction::Enzyme
-           3. precut string, e.g. '12/7'
+ Args    : 1. a Bio::Restriction::Enzyme
+           2. precut string, e.g. '12/7'
 
 
 The examples we have of multiply cutting enzymes cut only four
@@ -305,18 +408,15 @@ BEFORE the start of the recognition site, i.e. negative positions.
 
 =cut
 
+# removed the enzyme collection from arg list /maj
+
 sub _make_multicuts {
-    my ($self, $renzs, $re, $precut) = @_;
+    my ($self, $re, $precut) = @_;
 
     bless $re, 'Bio::Restriction::Enzyme::MultiCut';
-
+ 
     my ($cut, $comp_cut) = $precut =~ /(-?\d+)\/(-?\d+)/;
     
-    # Pads the front to prevent detection of sites when the 1st
-    # cut is off the end of the sequence.
-    my $site = $re->site;
-    $re->site(('N' x abs($cut)) . $site);
-
     my $re2 = $re->clone;
 
     $re2->cut("-$cut");
@@ -327,7 +427,6 @@ sub _make_multicuts {
     1;
 }
 
-
 =head2 _companies
 
  Title     : _companies
@@ -340,7 +439,6 @@ sub _make_multicuts {
 
 =cut
 
-
 sub _companies {
     # this is just so it is easy to set up the codes that REBASE uses
     my $self=shift;
@@ -367,7 +465,6 @@ sub _companies {
     $self->{company}=\%companies;
 }
 
-
 1;
 
 __DATA__
