@@ -1,7 +1,9 @@
-# $Id: IO.pm,v 1.56 2004/10/28 20:49:04 jason Exp $
+# $Id: IO.pm 16147 2009-09-22 01:26:32Z cjfields $
 #
 # BioPerl module for Bio::Root::IO
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Hilmar Lapp <hlapp@gmx.net>
 #
 # Copyright Hilmar Lapp
@@ -70,23 +72,35 @@ and other Bioperl modules. Send your comments and suggestions preferably
  to one of the Bioperl mailing lists.
 Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                 - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.
- Bug reports can be submitted via email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Hilmar Lapp
 
 Email hlapp@gmx.net
 
-Describe contact details here
+=head1 CONTRIBUTORS
+
+Mark A. Jensen ( maj -at- fortinbras -dot- us )
 
 =head1 APPENDIX
 
@@ -99,7 +113,7 @@ The rest of the documentation details each of the object methods. Internal metho
 
 
 package Bio::Root::IO;
-use vars qw(@ISA $FILESPECLOADED $FILETEMPLOADED $FILEPATHLOADED
+use vars qw($FILESPECLOADED $FILETEMPLOADED $FILEPATHLOADED
 	    $TEMPDIR $PATHSEP $ROOTDIR $OPENFLAGS $VERBOSE $ONMAC
             $HAS_LWP
            );
@@ -108,10 +122,9 @@ use strict;
 use Symbol;
 use POSIX qw(dup);
 use IO::Handle;
-use Bio::Root::Root;
 use Bio::Root::HTTPget;
 
-@ISA = qw(Bio::Root::Root);
+use base qw(Bio::Root::Root);
 
 my $TEMPCOUNTER;
 my $HAS_WIN32 = 0;
@@ -135,10 +148,10 @@ BEGIN {
     }
 
     eval {
-        require LWP::Simple;
+	require LWP::UserAgent;
     };
     if( $@ ) {
-	print STDERR "Cannot load LWP::Simple: $@" if( $VERBOSE > 0 );
+	print STDERR "Cannot load LWP::UserAgent: $@" if( $VERBOSE > 0 );
         $HAS_LWP = 0;
     } else {
         $HAS_LWP = 1;
@@ -242,6 +255,13 @@ sub new {
               -flush    boolean flag to autoflush after each write
               -noclose  boolean flag, when set to true will not close a
                         filehandle (must explictly call close($io->_fh)
+              -retries  number of times to try a web fetch before failure
+                        
+              -ua_parms hashref of key => value parameters to pass 
+                        to LWP::UserAgent->new()
+                        (only meaningful with -url is set)
+                        A useful value might be, for example,
+                        { timeout => 60 } (ua default is 180 sec)
  Returns : TRUE
  Args    : named parameters
 
@@ -253,27 +273,34 @@ sub _initialize_io {
 
     $self->_register_for_cleanup(\&_io_cleanup);
 
-    my ($input, $noclose, $file, $fh, $flush, $url) = $self->_rearrange([qw(INPUT 
-							    NOCLOSE
-							    FILE FH 
-							    FLUSH URL)], @args);
+    my ($input, $noclose, $file, $fh, $flush, $url,
+	$retries, $ua_parms) = 
+	$self->_rearrange([qw(INPUT 
+                              NOCLOSE
+                              FILE 
+                              FH 
+                              FLUSH
+                              URL
+                              RETRIES
+                              UA_PARMS)], @args);
 
     if($url){
-      my $trymax = 5;
+	$retries ||= 5;
 
-      if($HAS_LWP){ #use LWP::Simple::getstore()
-	require LWP::Simple;
-        #$self->warn("has lwp");
+      if($HAS_LWP){ #use LWP::UserAgent
+	  require LWP::UserAgent;
+	  my $ua = LWP::UserAgent->new(%$ua_parms);
         my $http_result;
         my($handle,$tempfile) = $self->tempfile();
-        close($handle);
+        CORE::close($handle);
+	  
 
-        for(my $try = 1 ; $try <= $trymax ; $try++){
-          $http_result = LWP::Simple::getstore($url, $tempfile);
-          $self->warn("[$try/$trymax] tried to fetch $url, but server threw $http_result.  retrying...") if $http_result != 200;
-          last if $http_result == 200;
+        for(my $try = 1 ; $try <= $retries ; $try++){
+	    $http_result = $ua->get($url, ':content_file' => $tempfile);
+	    $self->warn("[$try/$retries] tried to fetch $url, but server threw " . $http_result->code . ".  retrying...") if !$http_result->is_success;
+	    last if $http_result->is_success;
         }
-        $self->throw("failed to fetch $url, server threw $http_result") if $http_result != 200;
+	  $self->throw("failed to fetch $url, server threw " . $http_result->code) if !$http_result->is_success;
 
         $input = $tempfile;
         $file  = $tempfile;
@@ -315,6 +342,9 @@ sub _initialize_io {
 	    $self->throw("Could not open $file: $!");
 	$self->file($file);
     }
+    if ( defined($fh) && !(ref($fh) && ((ref($fh) eq "GLOB") || $fh->isa('IO::Handle') || $fh->isa('IO::String')))  ) {
+        $self->throw("file handle $fh doesn't appear to be a handle");
+    }
     $self->_fh($fh) if $fh; # if not provided, defaults to STDIN and STDOUT
 
     $self->_flush_on_write(defined $flush ? $flush : 1);
@@ -363,52 +393,29 @@ sub mode {
     my ($obj, @arg) = @_;
 	my %param = @arg;
     return $obj->{'_mode'} if defined $obj->{'_mode'} and !$param{-force};
-
-    print STDERR "testing mode... " if $obj->verbose;
-
-    # we need to dup() the original filehandle because
-    # doing fdopen() calls on an already open handle causes
-    # the handle to go stale. is this going to work for non-unix
-    # filehandles? -allen
-
-    my $fh = Symbol::gensym();
-
-    my $iotest = new IO::Handle;
-
-    #test for a readable filehandle;
-    $iotest->fdopen( dup(fileno($obj->_fh)) , 'r' );
-    if($iotest->error == 0){
-
-      # note the hack here, we actually have to try to read the line
-      # and if we get something, pushback() it into the readbuffer.
-      # this is because solaris and windows xp (others?) don't set
-      # IO::Handle::error.  for non-linux the r/w testing is done
-      # inside this read-test, instead of the write test below.  ugh.
-
-      if($^O eq 'linux'){
+    
+    # Previous system of:
+    #  my $iotest = new IO::Handle;
+    #  $iotest->fdopen( dup(fileno($fh)) , 'r' );
+    #  if ($iotest->error == 0) { ... }
+    # didn't actually seem to work under any platform, since there would no
+    # no error if the filehandle had been opened writable only. Couldn't be
+    # hacked around when dealing with unseekable (piped) filehandles.
+    #
+    # Just try and do a simple readline, turning io warnings off, instead:
+    
+    my $fh = $obj->_fh || return '?';
+    
+    no warnings "io"; # we expect a warning if this is writable only
+    my $line = <$fh>;
+    if (defined $line) {
+        $obj->_pushback($line);
         $obj->{'_mode'} = 'r';
-        my $line = $iotest->getline;
-        $obj->_pushback($line) if defined $line;
-        $obj->{'_mode'} = defined $line ? 'r' : 'w';
-        return $obj->{'_mode'};
-      } else {
-        my $line = $iotest->getline;
-        $obj->_pushback($line) if defined $line;
-        $obj->{'_mode'} = defined $line ? 'r' : 'w';
-	return $obj->{'_mode'};
-      }
     }
-    $iotest->clearerr;
-
-    #test for a writeable filehandle;
-    $iotest->fdopen( dup(fileno($obj->_fh)) , 'w' );
-    if($iotest->error == 0){
-      $obj->{'_mode'} = 'w';
-#      return $obj->{'_mode'};
+    else {
+        $obj->{'_mode'} = 'w';
     }
-
-    #wtf type of filehandle is this?
-#    $obj->{'_mode'} = '?';
+    
     return $obj->{'_mode'};
 }
 
@@ -489,9 +496,9 @@ sub _readline {
     # $line =~ s/\r\n/\n/g if( (!$param{-raw}) && (defined $line) );
     # Dave Howorth's fix
     if( (!$param{-raw}) && (defined $line) ) {
-       $line =~ s/\015?\012/\n/g;
-       $line =~ s/\015/\n/g unless $ONMAC;
-   }
+        $line =~ s/\015\012/\012/g; # Change all CR/LF pairs to LF
+        $line =~ tr/\015/\n/ unless $ONMAC; # Change all single CRs to NEWLINE
+    }
     return $line;
 }
 
@@ -501,15 +508,17 @@ sub _readline {
  Usage   : $obj->_pushback($newvalue)
  Function: puts a line previously read with _readline back into a buffer.
            buffer can hold as many lines as system memory permits.
- Example :
- Returns :
+ Example : $obj->_pushback($newvalue)
+ Returns : none
  Args    : newvalue
+ Note    : This is only supported for pushing back data ending with the
+		   current, localized value of $/. Using this method to push modified
+		   data back onto the buffer stack is not supported; see bug 843.
 
 =cut
 
 sub _pushback {
     my ($obj, $value) = @_;
-    $obj->{'_readbuffer'} ||= [];
     return unless $value;
     push @{$obj->{'_readbuffer'}}, $value;
 }
@@ -593,13 +602,13 @@ sub noclose{
 
 sub _io_cleanup {
     my ($self) = @_;
-
     $self->close();
     my $v = $self->verbose;
 
     # we are planning to cleanup temp files no matter what    
     if( exists($self->{'_rootio_tempfiles'}) &&
-	ref($self->{'_rootio_tempfiles'}) =~ /array/i) { 
+	ref($self->{'_rootio_tempfiles'}) =~ /array/i &&
+    !$self->save_tempfiles) { 
 	if( $v > 0 ) {
 	    warn( "going to remove files ", 
 		  join(",",  @{$self->{'_rootio_tempfiles'}}), "\n");
@@ -609,8 +618,8 @@ sub _io_cleanup {
     # cleanup if we are not using File::Temp
     if( $self->{'_cleanuptempdir'} &&
 	exists($self->{'_rootio_tempdirs'}) &&
-	ref($self->{'_rootio_tempdirs'}) =~ /array/i) {	
-
+	ref($self->{'_rootio_tempdirs'}) =~ /array/i &&
+    !$self->save_tempfiles) {	
 	if( $v > 0 ) {
 	    warn( "going to remove dirs ", 
 		  join(",",  @{$self->{'_rootio_tempdirs'}}), "\n");
@@ -638,9 +647,10 @@ sub _io_cleanup {
 
 sub exists_exe {
     my ($self, $exe) = @_;
-    $exe = $self if(!(ref($self) || $exe));
+	$self->throw("Must pass a defined value to exists_exe") unless defined $exe;
+    $exe = $self if (!(ref($self) || $exe));
     $exe .= '.exe' if(($^O =~ /mswin/i) && ($exe !~ /\.(exe|com|bat|cmd)$/i));
-    return $exe if(-e $exe); # full path and exists
+    return $exe if (-e $exe); # full path and exists
 
     # Ewan's comment. I don't think we need this. People should not be
     # asking for a program with a pathseparator starting it
@@ -722,10 +732,6 @@ sub tempfile {
 	    $file = Win32::GetShortPathName($file);
 	}
 
-	# taken from File::Temp
-	if ($] < 5.006) {
-	    $tfh = &Symbol::gensym;
-	}    
 	# Try to make sure this will be marked close-on-exec
 	# XXX: Win32 doesn't respect this, nor the proper fcntl,
 	#      but may have O_NOINHERIT. This may or may not be in Fcntl.
@@ -977,4 +983,23 @@ sub _flush_on_write {
     return $self->{'_flush_on_write'};
 }
 
+=head2 save_tempfiles
+
+ Title   : save_tempfiles
+ Usage   : $obj->save_tempfiles(1)
+ Function: Boolean flag to indicate whether to retain tempfiles/tempdir
+ Returns : Boolean value : 1 = save tempfiles/tempdirs, 0 = remove (default)
+ Args    : Value evaluating to TRUE or FALSE
+
+=cut
+
+sub save_tempfiles {
+    my $self = shift;
+    if (@_) {
+        my $value = shift;
+        $self->{save_tempfiles} = $value ? 1 : 0;
+    }
+    return $self->{save_tempfiles} || 0;
+}
+
 1;
