@@ -1,8 +1,10 @@
 #-----------------------------------------------------------------
-# $Id: HitI.pm,v 1.20 2003/08/09 21:48:11 jason Exp $
+# $Id: HitI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module Bio::Search::Hit::HitI
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Steve Chervitz <sac@bioperl.org>
 #
 # Originally created by Aaron Mackey <amackey@virginia.edu>
@@ -26,7 +28,7 @@ Bio::Search::Hit::HitI - Interface for a hit in a similarity search result
 
     # Get a HitI object from a SearchIO stream:
     use Bio::SeachIO;
-    my $searchio = new Bio::SearchIO(-format => 'blast', -file => 'result.bls');
+    my $searchio = Bio::SearchIO->new(-format => 'blast', -file => 'result.bls');
     my $result = $searchio->next_result;
     my $hit    = $result->next_hit;
 
@@ -62,17 +64,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Aaron Mackey, Steve Chervitz
 
@@ -98,12 +110,10 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Search::Hit::HitI;
 
-use Bio::Root::RootI;
 
-use vars qw(@ISA);
 use strict;
 
-@ISA = qw( Bio::Root::RootI );
+use base qw(Bio::Root::RootI);
 
 
 =head2 name
@@ -679,6 +689,45 @@ sub hit_description { shift->description(@_) }
 # aliasing for Steve's method names
 sub hit_length { shift->length(@_) }
 
+
+# sort method for HSPs
+
+=head2 sort_hits
+
+ Title		: sort_hsps
+ Usage		: $result->sort_hsps(\&sort_function)
+ Function	: Sorts the available HSP objects by a user-supplied function. Defaults to sort
+                  by descending score.
+ Returns	: n/a
+ Args		: A coderef for the sort function.  See the documentation on the Perl sort()
+                  function for guidelines on writing sort functions.  
+ Note		: To access the special variables $a and $b used by the Perl sort() function 
+                  the user function must access Bio::Search::Hit::HitI namespace. 
+                  For example, use :
+                  $hit->sort_hsps( sub{$Bio::Search::Result::HitI::a->length <=> 
+					  $Bio::Search::Result::HitI::b->length});
+                   NOT $hit->sort_hsps($a->length <=> $b->length);
+
+=cut
+
+sub sort_hsps {shift->throw_not_implemented }
+
+=head2 _default sort_hsps
+
+  Title	: _default_sort_hsps
+  Usage	: Do not call directly.
+  Function : Sort hsps in ascending order by evalue
+  Args	: None
+  Returns: 1 on success
+  Note	: Used by $hit->sort_hsps()
+
+=cut
+
+sub _default_sort_hsps {
+    $Bio::Search::Hit::HitI::a->evalue <=> 
+	    $Bio::Search::Hit::HitI::a->evalue;
+}
+
 1;
 
 
