@@ -17,28 +17,52 @@ a collection of methods, not an object.
 
 Steve Chervitz E<lt>sac@bioperl.orgE<gt>
 
-=cut
+=head1 CONTRIBUTORS
 
-#'
+Sendu Bala, bix@sendu.me.uk
+
+=cut
 
 package Bio::Search::SearchUtils;
 use Bio::Root::Version;
 
 use strict;
-use vars qw($DEBUG);
-$DEBUG = 1;
 
 =head2 tile_hsps
 
  Usage     : tile_hsps( $sbjct );
            : This is called automatically by methods in Bio::Search::Hit::GenericHit 
            : that rely on having tiled data.
+           :
+           : If you are interested in getting data about the constructed HSP contigs:
+           : my ($qcontigs, $scontigs) = Bio::Search::SearchUtils::tile_hsps($hit);
+           : if (ref $qcontigs) {
+           :    print STDERR "Query contigs:\n";
+           :    foreach (@{$qcontigs}) {
+           :         print "contig start is $_->{'start'}\n";
+           :         print "contig stop is $_->{'stop'}\n";
+           :    }
+           : }
+           : See below for more information about the contig data structure.
+           :
  Purpose   : Collect statistics about the aligned sequences in a set of HSPs.
            : Calculates the following data across all HSPs: 
            :    -- total alignment length 
            :    -- total identical residues 
            :    -- total conserved residues
- Returns   : true if tiling was possible, false if not
+ Returns   : If there was only a single HSP (so no tiling was necessary)
+               tile_hsps() returns a list of two non-zero integers.
+             If there were multiple HSP, 
+               tile_hsps() returns a list of two array references containin HSP contig data.
+             The first array ref contains a list of HSP contigs on the query sequence.
+             The second array ref contains a list of HSP contigs on the subject sequence.
+             Each contig is a hash reference with the following data fields:
+               'start' => start coordinate of the contig
+               'stop'  => start coordinate of the contig
+               'iden'  => number of identical residues in the contig
+               'cons'  => number of conserved residues in the contig
+               'strand'=> strand of the contig
+               'frame' => frame of the contig
  Argument  : A Bio::Search::Hit::HitI object 
  Throws    : n/a
  Comments  :
@@ -104,15 +128,15 @@ See Also   : L<_adjust_contigs>(), L<Bio::Search::Hit::GenericHit|Bio::Search::H
 sub tile_hsps {
 #--------------
     my $sbjct = shift;
-    
-    #print STDERR "tile_hsps(): $sbjct\n";
+
+    #print STDERR "Calling tile_hsps(): $sbjct\n";
     #$sbjct->verbose(1);  # to activate debugging
     $sbjct->tiled_hsps(1);
 
     if( $sbjct->num_hsps == 0 || $sbjct->n == 0 ) { 		
 	#print STDERR "_tile_hsps(): no hsps, nothing to tile! (", $sbjct->num_hsps, ")\n";
-        _warn_about_no_hsp($sbjct);
-        return undef;
+        _warn_about_no_hsps($sbjct);
+        return (undef, undef);
 
     } elsif( $sbjct->n == 1 or $sbjct->num_hsps == 1) {
         ## Simple summation scheme. Valid if there is only one HSP.
@@ -126,7 +150,7 @@ sub tile_hsps {
 	$sbjct->gaps('sbjct', $hsp->gaps('sbjct'));
 
         _adjust_length_aln($sbjct);
-	return 1;
+	return (1, 1);
     } else {
 	#print STDERR "Sbjct: _tile_hsps: summing multiple HSPs\n";
 	$sbjct->length_aln('query', 0);
@@ -159,7 +183,7 @@ sub tile_hsps {
 
 	($qstart, $qstop) = $hsp->range('query');
 	($sstart, $sstop) = $hsp->range('sbjct');
-	$frame = $hsp->frame;
+	$frame = $hsp->frame('hit');
 	$frame = -1 unless defined $frame;
 	
 	($qstrand, $sstrand) = ($hsp->query->strand,
@@ -173,13 +197,13 @@ sub tile_hsps {
 	$hit_len_aln += $hsp->length;
 
 	## Collect contigs in the query sequence.
- 	$qoverlap = &_adjust_contigs('query', $hsp, $qstart, $qstop, 
+ 	$qoverlap += &_adjust_contigs('query', $hsp, $qstart, $qstop, 
 				     \@qcontigs, $max_overlap, $frame, 
 				     $qstrand);
 
 	## Collect contigs in the sbjct sequence 
 	#  (needed for domain data and gapped Blast).
-	$soverlap = &_adjust_contigs('sbjct', $hsp, $sstart, $sstop, 
+	$soverlap += &_adjust_contigs('sbjct', $hsp, $sstart, $sstop, 
 				     \@scontigs, $max_overlap, $frame, 
 				     $sstrand);
 
@@ -220,16 +244,17 @@ sub tile_hsps {
     #           save the most significant one.
     my (%qctg_dat);
     foreach (@qcontigs) {
-	($frame, $strand) = ($_->{'frame'}, $_->{'strand'});
-	
-	if( $v > 0 ) {
-	    #$sbjct->debug(sprintf( "$frame/$strand len is getting %d for %d..%d\n", 
-	    #			   ($_->{'stop'} - $_->{'start'} + 1), $_->{'start'}, $_->{'stop'}));
-	}
-	$qctg_dat{ "$frame$strand" }->{'length_aln_query'} += $_->{'stop'} - $_->{'start'} + 1;
-	$qctg_dat{ "$frame$strand" }->{'totalIdentical'}   += $_->{'iden'};
-	$qctg_dat{ "$frame$strand" }->{'totalConserved'}   += $_->{'cons'};
-	$qctg_dat{ "$frame$strand" }->{'qstrand'}   = $strand;
+        ($frame, $strand) = ($_->{'frame'}, $_->{'strand'});
+        
+        if( $v > 0 ) {
+            #$sbjct->debug(sprintf( "$frame/$strand len is getting %d for %d..%d\n", 
+            #			   ($_->{'stop'} - $_->{'start'} + 1), $_->{'start'}, $_->{'stop'}));
+        }
+        
+        $qctg_dat{ "$frame$strand" }->{'length_aln_query'} += $_->{'stop'} - $_->{'start'} + 1;
+        $qctg_dat{ "$frame$strand" }->{'totalIdentical'}   += $_->{'iden'};
+        $qctg_dat{ "$frame$strand" }->{'totalConserved'}   += $_->{'cons'};
+        $qctg_dat{ "$frame$strand" }->{'qstrand'}   = $strand;
     }
 
     # Find longest contig.
@@ -284,7 +309,8 @@ sub tile_hsps {
     }
 
     _adjust_length_aln($sbjct);
-    return 1;
+
+    return ( [@qcontigs], [@scontigs] );
 }
 
 
@@ -346,8 +372,9 @@ sub logical_length {
 # Throws    : Exceptions propagated from Bio::Search::Hit::BlastHSP::matches()
 #           : for invalid sub-sequence ranges.
 # Status    : Experimental
-# Comments  : This method does not currently support gapped alignments.
-#           : Also, it does not keep track of the number of HSPs that
+# Comments  : This method supports gapped alignments through a patch by maj
+#           : to B:S:HSP:HSPI::matches().
+#           : It does not keep track of the number of HSPs that
 #           : overlap within the amount specified by overlap().
 #           : This will lead to significant tracking errors for large
 #           : overlap values.
@@ -356,81 +383,193 @@ sub logical_length {
 #
 #=cut
 
-#-------------------
 sub _adjust_contigs {
-#-------------------
     my ($seqType, $hsp, $start, $stop, $contigs_ref, 
 	$max_overlap, $frame, $strand) = @_;
-
     my $overlap = 0;
     my ($numID, $numCons);
-
     
-    foreach ( @$contigs_ref) {
-
-	# Don't merge things unless they have matching strand/frame.
-	next unless ($_->{'frame'} == $frame && 
-		     $_->{'strand'} == $strand);
-	
-	## Test special case of a nested HSP. Skip it.
-	if( $start >= $_->{'start'} && $stop <= $_->{'stop'}) { 
-	    # print SDTERR ( "----> Nested HSP. Skipping.\n");
-	    $overlap = 1; 
-	    next;
-	}
-
-	## Test for overlap at beginning of contig.
-	# to find left most
-	if($start < $_->{'start'} && 
-	   $stop > ($_->{'start'} + $max_overlap) ) { 
-	    # print STDERR "----> Overlaps beg: existing beg,end: $_->{'start'},$_->{'stop'}, new beg,end: $start,$stop\n";
-	    # Collect stats over the non-overlapping region.
-	    eval {
-		($numID, $numCons) = $hsp->matches(-SEQ   =>$seqType, 
-						   -START =>$start, 
-						   -STOP  =>$_->{'start'}-1); 
+    foreach (@$contigs_ref) {
+        # Don't merge things unless they have matching strand/frame.
+        next unless ($_->{'frame'} == $frame && $_->{'strand'} == $strand);
+        
+        # Test special case of a nested HSP. Skip it.
+        if ($start >= $_->{'start'} && $stop <= $_->{'stop'}) {
+            $overlap = 1; 
+            next;
+        }
+        
+        # Test for overlap at beginning of contig, or precedes consecutively
+        if ($start < $_->{'start'} && $stop >= ($_->{'start'} + $max_overlap - 1)) {
+            eval {
+                ($numID, $numCons) = $hsp->matches(-SEQ   =>$seqType, 
+                               -START => $start, 
+                               -STOP  => $_->{'start'} - 1); 
+		if ($numID eq '') {
+		    $hsp->warn("\$hsp->matches() returned '' for number identical; setting to 0");
+		    $numID = 0;
+		}
+		if ($numCons eq '') {
+		    $hsp->warn("\$hsp->matches() returned '' for number conserved; setting to 0");
+		    $numCons = 0;
+		}
+            };
+            if($@) { warn "\a\n$@\n"; }
+            else {
+                $_->{'start'} = $start;	# Assign a new start coordinate to the contig
+                $_->{'iden'} += $numID;	# and add new data to #identical, #conserved.
+                $_->{'cons'} += $numCons;
+                push(@{$_->{hsps}}, $hsp);
+                $overlap     = 1; 
+            }
+        }
+        
+        # Test for overlap at end of contig, or follows consecutively
+        if ($stop > $_->{'stop'} and $start <= ($_->{'stop'} - $max_overlap + 1)) {
+            eval {
+                ($numID,$numCons) = $hsp->matches(-SEQ   =>$seqType, 
+                              -START => $_->{'stop'} + 1, 
+                              -STOP  => $stop); 
+		if ($numID eq '') {
+		    $hsp->warn("\$hsp->matches() returned '' for number identical; setting to 0");
+		    $numID = 0;
+		}
+		if ($numCons eq '') {
+		    $hsp->warn("\$hsp->matches() returned '' for number conserved; setting to 0");
+		    $numCons = 0;
+		}
 	    };
-	    if($@) { warn "\a\n$@\n"; }
-	    else {
-		$_->{'start'} = $start;	# Assign a new start coordinate to the contig
-		$_->{'iden'} += $numID;	# and add new data to #identical, #conserved.
-		$_->{'cons'} += $numCons;
-		$overlap     = 1; 
-	    }
-	}
-
-	## Test for overlap at end of contig.
-	if($stop > $_->{'stop'} and 
-	   $start < ($_->{'stop'} - $max_overlap)) { 
-#	    print STDERR "----> Overlaps end: existing beg,end: $_->{'start'},$_->{'stop'}, new beg,end: $start,$stop\n";
-	    # Collect stats over the non-overlapping region.
-	    eval {
-		($numID,$numCons) = $hsp->matches(-SEQ   =>$seqType, 
-						  -START =>$_->{'stop'}, 
-						  -STOP  =>$stop); 
-	    };
-	    if($@) { warn "\a\n$@\n"; }
-	    else {
-		$_->{'stop'}  = $stop; # Assign a new stop coordinate to the contig
-		$_->{'iden'} += $numID;	# and add new data to #identical, #conserved.
-		$_->{'cons'} += $numCons;
-		$overlap    = 1; 
-	    }
-	}
-	$overlap && do {
-#		print STDERR " New Contig data:\n";
-#		print STDERR "  Contig: $_->{'start'} - $_->{'stop'}, iden= $_->{'iden'}, cons= $_->{'cons'}\n";
-	    last;
-	};
+            if($@) { warn "\a\n$@\n"; }
+            else {
+                $_->{'stop'}  = $stop; # Assign a new stop coordinate to the contig
+                $_->{'iden'} += $numID;	# and add new data to #identical, #conserved.
+                $_->{'cons'} += $numCons;
+                push(@{$_->{hsps}}, $hsp);
+                $overlap    = 1; 
+            }
+        }
+        
+        last if $overlap;
     }
-    ## If there is no overlap, add the complete HSP data.
-    ! $overlap && do {
-	($numID,$numCons) = $hsp->matches(-SEQ=>$seqType); 
-	push @$contigs_ref, {'start' =>$start, 'stop' =>$stop,
+    
+    if ($overlap && @$contigs_ref > 1) {
+        ## Merge any contigs that now overlap
+        my $max = $#{$contigs_ref};
+        for my $i (0..$max) {
+            ${$contigs_ref}[$i] || next;
+            my ($i_start, $i_stop) = (${$contigs_ref}[$i]->{start}, ${$contigs_ref}[$i]->{stop});
+            
+            for my $u ($i+1..$max) {
+                ${$contigs_ref}[$u] || next;
+                my ($u_start, $u_stop) = (${$contigs_ref}[$u]->{start}, ${$contigs_ref}[$u]->{stop});
+                
+                if ($u_start < $i_start && $u_stop >= ($i_start + $max_overlap - 1)) {
+                    # find the hsps within the contig that have sequence
+                    # extending before $i_start
+                    my ($ids, $cons) = (0, 0);
+                    my $use_start = $i_start;
+                    foreach my $hsp (sort { $b->end($seqType) <=> $a->end($seqType) } @{${$contigs_ref}[$u]->{hsps}}) {
+                        my $hsp_start = $hsp->start($seqType);
+                        $hsp_start < $use_start || next;
+                        
+                        my ($these_ids, $these_cons);
+                        eval {
+                            ($these_ids, $these_cons) = $hsp->matches(-SEQ => $seqType, -START => $hsp_start, -STOP => $use_start - 1);
+			    if ($these_ids eq '') {
+				$hsp->warn("\$hsp->matches() returned '' for number identical; setting to 0");
+				$these_ids = 0;
+			    }
+			    if ($these_cons eq '') {
+				$hsp->warn("\$hsp->matches() returned '' for number conserved; setting to 0");
+				$these_cons = 0;
+			    }
+			};
+                        if($@) { warn "\a\n$@\n"; }
+                        else {
+                            $ids  += $these_ids;
+                            $cons += $these_cons;
+                        }
+                        
+                        last if $hsp_start == $u_start;
+                        $use_start = $hsp_start;
+                    }
+                    ${$contigs_ref}[$i]->{start} = $u_start;
+                    ${$contigs_ref}[$i]->{'iden'} += $ids;
+                    ${$contigs_ref}[$i]->{'cons'} += $cons;
+                    push(@{${$contigs_ref}[$i]->{hsps}}, @{${$contigs_ref}[$u]->{hsps}});
+                    
+                    ${$contigs_ref}[$u] = undef;
+                }
+                elsif ($u_stop > $i_stop && $u_start <= ($i_stop - $max_overlap + 1)) {
+                    # find the hsps within the contig that have sequence
+                    # extending beyond $i_stop
+                    my ($ids, $cons) = (0, 0);
+                    my $use_stop = $i_stop;
+                    foreach my $hsp (sort { $a->start($seqType) <=> $b->start($seqType) } @{${$contigs_ref}[$u]->{hsps}}) {
+                        my $hsp_end = $hsp->end($seqType);
+                        $hsp_end > $use_stop || next;
+                        
+                        my ($these_ids, $these_cons);
+                        eval {
+                            ($these_ids, $these_cons) = $hsp->matches(-SEQ => $seqType, -START => $use_stop + 1, -STOP => $hsp_end);
+			    if ($these_ids eq '') {
+				$hsp->warn("\$hsp->matches() returned '' for number identical; setting to 0");
+				$these_ids = 0;
+			    }
+			    if ($these_cons eq '') {
+				$hsp->warn("\$hsp->matches() returned '' for number conserved; setting to 0");
+				$these_cons = 0;
+			    }
+                        };
+                        if($@) { warn "\a\n$@\n"; }
+                        else {
+                            $ids  += $these_ids;
+                            $cons += $these_cons;
+                        }
+                        
+                        last if $hsp_end == $u_stop;
+                        $use_stop = $hsp_end;
+                    }
+                    ${$contigs_ref}[$i]->{'stop'}  = $u_stop;
+                    ${$contigs_ref}[$i]->{'iden'} += $ids;
+                    ${$contigs_ref}[$i]->{'cons'} += $cons;
+                    push(@{${$contigs_ref}[$i]->{hsps}}, @{${$contigs_ref}[$u]->{hsps}});
+                    
+                    ${$contigs_ref}[$u] = undef;
+                }
+                elsif ($u_start >= $i_start && $u_stop <= $i_stop) {
+                    # nested, drop this contig
+                    #*** ideally we might do some magic to keep the stats of the
+                    #    better hsp...
+                    ${$contigs_ref}[$u] = undef;
+                }
+            }
+        }
+        
+        my @merged;
+        foreach (@$contigs_ref) {
+            push(@merged, $_ || next);
+        }
+        @{$contigs_ref} = @merged;
+    }
+    elsif (! $overlap) {
+        ## If there is no overlap, add the complete HSP data.
+        ($numID,$numCons) = $hsp->matches(-SEQ=>$seqType);
+	if ($numID eq '') {
+	    $hsp->warn("\$hsp->matches() returned '' for number identical; setting to 0");
+	    $numID = 0;
+	}
+	if ($numCons eq '') {
+	    $hsp->warn("\$hsp->matches() returned '' for number conserved; setting to 0");
+	    $numCons = 0;
+	}
+
+        push @$contigs_ref, {'start' =>$start, 'stop' =>$stop,
 			     'iden'  =>$numID, 'cons' =>$numCons,
-			     'strand'=>$strand,'frame'=>$frame};
-    };
-    $overlap;
+			     'strand'=>$strand,'frame'=>$frame,'hsps'=>[$hsp]};
+    }
+    
+    return $overlap;
 }
 
 =head2 get_exponent
@@ -451,9 +590,7 @@ sub _adjust_contigs {
 
 =cut
 
-#------------------
 sub get_exponent {
-#------------------
     my $data = shift;
 
     my($num, $exp) = split /[eE]/, $data;
@@ -502,9 +639,7 @@ See Also   : L<Bio::Search::Hit::BlastHit::seq_inds()|Bio::Search::Hit::BlastHit
 
 =cut
 
-#------------------
 sub collapse_nums {
-#------------------
 # This is probably not the slickest connectivity algorithm, but will do for now.
     my @a = @_;
     my ($from, $to, $i, @ca, $consec);
@@ -512,6 +647,8 @@ sub collapse_nums {
     $consec = 0;
     for($i=0; $i < @a; $i++) {
 	not $from and do{ $from = $a[$i]; next; };
+    # pass repeated positions (gap inserts)
+    next if $a[$i] == $a[$i-1];
 	if($a[$i] == $a[$i-1]+1) {
 	    $to = $a[$i];
 	    $consec++;
@@ -567,13 +704,9 @@ sub collapse_nums {
            : reconstitute parsable Blast reports from HTML-format versions
            : should be considered a temorary solution.
 
-See Also   : B<Bio::Search::Processor::BlastIO::new()>
-
 =cut
 
-#--------------------
 sub strip_blast_html {
-#--------------------
       # This may not best way to remove html tags. However, it is simple.
       # it won't work under following conditions:
       #    1) if quoted > appears in a tag  (does this ever happen?)
@@ -603,7 +736,53 @@ sub strip_blast_html {
     $stripped;
 }
 
+=head2 result2hash
 
+ Title    : result2hash
+ Usage    : my %data = &Bio::Search::SearchUtils($result)
+ Function : converts ResultI data to simple hash
+ Returns  : hash
+ Args     : ResultI
+ Note     : used mainly as a utility for running SearchIO tests
+
+=cut
+
+sub result2hash {
+    my ($result) = @_;
+    my %hash;
+    $hash{'query_name'} = $result->query_name;
+    my $hitcount = 1;
+    my $hspcount = 1;
+    foreach my $hit ( $result->hits ) {
+	$hash{"hit$hitcount\_name"}   =  $hit->name;
+	# only going to test order of magnitude
+	# too hard as these don't always match
+#	$hash{"hit$hitcount\_signif"} =  
+#	    ( sprintf("%.0e",$hit->significance) =~ /e\-?(\d+)/ );
+	$hash{"hit$hitcount\_bits"}   =  sprintf("%d",$hit->bits);
+
+	foreach my $hsp ( $hit->hsps ) {
+	    $hash{"hsp$hspcount\_bits"}   = sprintf("%d",$hsp->bits);
+	    # only going to test order of magnitude
+ 	    # too hard as these don't always match
+#	    $hash{"hsp$hspcount\_evalue"} =  
+#		( sprintf("%.0e",$hsp->evalue) =~ /e\-?(\d+)/ );
+	    $hash{"hsp$hspcount\_qs"}     = $hsp->query->start;
+	    $hash{"hsp$hspcount\_qe"}     = $hsp->query->end;
+	    $hash{"hsp$hspcount\_qstr"}   = $hsp->query->strand;
+	    $hash{"hsp$hspcount\_hs"}     = $hsp->hit->start;
+	    $hash{"hsp$hspcount\_he"}     = $hsp->hit->end;
+	    $hash{"hsp$hspcount\_hstr"}   = $hsp->hit->strand;
+
+	    #$hash{"hsp$hspcount\_pid"}     = sprintf("%d",$hsp->percent_identity);
+	    #$hash{"hsp$hspcount\_fid"}     = sprintf("%.2f",$hsp->frac_identical);
+	    $hash{"hsp$hspcount\_gaps"}    = $hsp->gaps('total');
+	    $hspcount++;
+	}
+	$hitcount++;
+    }
+    return %hash;
+}
 
 sub _warn_about_no_hsps {
     my $hit = shift;
@@ -618,9 +797,6 @@ sub _warn_about_no_hsps {
                "-b option (or equivalent if not using blastall) to increase the number\n".
                "of alignments.\n"
               );
-
 }
 
 1;
-
-
