@@ -1,7 +1,9 @@
-# $Id: HTMLResultWriter.pm,v 1.35 2004/12/08 10:48:04 heikki Exp $
+# $Id: HTMLResultWriter.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::SearchIO::Writer::HTMLResultWriter
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
@@ -9,24 +11,24 @@
 # You may distribute this module under the same terms as perl itself
 
 # Changes 2003-07-31 (jason)
-# Gary has cleaned up the code a lot to produce better looking
-# HTML
+# Gary has cleaned up the code a lot to produce better looking HTML
+
 # POD documentation - main docs before the code
 
 =head1 NAME
 
-Bio::SearchIO::Writer::HTMLResultWriter - Object to implement writing a Bio::Search::ResultI in HTML.
+Bio::SearchIO::Writer::HTMLResultWriter - write a Bio::Search::ResultI in HTML
 
 =head1 SYNOPSIS
 
   use Bio::SearchIO;
   use Bio::SearchIO::Writer::HTMLResultWriter;
 
-  my $in = new Bio::SearchIO(-format => 'blast',
+  my $in = Bio::SearchIO->new(-format => 'blast',
 			     -file   => shift @ARGV);
 
-  my $writer = new Bio::SearchIO::Writer::HTMLResultWriter();
-  my $out = new Bio::SearchIO(-writer => $writer);
+  my $writer = Bio::SearchIO::Writer::HTMLResultWriter->new();
+  my $out = Bio::SearchIO->new(-writer => $writer);
   $out->write_result($in->next_result);
 
 
@@ -41,9 +43,9 @@ Bio::SearchIO::Writer::HTMLResultWriter - Object to implement writing a Bio::Sea
       return $hsp->num_hits > 0;
   }
 
-  my $writer = new Bio::SearchIO::Writer::HTMLResultWriter
+  my $writer = Bio::SearchIO::Writer::HTMLResultWriter->new
                      (-filters => { 'HSP' => \&hsp_filter} );
-  my $out = new Bio::SearchIO(-writer => $writer);
+  my $out = Bio::SearchIO->new(-writer => $writer);
   $out->write_result($in->next_result);
 
   # can also set the filter via the writer object
@@ -52,7 +54,7 @@ Bio::SearchIO::Writer::HTMLResultWriter - Object to implement writing a Bio::Sea
 =head1 DESCRIPTION
 
 This object implements the SearchWriterI interface which will produce
-a set of HTML for a specific Bio::Search::Report::ReportI interface.
+a set of HTML for a specific L<Bio::Search::Report::ReportI> interface.
 
 See L<Bio::SearchIO::SearchWriterI> for more info on the filter method.
 
@@ -64,17 +66,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via
-email or the web:
+of the bugs and their resolution. Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -92,34 +104,31 @@ Internal methods are usually preceded with a _
 =cut
 
 
-
 package Bio::SearchIO::Writer::HTMLResultWriter;
-use vars qw(@ISA %RemoteURLDefault
-            $MaxDescLen $DATE $AlignmentLineWidth $Revision);
 use strict;
-$Revision = '$Id: HTMLResultWriter.pm,v 1.35 2004/12/08 10:48:04 heikki Exp $'; #'
+use vars qw(%RemoteURLDefault
+            $MaxDescLen $DATE $AlignmentLineWidth $Revision);
 
 # Object preamble - inherits from Bio::Root::RootI
 
 BEGIN {
+    $Revision = '$Id: HTMLResultWriter.pm 16123 2009-09-17 12:57:27Z cjfields $';
     $DATE = localtime(time);
-    %RemoteURLDefault = ( 'PROTEIN' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=protein&cmd=search&term=%s',			  
-			  'NUCLEOTIDE' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=nucleotide&cmd=search&term=%s'
-			  );
-
+    %RemoteURLDefault = ( 
+      'PROTEIN' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=protein&cmd=search&term=%s',			  
+      'NUCLEOTIDE' => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=nucleotide&cmd=search&term=%s'
+    );
     $MaxDescLen = 60;
     $AlignmentLineWidth = 60;
 }
 
-use Bio::Root::Root;
-use Bio::SearchIO::SearchWriterI;
 
-@ISA = qw(Bio::Root::Root Bio::SearchIO::SearchWriterI);
+use base qw(Bio::Root::Root Bio::SearchIO::SearchWriterI);
 
 =head2 new
 
  Title   : new
- Usage   : my $obj = new Bio::SearchIO::Writer::HTMLResultWriter();
+ Usage   : my $obj = Bio::SearchIO::Writer::HTMLResultWriter->new();
  Function: Builds a new Bio::SearchIO::Writer::HTMLResultWriter object 
  Returns : Bio::SearchIO::Writer::HTMLResultWriter
  Args    : -filters => hashref with any or all of the keys (HSP HIT RESULT)
@@ -212,7 +221,6 @@ sub to_string {
 
     my ($qtype,$dbtype,$dbseqtype,$type);
     my $alg = $result->algorithm;
-
     # This is actually wrong for the FASTAs I think
     if(  $alg =~ /T(FAST|BLAST)([XY])/i ) {
 	$qtype      = $dbtype = 'translated';
@@ -235,7 +243,7 @@ sub to_string {
         $dbtype     = 'PROTEIN';
 	$dbseqtype  = $type      = 'PROTEIN';
     } else { 
-	print STDERR "algorithm was ", $result->algorithm, " couldn't match\n";
+	$self->warn("algorithm was ", $result->algorithm, " couldn't match\n");
     }
     
     
@@ -243,7 +251,7 @@ sub to_string {
 		     'Query:'   => ( $qtype  eq 'translated' )  ? 3 : 1);
 
     my $str;
-    if( ! defined $num || $num <= 1 ) { 
+    if( $num <= 1 ) { 
 	$str = &{$self->start_report}($result);
     }
 
@@ -286,8 +294,8 @@ sub to_string {
 	    # no HSPs so no link 
 	    $str .= sprintf('<tr><td>%s %s</td><td>%s</td><td>%.2g</td></tr>'."\n",
 			    $url_desc, $descsub, 
-			    ($hit->raw_score ? $hit->raw_score : 
-			     (defined $hsps[0] ? $hsps[0]->score : ' ')),
+			    ($hit->bits ? $hit->bits : 
+			     (defined $hsps[0] ? $hsps[0]->bits : ' ')),
 			    ( $hit->significance ? $hit->significance :
 			      (defined $hsps[0] ? $hsps[0]->evalue : ' ')) 
 			    );
@@ -297,16 +305,16 @@ sub to_string {
 
 	    $str .= sprintf('<tr><td>%s %s</td><td>%s</td><td><a href="#%s">%.2g</a></td></tr>'."\n",
 			    $url_desc, $descsub, 
-			    ($hit->raw_score ? $hit->raw_score : 
-			     (defined $hsps[0] ? $hsps[0]->score : ' ')),
+			    ($hit->bits ? $hit->bits : 
+			     (defined $hsps[0] ? $hsps[0]->bits : ' ')),
 			    $acc,
 			    ( $hit->significance ? $hit->significance :
 			      (defined $hsps[0] ? $hsps[0]->evalue : ' ')) 
 			    );
+        my $dline = &{$self->hit_desc_line}($self, $hit, $result);
 	    $hspstr .= "<a name=\"$acc\">\n".
-		sprintf("><b>%s</b> %s\n<dd>Length = %s</dd><p>\n\n", $url_align, 
-			defined $hit->description ? $hit->description : '', 
-			&_numwithcommas($hit->length));
+		sprintf("><b>%s</b> %s</br><dd>Length = %s</dd><p>\n\n", $url_align, 
+			$dline , &_numwithcommas($hit->length));
 	    my $ct = 0;
 	    foreach my $hsp (@hsps ) {
 		next if( $hspfilter && ! &{$hspfilter}($hsp) );
@@ -421,7 +429,7 @@ sub to_string {
 						    length($hspvals[2]->{'start'}),
 						    length($hspvals[2]->{'end'}));
 		my $count = 0;
-		while ( $count <= $hsp->length('total') ) {
+		while ( $count < $hsp->length('total') ) {
 		    foreach my $v ( @hspvals ) {
 			my $piece = substr($v->{'seq'}, $v->{'index'} + $count,
 					   $AlignmentLineWidth);
@@ -457,19 +465,28 @@ sub to_string {
 #	$hspstr .= "</pre>\n";
     }
 
-
-    # make table of search statistics and end the web page
-    $str .= "</table><p>\n".$hspstr."<p><p><hr><h2>Search Parameters</h2><table border=1><tr><th>Parameter</th><th>Value</th>\n";
+    $str .= "</table><p>\n".$hspstr;
+    my ($pav, $sav) = ($result->available_parameters, $result->available_statistics);
+    if ($pav || $sav) {
+        # make table of search statistics and end the web page
+        $str .= "<p><p><hr><h2>Search Parameters</h2>";
+        if ($pav) {
+        $str .= "<table border=1><tr><th>Parameter</th><th>Value</th>\n";
+        foreach my $param ( sort $result->available_parameters ) {
+            $str .= "<tr><td>$param</td><td>". $result->get_parameter($param) ."</td></tr>\n";
+        }
+        $str .= "</table>";
+        }
         
-    foreach my $param ( sort $result->available_parameters ) {
-	$str .= "<tr><td>$param</td><td>". $result->get_parameter($param) ."</td></tr>\n";
-	
+        if ($sav) {
+        $str .= "<p><h2>Search Statistics</h2><table border=1><tr><th>Statistic</th><th>Value</th></tr>\n";
+        foreach my $stat ( sort $result->available_statistics ) {
+            $str .= "<tr><td>$stat</td><td>". $result->get_statistic($stat). "</td>\n";
+        }
+        $str .=  "</tr></table>";
+        }
     }
-    $str .= "</table><p><h2>Search Statistics</h2><table border=1><tr><th>Statistic</th><th>Value</th></tr>\n";
-    foreach my $stat ( sort $result->available_statistics ) {
-	$str .= "<tr><td>$stat</td><td>". $result->get_statistic($stat). "</td></tr>\n";
-    }
-    $str .=  "</table><P>".$self->footer() . "<P>\n";
+    $str .= $self->footer() . "<P>\n";
     return $str;
 }
 
@@ -499,7 +516,7 @@ sub hit_link_desc{
 
 =head2 default_hit_link_desc
 
- Title   : defaulthit_link_desc
+ Title   : default_hit_link_desc
  Usage   : $self->default_hit_link_desc($hit, $result)
  Function: Provides an HTML link(s) for the given hit to be used
            within the description section at the top of the BLAST report.
@@ -563,6 +580,79 @@ sub hit_link_align {
     return $self->{'_hit_link_align'} || \&default_hit_link_desc;
 }
 
+=head2 hit_desc_line
+
+ Title   : hit_desc_line
+ Usage   : $self->hit_desc_line(\&link_function);
+ Function: Get/Set the function which provides HTML for the description
+           information from a hit. This allows one to parse
+           the rest of the description and split up lines, add links, etc.
+ Returns : Function reference
+ Args    : Function reference
+ See Also: L<default_hit_link_desc()>
+
+=cut
+
+sub hit_desc_line{
+    my( $self, $code ) = @_; 
+    if ($code) {
+        $self->{'_hit_desc_line'} = $code;
+    }
+    return $self->{'_hit_desc_line'} || \&default_hit_desc_line;
+}
+
+=head2 default_hit_desc_line
+
+ Title   : default_hit_desc_line
+ Usage   : $self->default_hit_desc_line($hit, $result)
+ Function: Parses the description line information, splits based on the
+           hidden \x01 between independent descriptions, checks the lines for
+           possible web links, and adds HTML link(s) for the given hit to be
+           used.
+
+ Returns : string containing HTML markup "<a href...")
+           The default implementation returns an HTML link to the
+           URL supplied by the remote_database_url() method
+           and using the identifier supplied by the id_parser() method.
+           It will use the NCBI GI if present, and the accession if not.
+
+ Args    : First argument is a Bio::Search::Hit::HitI
+           Second argument is a Bio::Search::Result::ResultI
+
+See Also: L<hit_link_align>, L<remote_database>, L<id_parser>
+
+=cut
+
+sub default_hit_desc_line {
+    my($self, $hit, $result) = @_;
+    my $type = ( $result->algorithm =~ /(P|X|Y)$/i ) ? 'PROTEIN' : 'NUCLEOTIDE';
+    my @descs = split /\x01/, $hit->description;
+    #my $descline = join("</br>",@descs)."</br>";
+    my $descline = '';
+    #return $descline;
+    for my $sec (@descs) {
+        my $url = '';
+        if ($sec =~ s/((?:gi\|(\d+)\|)?        # optional GI
+                     (\w+)\|([A-Z\d\.\_]+) # main 
+                     (\|[A-Z\d\_]+)?) # optional secondary ID//xms) {
+            my ($name, $gi, $db, $acc) = ($1, $2, $3, $4);
+            #$acc ||= ($rest) ? $rest : $gi;
+            $acc =~ s/^\s+(\S+)/$1/;
+            $acc =~ s/(\S+)\s+$/$1/;
+            $url =
+            length($self->remote_database_url($type)) > 0 ? 
+              sprintf('<a href="%s">%s</a> %s',
+                      sprintf($self->remote_database_url($type),
+                      $gi || $acc || $db), 
+                      $name, $sec) :  $sec;
+        } else {
+            $url = $sec;
+        }
+        $descline .= "$url</br>\n";
+    }
+    return $descline;
+}
+
 =head2 start_report
 
   Title   : start_report
