@@ -1,7 +1,9 @@
-# $Id: TextResultWriter.pm,v 1.15 2004/12/14 21:18:42 jason Exp $
+# $Id: TextResultWriter.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::SearchIO::Writer::TextResultWriter
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
@@ -20,11 +22,11 @@ a Bio::Search::ResultI in Text.
   use Bio::SearchIO;
   use Bio::SearchIO::Writer::TextResultWriter;
 
-  my $in = new Bio::SearchIO(-format => 'blast',
+  my $in = Bio::SearchIO->new(-format => 'blast',
 			     -file   => shift @ARGV);
 
-  my $writer = new Bio::SearchIO::Writer::TextResultWriter();
-  my $out = new Bio::SearchIO(-writer => $writer);
+  my $writer = Bio::SearchIO::Writer::TextResultWriter->new();
+  my $out = Bio::SearchIO->new(-writer => $writer);
   $out->write_result($in->next_result);
 
 =head1 DESCRIPTION
@@ -52,7 +54,7 @@ Define a hit filter method
       return $hit->length E<gt> 100; # test if length of the hit sequence
                                      # long enough    
   }
-  my $writer = new Bio::SearchIO::Writer::TextResultWriter(
+  my $writer = Bio::SearchIO::Writer::TextResultWriter->new(
        -filters => { 'HIT' =E<gt> \&hit_filter }  
       );
 
@@ -63,7 +65,7 @@ only include HSPs which are 75% identical or better.
        my $hsp = shift;
        return $hsp->percent_identity E<gt> 75;
    }
-   my $writer = new Bio::SearchIO::Writer::TextResultWriter(
+   my $writer = Bio::SearchIO::Writer::TextResultWriter->new(
        -filters => { 'HSP' =E<gt> \&hsp_filter }  
       );
 
@@ -86,17 +88,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution. Bug reports can be submitted via
-email or the web:
+of the bugs and their resolution. Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich
 
@@ -114,8 +126,7 @@ Internal methods are usually preceded with a _
 
 
 package Bio::SearchIO::Writer::TextResultWriter;
-use vars qw(@ISA $MaxNameLen $MaxDescLen $AlignmentLineWidth 
-	    $DescLineLen $TextWrapLoaded);
+use vars qw($MaxNameLen $MaxDescLen $AlignmentLineWidth 	    $DescLineLen $TextWrapLoaded);
 use strict;
 
 # Object preamble - inherits from Bio::Root::RootI
@@ -129,16 +140,14 @@ BEGIN {
     }
 }
 
-use Bio::Root::Root;
-use Bio::SearchIO::SearchWriterI;
 use POSIX;
 
-@ISA = qw(Bio::Root::Root Bio::SearchIO::SearchWriterI);
+use base qw(Bio::Root::Root Bio::SearchIO::SearchWriterI);
 
 =head2 new
 
  Title   : new
- Usage   : my $obj = new Bio::SearchIO::Writer::TextResultWriter();
+ Usage   : my $obj = Bio::SearchIO::Writer::TextResultWriter->new();
  Function: Builds a new Bio::SearchIO::Writer::TextResultWriter object 
  Returns : Bio::SearchIO::Writer::TextResultWriter
  Args    : -filters => hashref with any or all of the keys (HSP HIT RESULT)
@@ -200,10 +209,12 @@ sub to_string {
 						  $self->filter('HIT'),
 						  $self->filter('HSP') );
     return '' if( defined $resultfilter && ! &{$resultfilter}($result) );    
-
+    
     my ($qtype,$dbtype,$dbseqtype,$type);
     my $alg = $result->algorithm;
-
+    
+    my $wublast = ($result->algorithm_version =~ /WashU/) ? 1 : 0;
+    
     # This is actually wrong for the FASTAs I think
     if(  $alg =~ /T(FAST|BLAST)([XY])/i ) {
 	$qtype      = $dbtype = 'translated';
@@ -268,11 +279,14 @@ Sequences producing significant alignments:                      (bits)    value
 	} else { 
 	    $descsub = sprintf($p,$desc);
 	}
-
-	$str .= sprintf("%s   %-4s  %s\n",
+	$str .= $wublast ? sprintf("%s   %-4s  %s\n",
 			$descsub,
 			defined $hit->raw_score ? $hit->raw_score : ' ',
-			defined $hit->significance ? $hit->significance : '?');
+			defined $hit->significance ? $hit->significance : '?') :
+                    sprintf("%s   %-4s  %s\n",
+			$descsub,
+			defined $hit->bits ? $hit->bits: ' ',
+			defined $hit->significance ? $hit->significance : '?');        
 	my @hsps = $hit->hsps;
 	if( @hsps ) { 
 	    $hspstr .= sprintf(">%s %s\n%9sLength = %d\n\n",
@@ -431,17 +445,17 @@ Sequences producing significant alignments:                      (bits)    value
     
     $str .= sprintf(qq{  Database: %s
     Posted date:  %s
-  Number of letters in database: %s
-  Number of sequences in database: %s
+	Number of letters in database: %s
+	Number of sequences in database: %s
 
-Matrix: %s
-}, 		   
-		    $result->database_name(),
-		    $result->get_statistic('posted_date') || 
-		    POSIX::strftime("%b %d, %Y %I:%M %p",localtime),
-		    &_numwithcommas($result->database_entries()), 
-		    &_numwithcommas($result->database_letters()),
-		    $result->get_parameter('matrix') || '');
+  Matrix: %s
+  },
+	        $result->database_name(),
+	        $result->get_statistic('posted_date') ||
+	        POSIX::strftime("%b %d, %Y %I:%M %p",localtime),
+	        &_numwithcommas($result->database_letters()),
+	        &_numwithcommas($result->database_entries()),
+	        $result->get_parameter('matrix') || '');
 
     if( defined (my $open = $result->get_parameter('gapopen')) ) {
 	$str .= sprintf("Gap Penalties Existence: %d, Extension: %d\n",
