@@ -1,7 +1,9 @@
-# $Id: Seq.pm,v 1.85 2004/08/11 07:27:29 lapp Exp $
+# $Id: Seq.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Seq
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
@@ -52,7 +54,7 @@ Bio::Seq - Sequence object, with features
     # sequences may have a species
 
     if( defined $seq->species ) {
-	print "Sequence is from ",$species->binomial_name," [",$species->common_name,"]\n";
+	print "Sequence is from ",$species->binomial," [",$species->common_name,"]\n";
     }
 
     # annotation objects are Bio::AnnotationCollectionI's
@@ -412,17 +414,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists. Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bio.perl.org/MailList.html  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney, inspired by Ian Korf objects
 
@@ -445,25 +457,12 @@ methods. Internal methods are usually preceded with a "_".
 
 
 package Bio::Seq;
-use vars qw(@ISA);
 use strict;
 
-
-# Object preamble - inherits from Bio::Root::Object
-
-use Bio::Root::Root;
-use Bio::SeqI;
 use Bio::Annotation::Collection;
 use Bio::PrimarySeq;
-use Bio::IdentifiableI;
-use Bio::DescribableI;
-use Bio::AnnotatableI;
-use Bio::FeatureHolderI;
 
-
-@ISA = qw(Bio::Root::Root Bio::SeqI
-	  Bio::IdentifiableI Bio::DescribableI
-	  Bio::AnnotatableI Bio::FeatureHolderI);
+use base qw(Bio::Root::Root Bio::SeqI Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI Bio::FeatureHolderI);
 
 =head2 new
 
@@ -583,7 +582,7 @@ sub seq {
            lead to an exception if passed to seq().
 
            The implementation provided here does not take alphabet() into
-           account. Allowed are all letters (A-Z) and '-','.', and '*'.
+           account. Allowed are all letters (A-Z), '-','.','*','=', and '~'.
 
  Example :
  Returns : 1 if the supplied sequence string is valid for the object, and
@@ -939,10 +938,11 @@ sub description {
 =head2 annotation
 
  Title   : annotation
- Usage   : $ann = $seq->annotation or $seq->annotation($annotation)
+ Usage   : $ann = $seq->annotation or 
+           $seq->annotation($ann)
  Function: Gets or sets the annotation
- Returns : L<Bio::AnnotationCollectionI> object
- Args    : None or L<Bio::AnnotationCollectionI> object
+ Returns : Bio::AnnotationCollectionI object
+ Args    : None or Bio::AnnotationCollectionI object
 
 See L<Bio::AnnotationCollectionI> and L<Bio::Annotation::Collection>
 for more information
@@ -957,7 +957,7 @@ sub annotation {
 	    unless $value->isa("Bio::AnnotationCollectionI");
 	$obj->{'_annotation'} = $value;
     } elsif( ! defined $obj->{'_annotation'}) {
-	$obj->{'_annotation'} = new Bio::Annotation::Collection;
+	$obj->{'_annotation'} = Bio::Annotation::Collection->new();
     }
     return $obj->{'_annotation'};
 }
