@@ -1,7 +1,9 @@
-# $Id: SequenceTrace.pm,v 1.8 2005/01/18 20:53:53 mlogghe Exp $
+# $Id: SequenceTrace.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Seq::SequenceTrace
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Chad Matsalla <bioinformatics@dieselwurks.com
 #
 # Copyright Chad Matsalla
@@ -30,17 +32,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Chad Matsalla
 
@@ -55,21 +67,14 @@ Internal methods are usually preceded with a _
 
 package Bio::Seq::SequenceTrace;
 
-use vars qw(@ISA);
 
 use strict;
-use Bio::Root::Root;
 use Bio::Seq::QualI;
 use Bio::PrimarySeqI;
 use Bio::PrimarySeq;
 use Bio::Seq::PrimaryQual;
-use Bio::Seq::TraceI;
-use Bio::Seq::SeqWithQuality;
-use Dumpvalue();
 
-my $dumper = new Dumpvalue();
-
-@ISA = qw(Bio::Root::Root Bio::Seq::SeqWithQuality Bio::Seq::TraceI);
+use base qw(Bio::Root::Root Bio::Seq::Quality Bio::Seq::TraceI);
 
 =head2 new()
 
@@ -112,12 +117,12 @@ sub new {
                ACCURACY_G
                ACCURACY_C )], @args);
           # first, deal with the sequence and quality information
-     if ($swq && ref($swq) eq "Bio::Seq::SeqWithQuality") {
+     if ($swq && ref($swq) eq "Bio::Seq::Quality") {
           $self->{swq} = $swq;
      }
      else {
           $self->throw("A Bio::Seq::SequenceTrace object must be created with a
-               Bio::Seq::SeqWithQuality object. You provided this type of object: "
+               Bio::Seq::Quality object. You provided this type of object: "
                .ref($swq));
      }
      if (!$acc_a) {
@@ -141,12 +146,13 @@ sub new {
           $self->trace('c',$trace_c);
           $self->peak_indices($peak_indices);
      }
-     $self->id($self->swq_obj()->id());
+     $self->id($self->seq_obj->id);
     return $self;
 }
 
 sub swq_obj {
      my $self = shift;
+     $self->warn('swq_obj() is deprecated: use seq_obj()');
      return $self->{swq};
 }
 
@@ -190,7 +196,7 @@ sub trace {
           return $self->{trace}->{$base_channel};
      }
      else {
-          return undef;
+          return;
      }
 }
 
@@ -293,7 +299,7 @@ sub peak_index_at {
 
 sub alphabet {
 	my $self = shift;
-	return $self->{swq}->{seq_ref}->alphabet(@_);
+	return $self->{swq}->alphabet;
 }
 
 =head2 display_id()
@@ -313,10 +319,10 @@ sub alphabet {
 	field for example, or extending the sequence object) to solve
 	this. Notice that $seq->id() maps to this function, mainly for
         legacy/convience issues.
-	This method sets the display_id for the SeqWithQuality object.
+	This method sets the display_id for the Quality object.
  Returns : A string
  Args    : If a scalar is provided, it is set as the new display_id for
-	the SeqWithQuality object.
+	the Quality object.
  Status  : Virtual
 
 =cut
@@ -341,11 +347,11 @@ sub display_id {
         for the implemetation, allowing multiple objects to have the same
         accession number in a particular implementation. For sequences
         with no accession number, this method should return "unknown".
-	This method sets the accession_number for the SeqWithQuality
+	This method sets the accession_number for the Quality
 	object. 
  Returns : A string (the value of accession_number)
  Args    : If a scalar is provided, it is set as the new accession_number
-	for the SeqWithQuality object.
+	for the Quality object.
  Status  : Virtual
 
 
@@ -371,11 +377,11 @@ sub accession_number {
         way the implementaiton can control clients can expect one id to
         map to one object. For sequences with no accession number, this
         method should return a stringified memory location.
-	This method sets the primary_id for the SeqWithQuality
+	This method sets the primary_id for the Quality
 	object.
  Returns : A string. (the value of primary_id)
  Args    : If a scalar is provided, it is set as the new primary_id for
-	the SeqWithQuality object.
+	the Quality object.
 
 =cut
 
@@ -393,15 +399,15 @@ sub primary_id {
  Title   : desc()
  Usage   : $qual->desc($newval); _or_ 
            $description = $qual->desc();
- Function: Get/set description text for this SeqWithQuality object.
+ Function: Get/set description text for this Quality object.
  Returns : A string. (the value of desc)
  Args    : If a scalar is provided, it is set as the new desc for the
-	SeqWithQuality object.
+	   Quality object.
 
 =cut
 
 sub desc {
-	# a mechanism to set the desc for the SeqWithQuality object.
+	# a mechanism to set the desc for the Quality object.
 	# probably will be used most often by set_common_features()
    my ($self,$value) = @_;
    if( defined $value) {
@@ -419,7 +425,7 @@ sub desc {
         for display_id().
  Returns : A string. (the value of id)
  Args    : If a scalar is provided, it is set as the new id for the
-	SeqWithQuality object.
+	   Quality object.
 
 =cut
 
@@ -438,9 +444,9 @@ sub id {
  Usage   : $string    = $obj->seq(); _or_
 	$obj->seq("atctatcatca");
  Function: Returns the sequence that is contained in the imbedded in the
-	PrimarySeq object within the SeqWithQuality object
+	PrimarySeq object within the Quality object
  Returns : A scalar (the seq() value for the imbedded PrimarySeq object.)
- Args    : If a scalar is provided, the SeqWithQuality object will
+ Args    : If a scalar is provided, the Quality object will
 	attempt to set that as the sequence for the imbedded PrimarySeq
 	object. Otherwise, the value of seq() for the PrimarySeq object
 	is returned.
@@ -456,9 +462,9 @@ sub id {
 sub seq {
 	my ($self,$value) = @_;
 	if( defined $value) {
-		$self->{swq}->{seq_ref}->seq($value);
+		$self->{swq}->seq($value);
 	}
-	return $self->{swq}->{seq_ref}->seq();
+	return $self->{swq}->seq();
 }
 
 =head2 qual()
@@ -467,10 +473,10 @@ sub seq {
  Usage   : @quality_values  = @{$obj->qual()}; _or_
 	$obj->qual("10 10 20 40 50");
  Function: Returns the quality as imbedded in the PrimaryQual object
-	within the SeqWithQuality object.
+	within the Quality object.
  Returns : A reference to an array containing the quality values in the 
 	PrimaryQual object.
- Args    : If a scalar is provided, the SeqWithQuality object will
+ Args    : If a scalar is provided, the Quality object will
 	attempt to set that as the quality for the imbedded PrimaryQual
 	object. Otherwise, the value of qual() for the PrimaryQual
 	object is returned.
@@ -486,9 +492,9 @@ sub qual {
 	my ($self,$value) = @_;
 
 	if( defined $value) {
-		$self->{swq}->{qual_ref}->qual($value);
+		$self->{swq}->qual($value);
 	}
-	return $self->{swq}->{qual_ref}->qual();
+	return $self->{swq}->qual();
 }
 
 
@@ -498,17 +504,15 @@ sub qual {
 
  Title   : length()
  Usage   : $length = $seqWqual->length();
- Function: Get the length of the SeqWithQuality sequence/quality.
- Returns : Returns the length of the sequence and quality if they are
-	both the same. Returns "DIFFERENT" if they differ.
+ Function: Get the length of the Quality sequence/quality.
+ Returns : Returns the length of the sequence and quality
  Args    : None.
 
 =cut
 
 sub length {
     my $self = shift;
-     return $self->seq_obj()->length();
-
+    return $self->seq_obj->length;
 }
 
 
@@ -517,36 +521,39 @@ sub length {
  Title   : qual_obj($different_obj)
  Usage   : $qualobj = $seqWqual->qual_obj(); _or_
 	$qualobj = $seqWqual->qual_obj($ref_to_primaryqual_obj);
- Function: Get the PrimaryQual object that is imbedded in the
-	SeqWithQuality object or if a reference to a PrimaryQual object
+ Function: Get the Qualilty object that is imbedded in the
+	Quality object or if a reference to a PrimaryQual object
 	is provided, set this as the PrimaryQual object imbedded in the
-	SeqWithQuality object.
- Returns : A reference to a Bio::Seq::SeqWithQuality object.
+	Quality object.
+ Returns : A reference to a Bio::Seq::Quality object.
+
+Identical to L<seq_obj>.
 
 =cut
 
 sub qual_obj {
     my ($self,$value) = @_;
-    return $self->{swq}->qual_obj($value);
+#    return $self->{swq}->qual_obj($value);
+    return $self->{swq};
 }
 
 
 =head2 seq_obj
 
  Title   : seq_obj()
- Usage   : $seqobj = $seqWqual->qual_obj(); _or_
+ Usage   : $seqobj = $seqWqual->seq_obj(); _or_
 	$seqobj = $seqWqual->seq_obj($ref_to_primary_seq_obj);
  Function: Get the PrimarySeq object that is imbedded in the
-	SeqWithQuality object or if a reference to a PrimarySeq object is
+	Quality object or if a reference to a PrimarySeq object is
 	provided, set this as the PrimarySeq object imbedded in the
-	SeqWithQuality object.
+	Quality object.
  Returns : A reference to a Bio::PrimarySeq object.
 
 =cut
 
 sub seq_obj {
     my ($self,$value) = @_;
-    return $self->{swq}->seq_obj($value);
+    return $self->{swq};
 }
 
 =head2 _set_descriptors
@@ -554,7 +561,7 @@ sub seq_obj {
  Title   : _set_descriptors()
  Usage   : $seqWqual->_qual_obj($qual,$seq,$id,$acc,$pid,$desc,$given_id,
 	$alphabet);
- Function: Set the descriptors for the SeqWithQuality object. Try to
+ Function: Set the descriptors for the Quality object. Try to
 	match the descriptors in the PrimarySeq object and in the
 	PrimaryQual object if descriptors were not provided with
 	construction.
@@ -570,7 +577,8 @@ sub seq_obj {
 
 sub _set_descriptors {
     my ($self,$qual,$seq,$id,$acc,$pid,$desc,$given_id,$alphabet) = @_;
-     $self->{swq}->_seq_descriptors($qual,$seq,$id,$acc,$pid,$desc,$given_id,$alphabet);
+    $self->{swq}->_seq_descriptors($qual,$seq,$id,$acc,$pid,
+				   $desc,$given_id,$alphabet);
 }
 
 =head2 subseq($start,$end)
@@ -765,8 +773,8 @@ sub sub_trace_object {
         my @subs = @{$self->sub_peak_index($start,$end)};
         $start2 = shift(@subs);
         $end2 =  pop(@subs);
-     my $new_object =  new Bio::Seq::SequenceTrace(
-               -swq =>   new Bio::Seq::SeqWithQuality(
+     my $new_object =  Bio::Seq::SequenceTrace->new(
+               -swq =>   Bio::Seq::Quality->new(
                              -seq => $self->subseq($start,$end),
                              -qual     =>   $self->subqual($start,$end),
                              -id    =>   $self->id()
@@ -974,7 +982,7 @@ sub _deprecated_get_scf_version_3_base_structure {
  Title   : trace_value_at($channel,$position)
  Usage   : $value = $trace_object->trace_value_at($channel,$position);
  Function: What is the value of the trace for this base at this position?
- Returns : A scalar represnting the trace value here.
+ Returns : A scalar representing the trace value here.
  Args    : a base channel (a,t,g,c)
            a position ( < $trace_object->trace_length() )
 
