@@ -1,7 +1,9 @@
-# $Id: Unflattener.pm,v 1.30 2005/01/24 02:39:25 jason Exp $
+# $Id: Unflattener.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # bioperl module for Bio::SeqFeature::Tools::Unflattener
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Chris Mungall <cjm@fruitfly.org>
 #
 # Copyright Chris Mungall
@@ -584,16 +586,26 @@ Bioperl modules. Send your comments and suggestions preferably to the
 Bioperl mailing lists  Your participation is much appreciated.
 
   bioperl-l@bioperl.org                         - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.  Bug reports can be submitted via
- email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Chris Mungall
 
@@ -610,17 +622,15 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::SeqFeature::Tools::Unflattener;
-use vars qw(@ISA);
 use strict;
 
 # Object preamble - inherits from Bio::Root::Root
-use Bio::Root::Root;
 use Bio::Location::Simple;
 use Bio::SeqFeature::Generic;
 use Bio::Range;
 
 
-@ISA = qw(Bio::Root::Root);
+use base qw(Bio::Root::Root);
 
 =head2 new
 
@@ -648,14 +658,16 @@ sub new {
     my($class,@args) = @_;
     my $self = $class->SUPER::new(@args);
 
-    my($seq, $group_tag) =
+    my($seq, $group_tag, $trust_grouptag) =
 	$self->_rearrange([qw(SEQ
                               GROUP_TAG
+                              TRUST_GROUPTAG
 			     )],
                           @args);
 
     $seq  && $self->seq($seq);
     $group_tag  && $self->group_tag($group_tag);
+    # $self->{'trust_grouptag'}= $trust_grouptag if($trust_grouptag); #dgg suggestion
     return $self; # success - we hope!
 }
 
@@ -832,8 +844,8 @@ typically contains
   source
   gene
   CDS
-  exon
-  intron
+  exon [optional]
+  intron [optional]
 
 there are no mRNA features
 
@@ -845,7 +857,7 @@ with this structure type, we want the seq_features to be nested like this
       intron
 
 exon and intron may or may not be present; they may be implicit from
-the mRNA 'join' location
+the CDS 'join' location
 
 =back
 
@@ -1147,7 +1159,18 @@ sub unflatten_seq{
        # use magic to guess the group tag
        my @sfs_with_locus_tag =
 	 grep {$_->has_tag("locus_tag")} @flat_seq_features;
+       my @sfs_with_gene_tag =
+	 grep {$_->has_tag("gene")} @flat_seq_features;
+       my @sfs_with_product_tag =
+	 grep {$_->has_tag("product")} @flat_seq_features;
+	 
+#        if ($group_tag && $self->{'trust_grouptag'}) { # dgg suggestion
+# 
+#         }
+#        elsif
        if (@sfs_with_locus_tag) {
+        # dgg note: would like to -use_magic with -group_tag = 'gene' for ensembl genomes
+        # where ensembl gene FT have both /locus_tag and /gene, but mRNA, CDS have /gene only
 	   if ($group_tag && $group_tag ne 'locus_tag') {
 	       $self->throw("You have explicitly set group_tag to be '$group_tag'\n".
 			    "However, I detect that some features use /locus_tag\n".
@@ -1155,15 +1178,28 @@ sub unflatten_seq{
 			    "You can resolve this by either NOT setting -group_tag\n".
 			    "OR you can unset -use_magic to regain control");
 	   }
-	   # I know what's best for you.
-	   # You want to be using /locus_tag=foo here
-	   #
+
+	   # use /locus_tag instead of /gene tag for grouping
 	   # see GenBank entry AE003677 (version 3) for an example
 	   $group_tag = 'locus_tag';
            if ($self->verbose > 0) {
                warn "Set group tag to: $group_tag\n";
            }
        }
+
+       # on rare occasions, records will have no /gene or /locus_tag
+       # but it WILL have /product tags. These serve the same purpose
+       # for grouping. For an example, see AY763288 (also in t/data)
+       if (@sfs_with_locus_tag==0 &&
+           @sfs_with_gene_tag==0 &&
+           @sfs_with_product_tag>0 &&
+           !$group_tag) {
+	   $group_tag = 'product';
+           if ($self->verbose > 0) {
+               warn "Set group tag to: $group_tag\n";
+           }
+           
+       }
    }
    if (!$group_tag) {
        $group_tag = 'gene';
@@ -1270,7 +1306,8 @@ sub unflatten_seq{
            my $type = $sf->primary_tag;
            # SO type is typically the same as the normal
            # type but preceeded by "pseudo"
-           if ($type eq 'misc_RNA') {
+           if ($type eq 'misc_RNA' || $type eq 'mRNA') { 
+            # dgg: see TypeMapper; both pseudo mRNA,misc_RNA should be pseudogenic_transcript
                $sf->primary_tag("pseudotranscript");
            }
            else {
@@ -1302,21 +1339,22 @@ sub unflatten_seq{
 			$_->has_tag($group_tag)} @flat_seq_features);
        my $n_cdss =
 	 scalar(grep {$_->primary_tag eq 'CDS'} @flat_seq_features);
-	   
+       my $n_rnas =
+	 scalar(grep {$_->primary_tag =~ /RNA/} @flat_seq_features);  
        # Are there any CDS features in the record?
-       if ($n_cdss) {
+       if ($n_cdss > 0) {
            # YES
            
 	   # - a pc gene model should contain at the least a CDS
 
            # Are there any mRNA features in the record?
-	   if (!$n_mrnas) {
-               # NO mRNAs
+	   if ($n_mrnas == 0) {
+               # NO mRNAs:
 	       # looks like structure_type == 1
 	       $structure_type = 1;
 	       $need_to_infer_mRNAs = 1;
 	   }
-	   elsif (!$n_mrnas_attached_to_gene) {
+	   elsif ($n_mrnas_attached_to_gene == 0) {
                # $n_mrnas > 0
                # $n_mrnas_attached_to_gene = 0
                #
@@ -1360,6 +1398,9 @@ sub unflatten_seq{
        }
        else {
 	   # this doesn't seem to be any kind of protein coding gene model
+	   if ( $n_rnas > 0 ) {
+	       $need_to_infer_exons = 1;
+	   }
        }
 
        if ($need_to_infer_exons) {
@@ -1398,9 +1439,7 @@ sub unflatten_seq{
    }
    # -
 
-   # >>>>>>>>>                   <<<<<<<<<<<<<
    # --------- FINISHED GROUPING -------------
-   # >>>>>>>>>                   <<<<<<<<<<<<<
 
 
    # TYPE CONTAINMENT HIERARCHY (aka partonomy)
@@ -1476,9 +1515,7 @@ sub unflatten_seq{
    $seq->remove_SeqFeatures;
    $seq->add_SeqFeature(@top_sfs);
 
-   # >>>>>>>>>                       <<<<<<<<<<<<<
    # --------- FINISHED UNFLATTENING -------------
-   # >>>>>>>>>                       <<<<<<<<<<<<<
 
    # lets see if there are any post-unflattening tasks we need to do
 
@@ -2305,11 +2342,9 @@ sub _write_sf {
 sub _write_sf_detail {
     my $self = shift;
     my $sf = shift;
-    if( $self->verbose > 0 ) {
-	printf STDERR "TYPE:%s\n", $sf->primary_tag;
-	my @locs = $sf->location->each_Location;
-	printf STDERR "  %s,%s [%s]\n", $_->start, $_->end, $_->strand foreach @locs;
-    }
+    printf STDERR "TYPE:%s\n", $sf->primary_tag;
+    my @locs = $sf->location->each_Location;
+    printf STDERR "  %s,%s [%s]\n", $_->start, $_->end, $_->strand foreach @locs;
     return;
 }
 
@@ -2447,17 +2482,23 @@ sub feature_from_splitloc{
        $sf->isa("Bio::SeqFeatureI") || $self->throw("$sf NOT A SeqFeatureI");
        $sf->isa("Bio::FeatureHolderI") || $self->throw("$sf NOT A FeatureHolderI");
 
-       # so far, we only infer exons from mRNA
        my $type = $sf->primary_tag;
        next unless $type eq 'mRNA' or $type =~ /RNA/;
 
        # an mRNA from genbank will have a discontinuous location,
        # with each sub-location being equivalent to an exon
        my @locs = $sf->location;
+
        if ($sf->location->isa("Bio::Location::SplitLocationI")) {
            @locs = $sf->location->each_Location;
        }
-       
+
+       if (!@locs) {
+           use Data::Dumper;
+           print Dumper $sf;
+	   $self->throw("ASSERTION ERROR: sf has no location objects");
+       }
+
        # make exons from locations
        my @subsfs =
          map {
@@ -2488,16 +2529,7 @@ sub feature_from_splitloc{
          } @locs;
        
        # PARANOID CHECK
-       my $ok =
-	 $self->_check_order_is_consistent($sf->location->strand,@subsfs);
-       if (!$ok) {
-           use Data::Dumper;
-           print Dumper $sf->location;
-	   printf STDERR "Unordered features [on strand:%s]:\n",
-             $sf->location->strand;
-	   $self->_write_sf_detail($_) foreach @subsfs;
-	   $self->throw("ASSERTION ERROR: inconsistent order");
-       }
+       $self->_check_order_is_consistent($sf->location->strand,@subsfs);
        #----
 
        $sf->location(Bio::Location::Simple->new());
@@ -2606,14 +2638,8 @@ sub infer_mRNA_from_CDS{
                if ($self->verbose > 0) {
                    print "    Inferring mRNA from CDS $cds\n";
                }
-	       my $ok;
-	       $ok =
-		 $self->_check_order_is_consistent($cds->location->strand,$cds->location->each_Location);
-	       if (!$ok) {
-		   $self->_write_sf_detail($cds);
-		   $self->throw("inconsistent order");
-	       }
-
+               $self->_check_order_is_consistent($cds->location->strand,$cds->location->each_Location);
+               
 	       my $loc = Bio::Location::Split->new;
 	       foreach my $cdsexonloc ($cds->location->each_Location) {
 		   my $subloc =
@@ -2631,11 +2657,7 @@ sub infer_mRNA_from_CDS{
                $mrna->seq_id($cds->seq_id) if $cds->seq_id;
                $mrna->source_tag($cds->source_tag) if $cds->source_tag;
 
-	       $ok =
-		 $self->_check_order_is_consistent($mrna->location->strand,$mrna->location->each_Location);
-	       if (!$ok) {
-		   $self->throw("inconsistent order");
-	       }
+               $self->_check_order_is_consistent($mrna->location->strand,$mrna->location->each_Location);
 
                # make the mRNA hold the CDS; no EXPAND option,
                # the CDS cannot be wider than the mRNA
@@ -2697,15 +2719,37 @@ sub remove_types{
 }
 
 
+# _check_order_is_consistent($strand,$ranges) RETURNS BOOL
+#
+# note: the value of this test is moot - there are many valid,
+# if unusual cases where it would flag an anomaly. for example
+# transpliced genes such as mod(mdg4) in dmel on AE003744, and
+# the following spliced gene on NC_001284:
+#
+#     mRNA            complement(join(20571..20717,21692..22086,190740..190761,
+#                     140724..141939,142769..142998))
+#                     /gene="nad5"
+#                     /note="trans-splicing, RNA editing"
+#                     /db_xref="GeneID:814567"
+#
+# note how the exons are not in order
+#  this will flag a level-3 warning, the user of this module
+#  can ignore this and deal appropriately with the resulting
+#  unordered exons
 sub _check_order_is_consistent {
     my $self = shift;
-    my $parent_strand = shift;
+
+    my $parent_strand = shift; # this does nothing..?
     my @ranges = @_;
     return unless @ranges;
+    my $rangestr =
+      join(" ",map{sprintf("[%s,%s]",$_->start,$_->end)} @ranges);
     my $strand = $ranges[0]->strand;
     for (my $i=1; $i<@ranges;$i++) {
 	if ($ranges[$i]->strand != $strand) {
-	    return 1; # mixed ranges - autopass
+            $self->problem(1,"inconsistent strands. Trans-spliced gene? Range: $rangestr");
+	    return 1; 
+            # mixed ranges - autopass
             # some mRNAs have exons on both strands; for
             # example, the dmel mod(mdg4) gene which is
             # trans-spliced (in actual fact two mRNAs)
@@ -2716,7 +2760,9 @@ sub _check_order_is_consistent {
 	my $rangeP = $ranges[$i-1];
 	my $range = $ranges[$i];
 	    if ($rangeP->start > $range->end) {
+                # failed - but still get one more chance..
 		$pass = 0;
+                $self->problem(2,"Ranges not in correct order. Strange ensembl genbank entry? Range: $rangestr");
                 last;
 	    }
     }
@@ -2730,6 +2776,7 @@ sub _check_order_is_consistent {
             my $rangeP = $ranges[$i-1];
             my $range = $ranges[$i];
 	    if ($rangeP->end < $range->start) {
+                $self->problem(3,"inconsistent order. Range: $rangestr");
                 return 0;
 	    }
         }
@@ -2771,3 +2818,4 @@ sub find_best_pairs {
     
 }
 
+1;
