@@ -1,8 +1,10 @@
-# $Id: SeqFeatureI.pm,v 1.60 2005/01/15 23:36:49 jason Exp $
+# $Id: SeqFeatureI.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::SeqFeatureI
 #
-# Cared for by Ewan Birney <birney@sanger.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
 #
@@ -19,28 +21,29 @@ Bio::SeqFeatureI - Abstract interface of a Sequence Feature
     # get a seqfeature somehow, eg, from a Sequence with Features attached
 
     foreach $feat ( $seq->get_SeqFeatures() ) {
-            print "Feature from ", $feat->start, "to ", 
-	          $feat->end, " Primary tag  ", $feat->primary_tag, 
+       print "Feature from ", $feat->start, "to ",
+	       $feat->end, " Primary tag  ", $feat->primary_tag,
 	          ", produced by ", $feat->source_tag(), "\n";
 
-            if( $feat->strand == 0 ) {
-		print "Feature applicable to either strand\n";
-            } else {
-                print "Feature on strand ", $feat->strand,"\n"; # -1,1
-            }
-            print "feature location is ",$feat->start, "..",
-                  $feat->end, " on strand ", $feat->strand, "\n";
-            print "easy utility to print locations in GenBank/EMBL way ",
-                  $feat->location->to_FTstring(), "\n";
+       if( $feat->strand == 0 ) {
+		    print "Feature applicable to either strand\n";
+       } else {
+          print "Feature on strand ", $feat->strand,"\n"; # -1,1
+       }
 
-            foreach $tag ( $feat->get_all_tags() ) {
-		print "Feature has tag ", $tag, " with values, ",
+       print "feature location is ",$feat->start, "..",
+          $feat->end, " on strand ", $feat->strand, "\n";
+       print "easy utility to print locations in GenBank/EMBL way ",
+          $feat->location->to_FTstring(), "\n";
+
+       foreach $tag ( $feat->get_all_tags() ) {
+		    print "Feature has tag ", $tag, " with values, ",
 		      join(' ',$feat->get_tag_values($tag)), "\n";
-            }
+       }
 	    print "new feature\n" if $feat->has_tag('new');
 	    # features can have sub features
 	    my @subfeat = $feat->get_SeqFeatures();
-	}
+	 }
 
 =head1 DESCRIPTION
 
@@ -57,8 +60,19 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org          - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
@@ -66,7 +80,7 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 APPENDIX
 
@@ -80,22 +94,19 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::SeqFeatureI;
-use vars qw(@ISA $HasInMemory);
+use vars qw($HasInMemory);
 use strict;
-
 BEGIN {
     eval { require Bio::DB::InMemoryCache };
     if( $@ ) { $HasInMemory = 0 }
     else { $HasInMemory = 1 }
 }
 
-use Bio::AnnotatableI;
-use Bio::RangeI;
 use Bio::Seq;
 
 use Carp;
 
-@ISA = qw(Bio::RangeI Bio::AnnotatableI);
+use base qw(Bio::RangeI);
 
 =head1 Bio::SeqFeatureI specific methods
 
@@ -129,7 +140,7 @@ sub get_SeqFeatures{
 
 =cut
 
-sub display_name { 
+sub display_name {
     shift->throw_not_implemented();
 }
 
@@ -139,7 +150,7 @@ sub display_name {
  Usage   : $tag = $feat->primary_tag()
  Function: Returns the primary tag for a feature,
            eg 'exon'
- Returns : a string 
+ Returns : a string
  Args    : none
 
 
@@ -157,8 +168,8 @@ sub primary_tag{
  Title   : source_tag
  Usage   : $tag = $feat->source_tag()
  Function: Returns the source tag for a feature,
-           eg, 'genscan' 
- Returns : a string 
+           eg, 'genscan'
+ Returns : a string
  Args    : none
 
 
@@ -170,6 +181,81 @@ sub source_tag{
    $self->throw_not_implemented();
 }
 
+=head2 has_tag
+
+ Title   : has_tag
+ Usage   : $tag_exists = $self->has_tag('some_tag')
+ Function: 
+ Returns : TRUE if the specified tag exists, and FALSE otherwise
+ Args    :
+
+=cut
+
+sub has_tag{
+   my ($self,@args) = @_;
+
+   $self->throw_not_implemented();
+
+}
+
+=head2 get_tag_values
+
+ Title   : get_tag_values
+ Usage   : @values = $self->get_tag_values('some_tag')
+ Function: 
+ Returns : An array comprising the values of the specified tag.
+ Args    : a string
+
+throws an exception if there is no such tag
+
+=cut
+
+sub get_tag_values {
+    shift->throw_not_implemented();
+}
+
+=head2 get_tagset_values
+
+ Title   : get_tagset_values
+ Usage   : @values = $self->get_tagset_values(qw(label transcript_id product))
+ Function: 
+ Returns : An array comprising the values of the specified tags, in order of tags
+ Args    : An array of strings
+
+does NOT throw an exception if none of the tags are not present
+
+this method is useful for getting a human-readable label for a
+SeqFeatureI; not all tags can be assumed to be present, so a list of
+possible tags in preferential order is provided
+
+=cut
+
+# interface + abstract method
+sub get_tagset_values {
+    my ($self, @args) = @_;
+    my @vals = ();
+    foreach my $arg (@args) {
+        if ($self->has_tag($arg)) {
+            push(@vals, $self->get_tag_values($arg));
+        }
+    }
+    return @vals;
+}
+
+=head2 get_all_tags
+
+ Title   : get_all_tags
+ Usage   : @tags = $feat->get_all_tags()
+ Function: gives all tags for this feature
+ Returns : an array of strings
+ Args    : none
+
+
+=cut
+
+sub get_all_tags{
+    shift->throw_not_implemented();
+}
 
 =head2 attach_seq
 
@@ -182,13 +268,13 @@ sub source_tag{
            Note that it is not guaranteed that if you obtain a feature from
            an object in bioperl, it will have a sequence attached. Also,
            implementors of this interface can choose to provide an empty
-           implementation of this method. I.e., there is also no guarantee 
+           implementation of this method. I.e., there is also no guarantee
            that if you do attach a sequence, seq() or entire_seq() will not
            return undef.
 
            The reason that this method is here on the interface is to enable
            you to call it on every SeqFeatureI compliant object, and
-           that it will be implemented in a useful way and set to a useful 
+           that it will be implemented in a useful way and set to a useful
            value for the great majority of use cases. Implementors who choose
            to ignore the call are encouraged to specifically state this in
            their documentation.
@@ -208,7 +294,7 @@ sub attach_seq {
 
  Title   : seq
  Usage   : $tseq = $sf->seq()
- Function: returns the truncated sequence (if there is a sequence attached) 
+ Function: returns the truncated sequence (if there is a sequence attached)
            for this feature
  Example :
  Returns : sub seq (a Bio::PrimarySeqI compliant object) on attached sequence
@@ -298,7 +384,7 @@ my $static_gff_formatter = undef;
  Usage   :
  Function:
  Example :
- Returns : 
+ Returns :
  Args    :
 
 
@@ -306,7 +392,7 @@ my $static_gff_formatter = undef;
 
 sub _static_gff_formatter{
    my ($self,@args) = @_;
-
+   require Bio::Tools::GFF; # on the fly inclusion -- is this better?
    if( !defined $static_gff_formatter ) {
        $static_gff_formatter = Bio::Tools::GFF->new('-gff_version' => 2);
    }
@@ -346,21 +432,70 @@ but can be validly overwritten by subclasses
             Implementing classes are free to override this method with their
             own magic if they have a better idea what the user wants.
 
-  Args    : [optional] A L<Bio::DB::RandomAccessI> compliant object if 
-                       one needs to retrieve remote seqs. 
-            [optional] boolean if the locations should not be sorted 
-                       by start location.
+  Args    : [optional]
+            -db        A L<Bio::DB::RandomAccessI> compliant object if
+                       one needs to retrieve remote seqs.
+            -nosort    boolean if the locations should not be sorted
+                       by start location.  This may occur, for instance,
+                       in a circular sequence where a gene span starts
+                       before the end of the sequence and ends after the
+                       sequence start. Example : join(15685..16260,1..207)
+					   (default = if sequence is_circular(), 1, otherwise 0)
+			-phase     truncates the returned sequence based on the
+					   intron phase (0,1,2).
+
   Returns : A L<Bio::PrimarySeqI> object
 
 =cut
 
 sub spliced_seq {
-    my ($self,$db,$nosort) = @_;
-   
-    if( ! $self->location->isa("Bio::Location::SplitLocationI") ) {
-	return $self->seq(); # nice and easy!
+    my $self = shift;
+	my @args = @_;
+	my ($db, $nosort, $phase) =
+	   $self->_rearrange([qw(DB NOSORT PHASE)], @args);
+	
+	# set no_sort based on the parent sequence status
+	if ($self->entire_seq->is_circular) {
+		$nosort = 1;
+	}
+	
+	# (added 7/7/06 to allow use old API (with warnings)
+	my $old_api = (!(grep {$_ =~ /(?:nosort|db|phase)/} @args)) ? 1 : 0;
+	if (@args && $old_api) {
+		$self->warn(q(API has changed; please use '-db' or '-nosort' ).
+                     qq(for args. See POD for more details.));
+		$db = shift @args if @args;
+		$nosort = shift @args if @args;
+		$phase = shift @args if @args;
+	};
+	
+	if (defined($phase) && ($phase < 0 || $phase > 2)) {
+	    $self->warn("Phase must be 0,1, or 2.  Setting phase to 0...");
+	    $phase = 0;
+	}
+	
+	if( $db && ref($db) && ! $db->isa('Bio::DB::RandomAccessI') ) {
+        $self->warn("Must pass in a valid Bio::DB::RandomAccessI object".
+                    " for access to remote locations for spliced_seq");
+        $db = undef;
+    } elsif( defined $db && $HasInMemory &&
+            $db->isa('Bio::DB::InMemoryCache') ) {
+        $db = Bio::DB::InMemoryCache->new(-seqdb => $db);
     }
 
+    if( ! $self->location->isa("Bio::Location::SplitLocationI") ) {
+        if ($phase) {
+	    $self->debug("Subseq start: ",$phase+1,"\tend: ",$self->end,"\n");
+	    my $seqstr = substr($self->seq->seq, $phase);
+	    my $out = Bio::Seq->new( -id => $self->entire_seq->display_id
+				. "_spliced_feat",
+			 -seq => $seqstr);
+	    return $out;
+	} else {
+	    return $self->seq(); # nice and easy!
+	}
+    }
+    
     # redundant test, but the above ISA is probably not ideal.
     if( ! $self->location->isa("Bio::Location::SplitLocationI") ) {
 	$self->throw("not atomic, not split, yikes, in trouble!");
@@ -372,22 +507,16 @@ sub spliced_seq {
     # so we are really sorting features 5' -> 3' on their strand
     # i.e. rev strand features will be sorted largest to smallest
     # as this how revcom CDSes seem to be annotated in genbank.
-    # Might need to eventually allow this to be programable?    
+    # Might need to eventually allow this to be programable?
     # (can I mention how much fun this is NOT! --jason)
-    
+
     my ($mixed,$mixedloc, $fstrand) = (0);
-    if( $db && ref($db) && ! $db->isa('Bio::DB::RandomAccessI') ) {
-	$self->warn("Must pass in a valid Bio::DB::RandomAccessI object for access to remote locations for spliced_seq");
-	$db = undef;
-    } elsif( defined $db && $HasInMemory && 
-	     ! $db->isa('Bio::DB::InMemoryCache') ) {
-	$db = new Bio::DB::InMemoryCache(-seqdb => $db);
-    }
+
     if( $self->isa('Bio::Das::SegmentI') &&
-	! $self->absolute ) { 
+	! $self->absolute ) {
 	$self->warn("Calling spliced_seq with a Bio::Das::SegmentI which does have absolute set to 1 -- be warned you may not be getting things on the correct strand");
     }
-    
+
     my @locset = $self->location->each_Location;
     my @locs;
     if( ! $nosort ) {
@@ -397,24 +526,24 @@ sub spliced_seq {
 	# rather than they way most are input which is on the fwd strand
 
 	sort { $a->[1] <=> $b->[1] } # Yes Tim, Schwartzian transformation
-	map { 
+	map {
 	    $fstrand = $_->strand unless defined $fstrand;
 	    $mixed = 1 if defined $_->strand && $fstrand != $_->strand;
 	    if( defined $_->seq_id ) {
 		$mixedloc = 1 if( $_->seq_id ne $seqid );
 	    }
-	    [ $_, $_->start * ($_->strand || 1)];	    
-	} @locset; 
+	    [ $_, $_->start * ($_->strand || 1)];
+	} @locset;
 
-	if ( $mixed ) { 
-	    $self->warn("Mixed strand locations, spliced seq using the input order rather than trying to sort");    
+	if ( $mixed ) {
+	    $self->warn("Mixed strand locations, spliced seq using the input order rather than trying to sort");
 	    @locs = @locset;
 	}
-    } else { 
+    } else {
 	# use the original order instead of trying to sort
 	@locs = @locset;
 	$fstrand = $locs[0]->strand;
-    } 
+    }
 
     foreach my $loc ( @locs ) {
 	if( ! $loc->isa("Bio::Location::Atomic") ) {
@@ -426,7 +555,7 @@ sub spliced_seq {
 	}
 	# deal with remote sequences
 
-	if( defined $loc->seq_id && 
+	if( defined $loc->seq_id &&
 	    $loc->seq_id ne $seqid ) {
 	    if( defined $db ) {
 		my $sid = $loc->seq_id;
@@ -449,11 +578,26 @@ sub spliced_seq {
 	} else {
 	    $called_seq = $self->entire_seq;
 	}
-	
+
+    # does the called sequence make sense? Bug 1780
+    if ($called_seq->length < $loc->end) {
+        my $accession = $called_seq->accession;
+        my $end = $loc->end;
+        my $length = $called_seq->length;
+        my $orig_id = $self->seq_id; # originating sequence
+        my ($locus) = $self->get_tagset_values("locus_tag");
+        $self->throw("Location end ($end) exceeds length ($length) of ".
+                     "called sequence $accession.\nCheck sequence version used in ".
+                     "$locus locus-tagged SeqFeature in $orig_id.");
+    }
+
 	if( $self->isa('Bio::Das::SegmentI') ) {
-	    my ($s,$e) = ($loc->start,$loc->end);	    
-	    $seqstr .= $called_seq->subseq($s,$e)->seq();
-	} else { 
+	    my ($s,$e) = ($loc->start,$loc->end);
+		# $called_seq is Bio::DB::GFF::RelSegment, as well as its subseq();
+		# Bio::DB::GFF::RelSegment::seq() returns a Bio::PrimarySeq, and using seq()
+		# in turn returns a string.  Confused?
+	    $seqstr .= $called_seq->subseq($s,$e)->seq()->seq();
+	} else {
 	    # This is dumb, subseq should work on locations...
 	    if( $loc->strand == 1 ) {
 		$seqstr .= $called_seq->subseq($loc->start,$loc->end);
@@ -466,10 +610,15 @@ sub spliced_seq {
 	    }
 	}
     }
-    my $out = Bio::Seq->new( -id => $self->entire_seq->display_id 
+    
+    if (defined($phase)) {
+	$seqstr = substr($seqstr, $phase);
+    }
+    
+    my $out = Bio::Seq->new( -id => $self->entire_seq->display_id
 			            . "_spliced_feat",
 			     -seq => $seqstr);
-    
+
     return $out;
 }
 
@@ -477,8 +626,8 @@ sub spliced_seq {
 
  Title   : location
  Usage   : my $location = $seqfeature->location()
- Function: returns a location object suitable for identifying location 
-	   of feature on sequence or parent feature  
+ Function: returns a location object suitable for identifying location
+	   of feature on sequence or parent feature
  Returns : Bio::LocationI object
  Args    : none
 
@@ -496,8 +645,8 @@ sub location {
 
  Title   : primary_id
  Usage   : $obj->primary_id($newval)
- Function: 
- Example : 
+ Function:
+ Example :
  Returns : value of primary_id (a scalar)
  Args    : on set, new value (a scalar or undef, optional)
 
@@ -530,14 +679,14 @@ sub primary_id{
 sub generate_unique_persistent_id {
     # DEPRECATED - us IDHandler
     my $self = shift;
-    require "Bio/SeqFeature/Tools/IDHandler.pm";
+    require Bio::SeqFeature::Tools::IDHandler;
     Bio::SeqFeature::Tools::IDHandler->new->generate_unique_persistent_id($self);
 }
 
 =head1 Bio::RangeI methods
 
 These methods are inherited from RangeI and can be used
-directly from a SeqFeatureI interface. Remember that a 
+directly from a SeqFeatureI interface. Remember that a
 SeqFeature is-a RangeI, and so wherever you see RangeI you
 can use a feature ($r in the below documentation).
 
@@ -575,34 +724,6 @@ can use a feature ($r in the below documentation).
 
  See L<Bio::RangeI>
 
-=head1 Bio::AnnotatableI methods
-
-=cut
-
-=head2 has_tag()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
-=head2 remove_tag()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
-=head2 add_tag_value()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
-=head2 get_tag_values()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
-=head2 get_tagset_values()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
-=head2 get_all_tags()
-
- B<Deprecated>.  See L<Bio::AnnotatableI>
-
 =cut
 
 1;
