@@ -1,9 +1,10 @@
-
-# $Id: SeqIO.pm,v 1.80 2005/01/12 14:07:36 bosborne Exp $
+# $Id: SeqIO.pm 16090 2009-09-15 21:57:56Z cjfields $
 #
 # BioPerl module for Bio::SeqIO
 #
-# Cared for by Ewan Birney <birney@sanger.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Ewan Birney <birney@ebi.ac.uk>
 #       and Lincoln Stein  <lstein@cshl.org>
 #
 # Copyright Ewan Birney
@@ -23,9 +24,9 @@ Bio::SeqIO - Handler for SeqIO Formats
 
     use Bio::SeqIO;
 
-    $in  = Bio::SeqIO->new(-file => "inputfilename" , 
+    $in  = Bio::SeqIO->new(-file => "inputfilename" ,
                            -format => 'Fasta');
-    $out = Bio::SeqIO->new(-file => ">outputfilename" , 
+    $out = Bio::SeqIO->new(-file => ">outputfilename" ,
                            -format => 'EMBL');
 
     while ( my $seq = $in->next_seq() ) {
@@ -37,7 +38,7 @@ Bio::SeqIO - Handler for SeqIO Formats
 
     use Bio::SeqIO;
 
-    $in  = Bio::SeqIO->new(-file => "inputfilename" , 
+    $in  = Bio::SeqIO->new(-file => "inputfilename" ,
                            -format => 'genbank');
 
     while ( my $seq = $in->next_seq() ) {
@@ -47,8 +48,8 @@ Bio::SeqIO - Handler for SeqIO Formats
 
 
   # The SeqIO system does have a filehandle binding. Most people find this
-  # a little confusing, but it does mean you write the world's smallest
-  # reformatter
+  # a little confusing, but it does mean you can write the world's
+  # smallest reformatter
 
     use Bio::SeqIO;
 
@@ -68,7 +69,7 @@ the format objects, which most people should use.
 
 The Bio::SeqIO system can be thought of like biological file handles.
 They are attached to filehandles with smart formatting rules (eg,
-genbank format, or EMBL format, or binary trace file format) and 
+genbank format, or EMBL format, or binary trace file format) and
 can either read or write sequence objects (Bio::Seq objects, or
 more correctly, Bio::SeqI implementing objects, of which Bio::Seq is
 one such object). If you want to know what to do with a Bio::Seq
@@ -89,8 +90,8 @@ and
    $stream->write_seq($seq);
 
 As an added bonus, you can recover a filehandle that is tied to the
-SeqIO object, allowing you to use the standard E<lt>E<gt> and print operations
-to read and write sequence objects:
+SeqIO object, allowing you to use the standard E<lt>E<gt> and print
+operations to read and write sequence objects:
 
     use Bio::SeqIO;
 
@@ -111,13 +112,14 @@ This makes the simplest ever reformatter
     #!/usr/bin/perl
 
     $format1 = shift;
-    $format2 = shift || die "Usage: reformat format1 format2 < input > output";
+    $format2 = shift || die
+       "Usage: reformat format1 format2 < input > output";
 
     use Bio::SeqIO;
 
     $in  = Bio::SeqIO->newFh(-format => $format1, -fh => \*ARGV );
     $out = Bio::SeqIO->newFh(-format => $format2 );
-    # Note: you might want to quote -format to keep older 
+    # Note: you might want to quote -format to keep older
     # perl's from complaining.
 
     print $out $_ while <$in>;
@@ -170,12 +172,12 @@ some HTML tags:
   use Bio::SeqIO;
   use IO::String;
   my $in  = Bio::SeqIO->new(-file => "emblfile",
-  			                   -format => 'EMBL');
+  		            -format => 'EMBL');
   while ( my $seq = $in->next_seq() ) {
       # the output handle is reset for every file
       my $stringio = IO::String->new($string);
       my $out = Bio::SeqIO->new(-fh => $stringio,
-  			                       -format => 'fasta');
+  			        -format => 'fasta');
       # output goes into $string
       $out->write_seq($seq);
       # modify $string
@@ -187,15 +189,15 @@ some HTML tags:
 =item -format
 
 Specify the format of the file.  Supported formats include fasta,
-genbank, embl, swiss (SwissProt), and tracefile formats such as
-abi (ABI) and scf. There are many more, for a complete listing see
-the SeqIO HOWTO (http://bioperl.org/HOWTOs/SeqIO/index.html).
+genbank, embl, swiss (SwissProt), Entrez Gene and tracefile formats
+such as abi (ABI) and scf. There are many more, for a complete listing
+see the SeqIO HOWTO (L<http://bioperl.open-bio.org/wiki/HOWTO:SeqIO>).
 
-If no format is specified and a filename is given then the module
-will attempt to deduce the format from the filename suffix. If
-there is no suffix that Bioperl understands then it will attempt
-to guess the format based on file content. If this is unsuccessful 
-then Fasta format is assumed.
+If no format is specified and a filename is given then the module will
+attempt to deduce the format from the filename suffix. If there is no
+suffix that Bioperl understands then it will attempt to guess the
+format based on file content. If this is unsuccessful then SeqIO will 
+throw a fatal error.
 
 The format name is case-insensitive: 'FASTA', 'Fasta' and 'fasta' are
 all valid.
@@ -215,16 +217,16 @@ may be important because Bioperl does not always guess correctly.
 
 By default, all files (or filehandles) opened for writing sequences
 will be flushed after each write_seq() (making the file immediately
-usable).  If you don't need this facility and would like to marginally
+usable).  If you do not need this facility and would like to marginally
 improve the efficiency of writing multiple sequences to the same file
 (or filehandle), pass the -flush option '0' or any other value that
 evaluates as defined but false:
 
-  my $gb = new Bio::SeqIO -file   => "<gball.gbk",
-                          -format => "gb";
-  my $fa = new Bio::SeqIO -file   => ">gball.fa",
-                          -format => "fasta",
-                          -flush  => 0; # go as fast as we can!
+  my $gb = Bio::SeqIO->new(-file   => "<gball.gbk",
+                           -format => "gb");
+  my $fa = Bio::SeqIO->new(-file   => ">gball.fa",
+                           -format => "fasta",
+                           -flush  => 0); # go as fast as we can!
   while($seq = $gb->next_seq) { $fa->write_seq($seq) }
 
 
@@ -244,8 +246,8 @@ read all sequence objects into an array like this:
 
   @sequences = <$fh>;
 
-Other operations, such as read(), sysread(), write(), close(), and printf() 
-are not supported.
+Other operations, such as read(), sysread(), write(), close(), and
+printf() are not supported.
 
 =head1 OBJECT METHODS
 
@@ -253,7 +255,7 @@ See below for more detailed summaries.  The main methods are:
 
 =head2 $sequence = $seqIO-E<gt>next_seq()
 
-Fetch the next sequence from the stream.
+Fetch the next sequence from the stream, or nothing if no more.
 
 =head2 $seqIO-E<gt>write_seq($sequence [,$another_sequence,...])
 
@@ -267,26 +269,38 @@ These provide the tie interface.  See L<perltie> for more details.
 
 =head2 Mailing Lists
 
-User feedback is an integral part of the evolution of this
-and other Bioperl modules. Send your comments and suggestions preferably
-to one of the Bioperl mailing lists.
+User feedback is an integral part of the evolution of this and other
+Bioperl modules. Send your comments and suggestions preferably to one
+of the Bioperl mailing lists.
 
 Your participation is much appreciated.
 
   bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/MailList.shtml      - About the mailing lists
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+ bioperl-l@bioperl.org
+
+rather than to the module maintainer directly. Many experienced and 
+responsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.
- Bug reports can be submitted via the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney, Lincoln Stein
 
 Email birney@ebi.ac.uk
+      lstein@cshl.org
 
 =head1 APPENDIX
 
@@ -300,28 +314,20 @@ methods. Internal methods are usually preceded with a _
 package Bio::SeqIO;
 
 use strict;
-use vars qw(@ISA);
 
-use Bio::Root::Root;
-use Bio::Root::IO;
-use Bio::Factory::SequenceStreamI;
 use Bio::Factory::FTLocationFactory;
 use Bio::Seq::SeqBuilder;
 use Bio::Tools::GuessSeqFormat;
-use Symbol();
+use Symbol;
 
-@ISA = qw(Bio::Root::Root Bio::Root::IO Bio::Factory::SequenceStreamI);
-
-sub BEGIN {
-    eval { require Bio::SeqIO::staden::read; };
-}
+use base qw(Bio::Root::Root Bio::Root::IO Bio::Factory::SequenceStreamI);
 
 my %valid_alphabet_cache;
 
 =head2 new
 
  Title   : new
- Usage   : $stream = Bio::SeqIO->new(-file => $filename, 
+ Usage   : $stream = Bio::SeqIO->new(-file => $filename,
                                      -format => 'Format')
  Function: Returns a new sequence stream
  Returns : A Bio::SeqIO stream initialised with the appropriate format
@@ -350,13 +356,13 @@ sub new {
 	# or do we want to call SUPER on an object if $caller is an
 	# object?
 	if( $class =~ /Bio::SeqIO::(\S+)/ ) {
-		my ($self) = $class->SUPER::new(@args);	
+		my ($self) = $class->SUPER::new(@args);
 		$self->_initialize(@args);
 		return $self;
 	} else {
 
 		my %param = @args;
-	@param{ map { lc $_ } keys %param } = values %param; # lowercase keys
+		@param{ map { lc $_ } keys %param } = values %param; # lowercase keys
 
 	if (!defined($param{-file}) && !defined($param{-fh})) {
 	  $class->throw("file argument provided, but with an undefined value") if exists($param{'-file'});
@@ -365,8 +371,8 @@ sub new {
 
 	my $format = $param{'-format'} ||
 	    $class->_guess_format( $param{-file} || $ARGV[0] );
-
-	if( ! $format ) { 
+        
+	if( ! $format ) {
 	    if ($param{-file}) {
 		$format = Bio::Tools::GuessSeqFormat->new(-file => $param{-file}||$ARGV[0] )->guess;
 	    } elsif ($param{-fh}) {
@@ -374,9 +380,13 @@ sub new {
 	    }
 	}
 	$format = "\L$format";	# normalize capitalization to lower case
-        $class->throw("Unknown format given or could not determine it [$format]")
-            unless $format;
-	return undef unless( $class->_load_format_module($format) );
+    $class->throw("Unknown format given or could not determine it [$format]")
+        unless $format;
+    if ($format =~ /-/) {
+        ($format, my $variant) = split('-', $format, 2);
+        push @args, (-variant => $variant);
+    }
+	return unless( $class->_load_format_module($format) );
 	return "Bio::SeqIO::$format"->new(@args);
     }
 }
@@ -431,21 +441,27 @@ sub _initialize {
 
 	# flush is initialized by the Root::IO init
 
-	my ($seqfact,$locfact,$objbuilder) =
+	my ($seqfact,$locfact,$objbuilder, $alphabet) =
 	  $self->_rearrange([qw(SEQFACTORY
 			      LOCFACTORY
-			      OBJBUILDER)
+			      OBJBUILDER
+                  ALPHABET)
 							  ], @args);
 
-	$locfact = Bio::Factory::FTLocationFactory->new(-verbose => $self->verbose) 
+	$locfact = Bio::Factory::FTLocationFactory->new(-verbose => $self->verbose)
 	  if ! $locfact;
-	$objbuilder = Bio::Seq::SeqBuilder->new(-verbose => $self->verbose) 
+	$objbuilder = Bio::Seq::SeqBuilder->new(-verbose => $self->verbose)
 	  unless $objbuilder;
 	$self->sequence_builder($objbuilder);
 	$self->location_factory($locfact);
+
 	# note that this should come last because it propagates the sequence
 	# factory to the sequence builder
 	$seqfact && $self->sequence_factory($seqfact);
+        
+    #bug 2160
+    $alphabet && $self->alphabet($alphabet);
+
 
 	# initialize the IO part
 	$self->_initialize_io(@args);
@@ -469,7 +485,9 @@ sub _initialize {
            recoverable errors into exceptions by calling
            $stream->verbose(2).
 
- Returns : a Bio::Seq sequence object
+ Returns : a Bio::Seq sequence object, or nothing if no more sequences
+           are available
+
  Args    : none
 
 See L<Bio::Root::RootI>, L<Bio::Factory::SeqStreamI>, L<Bio::Seq>
@@ -594,7 +612,6 @@ sub _concatenate_lines {
  Returns : value of _filehandle
  Args    : newvalue (optional)
 
-
 =cut
 
 sub _filehandle {
@@ -619,24 +636,31 @@ sub _filehandle {
 sub _guess_format {
    my $class = shift;
    return unless $_ = shift;
-   return 'fasta'   if /\.(fasta|fast|fas|seq|fa|fsa|nt|aa)$/i;
-   return 'genbank' if /\.(gb|gbank|genbank|gbk|gbs)$/i;
-   return 'scf'     if /\.scf$/i;
    return 'abi'     if /\.ab[i1]$/i;
-   return 'alf'     if /\.alf$/i;
-   return 'ctf'     if /\.ctf$/i;
-   return 'ztr'     if /\.ztr$/i;
-   return 'pln'     if /\.pln$/i;
-   return 'exp'     if /\.exp$/i;
-   return 'pir'     if /\.pir$/i;
-   return 'embl'    if /\.(embl|ebl|emb|dat)$/i;
-   return 'raw'     if /\.(txt)$/i;
-   return 'gcg'     if /\.gcg$/i;
    return 'ace'     if /\.ace$/i;
+   return 'alf'     if /\.alf$/i;
    return 'bsml'    if /\.(bsm|bsml)$/i;
-   return 'swiss'   if /\.(swiss|sp)$/i;
-   return 'phd'     if /\.(phd|phred)$/i;
+   return 'ctf'     if /\.ctf$/i;
+   return 'embl'    if /\.(embl|ebl|emb|dat)$/i;
+	return 'entrezgene' if /\.asn$/i;
+   return 'exp'     if /\.exp$/i;
+   return 'fasta'   if /\.(fasta|fast|fas|seq|fa|fsa|nt|aa|fna|faa)$/i;
    return 'fastq'   if /\.fastq$/i;
+   return 'gcg'     if /\.gcg$/i;
+   return 'genbank' if /\.(gb|gbank|genbank|gbk|gbs)$/i;
+   return 'phd'     if /\.(phd|phred)$/i;
+   return 'pir'     if /\.pir$/i;
+   return 'pln'     if /\.pln$/i;
+   return 'raw'     if /\.(txt)$/i;
+   return 'scf'     if /\.scf$/i;
+   return 'swiss'   if /\.(swiss|sp)$/i;
+
+   # from Strider 1.4 Release Notes: The file name extensions used by
+   # Strider 1.4 are ".xdna", ".xdgn", ".xrna" and ".xprt" for DNA,
+   # DNA Degenerate, RNA and Protein Sequence Files, respectively
+   return 'strider' if /\.(xdna|xdgn|xrna|xprt)$/i;
+
+   return 'ztr'     if /\.ztr$/i;
 }
 
 sub DESTROY {
@@ -670,7 +694,6 @@ sub PRINT {
  Returns : Bio::Factory::SequenceFactoryI
  Args    : [optional] Bio::Factory::SequenceFactoryI
 
-
 =cut
 
 sub sequence_factory{
@@ -694,11 +717,10 @@ sub sequence_factory{
  Title   : object_factory
  Usage   : $obj->object_factory($newval)
  Function: This is an alias to sequence_factory with a more generic name.
- Example : 
+ Example :
  Returns : value of object_factory (a scalar)
  Args    : on set, new value (a scalar or undef, optional)
 
-
 =cut
 
 sub object_factory{
@@ -719,7 +741,6 @@ sub object_factory{
  Returns : a Bio::Factory::ObjectBuilderI compliant object
  Args    : [optional] a Bio::Factory::ObjectBuilderI compliant object
 
-
 =cut
 
 sub sequence_builder{
@@ -743,7 +764,6 @@ sub sequence_builder{
  Args    : [optional] on set, a Bio::Factory::LocationFactoryI implementing
            object.
 
-
 =cut
 
 sub location_factory{
