@@ -7,9 +7,10 @@
 # POD documentation - main docs before the code
 #
 # The original version of the module can be found here:
-# http://www.lifecde.com/products/agave/agave.pm
+# http://www.agavexml.org/
 #
-# The DTD for AGAVE XML can be located here:
+# ### TODO: live link for this anymore?
+# The DTD for AGAVE XML was once located here (dead link):
 # http://www.lifecde.com/products/agave/schema/v2_3/agave.dtd
 #
 #
@@ -53,17 +54,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.
-Bug reports can be submitted via email or the web:
+Bug reports can be submitted via the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Simon K. Chan
 
@@ -81,15 +92,11 @@ methods. Internal methods are usually preceded with a _
 
 # Let the code begin...
 package Bio::SeqIO::agave;
-use vars qw(@ISA);
 use strict;
-# Object preamble - inherits from Bio::Root::Object
 
 use IO::File;
 
 
-use lib '/home/skchan/checkout/bioperl-live';
-use Bio::SeqIO;
 use Bio::SeqFeature::Generic;
 use Bio::Seq;
 use Bio::PrimarySeq;
@@ -97,12 +104,11 @@ use Bio::Seq::SeqFactory;
 use Bio::Annotation::Reference;
 use Bio::Species;
 
-use lib '/home/skchan/gq/BIO_SUPPORT/XML-Writer/XML-Writer-0.510/blib/lib';
 use XML::Writer;
 
 use Data::Dumper;
 
-@ISA = qw(Bio::SeqIO);
+use base qw(Bio::SeqIO);
 
 # ==================================================================================
 sub _initialize {
@@ -725,7 +731,7 @@ sub _process_seq_feature {
 
     } else {
 
-        die "Error.  Missing </seq_feature> tag.  Got this: $$line\n";
+        $self->throw("Error.  Missing </seq_feature> tag.  Got this: $$line\n");
 
     }
 
@@ -886,7 +892,7 @@ sub _process_related_annot { # NOT IN USE.
         }
 
         if ($element_id_count == 0) {
-            die "Error.  Missing <element_id> tag.  Got: $$line";
+            $self->throw("Error.  Missing <element_id> tag.  Got: $$line");
         }
 
         # Zero or more <sci_property>
@@ -898,7 +904,7 @@ sub _process_related_annot { # NOT IN USE.
         push @{$data_structure->{'related_annot'}}, $related_annot;
 
         unless ($$line =~ /<\/related_annot>/){
-            die "Error.  Missing </related_tag>. Got: $$line\n";
+            $self->throw("Error.  Missing </related_tag>. Got: $$line\n");
         }
 
     }
@@ -924,7 +930,7 @@ sub _process_result_group { # NOT IN USE.
             ++$count;
         }
 
-        die "Error.  No <comp_result></comp_result> tag! Got this: $$line"
+        $self->throw("Error.  No <comp_result></comp_result> tag! Got this: $$line")
             if $count == 0;
 
         # in the last iteration in the inner while loop, $line will
@@ -932,7 +938,7 @@ sub _process_result_group { # NOT IN USE.
         if ($line =~ /<\/result_group>/) {
             $$line = $self->_readline;
         } else {
-            die "Error.  No </result_tag>!  Got this: $$line";
+            $self->throw("Error.  No </result_tag>!  Got this: $$line");
         }
 
 
@@ -967,7 +973,7 @@ sub _process_match_region { # NOT IN USE.
             $$line = $self->_readline; # get the next line to be _processed by the next sub
             return;
         } else {
-            die "No closing tag </match_region>!  Got this: $$line\n";
+            $self->throw("No closing tag </match_region>!  Got this: $$line\n");
         }
 
     }
@@ -990,7 +996,7 @@ sub _process_query_region { # NOT IN USE.
             $$line = $self->_readline; # get the next line to _process.
             return;
         } else {
-            die "No closing tag </query_region>.  Got this: $$line\n";
+            $self->throw("No closing tag </query_region>.  Got this: $$line\n");
         }
 
     }
