@@ -1,10 +1,10 @@
-# $Id: gameWriter.pm,v 1.7 2004/04/17 09:31:09 heikki Exp $
+# $Id: gameWriter.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::SeqIO::game::gameWriter
 #
-# Cared for by Sheldon McKay <smckay@bcgsc.bc.ca>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
 #
-# Copyright Sheldon McKay
+# Cared for by Sheldon McKay <mckays@cshl.edu>
 #
 # You may distribute this module under the same terms as perl itself
 #
@@ -47,21 +47,30 @@ of the Bioperl mailing lists.
 Your participation is much appreciated.
 
   bioperl-l@bioperl.org                  - General discussion
-  http://bioperl.org/MailList.shtml      - About the mailing lists
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-of the bugs and their resolution.
+of the bugs and their resolution. Bug reports can be submitted via the
+web:
 
-Bug reports can be submitted via email or the web:
-
-  bioperl-bugs@bioperl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Sheldon McKay
 
-Email smckay@bcgsc.bc.ca
+Email mckays@cshl.edu
 
 =head1 APPENDIX
 
@@ -76,11 +85,9 @@ use strict;
 use IO::String;
 use XML::Writer;
 use Bio::SeqFeature::Generic;
-use Bio::SeqIO::game::gameSubs;
 use Bio::SeqFeature::Tools::Unflattener;
 
-use vars '@ISA';
-@ISA = qw/Bio::SeqIO::game::gameSubs/;
+use base qw(Bio::SeqIO::game::gameSubs);
 
 =head2 new
 
@@ -106,6 +113,7 @@ sub new {
 
     # make a <map_position> element only if requested 
     $self->{map} = 1 if $arg{map};
+    $self->{anon_set_counters} = {}; #counters for numbering anonymous result and feature sets
     return $self;
 }
 
@@ -167,7 +175,7 @@ sub write_to_game {
     $uf->unflatten_seq( -seq => $seq, use_magic => 1 );
     
     # rearrange snRNA and transposon hierarchies
-    # $self->_rearrange($seq, @gene_containers);
+    # $self->_rearrange_hierarchies($seq, @gene_containers);
 
     # add back nested feats
     $seq->add_SeqFeature( @nested_feats  );
@@ -183,11 +191,11 @@ sub write_to_game {
 				  NEWLINE     => 1
 				  );
     $self->{writer} = $writer;
-    $writer->xmlDecl("UTF-8");
-    $writer->doctype("game", 'game', "http://www.fruitfly.org/annot/gamexml.dtd.txt");
+#    $writer->xmlDecl("UTF-8");
+#    $writer->doctype("game", 'game', "http://www.fruitfly.org/annot/gamexml.dtd.txt");
     $writer->comment("GAME-XML generated by Bio::SeqIO::game::gameWriter");
     $writer->comment("Created " . localtime);
-    $writer->comment('Questions: smckay@bcgsc.bc.ca');
+    $writer->comment('Questions: mckays@cshl.edu');
     $writer->startTag('game', version => 1.2);
     
     my @sources = grep { $_->primary_tag =~ /source|origin|region/i } $seq->get_SeqFeatures;
@@ -202,9 +210,11 @@ sub write_to_game {
 	}
     }
     
-    my $seqname = $seq->accession unless $seq->accession eq 'unknown';
-    $seqname ||= $seq->display_name;
-    $atts->{name} ||= $seqname;
+
+    #set a name in the attributes if none was given
+    $atts->{name} ||= $seq->accession_number ne 'unknown'
+      ? $seq->accession_number : $seq->display_name;
+
     $self->_seq($seq, $atts);
     
     # make a map_position element if req'd
@@ -228,21 +238,23 @@ sub write_to_game {
 	$self->_span($start, $end);
 	$writer->endTag('map_position');
     }
-    
-    @feats = $seq->top_SeqFeatures;
-    my @addback;
-    
-    for ( @feats ) {
+
+    for ( $seq->top_SeqFeatures ) {
+
+      if($_->isa('Bio::SeqFeature::Computation')) {
+	$self->_comp_analysis($_);
+      }
+      else {
         # if the feature has subfeatures, we will assume it is a gene
 	# (hope this is safe!)
 	if ( $_->get_SeqFeatures ) {
-	    $self->_write_gene($_);
-	}
-	else {
-	    # non-gene stuff only
-	    next if $_->primary_tag =~ /CDS|mRNA|exon|UTR/;
-	    $self->_write_feature($_);
+	  $self->_write_gene($_);
+	} else {
+	  # non-gene stuff only
+	  next if $_->primary_tag =~ /CDS|mRNA|exon|UTR/;
+	  $self->_write_feature($_);
 	}
+      }
     }    
     
     $writer->endTag('game');
@@ -250,10 +262,10 @@ sub write_to_game {
     $xml;
 }
 
-=head2 _rearrange
+=head2 _rearrange_hierarchies
 
- Title   : _rearrange
- Usage   : $self->_rearrange($seq)
+ Title   : _rearrange_hierarchies
+ Usage   : $self->_rearrange_hierarchies($seq)
  Function: internal method to rearrange gene containment hierarchies
            so that snRNA or transposon features contain their genes
            rather than the other way around
@@ -263,7 +275,7 @@ sub write_to_game {
 
 =cut
 
-sub _rearrange {
+sub _rearrange_hierarchies { #renamed to not conflict with Bio::Root::_rearrange
     my ($self, $seq, @containers) = @_;
     my @feats   = $seq->remove_SeqFeatures;
     my @genes   = grep { $_->primary_tag eq 'gene' } @feats;
@@ -320,7 +332,11 @@ sub _write_feature {
     $writer->startTag('feature_set', id => $id);
     $self->_element('name', $id);
     $self->_element('type', $feat->primary_tag);
-    $self->_feature_set_tags($feat);
+    $self->_render_tags( $feat,
+			 \&_render_date_tags,
+			 \&_render_comment_tags,
+			 \&_render_tags_as_properties
+		       );
     $self->_feature_span($id, $feat);
     $writer->endTag('feature_set');
     $writer->endTag('annotation') unless $bare;
@@ -331,7 +347,7 @@ sub _write_feature {
  Title   : _write_gene
  Usage   : $self->_write_gene($feature)
  Function: internal method for rendering gene containment hierarchies into 
-           an nested <annotation> element 
+           a nested <annotation> element 
  Returns : nothing
  Args    : a nested Bio::SeqFeature::Generic gene feature
  Note    : A nested gene hierarchy (gene->mRNA->CDS->exon) is expected.  If other gene 
@@ -350,13 +366,21 @@ sub _write_gene {
 	  || $self->_find_name($feat, $feat->primary_tag)
 	  || $self->_find_name($feat, 'locus_tag') 
 	  || $self->_find_name($feat, 'symbol')
-          || $self->throw("Could not find a gene/feature ID");
+          || $self->throw(<<EOM."Feature name was: '".($feat->display_name || 'not set')."'");
+Could not find a gene/feature ID, feature must have a primary tag or a tag
+with one of the names: 'standard_name', 'gene', 'locus_tag', or 'symbol'.
+EOM
     my $gid = $self->_find_name($feat, 'gene') || $id;
 
     $writer->startTag('annotation', id => $id);
     $self->_element('name', $gid);
     $self->_element('type', $feat->primary_tag);
-    $self->_tags($feat);
+    $self->_render_tags( $feat,
+			 \&_render_date_tags,
+			 \&_render_dbxref_tags,
+			 \&_render_comment_tags,
+			 \&_render_tags_as_properties,
+		       );
     
     my @genes;
     
@@ -514,8 +538,11 @@ sub _write_gene {
 	    $writer->startTag('feature_set', id => $name);
 	    $self->_element('name', $name);
 	    $self->_element('type', 'transcript');
-	    $self->_feature_set_tags($mRNA);
-	    $self->_feature_set_tags($cds) if $cds;
+	    $self->_render_tags($_,
+				\&_render_date_tags,
+				\&_render_comment_tags,
+				\&_render_tags_as_properties,
+			       ) for ( $mRNA, ($cds) || () );
 	     
 	    # any UTR's, etc associated with this transcript?
 	    for my $thing ( @other_stuff ) {
@@ -574,7 +601,7 @@ sub _write_gene {
         $self->_element('description', "\n     $desc\n    ");
 
 	my $aa = $h{residues};
-	$aa =~ s/\w{60}/$&\n      /g;
+	$aa =~ s/(\w{60})/$1\n      /g;
 	$aa =~ s/\n\s+$//m;
 	$aa = "\n      " . $aa . "\n    ";
 	$self->_element('residues', $aa);
@@ -635,52 +662,334 @@ sub _check_cds {
 
 }
 
-=head2 _feature_set_tags
+=head2 _comp_analysis
 
- Title   : _feature_set_tags
- Usage   : $self->_feature_set_tags($feature)
- Function: an internal method to handle tag/value attributes
-           for a feature set element
- Returns : nothing
- Args    : a Bio::SeqFeatureI-compliant object
+  Usage:
+  Desc :
+  Ret  :
+  Args :
+  Side Effects:
+  Example:
 
 =cut
 
-sub _feature_set_tags {
-    my ($self, $feat) = @_;
-    my $writer = $self->{'writer'};
-    
-    my @tags = $feat->get_all_tags;
-    for my $tag ( @tags ) {
-	next if $tag eq 'timestamp';
-        
-        my %seen_val;	
-	for my $val ( $feat->get_tag_values($tag) ) { 
+sub _comp_analysis {
+  my ($self, $feat) = @_;
+  my $writer = $self->{writer};
 
-	    if ( $tag eq 'date' ) {
-		my ($date) = $feat->get_tag_values($tag);
-		my %timestamp;
-		if ( $feat->has_tag('timestamp') ) {
-		    ($timestamp{'timestamp'}) = $feat->get_tag_values('timestamp'); 
-		    $feat->remove_tag('timestamp');
-		}
-		$self->_element('date', $val, \%timestamp);
-	    }
-            elsif ( $tag eq 'comment' ) {
-                unless ( $val =~ /=.+?;.+=/ ) {
-                    $writer->startTag('comment');
-                    $self->_element('text', $val);
-                    $writer->endTag('comment');
-                }
-                else{
-                    $self->_unflatten_attribute('comment', $val);
-                }
-            }
-	    elsif ( ++$seen_val{$val} < 2 && $tag ne $feat->primary_tag ) {
-		$self->_property($tag, $val);
-	    }
-	}
+  $writer->startTag('computational_analysis');
+  $self->_element('program', $feat->program_name || 'unknown program');
+  $self->_element('database', $feat->database_name) if $feat->database_name;
+  $self->_element('version', $feat->program_version) if $feat->program_version;
+  $self->_element('type', $feat->primary_tag) if $feat->primary_tag;
+  $self->_render_tags($feat,
+		      \&_render_date_tags,
+		      \&_render_tags_as_properties,
+		     );
+  $self->_comp_result($feat);
+  $writer->endTag('computational_analysis');
+}
+
+=head2 _comp_result
+
+  Usage:
+  Desc : recursively render a feature and its subfeatures as
+         <result_set> and <result_span> elements
+  Ret  : nothing meaningful
+  Args : a feature
+
+=cut
+
+
+sub _comp_result {
+  my ($self,$feat) = @_;
+
+  #check that all our subfeatures have the same strand
+  
+
+  #write result sets for things that have subfeatures, or things
+  #that have some tags
+  if( my @subfeats = $feat->get_SeqFeatures or $feat->get_all_tags ) {
+    my $writer = $self->{writer};
+    $writer->startTag('result_set',
+		      ($feat->can('computation_id') && defined($feat->computation_id))
+		        ? (id => $feat->computation_id) : ()
+		     );
+    my $fakename = $feat->primary_tag || 'no_name';
+    $self->_element('name', $feat->display_name || ($fakename).'_'.++$self->{anon_result_set_counters}{$fakename} );
+    $self->_seq_relationship('query', $feat);
+    $self->_render_tags($feat,
+			\&_render_output_tags
+		       );
+    for (@subfeats) { #render the subfeats, if any
+      $self->_comp_result($_);
     }
+    $self->_comp_result_span($feat); #also have a span to hold this info
+    $writer->endTag('result_set');
+  } else {
+    #just write result spans for simple things
+    $self->_comp_result_span($feat);
+  }
+}
+
+=head2 _comp_result_span
+
+  Usage: _comp_result_span('foo12',$feature);
+  Desc : write GAME XML for a Bio::SeqFeature::Computation feature
+         that has no subfeatures
+  Ret  : nothing meaningful
+  Args : name for this span (some kind of identifier),
+         SeqFeature object to put into this span
+  Side Effects:
+  Example:
+
+=cut
+
+sub _comp_result_span {
+
+  my ($self, $feat) = @_;
+  my $writer = $self->{writer};
+
+  $writer->startTag('result_span',
+		    ($feat->can('computation_id') && defined($feat->computation_id) ? (id => $feat->computation_id) : ())
+		   );
+  $self->_element('name', $feat->display_name) if $feat->display_name;
+  $self->_element('type', $feat->primary_tag) if $feat->primary_tag;
+  my $has_score = $feat->can('has_score') ? $feat->has_score : defined($feat->score);
+  $self->_element('score', $feat->score) if $has_score;
+  $self->_render_tags($feat,
+		      \&_render_output_tags
+		     );
+  $self->_seq_relationship('query', $feat);
+  $self->_render_tags($feat,
+		      \&_render_target_tags,
+		     );
+  $writer->endTag('result_span');
+}
+
+=head2 _render_tags
+
+  Usage:
+  Desc :
+  Ret  :
+  Args :
+  Side Effects:
+  Example:
+
+=cut
+
+sub _render_tags {
+  my ($self,$feat,@render_funcs) = @_;
+
+  my @tagnames = $feat->get_all_tags;
+
+  #do a chain-of-responsibility down the allowed
+  #tag handlers types for the context in which this is
+  #called
+  foreach my $func (@render_funcs) {
+    @tagnames = $self->$func($feat,@tagnames);
+  }
+}
+
+=head2 _render_output_tags
+
+  Usage:
+  Desc : print out <output> elements, with contents
+         taken from the SeqFeature::Computation's 'output' tag
+  Ret  : array of tag names this did not render
+  Args : feature object, list of tag names to maybe render
+
+  In game xml, only <result_span> and <result_set> elements can
+  have <output> elements.
+
+=cut
+
+sub _render_output_tags {
+  my ($self, $feat, @tagnames) = @_;
+  my $writer = $self->{writer};
+  my @passed_up;
+
+  for my $tag (@tagnames) {
+    if(lc($tag) eq 'output') {
+      my @outputs = $feat->get_tag_values($tag);
+      while(my($type,$val) = splice @outputs,0,2) {
+	$writer->startTag('output');
+	$self->_element('type',$type);
+	$self->_element('value',$val);
+	$writer->endTag('output');
+      }
+    }
+    else {
+      push @passed_up,$tag;
+    }
+  }
+  return @passed_up;
+}
+
+=head2 _render_tags_as_properties
+
+  Usage:
+  Desc :
+  Ret  : empty array
+  Args : feature object, array of tag names
+  Side Effects:
+  Example:
+
+  In game xml, <annotation>, <computational_analysis>,
+  and <feature_set> elements can have properties.
+
+=cut
+
+sub _render_tags_as_properties {
+  my ($self,$feat,@tagnames) = @_;
+
+  foreach my $tag (@tagnames) {
+    if( $tag ne $feat->primary_tag ) {
+      $self->_property($tag,$_) for $feat->get_tag_values($tag);
+    }
+  }
+  return ();
+}
+
+=head2 _render_comment_tags
+
+  Usage:
+  Desc :
+  Ret  : names of tags that were not comment tags
+  Args : feature object, tag names available for us to render
+  Side Effects: writes XML
+  Example:
+
+  In game xml, <annotation> and <feature_set> elements can
+  have comments.
+
+=cut
+
+sub _render_comment_tags {
+  my ($self,$feat,@tagnames) = @_;
+  my $writer = $self->{writer};
+  my @passed_up;
+  for my $tag ( @tagnames ) {
+    if( lc($tag) eq 'comment' ) {
+      for my $val ($feat->get_tag_values($tag)) {
+	if ( $val =~ /=.+?;.+=/ ) {
+	  $self->_unflatten_attribute('comment', $val);
+	} else {
+	  $writer->startTag('comment');
+	  $self->_element('text', $val);
+	  $writer->endTag('comment');
+	}
+      }
+    } else {
+      push @passed_up,$tag;
+    }
+  }
+  return @passed_up;
+}
+
+=head2 _render_date_tags
+
+  Usage:
+  Desc :
+  Ret  : names of tags that were not date tags
+  Args : feature, list of tag names available for us to render
+  Side Effects: writes XML for <date> elements
+  Example:
+
+  In game xml, <annotation>, <computational_analysis>,
+  <transaction>, <comment>, and <feature_set> elements
+  can have <date>s.
+
+=cut
+
+sub _render_date_tags {
+  my ($self,$feat,@tagnames) = @_;
+  my @passed_up;
+  my $date;
+  my %timestamp;
+  foreach my $tag (@tagnames) {
+    if ( lc($tag) eq 'date' ) {
+      ($date) = $feat->get_tag_values($tag);
+    } elsif ( lc($tag) eq 'timestamp' ) {
+      ($timestamp{'timestamp'}) = $feat->get_tag_values($tag);
+      #ignore timestamps, they are folded in with date elem above
+    } else {
+      push @passed_up,$tag;
+    }
+  }
+  $self->_element('date', $date, \%timestamp) if defined($date);
+  return @passed_up;
+}
+
+=head2 _render_dbxref_tags
+
+  Desc : look for xref tags and render them if they are there
+  Ret  : tag names that we didn't render
+  Args : feature object, list of tag names to render
+  Side Effects: writes a <dbxref> element if a tag with name
+                matching /xref$/i is present
+
+
+  In game xml, <annotation> and <seq> elements can have dbxrefs.
+
+=cut
+
+#TODO: can't sequences also have database xrefs?  how to find those?
+sub _render_dbxref_tags {
+  my ($self, $feat, @tagnames) = @_;
+  my @passed_up;
+  for my $tag ( @tagnames ) {                           #look through all the tags
+    if( $tag =~ /xref$/i ) {                            #if they are xref tags
+      my $writer = $self->{writer};
+      for my $val ( $feat->get_all_tag_values($tag) ) { #get all their values
+	if( my ($db,$dbid) = $val =~ /(\S+):(\S+)/ ) {  #and render them as xrefs
+	  $writer->startTag('dbxref');
+	  $self->_element('xref_db', $db);
+	  $dbid = $val if $db =~ /^[A-Z]O$/; # -> ontology, like GO
+	  $self->_element('db_xref_id', $dbid);
+	  $writer->endTag('dbxref');
+	}
+      }
+    } else {
+      push @passed_up,$tag;
+    }
+  }
+  return @passed_up;
+}
+
+
+=head2 _render_target_tags
+
+  Usage:
+  Desc : process any 'Target' tags that would indicate a sequence alignment subject
+  Ret  : array of tag names that we didn't render
+  Args : feature object
+  Side Effects: writes a <seq_relationship> of type 'subject' if it finds
+                any properly formed tags named 'Target'
+  Example:
+
+  In game xml, <result_span>, <feature_span>, and <result_set> can have
+  <seq_relationship>s.  <result_set> can only have one, a 'query' relation.
+
+=cut
+
+sub _render_target_tags {
+  my ($self,$feat,@tagnames) = @_;
+  my @passed_up;
+  foreach my $tag (@tagnames) {
+    if($tag eq 'Target' && (my @alignment = $feat->get_tag_values('Target')) >= 3) {
+      $self->_seq_relationship('subject',
+			       Bio::Location::Simple->new( -start => $alignment[1],
+							   -end   => $alignment[2],
+							 ),
+			       $alignment[0],
+			       $alignment[3],
+			      );
+    } else {
+      push @passed_up, $tag;
+    }
+  }
+  return @passed_up;
 }
 
 
@@ -697,7 +1006,7 @@ sub _feature_set_tags {
 sub _property {
     my ($self, $tag, $val) = @_;
     my $writer = $self->{writer};
-    
+
     if ( length $val > 45 ) {
 	my @val = split /\s+/, $val;
 	$val = '';
@@ -716,46 +1025,6 @@ sub _property {
     $writer->endTag('property');
 }
 
-=head2 _tags
-
- Title   : _tags
- Usage   : $self->_tags($feat)
- Function: an internal method to intercept GO terms and
-           db_xrefs and handle generic tag/value pairs for a gene
- Returns : nothing 
- Args    : a Bio::SeqFeatureI-compliant object
-
-=cut
-
-sub _tags {
-    my ($self, $feat) = @_;
-    my $writer = $self->{writer};
-    my @tags = $feat->get_all_tags;
-    
-    for my $tag ( @tags ) {
-	for my $val ( $feat->get_tag_values($tag) ) {
-	    if ( $tag =~ /xref/ && $val =~ /GO/ ) {
-		    $writer->startTag('aspect');
-		    $self->_xref($val);
-		    $writer->endTag('aspect');
-	    }
-	    elsif ( $tag eq 'comment' ) {
-                unless ( $val =~ /=.+?;.+=/ ) {
-                    $writer->startTag('comment');
-                    $self->_element('text', $val);
-                    $writer->endTag('comment');
-                }
-                else{
-		    $self->_unflatten_attribute('comment', $val);
-                }
-            }
-	    else {
-		$self->_property($tag, $val);
-	    }
-	}
-    }
-}
-
 =head2 _unflatten_attribute
 
  Title   : _unflatten_attribute
@@ -843,25 +1112,27 @@ sub _feature_span {
  Title   : _seq_relationship
  Usage   : $self->_seq_relationship($type, $loc)
  Function: an internal method to handle feature_span sequence relationships
- Returns : nothing 
- Args    : feature type and a Bio::LocationI-compliant object
+ Returns : nothing
+ Args    : feature type, a Bio::LocationI-compliant object,
+           (optional) sequence name (defaults to the query seq)
+           and (optional) alignment string
 
 =cut
 
 sub _seq_relationship {
-    my ($self, $type, $loc) = @_;
+    my ($self, $type, $loc, $seqname, $alignment) = @_;
     my $writer = $self->{'writer'};
-    my $id = $self->{seq}->accession;
-    if ( $id eq 'unknown' ) {
-	$id = $self->{seq}->display_name;
-    }
-    
+
+    $seqname ||= #if no seqname passed in, use the name of our annotating seq
+      $self->{seq}->accession_number ne 'unknown' && $self->{seq}->accession_number
+	|| $self->{seq}->display_id || 'unknown';
     $writer->startTag(
 		      'seq_relationship',
 		      type => $type,
-		      seq  => $id
+		      seq  => $seqname,
 		     );
     $self->_span($loc);
+    $writer->_element('alignment',$alignment) if $alignment;
     $writer->endTag('seq_relationship');
 }
 
@@ -941,7 +1212,8 @@ sub _seq {
     my ($self, $seq, $atts) = @_;
 
     my $writer = $self->{'writer'};
-    
+
+   
     # game moltypes
     my $alphabet = $seq->alphabet;
     $alphabet ||= $seq->mol_type if $seq->can('mol_type');
@@ -974,7 +1246,7 @@ sub _seq {
     # nicer xml formatting/indentation
     my $sp  = (' ' x 6);
     my $dna = $seq->seq;
-    $dna =~ s/\w{60}/$&\n$sp/g;
+    $dna =~ s/(\w{60})/$1\n$sp/g;
     $dna = "\n$sp" . $dna . "\n    ";
     
     if ( $seq->species && !$self->{has_organism}) {
@@ -1005,6 +1277,7 @@ sub _find_name {
 	return $name;
     }
     else {
+#      warn "Could not find name '$key'\n";
 	return '';
     }
 }
