@@ -1,6 +1,8 @@
-# $Id: tigr.pm,v 1.6 2004/06/16 19:39:54 laurichj Exp $
+# $Id: tigr.pm 16123 2009-09-17 12:57:27Z cjfields $
 # BioPerl module for Bio::SeqIO::tigr
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Josh Lauricha (laurichj@bioinfo.ucr.edu)
 #
 # Copyright Josh Lauricha
@@ -30,17 +32,27 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org            - General discussion
-  http://bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.
- Bug reports can be submitted via email or the web:
+the bugs and their resolution.
+Bug reports can be submitted via the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org
 
 =head1 AUTHORS - Josh Lauricha
 
@@ -60,10 +72,8 @@ methods. Internal methods are usually preceded with a _
 
 # Let the code begin...
 package Bio::SeqIO::tigr;
-use vars qw(@ISA);
 use strict;
 
-use Bio::SeqIO;
 use Bio::Seq::RichSeq;
 use Bio::Species;
 use Bio::Annotation::Comment;
@@ -73,14 +83,14 @@ use Bio::Seq::RichSeq;
 use Data::Dumper;
 use Error qw/:try/;
 
-@ISA = qw(Bio::SeqIO);
+use base qw(Bio::SeqIO);
 
 sub _initialize
 {
 	my($self, @args) = @_;
 
 	$self->SUPER::_initialize(@args);
-	$self->sequence_factory(new Bio::Seq::SeqFactory(
+	$self->sequence_factory(Bio::Seq::SeqFactory->new(
 			-type => 'Bio::Seq::RichSeq')
 	);
 
@@ -103,7 +113,7 @@ sub next_seq()
 	my ($self) = @_;
 	
 	# Check for any more sequences
-	return undef if !defined($self->{_sequences}) or scalar(@{$self->{_sequences}}) < 1;
+	return if !defined($self->{_sequences}) or scalar(@{$self->{_sequences}}) < 1;
 
 	# get the next sequence
 	my $seq = shift(@{ $self->{_sequences} } );
@@ -146,7 +156,7 @@ sub next_seq()
 		# Check for an invalid protein
 		try {
 			# Get the subsq
-			my $cds = new Bio::PrimarySeq(
+			my $cds = Bio::PrimarySeq->new(
 				-strand => 1,
 				-id  => $seq->accession_number(),
 				-seq => $seq->subseq($feat->location())
@@ -430,7 +440,7 @@ sub _process_header
 
 	if($line =~ /<ORGANISM>\s*(.+)\s*<\/ORGANISM>/o) {
 		my( $genus, $species, @ss ) = split(/\s+/o, $1);
-		$self->{_assembly}->{species} = new Bio::Species();
+		$self->{_assembly}->{species} = Bio::Species->new();
 		$self->{_assembly}->{species}->genus($genus);
 		$self->{_assembly}->{species}->species($species);
 		$self->{_assembly}->{species}->sub_species(join(' ', @ss)) if scalar(@ss) > 0;
@@ -609,7 +619,7 @@ sub _process_repeat_list
 sub _process_tiling_path
 {
 	my($self) = @_;
-	my $line = $self->_readine();
+	my $line = $self->_readline();
 
 
 	if($line =~ /<TILING_PATH>/o) {
@@ -641,11 +651,11 @@ sub _process_tu
 	my $line = $self->_readline();
 
 	try {
-		my $tu = new Bio::Seq::RichSeq(-strand => 1);
+		my $tu = Bio::Seq::RichSeq->new(-strand => 1);
 		$tu->species( $self->{_assembly}->{species} );
 
 		# Add the source tag, so we can add the GO annotations to it
-		$tu->add_SeqFeature(new Bio::SeqFeature::Generic(-source_tag => 'TIGR', -primary_tag => 'source'));
+		$tu->add_SeqFeature(Bio::SeqFeature::Generic->new(-source_tag => 'TIGR', -primary_tag => 'source'));
 		
 		if($line !~ /<TU>/o) {
 			$self->throw("Process_tu called when no <TU> tag");
@@ -820,7 +830,7 @@ sub _process_gene_info
 	}
 
 	while($line =~ /<COMMENT>([^<]+)<\/COMMENT>/o) {
-		my $comment = new Bio::Annotation::Comment(
+		my $comment = Bio::Annotation::Comment->new(
 			-text => $1
 		);
 		$tu->annotation()->add_Annotation('comment', $comment);
@@ -828,7 +838,7 @@ sub _process_gene_info
 	}
 
 	while($line =~ /<PUB_COMMENT>([^<]+)<\/PUB_COMMENT>/o) {
-		my $comment = new Bio::Annotation::Comment(
+		my $comment = Bio::Annotation::Comment->new(
 			-text => $1
 		);
 		$tu->annotation()->add_Annotation('comment', $comment);
@@ -912,7 +922,7 @@ sub _build_location
 		);
 	}
 
-	return new Bio::Location::Simple( -start => $start, -end => $end, -strand => 1 );
+	return Bio::Location::Simple->new( -start => $start, -end => $end, -strand => 1 );
 }
 
 sub _process_model
@@ -920,7 +930,7 @@ sub _process_model
 	my($self, $tu, $end5, $end3) = @_;
 	my $line;
 	my( $source ) = grep { $_->primary_tag() eq 'source' } $tu->get_SeqFeatures();
-	my $model = new Bio::SeqFeature::Generic(
+	my $model = Bio::SeqFeature::Generic->new(
 		-source_tag  => 'TIGR',
 		-primary_tag => 'MODEL',
 	);
@@ -979,18 +989,18 @@ sub _process_model
 		$self->throw("Invalid Coordset: $line");
 	}
 
-	my $exon = new Bio::SeqFeature::Generic(
+	my $exon = Bio::SeqFeature::Generic->new(
 		-source_tag  => 'TIGR',
 		-primary_tag => 'EXON',
-		-location => new Bio::Location::Split(),
+		-location => Bio::Location::Split->new(),
 		-tags => [ locus => $tu->accession_number() ],
 	);
 	$exon->add_tag_value( model => $model->get_tag_values('feat_name') );
 
-	my $cds  = new Bio::SeqFeature::Generic(
+	my $cds  = Bio::SeqFeature::Generic->new(
 		-source_tag  => 'TIGR',
 		-primary_tag => 'CDS',
-		-location => new Bio::Location::Split(),
+		-location => Bio::Location::Split->new(),
 		-tags => [ locus => $tu->accession_number() ],
 	);
 	$cds->add_tag_value( model => $model->get_tag_values('feat_name') );
@@ -1202,7 +1212,7 @@ sub _process_left_utr
 		my $cs = $self->_process_coordset();
 		my $loc = $self->_build_location($end5, $end3, $tu->length(), $cs);
 
-		push(@$utrs, new Bio::SeqFeature::Generic(
+		push(@$utrs, Bio::SeqFeature::Generic->new(
 		        -source_tag  => 'TIGR',
 				-primary_tag => 'LEFT_UTR',
 				-strand => 1,
@@ -1240,7 +1250,7 @@ sub _process_right_utr
 		my $cs = $self->_process_coordset();
 		my $loc = $self->_build_location($end5, $end3, $tu->length(), $cs);
 
-		push(@$utrs, new Bio::SeqFeature::Generic(
+		push(@$utrs, Bio::SeqFeature::Generic->new(
 		        -source_tag  => 'TIGR',
 				-primary_tag => 'RIGHT_UTR',
 				-strand => 1,
@@ -1276,7 +1286,7 @@ sub _process_ext_utr
 		my $cs = $self->_process_coordset();
 		my $loc = $self->_build_location($end5, $end3, $tu->length(), $cs);
 
-		push(@$utrs, new Bio::SeqFeature::Generic(
+		push(@$utrs, Bio::SeqFeature::Generic->new(
 		        -source_tag  => 'TIGR',
 				-primary_tag => 'EXTENDED_UTR',
 				-strand => 1,
@@ -1342,3 +1352,5 @@ sub throw
 	my $string = "[$.]" . join('', @s);
 	$self->SUPER::throw($string);
 }
+
+1;
