@@ -1,5 +1,8 @@
+# $Id: Trim.pm 16123 2009-09-17 12:57:27Z cjfields $
 # Bio::Tools::Alignment::Trim.pm
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Chad Matsalla
 #
 # Copyright Chad Matsalla
@@ -16,7 +19,7 @@ Bio::Tools::Alignment::Trim - A kludge to do specialized trimming of
 =head1 SYNOPSIS
 
   use Bio::Tools::Alignment::Trim;
-  $o_trim = new Bio::Tools::Alignment::Trim;
+  $o_trim = Bio::Tools::Alignment::Trim->new();
   $o_trim->set_reverse_designator("R");
   $o_trim->set_forward_designator("F");
 
@@ -45,22 +48,32 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                 - General discussion
-  http://bio.perl.org/MailList.html     - About the mailing
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists     - About the mailing
 lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via
-email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Chad Matsalla
 
-Email bioinformatics@dieselwurks.com
+Email bioinformatics-at-dieselwurks.com
 
 =head1 APPENDIX
 
@@ -71,13 +84,12 @@ Internal methods are usually preceded with a _
 
 package Bio::Tools::Alignment::Trim;
 
-use Bio::Root::Root;
 use strict;
 use Dumpvalue;
 
-use vars qw(@ISA %DEFAULTS);
+use vars qw(%DEFAULTS);
 
-@ISA = qw(Bio::Root::Root);
+use base qw(Bio::Root::Root);
 
 BEGIN {
     %DEFAULTS = ( 'f_designator' => 'f',
