@@ -1,8 +1,10 @@
-# $Id: CodonTable.pm,v 1.29 2004/09/28 13:05:37 radams Exp $
+# $Id: CodonTable.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # bioperl module for Bio::Tools::CodonTable
 #
-# Cared for by Heikki Lehvaslaiho <heikki@ebi.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Heikki Lehvaslaiho <heikki-at-bioperl-dot-org>
 #
 # Copyright Heikki Lehvaslaiho
 #
@@ -12,27 +14,32 @@
 
 =head1 NAME
 
-Bio::Tools::CodonTable - Bioperl codon table object
+Bio::Tools::CodonTable - Codon table object
 
 =head1 SYNOPSIS
 
   # This is a read-only class for all known codon tables.  The IDs are
   # the ones used by nucleotide sequence databases.  All common IUPAC
-  # ambiguity codes for DNA, RNA and animo acids are recognized.
+  # ambiguity codes for DNA, RNA and amino acids are recognized.
 
-  # to use
   use Bio::Tools::CodonTable;
 
   # defaults to ID 1 "Standard"
   $myCodonTable   = Bio::Tools::CodonTable->new();
-  $myCodonTable2  = Bio::Tools::CodonTable -> new ( -id => 3 );
+  $myCodonTable2  = Bio::Tools::CodonTable->new( -id => 3 );
 
   # change codon table
   $myCodonTable->id(5);
 
   # examine codon table
   print  join (' ', "The name of the codon table no.", $myCodonTable->id(4),
-	       "is:", $myCodonTable->name(), "\n");
+           "is:", $myCodonTable->name(), "\n");
+
+  # print possible codon tables
+  $tables = Bio::Tools::CodonTable->tables;
+  while ( ($id,$name) = each %{$tables} ) {
+    print "$id = $name\n";
+  }
 
   # translate a codon
   $aa = $myCodonTable->translate('ACU');
@@ -51,17 +58,17 @@ Bio::Tools::CodonTable - Bioperl codon table object
   my $seqobj    = Bio::PrimarySeq->new(-seq => 'FHGERHEL');
   my $iupac_str = $myCodonTable->reverse_translate_all($seqobj);
 
-  #boolean tests
+  # boolean tests
   print "Is a start\n"       if $myCodonTable->is_start_codon('ATG');
-  print "Is a termianator\n" if $myCodonTable->is_ter_codon('tar');
+  print "Is a terminator\n" if $myCodonTable->is_ter_codon('tar');
   print "Is a unknown\n"     if $myCodonTable->is_unknown_codon('JTG');
 
 =head1 DESCRIPTION
 
-Codon tables are also called translation tables or genetics codes
-since that is what they try to represent. A bit more complete picture
+Codon tables are also called translation tables or genetic codes
+since that is what they represent. A bit more complete picture
 of the full complexity of codon usage in various taxonomic groups
-presented at the NCBI Genetic Codes Home page.
+is presented at the NCBI Genetic Codes Home page.
 
 CodonTable is a BioPerl class that knows all current translation
 tables that are used by primary nucleotide sequence databases
@@ -77,10 +84,12 @@ acid transcripts. The CodonTable object accepts codons of both type as
 input and allows the user to set the mode for output when reverse
 translating. Its default for output is DNA.
 
-Note: This class deals primarily with individual codons and amino
-      acids. However in the interest of speed you can L<translate>
-      longer sequence, too. The full complexity of protein translation
-      is tackled by L<Bio::PrimarySeqI::translate>.
+Note: 
+
+This class deals primarily with individual codons and amino
+acids. However in the interest of speed you can L<translate>
+longer sequence, too. The full complexity of protein translation
+is tackled by L<Bio::PrimarySeqI::translate>.
 
 
 The amino acid codes are IUPAC recommendations for common amino acids:
@@ -100,6 +109,8 @@ The amino acid codes are IUPAC recommendations for common amino acids:
           M           Met            Methionine
           F           Phe            Phenylalanine
           P           Pro            Proline
+          O           Pyl            Pyrrolysine (22nd amino acid)
+          U           Sec            Selenocysteine (21st amino acid)
           S           Ser            Serine
           T           Thr            Threonine
           W           Trp            Tryptophan
@@ -107,6 +118,7 @@ The amino acid codes are IUPAC recommendations for common amino acids:
           V           Val            Valine
           B           Asx            Aspartic acid or Asparagine
           Z           Glx            Glutamine or Glutamic acid
+          J           Xle            Isoleucine or Valine (mass spec ambiguity)
           X           Xaa            Any or unknown amino acid
 
 
@@ -116,7 +128,7 @@ only differences are in available initiator codons.
 
 
 NCBI Genetic Codes home page:
-     http://www.ncbi.nlm.nih.gov/htbin-post/Taxonomy/wprintgc?mode=c
+     http://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi?mode=c
 
 EBI Translation Table Viewer:
      http://www.ebi.ac.uk/cgi-bin/mutations/trtables.cgi
@@ -124,7 +136,7 @@ EBI Translation Table Viewer:
 Amended ASN.1 version with ids 16 and 21 is at:
      ftp://ftp.ebi.ac.uk/pub/databases/geneticcode/
 
-Thank your for Matteo diTomasso for the original Perl implementation
+Thanks to Matteo diTomasso for the original Perl implementation
 of these tables.
 
 =head1 FEEDBACK
@@ -135,26 +147,31 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the
 Bioperl mailing lists  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                         - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
-report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.  Bug reports can be submitted via
- email or the web:
+Report bugs to the Bioperl bug tracking system to help us keep track
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Heikki Lehvaslaiho
 
-Email:  heikki@ebi.ac.uk
-Address:
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom
+Email:  heikki-at-bioperl-dot-org
 
 =head1 APPENDIX
 
@@ -167,17 +184,15 @@ methods. Internal methods are usually preceded with a _
 # Let the code begin...
 
 package Bio::Tools::CodonTable;
-use vars qw(@ISA @NAMES @TABLES @STARTS $TRCOL $CODONS %IUPAC_DNA 
-	    $CODONGAP $GAP
-	    %IUPAC_AA %THREELETTERSYMBOLS $VALID_PROTEIN $TERMINATOR);
+use vars qw(@NAMES @TABLES @STARTS $TRCOL $CODONS %IUPAC_DNA $CODONGAP $GAP
+                %IUPAC_AA %THREELETTERSYMBOLS $VALID_PROTEIN $TERMINATOR);
 use strict;
 
 # Object preamble - inherits from Bio::Root::Root
-use Bio::Root::Root;
 use Bio::Tools::IUPAC;
 use Bio::SeqUtils;
 
-@ISA = qw(Bio::Root::Root);
+use base qw(Bio::Root::Root);
 
 
 # first set internal values for all translation tables
@@ -187,88 +202,88 @@ BEGIN {
     $GAP = '-';
     $CODONGAP = $GAP x CODONSIZE;
 
-    @NAMES =			#id
-	(
-	 'Standard',		#1
-	 'Vertebrate Mitochondrial',#2
-	 'Yeast Mitochondrial',# 3
-	 'Mold, Protozoan, and CoelenterateMitochondrial and Mycoplasma/Spiroplasma',#4
-	 'Invertebrate Mitochondrial',#5
-	 'Ciliate, Dasycladacean and Hexamita Nuclear',# 6
-	 '', '',
-	 'Echinoderm Mitochondrial',#9
-	 'Euplotid Nuclear',#10
-	 '"Bacterial"',# 11
-	 'Alternative Yeast Nuclear',# 12
-	 'Ascidian Mitochondrial',# 13
-	 'Flatworm Mitochondrial',# 14
-	 'Blepharisma Nuclear',# 15
-	 'Chlorophycean Mitochondrial',# 16
-	 '', '',  '', '',
-	 'Trematode Mitochondrial',# 21
-	 'Scenedesmus obliquus Mitochondrial', #22
-	 'Thraustochytrium Mitochondrial' #23
-	 );
+    @NAMES =            #id
+    (
+     'Standard',        #1
+     'Vertebrate Mitochondrial',#2
+     'Yeast Mitochondrial',# 3
+     'Mold, Protozoan, and CoelenterateMitochondrial and Mycoplasma/Spiroplasma',#4
+     'Invertebrate Mitochondrial',#5
+     'Ciliate, Dasycladacean and Hexamita Nuclear',# 6
+     '', '',
+     'Echinoderm Mitochondrial',#9
+     'Euplotid Nuclear',#10
+     '"Bacterial"',# 11
+     'Alternative Yeast Nuclear',# 12
+     'Ascidian Mitochondrial',# 13
+     'Flatworm Mitochondrial',# 14
+     'Blepharisma Nuclear',# 15
+     'Chlorophycean Mitochondrial',# 16
+     '', '',  '', '',
+     'Trematode Mitochondrial',# 21
+     'Scenedesmus obliquus Mitochondrial', #22
+     'Thraustochytrium Mitochondrial' #23
+     );
 
     @TABLES =
-	qw(
-	   FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSS**VVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCWWTTTTPPPPHHQQRRRRIIMMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSSSVVVVAAAADDEEGGGG
-	   FFLLSSSSYYQQCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   '' ''
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNNKSSSSVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCCWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CC*WLLLSPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSGGVVVVAAAADDEEGGGG
-	   FFLLSSSSYYY*CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNNKSSSSVVVVAAAADDEEGGGG
-	   FFLLSSSSYY*QCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FFLLSSSSYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   '' '' '' ''
-	   FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNNKSSSSVVVVAAAADDEEGGGG   
-	   FFLLSS*SYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   FF*LSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
-	   );
+    qw(
+       FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSS**VVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCWWTTTTPPPPHHQQRRRRIIMMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSSSVVVVAAAADDEEGGGG
+       FFLLSSSSYYQQCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       '' ''
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNNKSSSSVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCCWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CC*WLLLSPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSGGVVVVAAAADDEEGGGG
+       FFLLSSSSYYY*CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNNKSSSSVVVVAAAADDEEGGGG
+       FFLLSSSSYY*QCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FFLLSSSSYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       '' '' '' ''
+       FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNNKSSSSVVVVAAAADDEEGGGG   
+       FFLLSS*SYY*LCC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       FF*LSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG
+       );
 
 
     @STARTS =
-	qw(
-	   ---M---------------M---------------M----------------------------
-	   --------------------------------MMMM---------------M------------
-	   ----------------------------------MM----------------------------
-	   --MM---------------M------------MMMM---------------M------------
-	   ---M----------------------------MMMM---------------M------------
-	   -----------------------------------M----------------------------
-	   '' ''
-	   -----------------------------------M----------------------------
-	   -----------------------------------M----------------------------
-	   ---M---------------M------------MMMM---------------M------------
-	   -------------------M---------------M----------------------------
-	   -----------------------------------M----------------------------
-	   -----------------------------------M----------------------------
-	   -----------------------------------M----------------------------
-	   -----------------------------------M----------------------------
-	   '' ''  '' ''
-	   -----------------------------------M---------------M------------  
-	   -----------------------------------M----------------------------
-	   --------------------------------M--M---------------M------------
-	   );
+    qw(
+       ---M---------------M---------------M----------------------------
+       --------------------------------MMMM---------------M------------
+       ----------------------------------MM----------------------------
+       --MM---------------M------------MMMM---------------M------------
+       ---M----------------------------MMMM---------------M------------
+       -----------------------------------M----------------------------
+       '' ''
+       -----------------------------------M----------------------------
+       -----------------------------------M----------------------------
+       ---M---------------M------------MMMM---------------M------------
+       -------------------M---------------M----------------------------
+       -----------------------------------M----------------------------
+       -----------------------------------M----------------------------
+       -----------------------------------M----------------------------
+       -----------------------------------M----------------------------
+       '' ''  '' ''
+       -----------------------------------M---------------M------------  
+       -----------------------------------M----------------------------
+       --------------------------------M--M---------------M------------
+       );
 
     my @nucs = qw(t c a g);
     my $x = 0;
     ($CODONS, $TRCOL) = ({}, {});
     for my $i (@nucs) {
-	for my $j (@nucs) {
-	    for my $k (@nucs) {
-		my $codon = "$i$j$k";
-		$CODONS->{$codon} = $x;
-		$TRCOL->{$x} = $codon;
-		$x++;
-	    }
-	}
+    for my $j (@nucs) {
+        for my $k (@nucs) {
+        my $codon = "$i$j$k";
+        $CODONS->{$codon} = $x;
+        $TRCOL->{$x} = $codon;
+        $x++;
+        }
+    }
     }
     %IUPAC_DNA = Bio::Tools::IUPAC->iupac_iub();    
     %IUPAC_AA = Bio::Tools::IUPAC->iupac_iup();
@@ -282,9 +297,9 @@ sub new {
     my $self = $class->SUPER::new(@args);
 
     my($id) =
-	$self->_rearrange([qw(ID
-			     )],
-			 @args);
+    $self->_rearrange([qw(ID
+                 )],
+             @args);
 
     $id = 1 if ( ! $id );
     $id  && $self->id($id);
@@ -295,14 +310,10 @@ sub new {
 
  Title   : id
  Usage   : $obj->id(3); $id_integer = $obj->id();
- Function:
-
-           Sets or returns the id of the translation table.  IDs are
+ Function: Sets or returns the id of the translation table.  IDs are
            integers from 1 to 15, excluding 7 and 8 which have been
            removed as redundant. If an invalid ID is given the method
            returns 0, false.
-
-
  Example :
  Returns : value of id, a scalar, 0 if not a valid
  Args    : newvalue (optional)
@@ -313,8 +324,8 @@ sub id{
    my ($self,$value) = @_;
    if( defined $value) {
        if (  !(defined $TABLES[$value-1]) or $TABLES[$value-1] eq '') {
-	   $self->warn("Not a valid codon table ID [$value] ");
-	   $value = 0;
+       $self->warn("Not a valid codon table ID [$value] ");
+       $value = 0;
        }
        $self->{'id'} = $value;
    }
@@ -340,6 +351,29 @@ sub name{
    return $NAMES[$id-1];
 }
 
+=head2 tables
+
+ Title   : tables
+ Usage   : $obj->tables()  or  Bio::Tools::CodonTable->tables()
+ Function: returns a hash reference where each key is a valid codon
+           table id() number, and each value is the corresponding
+           codon table name() string
+ Example :
+ Returns : A hashref
+ Args    : None
+
+
+=cut
+
+sub tables{
+  my %tables;
+  for my $id (1 .. @NAMES) {
+    my $name = $NAMES[$id-1];
+    $tables{$id} = $name if $name;
+  }
+  return \%tables;
+}
+
 =head2 translate
 
  Title   : translate
@@ -385,37 +419,37 @@ sub translate {
     if ($seq =~ /[^actg]/ ) { #ambiguous chars
         for (my $i = 0; $i < (length($seq) - (CODONSIZE-1)); $i+= CODONSIZE) {
             my $triplet = substr($seq, $i, CODONSIZE);
-	    if( $triplet eq $CODONGAP ) {
-		$protein .= $GAP;
-	    } elsif (exists $CODONS->{$triplet}) {
-		$protein .= substr($TABLES[$id-1], 
-				   $CODONS->{$triplet},1);
-	    } else {
-		$protein .= $self->_translate_ambiguous_codon($triplet);
-	    }
-	}
+        if( $triplet eq $CODONGAP ) {
+        $protein .= $GAP;
+        } elsif (exists $CODONS->{$triplet}) {
+        $protein .= substr($TABLES[$id-1], 
+                   $CODONS->{$triplet},1);
+        } else {
+        $protein .= $self->_translate_ambiguous_codon($triplet);
+        }
+    }
     } else { # simple, strict translation
-	for (my $i = 0; $i < (length($seq) - (CODONSIZE -1)); $i+=CODONSIZE) {
+    for (my $i = 0; $i < (length($seq) - (CODONSIZE -1)); $i+=CODONSIZE) {
             my $triplet = substr($seq, $i, CODONSIZE); 
             if( $triplet eq $CODONGAP ) {
-		$protein .= $GAP;
-	    } if (exists $CODONS->{$triplet}) {
+        $protein .= $GAP;
+        } if (exists $CODONS->{$triplet}) {
                 $protein .= substr($TABLES[$id-1], $CODONS->{$triplet}, 1);
-	    } else {
+        } else {
                 $protein .= 'X';
             }
         }
     }
     if ($partial == 2) { # 2 overhanging nucleotides
-	my $triplet = substr($seq, ($partial -4)). "n";
-	if( $triplet eq $CODONGAP ) {
-	    $protein .= $GAP;
-	} elsif (exists $CODONS->{$triplet}) {
-	    my $aa = substr($TABLES[$id-1], $CODONS->{$triplet},1);       
-	    $protein .= $aa;
-	} else {
-	    $protein .= $self->_translate_ambiguous_codon($triplet, $partial);
-	}
+    my $triplet = substr($seq, ($partial -4)). "n";
+    if( $triplet eq $CODONGAP ) {
+        $protein .= $GAP;
+    } elsif (exists $CODONS->{$triplet}) {
+        my $aa = substr($TABLES[$id-1], $CODONS->{$triplet},1);       
+        $protein .= $aa;
+    } else {
+        $protein .= $self->_translate_ambiguous_codon($triplet, $partial);
+    }
     }
     return $protein;
 }
@@ -428,23 +462,23 @@ sub _translate_ambiguous_codon {
     my @codons = _unambiquous_codons($triplet);
     my %aas =();
     foreach my $codon (@codons) {
-	$aas{substr($TABLES[$id-1],$CODONS->{$codon},1)} = 1;
+    $aas{substr($TABLES[$id-1],$CODONS->{$codon},1)} = 1;
     }
     my $count = scalar keys %aas;
     if ( $count == 1 ) {
-	$aa = (keys %aas)[0];
+    $aa = (keys %aas)[0];
     }
     elsif ( $count == 2 ) {
-	if ($aas{'D'} and $aas{'N'}) {
-	    $aa = 'B';
-	}
-	elsif ($aas{'E'} and $aas{'Q'}) {
-	    $aa = 'Z';
-	} else {
-	    $partial ? ($aa = '') : ($aa = 'X');
-	}
+    if ($aas{'D'} and $aas{'N'}) {
+        $aa = 'B';
+    }
+    elsif ($aas{'E'} and $aas{'Q'}) {
+        $aa = 'Z';
     } else {
-	$partial ? ($aa = '') :  ($aa = 'X');
+        $partial ? ($aa = '') : ($aa = 'X');
+    }
+    } else {
+    $partial ? ($aa = '') :  ($aa = 'X');
     }
     return $aa;
 }
@@ -517,32 +551,33 @@ sub revtranslate {
     my (@codons) = ();
 
     if (length($value) == 3 ) {
-	$value = lc $value;
-	$value = ucfirst $value;
-	$value = $THREELETTERSYMBOLS{$value};
+        $value = lc $value;
+        $value = ucfirst $value;
+        $value = $THREELETTERSYMBOLS{$value};
     }
     if ( defined $value and $value =~ /$VALID_PROTEIN/ 
-	 and length($value) == 1 ) {
-	$value = uc $value;
-	@aas = @{$IUPAC_AA{$value}};	
-	foreach my $aa (@aas) {
-	    #print $aa, " -2\n";
-	    $aa = '\*' if $aa eq '*';
-	    while ($TABLES[$id-1] =~ m/$aa/g) {
-		$p = pos $TABLES[$id-1];
-		push (@codons, $TRCOL->{--$p});
-	    }
-	}
+          and length($value) == 1 ) {
+        $value = uc $value;
+        @aas = @{$IUPAC_AA{$value}};    
+        foreach my $aa (@aas) {
+            #print $aa, " -2\n";
+            $aa = '\*' if $aa eq '*';
+          while ($TABLES[$id-1] =~ m/$aa/g) {
+              $p = pos $TABLES[$id-1];
+              push (@codons, $TRCOL->{--$p});
+          }
+        }
     }
 
-    if ($coding and uc ($coding) eq 'RNA') {
-	for my $i (0..$#codons)  {
-	    $codons[$i] =~ tr/t/u/;
-	}
-    }
-
-    return @codons;
+   if ($coding and uc ($coding) eq 'RNA') {
+       for my $i (0..$#codons)  {
+          $codons[$i] =~ tr/t/u/;
+       }
+   }
+    
+   return @codons;
 }
+
 =head2 reverse_translate_all
 
  Title   : reverse_translate_all
@@ -557,60 +592,105 @@ sub revtranslate {
  Args    : a Bio::PrimarySeqI compatible object (mandatory)
            a Bio::CodonUsage::Table object and a threshold if only
              codons with a relative frequency above the threshold are
-             to be considered. 
-
-
+             to be considered.
 =cut
 
 sub reverse_translate_all {
-	
-	my ($self, $obj, $cut, $threshold) = @_;
+
+    my ($self, $obj, $cut, $threshold) = @_;
 
     ## check args are OK
 
-	if (!$obj || !$obj->isa('Bio::PrimarySeqI')){
-		$self->throw(" I need a Bio::PrimarySeqI object, not a [".
-						ref($obj) . "]");
-		}
-	if($obj->alphabet ne 'protein')	{
-		$self->throw("Cannot reverse translate, need an amino acid sequence .".
+    if (!$obj || !$obj->isa('Bio::PrimarySeqI')){
+        $self->throw(" I need a Bio::PrimarySeqI object, not a [".
+                        ref($obj) . "]");
+        }
+    if($obj->alphabet ne 'protein') {
+        $self->throw("Cannot reverse translate, need an amino acid sequence .".
                      "This sequence is of type [" . $obj->alphabet ."]");
-		}
-	my @data;
-	my @seq = split '', $obj->seq;
+        }
+    my @data;
+    my @seq = split '', $obj->seq;
 
-	## if we're not supplying a codon usage table...
-	if( !$cut && !$threshold) {
-		## get lists of possible codons for each aa. 
-		for my $aa (@seq) {
-			if ($aa =~ /x/i) {
-				push @data, (['NNN']);
-			}else {
-				my @cods = $self->revtranslate($aa);
-				push @data, \@cods;
-			}
-		}
-	}else{
-	#else we are supplying a codon usage table, we just want common codons
-	#check args first. 
-		if(!$cut->isa('Bio::CodonUsage::Table'))	{
-			$self->throw("I need a Bio::CodonUsage::Table object, not a [".
+    ## if we're not supplying a codon usage table...
+    if( !$cut && !$threshold) {
+        ## get lists of possible codons for each aa. 
+        for my $aa (@seq) {
+            if ($aa =~ /x/i) {
+                push @data, (['NNN']);
+            }else {
+                my @cods = $self->revtranslate($aa);
+                push @data, \@cods;
+            }
+        }
+    }else{
+    #else we are supplying a codon usage table, we just want common codons
+    #check args first. 
+        if(!$cut->isa('Bio::CodonUsage::Table'))    {
+            $self->throw("I need a Bio::CodonUsage::Table object, not a [".
                      ref($cut). "].");
-			}
-		my $cod_ref = $cut->probable_codons($threshold);
-		for my $aa (@seq) {
-			if ($aa =~ /x/i) {
-				push @data, (['NNN']);
-				next;
-				}
-			push @data, $cod_ref->{$aa};
-		}
-	}
+            }
+        my $cod_ref = $cut->probable_codons($threshold);
+        for my $aa (@seq) {
+            if ($aa =~ /x/i) {
+                push @data, (['NNN']);
+                next;
+                }
+            push @data, $cod_ref->{$aa};
+        }
+    }
 
-	return $self->_make_iupac_string(\@data);
+    return $self->_make_iupac_string(\@data);
 
 }
 
+=head2 reverse_translate_best
+
+ Title   : reverse_translate_best
+ Usage   : my $str = $cttable->reverse_translate_best($seq_object,$cutable);
+ Function: Reverse translates a protein sequence into plain nucleotide
+           sequence (GATC), uses the most common codon for each amino acid
+ Returns : A string
+ Args    : A Bio::PrimarySeqI compatible object and a Bio::CodonUsage::Table object
+
+=cut
+
+sub reverse_translate_best {
+
+    my ($self, $obj, $cut) = @_;
+
+    if (!$obj || !$obj->isa('Bio::PrimarySeqI')){
+        $self->throw(" I need a Bio::PrimarySeqI object, not a [".
+                         ref($obj) . "]");
+    }
+    if ($obj->alphabet ne 'protein')    {
+        $self->throw("Cannot reverse translate, need an amino acid sequence .".
+                         "This sequence is of type [" . $obj->alphabet ."]");
+    }
+    if ( !$cut | !$cut->isa('Bio::CodonUsage::Table'))  {
+        $self->throw("I need a Bio::CodonUsage::Table object, not a [".
+                         ref($cut). "].");
+    }
+
+    my $str = '';
+    my @seq = split '', $obj->seq;
+
+    my $cod_ref = $cut->most_common_codons();
+
+    for my $aa ( @seq ) {
+        if ($aa =~ /x/i) {
+            $str .= 'NNN';
+            next;
+        }
+        if ( defined $cod_ref->{$aa} ) {
+            $str .= $cod_ref->{$aa};
+        } else {
+            $self->throw("Input sequence contains invalid character: $aa");         
+        }
+    }
+   $str;
+}
+
 =head2 is_start_codon
 
  Title   : is_start_codon
@@ -621,7 +701,6 @@ sub reverse_translate_all {
  Returns : boolean
  Args    : codon
 
-
 =cut
 
 sub is_start_codon{
@@ -638,7 +717,7 @@ sub is_start_codon{
        my $result = 1;
        my @ms = map { substr($STARTS[$id-1],$CODONS->{$_},1) } _unambiquous_codons($value);
        foreach my $c (@ms) {
-	   $result = 0 if $c ne 'M';
+       $result = 0 if $c ne 'M';
        }
        return $result;
    }
@@ -656,7 +735,6 @@ sub is_start_codon{
  Returns : boolean
  Args    : codon
 
-
 =cut
 
 sub is_ter_codon{
@@ -673,7 +751,7 @@ sub is_ter_codon{
        my $result = 1;
        my @ms = map { substr($TABLES[$id-1],$CODONS->{$_},1) } _unambiquous_codons($value);
        foreach my $c (@ms) {
-	   $result = 0 if $c ne $TERMINATOR;
+       $result = 0 if $c ne $TERMINATOR;
        }
        return $result;
    }
@@ -684,7 +762,7 @@ sub is_ter_codon{
  Title   : is_unknown_codon
  Usage   : $obj->is_unknown_codon('GAJ')
  Function: returns false (0) for all codons that are valid,
-	    true (1) for others.
+        true (1) for others.
  Example : $myCodonTable->is_unknown_codon('NTG')
  Returns : boolean
  Args    : codon
@@ -728,11 +806,11 @@ sub _unambiquous_codons{
     my ($i, $j, $k);
     @nts = map { $IUPAC_DNA{uc $_} }  split(//, $value);
     for my $i (@{$nts[0]}) {
-	for my $j (@{$nts[1]}) {
-	    for my $k (@{$nts[2]}) {
-		push @codons, lc "$i$j$k";
-	    }
-	}
+    for my $j (@{$nts[1]}) {
+        for my $k (@{$nts[2]}) {
+        push @codons, lc "$i$j$k";
+        }
+    }
     }
     return @codons;
 }
@@ -769,27 +847,27 @@ sub add_table {
 
 sub _make_iupac_string {
 
-	my ($self, $cod_ref) = @_;
-	if(ref($cod_ref) ne 'ARRAY') {
-		$self->throw(" I need a reference to a list of references to codons, ".
-					 " not a [". ref($cod_ref) . "].");
-		}
+    my ($self, $cod_ref) = @_;
+    if(ref($cod_ref) ne 'ARRAY') {
+        $self->throw(" I need a reference to a list of references to codons, ".
+                     " not a [". ref($cod_ref) . "].");
+        }
     my %iupac_hash   = Bio::Tools::IUPAC->iupac_rev_iub();
-	my $iupac_string = ''; ## the string to be returned
-	for my $aa (@$cod_ref) {
+    my $iupac_string = ''; ## the string to be returned
+    for my $aa (@$cod_ref) {
 
-		## scan through codon positions, record the differing values,	
-		# then look up in the iub hash
-		for my $index(0..2) {
-			my %h;
-			map { my $k = substr($_,$index,1);
-		 		$h{$k}  = undef;} @$aa;
-			my $lookup_key = join '', sort{$a cmp $b}keys %h;
+        ## scan through codon positions, record the differing values,   
+        # then look up in the iub hash
+        for my $index(0..2) {
+            my %h;
+            map { my $k = substr($_,$index,1);
+                $h{$k}  = undef;} @$aa;
+            my $lookup_key = join '', sort{$a cmp $b}keys %h;
 
             ## extend string 
-			$iupac_string .= $iupac_hash{uc$lookup_key};
-		}
-	}
+            $iupac_string .= $iupac_hash{uc$lookup_key};
+        }
+    }
     return $iupac_string;
 
 }
