@@ -1,7 +1,9 @@
-# $Id: GFF.pm,v 1.47 2004/12/28 19:21:39 scain Exp $
+# $Id: GFF.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Tools::GFF
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by the Bioperl core team
 #
 # Copyright Matthew Pocock
@@ -48,10 +50,9 @@ your result parser.
 
 =head1 GFF3 AND SEQUENCE DATA
 
-[added by cjm 2004/07/09]
-
 GFF3 supports sequence data; see
-http://song.sourceforge.net/gff3-jan04.shtml
+
+http://www.sequenceontology.org/gff3.shtml
 
 There are a number of ways to deal with this -
 
@@ -102,27 +103,37 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org          - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Matthew Pocock
 
-Email mrp@sanger.ac.uk
+Email mrp-at-sanger.ac.uk
 
 =head1 CONTRIBUTORS 
 
 Jason Stajich, jason-at-biperl-dot-org
 Chris Mungall, cjm-at-fruitfly-dot-org
 Steffen Grossmann [SG], grossman at molgen.mpg.de
+Malcolm Cook, mec-at-stowers-institute.org
 
 =head1 APPENDIX
 
@@ -134,24 +145,25 @@ The rest of the documentation details each of the object methods. Internal metho
 
 package Bio::Tools::GFF;
 
-use vars qw(@ISA $HAS_HTML_ENTITIES);
+use vars qw($HAS_HTML_ENTITIES);
 use strict;
 
-use Bio::Root::IO;
-use Bio::SeqAnalysisParserI;
 use Bio::Seq::SeqFactory;
 use Bio::LocatableSeq;
 use Bio::SeqFeature::Generic;
 
-@ISA = qw(Bio::Root::Root Bio::SeqAnalysisParserI Bio::Root::IO);
+use base qw(Bio::Root::Root Bio::SeqAnalysisParserI Bio::Root::IO);
+
+my $i = 0;
+my %GFF3_ID_Tags = map { $_ => $i++ } qw(ID Parent Target);
 
 =head2 new
 
  Title   : new
- Usage   : my $parser = new Bio::Tools::GFF(-gff_version => 2,
+ Usage   : my $parser = Bio::Tools::GFF->new(-gff_version => 2,
 					    -file        => "filename.gff");
            or
-           my $writer = new Bio::Tools::GFF(-gff_version => 3,
+           my $writer = Bio::Tools::GFF->new(-gff_version => 3,
 					    -file        => ">filename.gff3");
  Function: Creates a new instance. Recognized named parameters are -file, -fh,
            and -gff_version.
@@ -224,6 +236,7 @@ sub _parse_header{
 		-id    => unescape($seqid),
 		-start => $start,
 		-end   => $end,
+                -length => ($end - $start + 1),  ## make the length explicit
 		);
 	   $handled = 1;
 	 } elsif($line =~ /^(\#\#feature-ontology)/) {
@@ -332,7 +345,7 @@ sub _parse_sequence {
 sub next_segment{
    my ($self,@args) = @_;
    return shift @{ $self->{'segments'} } if defined $self->{'segments'};
-   return undef;
+   return;
 }
 
 =head2 next_feature
@@ -360,7 +373,7 @@ sub next_feature {
             # all forward refs have been seen; TODO
         }
         next if($gff_string =~ /^\#/ || $gff_string =~ /^\s*$/ ||
-                $gff_string =~ /^\/\//);
+                $gff_string =~ m{^//});
 
         while ($gff_string =~ /^\>(.+)/) {
             # fasta can be in header or footer
@@ -373,7 +386,7 @@ sub next_feature {
         }
         last; 
     }
-    return undef unless $gff_string;
+    return unless $gff_string;
 
     my $feat = Bio::SeqFeature::Generic->new();
     $self->from_gff_string($feat, $gff_string);
@@ -493,11 +506,15 @@ sub _from_gff2_string {
    # text-translated tab symbols but no "real" tabs, so splitting on
    # \t is safe, and $attribs gets the entire attributes field to be
    # parsed later
+   
+   # sendu: but the tag value pair can (should?) be separated by a tab. The
+   # 'no tabs' thing seems to apply only to the free text that is allowed for
+   # the value
 
    my ($seqname, $source, $primary, $start, 
        $end, $score, $strand, $frame, @attribs) = split(/\t+/, $string);
-   my $attribs = join '', @attribs;  # just in case the rule 
-                                     # against tab characters has been broken
+   my $attribs = join ' ', @attribs;
+   
    if ( !defined $frame ) {
        $feat->throw("[$string] does not look like GFF2 to me");
    }
@@ -532,7 +549,6 @@ sub _from_gff2_string {
    # run through each character one at a time and check it
    # NOTE: changed to foreach loop which is more efficient in perl
    # --jasons
-
    for my $a ( split //, $attribs ) { 
        # flag up on entering quoted text, down on leaving it
        if( $a eq '"') { $flag = ( $flag == 0 ) ? 1:0 }
@@ -735,7 +751,7 @@ sub _gff1_string{
 
    foreach my $tag ( $feat->all_tags ) {
        foreach my $value ( $feat->each_tag_value($tag) ) {
-	   $str .= " $tag=$value";
+	   $str .= " $tag=$value" if $value;
        }
    }
 
@@ -1001,53 +1017,87 @@ sub _gff3_string {
     } else {
 	$name = 'SEQ';
     }
-    my @groups,
-    my @all_tags = $feat->all_tags;
+
+    my @groups;
+
+    # force leading ID and Parent tags
+    my @all_tags =  grep { ! exists $GFF3_ID_Tags{$_} } $feat->all_tags;
+    for my $t ( sort { $GFF3_ID_Tags{$b} <=> $GFF3_ID_Tags{$a} }
+		keys %GFF3_ID_Tags ) {
+	unshift @all_tags, $t if $feat->has_tag($t);
+    }
+
     for my $tag ( @all_tags ) {
-	my $valuestr;	# a string which will hold one or more values 
-# for this tag, with quoted free text and 
-# space-separated individual values.
+	# next if $tag eq 'Target';
+	if ($tag eq 'Target' && ! $origfeat->isa('Bio::SeqFeature::FeaturePair')){  
+	    # simple Target,start,stop
+	    my($target_id, $b,$e,$strand) = $feat->get_tag_values($tag); 
+	    next unless(defined($e) && defined($b) && $target_id);			
+	    ($b,$e)= ($e,$b) if(defined $strand && $strand<0);
+	    $target_id =~ s/([\t\n\r%&\=;,])/sprintf("%%%X",ord($1))/ge;    
+	    push @groups, sprintf("Target=%s %d %d", $target_id,$b,$e);
+	    next;
+	}
+
+	my $valuestr;	
+	# a string which will hold one or more values 
+	# for this tag, with quoted free text and 
+	# space-separated individual values.
 	my @v;
 	for my $value ( $feat->each_tag_value($tag) ) {	    
 	    if(  defined $value && length($value) ) { 
-		$value =~ tr/ /+/;
-		if ($value =~ /[^a-zA-Z0-9\,\;\=\.:\%\^\*\$\@\!\+\_\?\-]/) {
-		    $value =~ s/\t/\\t/g;	# substitute tab and newline 
-# characters
-		    $value =~ s/\n/\\n/g;		# to their UNIX equivalents
+				#$value =~ tr/ /+/;  #spaces are allowed now
+                if ( ref $value eq 'Bio::Annotation::Comment') {
+                    $value = $value->text;
+                }
 
-# Unescaped quotes are not allowed in GFF3
-#		    $value = '"' . $value . '"';
+		if ($value =~ /[^a-zA-Z0-9\,\;\=\.:\%\^\*\$\@\!\+\_\?\-]/) {
+		    $value =~ s/\t/\\t/g; # substitute tab and newline 
+		    # characters
+		    $value =~ s/\n/\\n/g; # to their UNIX equivalents
+
+		    # Unescaped quotes are not allowed in GFF3
+		    #		    $value = '"' . $value . '"';
 		}
-		$value =~ s/([\t\n\=;,])/sprintf("%%%X",ord($1))/ge;
+		$value =~ s/([\t\n\r%&\=;,])/sprintf("%%%X",ord($1))/ge;
 	    } else {
-		# if it is completely empty, 
-		# then just make empty double 
-		# quotes
+				# if it is completely empty, 
+				# then just make empty double 
+				# quotes
 		$value = '""';
 	    }
 	    push @v, $value;
 	}
-	$tag= lcfirst($tag) unless ($tag =~/^ID|Name|Alias|Parent|Gap|Target$/);
+	# can we figure out how to improve this?
+	$tag= lcfirst($tag) unless ($tag 
+				    =~ /^(ID|Name|Alias|Parent|Gap|Target|Derives_from|Note|Dbxref|Ontology_term)$/);
 
 	push @groups, "$tag=".join(",",@v);
     }
-# Add Target information for Feature Pairs
-    if( ! $feat->has_tag('Target') && 
+    # Add Target information for Feature Pairs
+    if( $feat->has_tag('Target') && 
 	! $feat->has_tag('Group') &&
 	$origfeat->isa('Bio::SeqFeature::FeaturePair') ) {
-	push @groups, sprintf("Target=%s+%d+%d", 
-			      $origfeat->feature1->seq_id,
+
+	my $target_id = $origfeat->feature1->seq_id;
+	$target_id =~ s/([\t\n\r%&\=;,])/sprintf("%%%X",ord($1))/ge;    
+
+	push @groups, sprintf("Target=%s %d %d", 
+			      $target_id,
 			      ( $origfeat->feature1->strand < 0 ? 
 				( $origfeat->feature1->end,
 				  $origfeat->feature1->start) :
 				( $origfeat->feature1->start,
 				  $origfeat->feature1->end) 
-			      ));
+				));
     }
-    
-# unshift @groups, "ID=autogenerated$ID" unless ($feat->has_tag('ID'));
-    
+
+    # unshift @groups, "ID=autogenerated$ID" unless ($feat->has_tag('ID'));
+    if ( $feat->can('name') && defined($feat->name) ) {
+	# such as might be for Bio::DB::SeqFeature
+	unshift @groups, 'Name=' . $feat->name;
+    }
+
     my $gff_string = "";
     if ($feat->location->isa("Bio::Location::SplitLocationI")) {
 	my @locs = $feat->location->each_Location;
