@@ -1,7 +1,9 @@
-# $Id: Result.pm,v 1.15 2004/12/08 10:48:05 heikki Exp $ 
+# $Id: Result.pm 16123 2009-09-17 12:57:27Z cjfields $ 
 #
 # BioPerl module for Bio::Tools::Phylo::PAML::Result
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason-at-bioperl.org>
 #
 # Copyright Jason Stajich, Aaron Mackey
@@ -18,23 +20,72 @@ Bio::Tools::Phylo::PAML::Result - A PAML result set object
 
   # see Bio::Tools::Phylo::PAML for example usage
   use Bio::Tools::Phylo::PAML;
-  my $parser = new Bio::Tools::Phylo::PAML
+  my $parser = Bio::Tools::Phylo::PAML->new
     (-file => "./results/mlc", -dir => "./results/");
 
   # get the first/next result; a Bio::Tools::Phylo::PAML::Result object,
   # which isa Bio::SeqAnalysisResultI object.
   my $result = $parser->next_result();
 
-  my @seqs       = $result->get_seqs;
-  my $MLmatrix   = $result->get_MLmatrix; # get MaxLikelihood Matrix
-  my $NGmatrix   = $result->get_NGmatrix; # get Nei-Gojoburi Matrix
-  my @basfreq    = $result->get_codon_pos_basefreq;
+  my @seqs         = $result->get_seqs;
+  my %input_params = $result->get_input_parameters;
+  my @basfreq      = $result->get_codon_pos_basefreq;
+  my $MLmatrix     = $result->get_MLmatrix; # get MaxLikelihood Matrix
+  my $NGmatrix     = $result->get_NGmatrix; # get Nei-Gojoburi Matrix
 
 
   # for AAML runs
   my $AAmatrix   = $result->get_AADistMatrix;
   my $AAMLmatrix   = $result->get_AAMLDistMatrix;
 
+  # if -dir contains an rst file get list of
+  # Bio::PrimarySeq ancestral state reconstructions of the sequences
+  my @rsts          = $result->get_rst_seqs; 
+
+
+  # if you want to print the changes on the tree
+  # this will print out the 
+  # anc_aa       => ANCESTRAL AMINO ACID
+  # anc_prob     => ANCESTRAL AA PROBABILITY 
+  # derived_aa   => DERIVED AA
+  # derived_prob => DERIVE AA PROBABILITY (where appropriate - NA for extant/tip taxas)
+  # site         => which codon site this in the alignment
+    @trees = $result->get_rst_trees;
+    for my $t ( @trees ) {
+	for my $node ( $t->get_nodes ) {	
+	    next unless $node->ancestor; # skip root node
+	    my @changes = $node->get_tag_values('changes');
+	    my $chgstr = '';
+	    for my $c ( @changes ) { 
+		for my $k ( sort keys %$c ) {
+		    $chgstr .= "$k => $c->{$k} ";
+		}
+		$chgstr .= "\n\t";
+	    }
+
+	    printf "node:%s n=%s s=%s\n\t%s\n",
+	    $node->id, 
+	    $node->get_tag_values('n'),
+	    $node->get_tag_values('s'),
+	    $chgstr;
+	}
+    }
+
+  # Persite probabilities
+  my $persite = $result->get_rst_persite;
+  # let's score site 1
+  $site = $persite->[2]; 
+  # so site 2, node 2 (extant node, node 2)
+  print $site->[2]->{'codon'}, ' ',$site->[2]->{'aa'},"\n";
+  # site 2, node 3
+  print $site->[3]->{'codon'}, ' ',$site->[3]->{'aa'}, "\n";
+
+  # ancestral node 9, codon, aa, marginal probabilities; Yang95 is listed as 
+  #  (eqn. 4 in Yang et al. 1995 Genetics 141:1641-1650) in PAML rst file.
+  print $site->[9]->{'codon'}, ' ',$site->[9]->{'aa'}, ' ', $site->[9]->{'prob'}, ' ',
+        $site->[9]->{'Yang95_aa'},' ', $site->[9]->{'Yang95_aa_prob'},"\n";
+
+
 =head1 DESCRIPTION
 
 This is a container object for PAML Results.
@@ -47,8 +98,19 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to
 the Bioperl mailing list.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org              - General discussion
-  http://bioperl.org/MailList.shtml  - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
@@ -56,18 +118,16 @@ Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via
 email or the web:
 
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Jason Stajich, Aaron Mackey
 
-Email jason-at-bioperl-dot-org
-Email amackey-at-virginia-dot-edu
-
-Describe contact details here
+ Email jason-at-bioperl-dot-org
+ Email amackey-at-virginia-dot-edu
 
 =head1 CONTRIBUTORS
 
-Additional contributors names and emails here
+Albert Vilella avilella-AT-gmail-DOT-com
 
 =head1 APPENDIX
 
@@ -81,23 +141,20 @@ Internal methods are usually preceded with a _
 
 
 package Bio::Tools::Phylo::PAML::Result;
-use vars qw(@ISA);
 use strict;
 
-use Bio::Root::Root;
-use Bio::AnalysisResultI;
 
-@ISA = qw(Bio::Root::Root Bio::AnalysisResultI);
+use base qw(Bio::Root::Root Bio::AnalysisResultI);
 
 =head2 new
 
  Title   : new
- Usage   : my $obj = new Bio::Tools::Phylo::PAML::Result(%data);
+ Usage   : my $obj = Bio::Tools::Phylo::PAML::Result->new(%data);
  Function: Builds a new Bio::Tools::Phylo::PAML::Result object
  Returns : Bio::Tools::Phylo::PAML::Result
- Args    : -trees     => array reference of L<Bio::Tree::TreeI> objects
+ Args    : -trees     => array reference of Bio::Tree::TreeI objects
            -MLmatrix  => ML matrix
-           -seqs      => array reference of L<Bio::PrimarySeqI> objects
+           -seqs      => array reference of Bio::PrimarySeqI objects
            -codonpos  => array reference of codon positions 
            -codonfreq => array reference of codon frequencies
            -version   => version string
@@ -108,9 +165,18 @@ use Bio::AnalysisResultI;
            -aadistmat => Bio::Matrix::PhylipDist   (only for AAML)
            -aamldistmat => Bio::Matrix::PhylipDist   (only for pairwise AAML)
            -ntfreq    => array ref of NT frequencies (only for BASEML)
+           -seqfile    => seqfile used
            -kappa_mat => Bio::Matrix::PhylipDist of kappa values (only for BASEML)
            -alpha_mat => Bio::Matrix::PhylipDist of alpha values (only for BASEML)
            -NSSitesresult => arrayref of PAML::ModelResult 
+           -input_params  => input params from .ctl file 
+           -rst       => array reference of Bio::PrimarySeqI objects
+                         of ancestral state reconstruction
+           -rst_persite=> arrayref of persite data, this is a complicated set of AoH
+           -rst_trees  => rst trees with changes coded on the tree
+
+See Also: L<Bio::Tree::TreeI>, L<Bio::PrimarySeqI>, L<Bio::Matrix::PhylipDist>, L<Bio::Tools::Phylo::PAML>
+
 
 =cut
 
@@ -123,22 +189,26 @@ sub new {
       $model,$patterns, $stats,
       $aafreq, $aadistmat, 
       $aamldistmat,
-      $ntfreqs, $kappa_mat,$alpha_mat,
-      $NSSitesresults ) = $self->_rearrange([qw(TREES MLMATRIX 
-						SEQS NGMATRIX
-						CODONPOS CODONFREQ
-						VERSION MODEL PATTERNS
-						STATS AAFREQ AADISTMAT
-						AAMLDISTMAT 
-						NTFREQ
-						KAPPA_DISTMAT
-						ALPHA_DISTMAT
-						NSSITESRESULTS)], 
-				       @args);
+      $ntfreqs, $seqfile, $kappa_mat, $alpha_mat,
+      $NSSitesresults,$input_params,$rst,$rst_persite,$rst_trees ) = 
+	  $self->_rearrange([qw
+			     (TREES MLMATRIX 
+			      SEQS NGMATRIX
+			      CODONPOS CODONFREQ
+			      VERSION MODEL PATTERNS
+			      STATS AAFREQ AADISTMAT
+			      AAMLDISTMAT 
+			      NTFREQ SEQFILE
+			      KAPPA_DISTMAT
+			      ALPHA_DISTMAT
+			      NSSITESRESULTS
+			      INPUT_PARAMS
+			      RST RST_PERSITE RST_TREES)], 
+			    @args);
   $self->reset_seqs;
   if( $trees ) {
       if(ref($trees) !~ /ARRAY/i ) { 
-	  $self->warn("Must have provided a valid array reference to initialize trees");
+	  $self->warn("Must provide a valid array reference to initialize trees");
       } else { 
 	  foreach my $t ( @$trees ) {
 	      $self->add_tree($t);
@@ -149,14 +219,14 @@ sub new {
 
   if( $mlmat ) {
       if( ref($mlmat) !~ /ARRAY/i ) {
-	  $self->warn("Must have provided a valid array reference to initialize MLmatrix");
+	  $self->warn("Must provide a valid array reference to initialize MLmatrix");
       } else { 
 	  $self->set_MLmatrix($mlmat);
       }
   } 
   if( $seqs ) { 
       if( ref($seqs) !~ /ARRAY/i ) {
-	  $self->warn("Must have provided a valid array reference to initialize seqs");
+	  $self->warn("Must provide a valid array reference to initialize seqs");
       } else {
 	  foreach my $s ( @$seqs ) {
 	      $self->add_seq($s);
@@ -165,7 +235,7 @@ sub new {
   }
   if( $ngmatrix ) {
       if( ref($ngmatrix) !~ /ARRAY/i ) {
-	  $self->warn("Must have provided a valid array reference to initialize NGmatrix");
+	  $self->warn("Must provide a valid array reference to initialize NGmatrix");
       } else { 
 	  $self->set_NGmatrix($ngmatrix);
       }
@@ -174,25 +244,26 @@ sub new {
       if( ref($codonfreq) =~ /ARRAY/i ) {
 	  $self->set_CodonFreqs($codonfreq);
       } else { 
-	  $self->warn("Must have provided a valid array reference to initialize codonfreq");
+	  $self->warn("Must provide a valid array reference to initialize codonfreq");
       }
   }
 
   if( $codonpos ) {
       if( ref($codonpos) !~ /ARRAY/i ) {
-	  $self->warn("Must have provided a valid array reference to initialize codonpos");
+	  $self->warn("Must provide a valid array reference to initialize codonpos");
       } else { 
 	  $self->set_codon_pos_basefreq(@$codonpos);
       }
   }
 
   $self->version($version)   if defined $version;
+  $self->seqfile($seqfile)   if defined $seqfile;
   $self->model($model)       if defined $model;
   if( defined $patterns ) {
       if( ref($patterns) =~ /HASH/i ) {
 	  $self->patterns($patterns);
       } else {
-	  $self->warn("Must have provided a valid array reference to initialize patterns");
+	  $self->warn("Must provide a valid array reference to initialize patterns");
       }
   }
 
@@ -201,7 +272,7 @@ sub new {
       if( ref($aafreq) =~ /HASH/i ) {
 	  $self->set_AAFreqs($aafreq);
       } else { 
-	  $self->warn("Must have provided a valid hash reference to initialize aafreq");
+	  $self->warn("Must provide a valid hash reference to initialize aafreq");
       }
   }
   if( $stats ) {
@@ -210,7 +281,7 @@ sub new {
 	      $self->add_stat($stat,$val);
 	  }
       } else { 
-	  $self->warn("Must have provided a valid hash reference initialize stats");
+	  $self->warn("Must provide a valid hash reference initialize stats");
       }
   }
   $self->set_AADistMatrix($aadistmat) if defined $aadistmat;
@@ -231,7 +302,7 @@ sub new {
       if( ref($ntfreqs) =~ /HASH/i ) {
 	  $self->set_NTFreqs($ntfreqs);
       } else { 
-	  $self->warn("Must have provided a valid hash reference to initialize ntfreq");
+	  $self->warn("Must provide a valid hash reference to initialize ntfreq");
       }
   }
 
@@ -241,6 +312,41 @@ sub new {
   if( $alpha_mat ) {
       $self->set_AlphaMatrix($alpha_mat);
   }
+
+  if( $input_params ) {
+      if(  ref($input_params) !~ /HASH/i ) {
+	  $self->warn("Must provide a valid hash object for input_params\n");
+      } else {
+	  while( my ($p,$v) = each %$input_params ) {
+	      $self->set_input_parameter($p,$v);
+	  }
+      }
+      
+  }
+  $self->reset_rst_seqs;
+  if( $rst ) {
+      if( ref($rst) =~ /ARRAY/i ) {	  
+	  for ( @$rst ) {
+	      $self->add_rst_seq($_);
+	  }
+      } else {
+	  $self->warn("Need a valid array ref for -rst option\n");
+      }
+  }
+  if( defined $rst_persite ) {
+      $self->set_rst_persite($rst_persite);
+  }
+  $self->reset_rst_trees;
+  if( $rst_trees ) {
+      if( ref($rst_trees) =~ /ARRAY/i ) {	  
+	  for ( @$rst_trees ) {
+	      $self->add_rst_tree($_);
+	  }
+      } else {
+	  $self->warn("Need a valid array ref for -rst_trees option\n");
+      }
+  }
+
   return $self;
 }
 
@@ -275,10 +381,10 @@ sub get_trees{
    return @{$self->{'_trees'} || []};
 }
 
-=head2 rewind_tree
+=head2 rewind_tree_iterator
 
  Title   : rewind_tree_iterator
- Usage   : $result->rewind_tree()
+ Usage   : $result->rewind_tree_iterator()
  Function: Rewinds the tree iterator so that next_tree can be 
            called again from the beginning
  Returns : none
@@ -504,6 +610,23 @@ sub version{
    return $self->{'_version'};
 }
 
+=head2 seqfile
+
+ Title   : seqfile
+ Usage   : $obj->seqfile($newval)
+ Function: Get/Set seqfile
+ Returns : value of seqfile
+ Args    : newvalue (optional)
+
+
+=cut
+
+sub seqfile{
+   my $self = shift;
+   $self->{'_seqfile'} = shift if @_;
+   return $self->{'_seqfile'};
+}
+
 =head2 model
 
  Title   : model
@@ -724,7 +847,7 @@ sub set_AADistMatrix{
        $self->warn("Must provide a valid Bio::Matrix::MatrixI for set_AADistMatrix");
    }
    $self->{'_AADistMatix'} = $d;
-   return undef;
+   return;
 }
 
 =head2 get_AAMLDistMatrix
@@ -762,7 +885,7 @@ sub set_AAMLDistMatrix{
        $self->warn("Must provide a valid Bio::Matrix::MatrixI for set_AAMLDistMatrix");
    }
    $self->{'_AAMLDistMatix'} = $d;
-   return undef;
+   return;
 }
 
 =head2 add_NSSite_result
@@ -874,7 +997,7 @@ sub set_KappaMatrix{
        $self->warn("Must provide a valid Bio::Matrix::MatrixI for set_NTDistMatrix");
    }
    $self->{'_KappaMatix'} = $d;
-   return undef;
+   return;
 }
 
 
@@ -913,7 +1036,204 @@ sub set_AlphaMatrix{
        $self->warn("Must provide a valid Bio::Matrix::MatrixI for set_NTDistMatrix");
    }
    $self->{'_AlphaMatix'} = $d;
-   return undef;
+   return;
 }
 
+=head2 set_input_parameter
+
+ Title   : set_input_parameter
+ Usage   : $obj->set_input_parameter($p,$vl);
+ Function: Set an Input Parameter 
+ Returns : none
+ Args    : $parameter and $value
+
+
+=cut
+
+sub set_input_parameter{
+   my ($self,$p,$v) = @_;
+   return unless defined $p;
+   $self->{'_input_parameters'}->{$p} = $v;
+}
+
+=head2 get_input_parameters
+
+ Title   : get_input_parameters
+ Usage   : $obj->get_input_parameters;
+ Function: Get Input Parameters 
+ Returns : Hash of key/value pairs
+ Args    : none
+
+
+=cut
+
+sub get_input_parameters{
+   my ($self) = @_;
+   return %{$self->{'_input_parameters'} || {}};
+}
+
+=head2 reset_input_parameters
+
+ Title   : reset_input_parameters
+ Usage   : $obj->reset_input_parameters;
+ Function: Reset the Input Parameters hash 
+ Returns : none
+ Args    : none
+
+
+=cut
+
+sub reset_input_parameters{
+   my ($self) = @_;
+   $self->{'_input_parameters'} = {};
+}
+
+=head1 Reconstructed Ancestral State relevant options 
+
+=head2 add_rst_seq
+
+ Title   : add_rst_seq
+ Usage   : $obj->add_rst_seq($seq)
+ Function: Add a Bio::PrimarySeq to the RST Result
+ Returns : none
+ Args    : Bio::PrimarySeqI
+See also : L<Bio::PrimarySeqI>
+
+=cut
+
+sub add_rst_seq{
+   my ($self,$seq) = @_;
+   if( $seq ) { 
+       unless( $seq->isa("Bio::PrimarySeqI") ) {
+	   $self->warn("Must provide a valid Bio::PrimarySeqI to add_rst_seq");
+	   return;
+       }
+       push @{$self->{'_rstseqs'}},$seq;
+   }
+
+}
+
+=head2 reset_rst_seqs
+
+ Title   : reset_rst_seqs
+ Usage   : $result->reset_rst_seqs
+ Function: Reset the RST seqs stored
+ Returns : none
+ Args    : none
+
+
+=cut
+
+sub reset_rst_seqs{
+   my ($self) = @_;
+   $self->{'_rstseqs'} = [];
+}
+
+=head2 get_rst_seqs
+
+ Title   : get_rst_seqs
+ Usage   : my @otus = $result->get_rst_seqs
+ Function: Get the seqs Bio::PrimarySeq
+ Returns : Array of Bio::PrimarySeqI objects
+ Args    : None
+See also : L<Bio::PrimarySeq>
+
+=cut
+
+sub get_rst_seqs{
+   my ($self) = @_;
+   return @{$self->{'_rstseqs'} || []};
+}
+
+
+=head2 add_rst_tree
+
+ Title   : add_rst_tree
+ Usage   : $obj->add_rst_tree($tree)
+ Function: Add a Bio::Tree::TreeI to the RST Result
+ Returns : none
+ Args    : Bio::Tree::TreeI
+See also : L<Bio::Tree::TreeI>
+
+=cut
+
+sub add_rst_tree{
+   my ($self,$tree) = @_;
+   if( $tree ) { 
+       unless( $tree->isa("Bio::Tree::TreeI") ) {
+	   $self->warn("Must provide a valid Bio::Tree::TreeI to add_rst_tree not $tree");
+	   return;
+       }
+       push @{$self->{'_rsttrees'}},$tree;
+   }
+}
+
+=head2 reset_rst_trees
+
+ Title   : reset_rst_trees
+ Usage   : $result->reset_rst_trees
+ Function: Reset the RST trees stored
+ Returns : none
+ Args    : none
+
+
+=cut
+
+sub reset_rst_trees{
+   my ($self) = @_;
+   $self->{'_rsttrees'} = [];
+}
+
+=head2 get_rst_trees
+
+ Title   : get_rst_trees
+ Usage   : my @otus = $result->get_rst_trees
+ Function: Get the trees Bio::Tree::TreeI
+ Returns : Array of Bio::Tree::TreeI objects
+ Args    : None
+See also : L<Bio::Tree::TreeI>
+
+=cut
+
+sub get_rst_trees{
+   my ($self) = @_;
+   return @{$self->{'_rsttrees'} || []};
+}
+
+=head2 set_rst_persite
+
+ Title   : set_rst_persite
+ Usage   : $obj->set_rst_persite($newval)
+ Function: Get/Set the per-site RST values
+ Returns : value of set_rst_persite (a scalar)
+ Args    : on set, new value (a scalar or undef, optional)
+
+
+=cut
+
+sub set_rst_persite{
+    my $self = shift;
+
+    return $self->{'_rstpersite'} = shift if @_;
+    return $self->{'_rstpersite'};
+}
+
+=head2 get_rst_persite
+
+ Title   : get_rst_persite
+ Usage   : my @rst_persite = @{$result->get_rst_persite()} 
+ Function: Get the per-site RST values
+ Returns : Array
+ Args    : none
+
+
+=cut
+
+sub get_rst_persite{
+   my ($self) = @_;
+   return $self->{'_rstpersite'} || [];
+}
+
+
+
 1;
