@@ -1,10 +1,14 @@
-# $Id: RemoteBlast.pm,v 1.21 2004/11/17 23:54:50 bosborne Exp $
+# $Id: RemoteBlast.pm 16123 2009-09-17 12:57:27Z cjfields $
 #
 # BioPerl module for Bio::Tools::Run::RemoteBlast
 #
-# Cared for by Jason Stajich, Mat Wiepert
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
 #
-# Copyright Jason Stajich
+# FORMERLY Cared for by Jason Stajich, Mat Wiepert
+#
+# Somewhat cared for by Roger Hall, Chris Fields (when they have time)
+#
+# Copyright Jason Stajich, Bioperl
 #
 # You may distribute this module under the same terms as perl itself
 
@@ -86,7 +90,8 @@ via HTTP
   }
 
   # This example shows how to change a CGI parameter:
-  $Bio::Tools::Run::RemoteBlast::HEADER{'MATRIX_NAME'} = 'BLOSUM25';
+  $Bio::Tools::Run::RemoteBlast::HEADER{'MATRIX_NAME'} = 'BLOSUM45';
+  $Bio::Tools::Run::RemoteBlast::HEADER{'GAPCOSTS'} = '15 2';
 
   # And this is how to delete a CGI parameter:
   delete $Bio::Tools::Run::RemoteBlast::HEADER{'FILTER'};
@@ -109,21 +114,36 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org               - General discussion
-  http://bio.perl.org/MailList.html   - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bio.perl.org/bioperl-bugs/
+  http://bugzilla.bioperl.org
 
-=head1 AUTHOR -  Jason Stajich
+=head1 AUTHOR 
 
-Email jason@bioperl.org
+Please do NOT contact Jason directly about this module.  Please post to
+the bioperl mailing list (L<FEEDBACK>). If you would like to be the
+official maintainer of this module, please volunteer on the list and
+we will make it official in this POD.
+
+First written by Jason Stajich, many others have helped keep it running.
 
 =head1 APPENDIX
 
@@ -134,31 +154,29 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Tools::Run::RemoteBlast;
 
-use vars qw($AUTOLOAD @ISA $URLBASE %HEADER %RETRIEVALHEADER
+use vars qw($AUTOLOAD $URLBASE %HEADER %RETRIEVALHEADER
 	    $RIDLINE $MODVERSION %PUTPARAMS %GETPARAMS);
 use strict;
 
-use Bio::Root::Root;
-use Bio::Root::IO;
 use Bio::SeqIO;
 use IO::String;
-use Bio::Tools::BPlite;
 use Bio::SearchIO;
 use LWP;
 use HTTP::Request::Common;
 
-@ISA = qw(Bio::Root::Root Bio::Root::IO);
+use base qw(Bio::Root::Root Bio::Root::IO);
 
 BEGIN {
     $MODVERSION = $Bio::Root::Version::VERSION;
-    $URLBASE = 'http://www.ncbi.nlm.nih.gov/blast/Blast.cgi';
+    $URLBASE = 'http://blast.ncbi.nlm.nih.gov/Blast.cgi';
 
     # In GET/PUTPARAMS the values are regexes which validate the input.
     %PUTPARAMS = (
 	'AUTO_FORMAT' 	=> '(Off|(Semi|Full)auto)',	# Off, Semiauto, Fullauto
-	'COMPOSITION_BASED_STATISTICS'	=> '(yes|no)',	# yes, no
+	'COMPOSITION_BASED_STATISTICS'	=> '(0|1)',	# yes, no on NCBI's site, but actually binary 0/1
 	'DATABASE' 	=>  '.*',
 	'DB_GENETIC_CODE' => '([1-9]|1[1-6]|2(1|2))',   # 1..16,21,22
+    'DISPLAY_SORT'   => '\d',
 	'ENDPOINTS'	=> '(yes|no)',			# yes,no
 	'ENTREZ_QUERY'	=> '.*',
 	'EXPECT'	=> '\d+(\.\d+)?([eE]-\d+)?',	# Positive double
@@ -186,6 +204,7 @@ BEGIN {
 	'SEARCHSP_EFF'	=> '\d+',			# Positive integer
 	'SERVICE'	=> '(plain|p[sh]i|(rps|mega)blast)',
 					# plain,psi,phi,rpsblast,megablast
+    'SHORT_QUERY_ADJUST' => '(true|false)',
 	'THRESHOLD'	=> '-?\d+',			# Integer
 	'UNGAPPED_ALIGNMENT' => '(yes|no)',		# yes, no
 	'WORD_SIZE'	=> '\d+'			# Positive integer
@@ -193,9 +212,10 @@ BEGIN {
     %GETPARAMS = (
    'ALIGNMENTS'	=> '\d+',			# Positive integer
 	'ALIGNMENT_VIEW' =>
-   '(Pairwise|(Flat)?QueryAnchored(NoIdentities)?|Tabular)',
+		  '(Pairwise|(Flat)?QueryAnchored(NoIdentities)?|Tabular)',
 	 # Pairwise, QueryAnchored, QueryAnchoredNoIdentities, 
-  	 # FlatQueryAnchored, FlatQueryAnchoredNoIdentities, Tabular 
+  	 # FlatQueryAnchored, FlatQueryAnchoredNoIdentities, Tabular
+     'DATABASE_SORT' => '\d',
 	 'DESCRIPTIONS'	=> '\d+',			# Positive integer
 	 'ENTREZ_LINKS_NEW_WINDOW' => '(yes|no)',	# yes, no
 	 'EXPECT_LOW'	=> '\d+(\.\d+)?([eE]-\d+)?',	# Positive double
@@ -208,6 +228,7 @@ BEGIN {
 	 'FORMAT_TYPE'	=> '((HT|X)ML|ASN\.1|Text)',
 					# HTML, Text, ASN.1, XML
 	 'NCBI_GI'	=> '(yes|no)',			# yes, no
+     'NEW_VIEW' => '(true|false)',
 	 'RID' 		=>  '.*',
 	 'RESULTS_FILE' 	=>  '(yes|no)',			# yes, no
 	 'SERVICE' 	=>  '(plain|p[sh]i|(rps|mega)blast)',
@@ -217,25 +238,25 @@ BEGIN {
 
     # Default values go in here for PUT
     %HEADER = (
-          'CMD'                          => 'Put',
+	       'CMD'                          => 'Put',
 	       'FORMAT_OBJECT'                => 'Alignment',
 	       'COMPOSITION_BASED_STATISTICS' => 'off', 
-	       'DATABASE'	    	              => 'nr',
-	       'EXPECT'			              => '1e-3', 
-	       'FILTER'			              => 'L', 
-	       'PROGRAM'		                 => 'blastp', 
-	       'SERVICE'		                 => 'plain' 
+	       'DATABASE'	    	      => 'nr',
+	       'EXPECT'			      => '1e-3', 
+	       'FILTER'			      => 'L', 
+	       'PROGRAM'		      => 'blastp', 
+	       'SERVICE'		      => 'plain' 
 	       );
-
+    
     # Default values go in here for GET
     %RETRIEVALHEADER = (
-         'CMD'            => 'Get',
-			'ALIGNMENTS'	  => '50',
+			'CMD'            => 'Get',
+			'ALIGNMENTS'	 => '50',
 			'ALIGNMENT_VIEW' => 'Pairwise',
-			'DESCRIPTIONS'	  => '100',
-			'FORMAT_TYPE'	  => 'Text'
+			'DESCRIPTIONS'	 => '100',
+			'FORMAT_TYPE'	 => 'Text',
 			);
-
+    
     $RIDLINE = 'RID\s+=\s+(\S+)';
 }
 
@@ -245,9 +266,8 @@ sub new {
 	my $self = $caller->SUPER::new(@args);
 	# so that tempfiles are cleaned up
 	$self->_initialize_io();
-	my ($prog, $data,
-		 $readmethod) = $self->_rearrange([qw(PROG DATA
-					     READMETHOD)],
+	my ($prog, $data, $readmethod, $url_base) =
+        $self->_rearrange([qw(PROG DATA READMETHOD URL_BASE)],
 					 @args);
 	# Use these two parameters for backward-compatibility. 
 	# Overridden by PROGRAM and DATABASE if supplied.
@@ -273,7 +293,10 @@ sub new {
 	foreach my $getName (keys %getNames) {
 		$self->retrieve_parameter($getName,$getNames{$getName});
 	}
-
+        # private variable to keep track of total rids
+    $self->{'_total_rids'} = 0;
+    $url_base ||= $URLBASE;  # default to regular NCBI BLAST URL
+    $self->set_url_base($url_base);
 	return $self;
 }
 
@@ -348,14 +371,17 @@ sub header {
  Usage   : my $readmethod = $self->readmethod
  Function: Get/Set the method to read the blast report
  Returns : string
- Args    : string [ Blast, BPlite ]
+ Args    : string [ blast, blasttable, xml ]
 
 =cut
 
 sub readmethod {
     my ($self, $val) = @_;
     if( defined $val ) {
-	$self->{'_readmethod'} = $val;
+        if ($val =~ /bplite/i) {
+            $self->throw("Use of Bio::Tools::BPlite is deprecated; use Bio::SearchIO modules instead");
+        }
+        $self->{'_readmethod'} = $val;
     }
     return $self->{'_readmethod'};
 }
@@ -423,7 +449,7 @@ sub expect {
 sub ua {
     my ($self, $value) = @_;    
     if( ! defined $self->{'_ua'} ) {
-	$self->{'_ua'} = new LWP::UserAgent();
+	$self->{'_ua'} = LWP::UserAgent->new(env_proxy => 1, parse_head => 0);
 	my $nm = ref($self);
 	$nm =~ s/::/_/g;
 	$self->{'_ua'}->agent("bioperl-$nm/$MODVERSION");
@@ -445,7 +471,7 @@ sub ua {
 
 sub proxy {
     my ($self,$protocol,$proxy) = @_;
-    return undef if ( !defined $self->ua || !defined $protocol
+    return if ( !defined $self->ua || !defined $protocol
 		      || !defined $proxy );
     return $self->ua->proxy($protocol,$proxy);
 }
@@ -453,7 +479,8 @@ sub proxy {
 sub add_rid {
     my ($self, @vals) = @_;
     foreach ( @vals ) {
-	$self->{'_rids'}->{$_} = 1;
+	$self->{'_rids'}->{$_} = $self->{'_total_rids'};
+        $self->{'_total_rids'}++; 
     }
     return scalar keys %{$self->{'_rids'}};
 }
@@ -468,7 +495,9 @@ sub remove_rid {
 
 sub each_rid {
     my ($self) = @_;
-    return keys %{$self->{'_rids'}};
+    # sort on key value, a little tricky...
+    my @sort_rids = sort {$self->{'_rids'}->{$a} <=> $self->{'_rids'}->{$b}} keys %{$self->{'_rids'}};
+    return @sort_rids;
 }
 
 =head2 submit_blast
@@ -487,14 +516,16 @@ sub each_rid {
 sub submit_blast {
     my ($self, $input) = @_;
     my @seqs = $self->_load_input($input);
+    my $url_base = $self->get_url_base;
     return 0 unless ( @seqs );
     my $tcount = 0;
     my %header = $self->header;
+    $header{$_} ||= $RETRIEVALHEADER{$_} foreach (keys %RETRIEVALHEADER);    
     foreach my $seq ( @seqs ) {
 	#If query has a fasta header, the output has the query line.
 	$header{'QUERY'} = ">".(defined $seq->display_id() ? $seq->display_id() : "").
 		" ".(defined $seq->desc() ? $seq->desc() : "")."\n".$seq->seq();
-	my $request = POST $URLBASE, [%header];
+	my $request = POST $url_base, [%header];
 	$self->warn($request->as_string) if ( $self->verbose > 0);
 	my $response = $self->ua->request( $request);
 
@@ -504,15 +535,15 @@ sub submit_blast {
 	    foreach ( @subdata ) {
 			if( /$RIDLINE/ ) {
 		    	$count++;
-		    	print STDERR $_ if( $self->verbose > 0);
-		    	$self->add_rid($1);		
+		    	$self->debug("RID: $1\n");
+		    	$self->add_rid($1);
 		    	last;
-			}	
+			}
 	    }
 	    if( $count == 0 ) {
-			$self->warn("req was ". $request->as_string() . "\n");
-			$self->warn(join('', @subdata));
-	    	}    	
+            $self->warn("req was ". $request->as_string() . "\n");
+            $self->warn(join('', @subdata));
+	    }    	
 	    $tcount += $count;
 	} else {
 	    # should try and be a little more verbose here
@@ -531,56 +562,98 @@ sub submit_blast {
  Function: Attempts to retrieve a blast report from remote blast queue
  Returns : -1 on error,
            0 on 'job not finished',
-           Bio::Tools::BPlite or Bio::Tools::Blast object
-           (depending on how object was initialized) on success
+           Bio::SearchIO object
  Args    : Remote Blast ID (RID)
 
 =cut
 
 sub retrieve_blast {
-	my($self, $rid) = @_;
-	my ($fh,$tempfile) = $self->tempfile();
-	close $fh; #explicit close
-	my %hdr = %RETRIEVALHEADER;
-	$hdr{'RID'} = $rid;
-	my $req = POST $URLBASE, [%hdr];
-	if( $self->verbose > 0 ) {
-		$self->warn("retrieve request is " . $req->as_string());
-	}
-	my $response = $self->ua->request($req, $tempfile);
-	if( $response->is_success ) {
+    my($self, $rid) = @_;
+    my ($fh,$tempfile) = $self->tempfile();
+    close $fh;			#explicit close
+    my $url_base = $self->get_url_base;
+    my %hdr = %RETRIEVALHEADER;
+    $hdr{'RID'} = $rid;
+    my $req = POST $url_base, [%hdr];
+    $self->debug("retrieve request is " . $req->as_string());
+    my $response = $self->ua->request($req, $tempfile);
+    if( $response->is_success ) {
     	if( $self->verbose > 0 ) {
-			#print content of reply if verbose > 1
-			open(TMP, $tempfile) or $self->throw("cannot open $tempfile");
-			while(<TMP>) { print $_; }
-			close TMP;
-		}
-		## if proper reply 
-		my $size = -s $tempfile;
-		if( $size > 1000 ) {
-	    	my $blastobj;
-			if( $self->readmethod =~ /BPlite/ ) {
-				$blastobj = new Bio::Tools::BPlite(-file => $tempfile);
-	    	} else {
-				$blastobj = new Bio::SearchIO( -file     => $tempfile,
-														 -format => 'blast');
-			}
-
-			## store filename in object ##
-			$self->file($tempfile);
-	    	return $blastobj;
-		} elsif( $size < 500 ) { # search had a problem
-			open(ERR, "<$tempfile") or $self->throw("cannot open file $tempfile"); 
-			$self->warn(join("", <ERR>));
-			close ERR;
-	    	return -1;
-		} else { # still working
-	    	return 0;
-		}
-	} else {
-		$self->warn($response->error_as_HTML);
-		return -1;
-	}
+            #print content of reply if verbose > 1
+            open(my $DEBUG, $tempfile) || $self->throw("cannot open $tempfile");
+            while(<$DEBUG>) { print $_; }
+            close $DEBUG;
+    	}
+        open(my $TMP, $tempfile) || $self->throw("Error opening $tempfile");
+        ## if proper reply 
+        my $waiting = 1;
+        my $s = 0;
+        my $got_content = 0;
+        my $is_tabular = 0;
+        while(my $line = <$TMP>) {
+            if ($line =~ /./) {
+                $got_content = 1;
+            }
+            if($line =~ /<\?xml version=/ ) { # xml time
+                $waiting = 0;
+                $self->readmethod('blastxml');
+                last;
+            }
+            if($line =~ /QBlastInfoBegin/i ) {
+                $s = 1;
+            } elsif( $s ) {
+                if($line =~ /Status=(WAITING|ERROR|READY)/i ) {
+                    my $status = $1;
+                    if( $status eq 'WAITING' ) {
+                        $waiting = 1;
+                    } elsif( $status eq 'ERROR' ) {
+                        close($TMP);
+                        open(my $ERR, "<$tempfile") or $self->throw("cannot open file $tempfile");
+                        $self->warn(join("", <$ERR>));
+                        close $ERR;
+                        return -1;
+                    } elsif( $status eq 'READY' ) {
+                        $waiting = 0;
+                        last;
+                    } else {
+                        $self->warn("Unknown status $1:\n");
+                        last;
+                    }
+                }
+            } elsif ($line =~ /ERROR/i ) {
+                close($TMP);
+                open(my $ERR, "<$tempfile") or $self->throw("cannot open file $tempfile");
+                $self->warn(join("", <$ERR>));
+                close $ERR;
+                return -1;
+            }
+            
+        }
+        close($TMP);
+        if( ! $waiting ) {
+            my $blastobj;
+            my $mthd = $self->readmethod;
+            $mthd = ($mthd =~ /blasttable/i) ? 'blasttable' :
+                    ($mthd =~ /xml/i)        ? 'blastxml'   :
+                    ($mthd =~ /pull/i)       ? 'blast_pull' :
+                    'blast';
+            $blastobj = Bio::SearchIO->new( -file => $tempfile,
+                               -format => $mthd);                    
+            ## store filename in object ##
+            $self->file($tempfile);
+            return $blastobj;
+        } elsif (!$got_content) {
+            # server returned no content, can't be good
+            $self->warn("Server failed to return any data");
+            return -1
+        } else {		# still working
+            return 0;
+        }
+	
+    } else {
+        $self->warn($response->error_as_HTML);
+        return -1;
+    }
 }
 
 =head2 save_output
@@ -600,58 +673,93 @@ sub save_output {
 	}
 	my $blastfile = $self->file;
 	#open temp file and output file, have to filter out some HTML
-	open(TMP, $blastfile) or $self->throw("cannot open $blastfile");
+	open(my $TMP, $blastfile) or $self->throw("cannot open $blastfile");
 
-	open(SAVEOUT, ">$filename") or $self->throw("cannot open $filename");
+	open(my $SAVEOUT, ">", $filename) or $self->throw("cannot open $filename");
 	my $seentop = 0;
-	while(my $l = <TMP>) {
-		next if ($l =~ /<pre>/);
-		if( $l =~ /^(?:[T]?BLAST[NPX])\s*.+$/i ||
-			 $l =~/^RPS-BLAST\s*.+$/i ) {
+	while(<$TMP>) {
+		next if (/<pre>/);
+		if(/^(?:[T]?BLAST[NPX])\s*.+$/i ||
+           /^RPS-BLAST\s*.+$/i ||
+           /<\?xml\sversion=/ ||
+           /^#\s+(?:[T]?BLAST[NPX])\s*.+$/) {
 			$seentop=1;
-		}
-		next if !$seentop;
+		} 
+        next if !$seentop;
 		if( $seentop ) {
-			print SAVEOUT $l;
+			print $SAVEOUT $_;
 		}
 	}
-	close TMP;
-	close SAVEOUT;
 	return 1;
 }
 
 sub _load_input {
-    my ($self, $input) = @_;
+	my ($self, $input) = @_;
 
-    if( ! defined $input ) {
+	if( ! defined $input ) {
 		$self->throw("Calling remote blast with no input");
-    }
-    my @seqs;
-    if( ! ref $input ) {
-	if( -e $input ) {
-	    my $seqio = new Bio::SeqIO(-format => 'fasta', -file => $input);
-	    while( my $seq = $seqio->next_seq ) {
-		push @seqs, $seq;
-	    }
-	} else {
-	    $self->throw("Input $input was not a valid filename");
-	}	
-    } elsif( ref($input) =~ /ARRAY/i ) {
-	foreach ( @$input ) {
-	    if( ref($_) && $_->isa('Bio::PrimarySeqI') ) {
-		push @seqs, $_;
-	    } else {
-		$self->warn("Trying to add a " . ref($_) .
-			    " but expected a Bio::PrimarySeqI");
-	    }
 	}
-	if( ! @seqs) {
-	    $self->throw("Did not pass in valid input -- no sequence objects found");
+	my @seqs;
+	if( ! ref $input ) {
+		if( -e $input ) {
+			my $seqio = Bio::SeqIO->new(-format => 'fasta',
+												-file => $input);
+			while( my $seq = $seqio->next_seq ) {
+				push @seqs, $seq;
+			}
+		} else {
+			$self->throw("Input $input was not a valid filename");
+		}
+	} elsif( ref($input) =~ /ARRAY/i ) {
+		foreach ( @$input ) {
+			if( ref($_) && $_->isa('Bio::PrimarySeqI') ) {
+				push @seqs, $_;
+			} else {
+				$self->warn("Trying to add a " . ref($_) .
+								" but expected a Bio::PrimarySeqI");
+			}
+		}
+		if( ! @seqs) {
+			$self->throw("Did not pass in valid input -- no sequence objects found");
+		}
+	} elsif( $input->isa('Bio::PrimarySeqI') ) {
+		push @seqs, $input;
 	}
-    } elsif( $input->isa('Bio::PrimarySeqI') ) {
-	push @seqs, $input;
-    }
-    return @seqs;
+	return @seqs;
 }
+
+=head2 set_url_base
+
+ Title   : set_url_base
+ Usage   : $self->set_url_base($url)
+ Function: Method to override the default NCBI BLAST database
+ Returns : None
+ Args    : string (database url like
+ NOTE    : This is highly experimental; we cannot maintain support on
+           databases other than the default NCBI database at this time
+
+=cut
+
+sub set_url_base {
+    my $self = shift;
+    $self->{'_urlbase'} = shift if @_;
+}
+
+=head2 get_url_base
+
+ Title   : get_url_base
+ Usage   : my $url = $self->set_url_base
+ Function: Get the current URL for BLAST database searching
+ Returns : string (URL used for remote blast searches)
+ Args    : None
+
+=cut
+
+sub get_url_base {
+    my $self = shift;
+    return $self->{'_urlbase'};
+}
+
 1;
+
 __END__
