@@ -1,7 +1,9 @@
-# $Id: flat.pm,v 1.13 2003/06/04 08:36:44 heikki Exp $
+# $Id: flat.pm 16123 2009-09-17 12:57:27Z cjfields $
 # BioPerl module for Bio::Variation::IO::flat
 #
-# Cared for by Heikki Lehvaslaiho <Heikki@ebi.ac.uk>
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
+# Cared for by Heikki Lehvaslaiho <heikki-at-bioperl-dot-org>
 #
 # Copyright Heikki Lehvaslaiho
 #
@@ -16,18 +18,12 @@ Bio::Variation::IO::flat - flat file sequence variation input/output stream
 
 =head1 SYNOPSIS
 
-Do not use this module directly.  Use it via the Bio::Variation::IO class.
+Do not use this module directly. Use it via the Bio::Variation::IO class.
 
 =head1 DESCRIPTION
 
 This object can transform Bio::Variation::SeqDiff objects to and from
-flat file databases. The format used is EMBL like extension of what is
-used by the "EBI Mutation Checker" at
-http://www.ebi.ac.uk/cgi-bin/mutations/check.cgi and will eventually
-replace it.
-
-More information of the attributes and values use can be found at
-http://www.ebi.ac.uk/mutations/recommendations/.
+flat file databases.
 
 =head1 FEEDBACK
 
@@ -37,28 +33,31 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to the 
 Bioperl mailing lists  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                         - General discussion
-  http://bio.perl.org/MailList.html             - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
+
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
 
 =head2 Reporting Bugs
 
 report bugs to the Bioperl bug tracking system to help us keep track
- the bugs and their resolution.  Bug reports can be submitted via
- email or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Heikki Lehvaslaiho
 
-Email:  heikki@ebi.ac.uk
-Address: 
-
-     EMBL Outstation, European Bioinformatics Institute
-     Wellcome Trust Genome Campus, Hinxton
-     Cambs. CB10 1SD, United Kingdom 
-
-
+Email:  heikki-at-bioperl-dot-org
 
 =head1 APPENDIX
 
@@ -71,11 +70,9 @@ methods. Internal methods are usually preceded with a _
 
 package Bio::Variation::IO::flat;
 
-use vars qw(@ISA);
 use strict;
 
 use Text::Wrap;
-use Bio::Variation::IO;
 use Bio::Variation::SeqDiff;
 use Bio::Variation::DNAMutation;
 use Bio::Variation::RNAChange;
@@ -83,7 +80,7 @@ use Bio::Variation::AAChange;
 use Bio::Variation::Allele;
 
 
-@ISA = qw(Bio::Variation::IO);
+use base qw(Bio::Variation::IO);
 
 sub new {
     my($class, @args) = @_;
@@ -165,7 +162,7 @@ sub next {
 	    #$self->warn($s);
 	    #exit;
 	    my ($start, $sep, $end) = $location =~ /(-?\d+)(.)?\D?(-?\d+)?/;
-	    $end = $start if not $end ;
+	    $end = $start if not defined $end ;
 	    my ($len) = $end - $start +1; 
 	    $len = 0, $start = $end if defined $sep and $sep eq '^';
 	    my $ismut = 0;
@@ -230,7 +227,7 @@ sub next {
 	    $prevrnaobj->add_Allele($a3);
 	} else {
 	    my ($start, $sep, $end) = $location =~ /(-?\d+)(.)?\D?(-?\d+)?/;
-	    $end = $start if not $end ;
+	    $end = $start if not defined $end ;
 	    my ($len) = $end - $start + 1; 
 	    $len = 0, $start = $end if defined $sep and $sep eq '^'; 
 	    my $ismut;
@@ -305,7 +302,7 @@ sub next {
 	    $prevaaobj->add_Allele($a3);
 	} else {
 	    my ($start, $sep, $end) = $location =~ /(-?\d+)(.)?\D?(-?\d+)?/;
-	    $end = $start if not $end ;
+	    $end = $start if not defined $end ;
 	    my ($len) = $end - $start + 1; 
 	    $len = 0, $start = $end if defined $sep and $sep eq '^'; 
 	    my $ismut;
